// Platform: android

// commit cd29cf0f224ccf25e9d422a33fd02ef67d3a78f4

// File generated at :: Thu Apr 25 2013 14:53:10 GMT-0700 (PDT)

/*
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at
 
     http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
*/

;(function() {

// file: lib/scripts/require.js

var require,
    define;

(function () {
    var modules = {};
    // Stack of moduleIds currently being built.
    var requireStack = [];
    // Map of module ID -> index into requireStack of modules currently being built.
    var inProgressModules = {};

    function build(module) {
        var factory = module.factory;
        module.exports = {};
        delete module.factory;
        factory(require, module.exports, module);
        return module.exports;
    }

    require = function (id) {
        if (!modules[id]) {
            throw "module " + id + " not found";
        } else if (id in inProgressModules) {
            var cycle = requireStack.slice(inProgressModules[id]).join('->') + '->' + id;
            throw "Cycle in require graph: " + cycle;
        }
        if (modules[id].factory) {
            try {
                inProgressModules[id] = requireStack.length;
                requireStack.push(id);
                return build(modules[id]);
            } finally {
                delete inProgressModules[id];
                requireStack.pop();
            }
        }
        return modules[id].exports;
    };

    define = function (id, factory) {
        if (modules[id]) {
            throw "module " + id + " already defined";
        }

        modules[id] = {
            id: id,
            factory: factory
        };
    };

    define.remove = function (id) {
        delete modules[id];
    };

    define.moduleMap = modules;
})();

//Export for use in node
if (typeof module === "object" && typeof require === "function") {
    module.exports.require = require;
    module.exports.define = define;
}

// file: lib/cordova.js
define("cordova", function(require, exports, module) {


var channel = require('cordova/channel');

/**
 * Listen for DOMContentLoaded and notify our channel subscribers.
 */
document.addEventListener('DOMContentLoaded', function() {
    channel.onDOMContentLoaded.fire();
}, false);
if (document.readyState == 'complete' || document.readyState == 'interactive') {
    channel.onDOMContentLoaded.fire();
}

/**
 * Intercept calls to addEventListener + removeEventListener and handle deviceready,
 * resume, and pause events.
 */
var m_document_addEventListener = document.addEventListener;
var m_document_removeEventListener = document.removeEventListener;
var m_window_addEventListener = window.addEventListener;
var m_window_removeEventListener = window.removeEventListener;

/**
 * Houses custom event handlers to intercept on document + window event listeners.
 */
var documentEventHandlers = {},
    windowEventHandlers = {};

document.addEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
    if (typeof documentEventHandlers[e] != 'undefined') {
        documentEventHandlers[e].subscribe(handler);
    } else {
        m_document_addEventListener.call(document, evt, handler, capture);
    }
};

window.addEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
    if (typeof windowEventHandlers[e] != 'undefined') {
        windowEventHandlers[e].subscribe(handler);
    } else {
        m_window_addEventListener.call(window, evt, handler, capture);
    }
};

document.removeEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
    // If unsubscribing from an event that is handled by a plugin
    if (typeof documentEventHandlers[e] != "undefined") {
        documentEventHandlers[e].unsubscribe(handler);
    } else {
        m_document_removeEventListener.call(document, evt, handler, capture);
    }
};

window.removeEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
    // If unsubscribing from an event that is handled by a plugin
    if (typeof windowEventHandlers[e] != "undefined") {
        windowEventHandlers[e].unsubscribe(handler);
    } else {
        m_window_removeEventListener.call(window, evt, handler, capture);
    }
};

function createEvent(type, data) {
    var event = document.createEvent('Events');
    event.initEvent(type, false, false);
    if (data) {
        for (var i in data) {
            if (data.hasOwnProperty(i)) {
                event[i] = data[i];
            }
        }
    }
    return event;
}

if(typeof window.console === "undefined") {
    window.console = {
        log:function(){}
    };
}

var cordova = {
    define:define,
    require:require,
    /**
     * Methods to add/remove your own addEventListener hijacking on document + window.
     */
    addWindowEventHandler:function(event) {
        return (windowEventHandlers[event] = channel.create(event));
    },
    addStickyDocumentEventHandler:function(event) {
        return (documentEventHandlers[event] = channel.createSticky(event));
    },
    addDocumentEventHandler:function(event) {
        return (documentEventHandlers[event] = channel.create(event));
    },
    removeWindowEventHandler:function(event) {
        delete windowEventHandlers[event];
    },
    removeDocumentEventHandler:function(event) {
        delete documentEventHandlers[event];
    },
    /**
     * Retrieve original event handlers that were replaced by Cordova
     *
     * @return object
     */
    getOriginalHandlers: function() {
        return {'document': {'addEventListener': m_document_addEventListener, 'removeEventListener': m_document_removeEventListener},
        'window': {'addEventListener': m_window_addEventListener, 'removeEventListener': m_window_removeEventListener}};
    },
    /**
     * Method to fire event from native code
     * bNoDetach is required for events which cause an exception which needs to be caught in native code
     */
    fireDocumentEvent: function(type, data, bNoDetach) {
        var evt = createEvent(type, data);
        if (typeof documentEventHandlers[type] != 'undefined') {
            if( bNoDetach ) {
              documentEventHandlers[type].fire(evt);
            }
            else {
              setTimeout(function() {
                  // Fire deviceready on listeners that were registered before cordova.js was loaded.
                  if (type == 'deviceready') {
                      document.dispatchEvent(evt);
                  }
                  documentEventHandlers[type].fire(evt);
              }, 0);
            }
        } else {
            document.dispatchEvent(evt);
        }
    },
    fireWindowEvent: function(type, data) {
        var evt = createEvent(type,data);
        if (typeof windowEventHandlers[type] != 'undefined') {
            setTimeout(function() {
                windowEventHandlers[type].fire(evt);
            }, 0);
        } else {
            window.dispatchEvent(evt);
        }
    },

    /**
     * Plugin callback mechanism.
     */
    // Randomize the starting callbackId to avoid collisions after refreshing or navigating.
    // This way, it's very unlikely that any new callback would get the same callbackId as an old callback.
    callbackId: Math.floor(Math.random() * 2000000000),
    callbacks:  {},
    callbackStatus: {
        NO_RESULT: 0,
        OK: 1,
        CLASS_NOT_FOUND_EXCEPTION: 2,
        ILLEGAL_ACCESS_EXCEPTION: 3,
        INSTANTIATION_EXCEPTION: 4,
        MALFORMED_URL_EXCEPTION: 5,
        IO_EXCEPTION: 6,
        INVALID_ACTION: 7,
        JSON_EXCEPTION: 8,
        ERROR: 9
    },

    /**
     * Called by native code when returning successful result from an action.
     */
    callbackSuccess: function(callbackId, args) {
        try {
            cordova.callbackFromNative(callbackId, true, args.status, [args.message], args.keepCallback);
        } catch (e) {
            console.log("Error in error callback: " + callbackId + " = "+e);
        }
    },

    /**
     * Called by native code when returning error result from an action.
     */
    callbackError: function(callbackId, args) {
        // TODO: Deprecate callbackSuccess and callbackError in favour of callbackFromNative.
        // Derive success from status.
        try {
            cordova.callbackFromNative(callbackId, false, args.status, [args.message], args.keepCallback);
        } catch (e) {
            console.log("Error in error callback: " + callbackId + " = "+e);
        }
    },

    /**
     * Called by native code when returning the result from an action.
     */
    callbackFromNative: function(callbackId, success, status, args, keepCallback) {
        var callback = cordova.callbacks[callbackId];
        if (callback) {
            if (success && status == cordova.callbackStatus.OK) {
                callback.success && callback.success.apply(null, args);
            } else if (!success) {
                callback.fail && callback.fail.apply(null, args);
            }

            // Clear callback if not expecting any more results
            if (!keepCallback) {
                delete cordova.callbacks[callbackId];
            }
        }
    },
    addConstructor: function(func) {
        channel.onCordovaReady.subscribe(function() {
            try {
                func();
            } catch(e) {
                console.log("Failed to run constructor: " + e);
            }
        });
    }
};

// Register pause, resume and deviceready channels as events on document.
channel.onPause = cordova.addDocumentEventHandler('pause');
channel.onResume = cordova.addDocumentEventHandler('resume');
channel.onDeviceReady = cordova.addStickyDocumentEventHandler('deviceready');

module.exports = cordova;

});

// file: lib/common/argscheck.js
define("cordova/argscheck", function(require, exports, module) {

var exec = require('cordova/exec');
var utils = require('cordova/utils');

var moduleExports = module.exports;

var typeMap = {
    'A': 'Array',
    'D': 'Date',
    'N': 'Number',
    'S': 'String',
    'F': 'Function',
    'O': 'Object'
};

function extractParamName(callee, argIndex) {
  return (/.*?\((.*?)\)/).exec(callee)[1].split(', ')[argIndex];
}

function checkArgs(spec, functionName, args, opt_callee) {
    if (!moduleExports.enableChecks) {
        return;
    }
    var errMsg = null;
    var typeName;
    for (var i = 0; i < spec.length; ++i) {
        var c = spec.charAt(i),
            cUpper = c.toUpperCase(),
            arg = args[i];
        // Asterix means allow anything.
        if (c == '*') {
            continue;
        }
        typeName = utils.typeName(arg);
        if ((arg === null || arg === undefined) && c == cUpper) {
            continue;
        }
        if (typeName != typeMap[cUpper]) {
            errMsg = 'Expected ' + typeMap[cUpper];
            break;
        }
    }
    if (errMsg) {
        errMsg += ', but got ' + typeName + '.';
        errMsg = 'Wrong type for parameter "' + extractParamName(opt_callee || args.callee, i) + '" of ' + functionName + ': ' + errMsg;
        // Don't log when running jake test.
        if (typeof jasmine == 'undefined') {
            console.error(errMsg);
        }
        throw TypeError(errMsg);
    }
}

function getValue(value, defaultValue) {
    return value === undefined ? defaultValue : value;
}

moduleExports.checkArgs = checkArgs;
moduleExports.getValue = getValue;
moduleExports.enableChecks = true;


});

// file: lib/common/builder.js
define("cordova/builder", function(require, exports, module) {

var utils = require('cordova/utils');

function each(objects, func, context) {
    for (var prop in objects) {
        if (objects.hasOwnProperty(prop)) {
            func.apply(context, [objects[prop], prop]);
        }
    }
}

function clobber(obj, key, value) {
    exports.replaceHookForTesting(obj, key);
    obj[key] = value;
    // Getters can only be overridden by getters.
    if (obj[key] !== value) {
        utils.defineGetter(obj, key, function() {
            return value;
        });
    }
}

function assignOrWrapInDeprecateGetter(obj, key, value, message) {
    if (message) {
        utils.defineGetter(obj, key, function() {
            console.log(message);
            delete obj[key];
            clobber(obj, key, value);
            return value;
        });
    } else {
        clobber(obj, key, value);
    }
}

function include(parent, objects, clobber, merge) {
    each(objects, function (obj, key) {
        try {
          var result = obj.path ? require(obj.path) : {};

          if (clobber) {
              // Clobber if it doesn't exist.
              if (typeof parent[key] === 'undefined') {
                  assignOrWrapInDeprecateGetter(parent, key, result, obj.deprecated);
              } else if (typeof obj.path !== 'undefined') {
                  // If merging, merge properties onto parent, otherwise, clobber.
                  if (merge) {
                      recursiveMerge(parent[key], result);
                  } else {
                      assignOrWrapInDeprecateGetter(parent, key, result, obj.deprecated);
                  }
              }
              result = parent[key];
          } else {
            // Overwrite if not currently defined.
            if (typeof parent[key] == 'undefined') {
              assignOrWrapInDeprecateGetter(parent, key, result, obj.deprecated);
            } else {
              // Set result to what already exists, so we can build children into it if they exist.
              result = parent[key];
            }
          }

          if (obj.children) {
            include(result, obj.children, clobber, merge);
          }
        } catch(e) {
          utils.alert('Exception building cordova JS globals: ' + e + ' for key "' + key + '"');
        }
    });
}

/**
 * Merge properties from one object onto another recursively.  Properties from
 * the src object will overwrite existing target property.
 *
 * @param target Object to merge properties into.
 * @param src Object to merge properties from.
 */
function recursiveMerge(target, src) {
    for (var prop in src) {
        if (src.hasOwnProperty(prop)) {
            if (target.prototype && target.prototype.constructor === target) {
                // If the target object is a constructor override off prototype.
                clobber(target.prototype, prop, src[prop]);
            } else {
                if (typeof src[prop] === 'object' && typeof target[prop] === 'object') {
                    recursiveMerge(target[prop], src[prop]);
                } else {
                    clobber(target, prop, src[prop]);
                }
            }
        }
    }
}

exports.buildIntoButDoNotClobber = function(objects, target) {
    include(target, objects, false, false);
};
exports.buildIntoAndClobber = function(objects, target) {
    include(target, objects, true, false);
};
exports.buildIntoAndMerge = function(objects, target) {
    include(target, objects, true, true);
};
exports.recursiveMerge = recursiveMerge;
exports.assignOrWrapInDeprecateGetter = assignOrWrapInDeprecateGetter;
exports.replaceHookForTesting = function() {};

});

// file: lib/common/channel.js
define("cordova/channel", function(require, exports, module) {

var utils = require('cordova/utils'),
    nextGuid = 1;

/**
 * Custom pub-sub "channel" that can have functions subscribed to it
 * This object is used to define and control firing of events for
 * cordova initialization, as well as for custom events thereafter.
 *
 * The order of events during page load and Cordova startup is as follows:
 *
 * onDOMContentLoaded*         Internal event that is received when the web page is loaded and parsed.
 * onNativeReady*              Internal event that indicates the Cordova native side is ready.
 * onCordovaReady*             Internal event fired when all Cordova JavaScript objects have been created.
 * onCordovaInfoReady*         Internal event fired when device properties are available.
 * onCordovaConnectionReady*   Internal event fired when the connection property has been set.
 * onDeviceReady*              User event fired to indicate that Cordova is ready
 * onResume                    User event fired to indicate a start/resume lifecycle event
 * onPause                     User event fired to indicate a pause lifecycle event
 * onDestroy*                  Internal event fired when app is being destroyed (User should use window.onunload event, not this one).
 *
 * The events marked with an * are sticky. Once they have fired, they will stay in the fired state.
 * All listeners that subscribe after the event is fired will be executed right away.
 *
 * The only Cordova events that user code should register for are:
 *      deviceready           Cordova native code is initialized and Cordova APIs can be called from JavaScript
 *      pause                 App has moved to background
 *      resume                App has returned to foreground
 *
 * Listeners can be registered as:
 *      document.addEventListener("deviceready", myDeviceReadyListener, false);
 *      document.addEventListener("resume", myResumeListener, false);
 *      document.addEventListener("pause", myPauseListener, false);
 *
 * The DOM lifecycle events should be used for saving and restoring state
 *      window.onload
 *      window.onunload
 *
 */

/**
 * Channel
 * @constructor
 * @param type  String the channel name
 */
var Channel = function(type, sticky) {
    this.type = type;
    // Map of guid -> function.
    this.handlers = {};
    // 0 = Non-sticky, 1 = Sticky non-fired, 2 = Sticky fired.
    this.state = sticky ? 1 : 0;
    // Used in sticky mode to remember args passed to fire().
    this.fireArgs = null;
    // Used by onHasSubscribersChange to know if there are any listeners.
    this.numHandlers = 0;
    // Function that is called when the first listener is subscribed, or when
    // the last listener is unsubscribed.
    this.onHasSubscribersChange = null;
},
    channel = {
        /**
         * Calls the provided function only after all of the channels specified
         * have been fired. All channels must be sticky channels.
         */
        join: function(h, c) {
            var len = c.length,
                i = len,
                f = function() {
                    if (!(--i)) h();
                };
            for (var j=0; j<len; j++) {
                if (c[j].state === 0) {
                    throw Error('Can only use join with sticky channels.');
                }
                c[j].subscribe(f);
            }
            if (!len) h();
        },
        create: function(type) {
            return channel[type] = new Channel(type, false);
        },
        createSticky: function(type) {
            return channel[type] = new Channel(type, true);
        },

        /**
         * cordova Channels that must fire before "deviceready" is fired.
         */
        deviceReadyChannelsArray: [],
        deviceReadyChannelsMap: {},

        /**
         * Indicate that a feature needs to be initialized before it is ready to be used.
         * This holds up Cordova's "deviceready" event until the feature has been initialized
         * and Cordova.initComplete(feature) is called.
         *
         * @param feature {String}     The unique feature name
         */
        waitForInitialization: function(feature) {
            if (feature) {
                var c = channel[feature] || this.createSticky(feature);
                this.deviceReadyChannelsMap[feature] = c;
                this.deviceReadyChannelsArray.push(c);
            }
        },

        /**
         * Indicate that initialization code has completed and the feature is ready to be used.
         *
         * @param feature {String}     The unique feature name
         */
        initializationComplete: function(feature) {
            var c = this.deviceReadyChannelsMap[feature];
            if (c) {
                c.fire();
            }
        }
    };

function forceFunction(f) {
    if (typeof f != 'function') throw "Function required as first argument!";
}

/**
 * Subscribes the given function to the channel. Any time that
 * Channel.fire is called so too will the function.
 * Optionally specify an execution context for the function
 * and a guid that can be used to stop subscribing to the channel.
 * Returns the guid.
 */
Channel.prototype.subscribe = function(f, c) {
    // need a function to call
    forceFunction(f);
    if (this.state == 2) {
        f.apply(c || this, this.fireArgs);
        return;
    }

    var func = f,
        guid = f.observer_guid;
    if (typeof c == "object") { func = utils.close(c, f); }

    if (!guid) {
        // first time any channel has seen this subscriber
        guid = '' + nextGuid++;
    }
    func.observer_guid = guid;
    f.observer_guid = guid;

    // Don't add the same handler more than once.
    if (!this.handlers[guid]) {
        this.handlers[guid] = func;
        this.numHandlers++;
        if (this.numHandlers == 1) {
            this.onHasSubscribersChange && this.onHasSubscribersChange();
        }
    }
};

/**
 * Unsubscribes the function with the given guid from the channel.
 */
Channel.prototype.unsubscribe = function(f) {
    // need a function to unsubscribe
    forceFunction(f);

    var guid = f.observer_guid,
        handler = this.handlers[guid];
    if (handler) {
        delete this.handlers[guid];
        this.numHandlers--;
        if (this.numHandlers === 0) {
            this.onHasSubscribersChange && this.onHasSubscribersChange();
        }
    }
};

/**
 * Calls all functions subscribed to this channel.
 */
Channel.prototype.fire = function(e) {
    var fail = false,
        fireArgs = Array.prototype.slice.call(arguments);
    // Apply stickiness.
    if (this.state == 1) {
        this.state = 2;
        this.fireArgs = fireArgs;
    }
    if (this.numHandlers) {
        // Copy the values first so that it is safe to modify it from within
        // callbacks.
        var toCall = [];
        for (var item in this.handlers) {
            toCall.push(this.handlers[item]);
        }
        for (var i = 0; i < toCall.length; ++i) {
            toCall[i].apply(this, fireArgs);
        }
        if (this.state == 2 && this.numHandlers) {
            this.numHandlers = 0;
            this.handlers = {};
            this.onHasSubscribersChange && this.onHasSubscribersChange();
        }
    }
};


// defining them here so they are ready super fast!
// DOM event that is received when the web page is loaded and parsed.
channel.createSticky('onDOMContentLoaded');

// Event to indicate the Cordova native side is ready.
channel.createSticky('onNativeReady');

// Event to indicate that all Cordova JavaScript objects have been created
// and it's time to run plugin constructors.
channel.createSticky('onCordovaReady');

// Event to indicate that device properties are available
channel.createSticky('onCordovaInfoReady');

// Event to indicate that the connection property has been set.
channel.createSticky('onCordovaConnectionReady');

// Event to indicate that all automatically loaded JS plugins are loaded and ready.
channel.createSticky('onPluginsReady');

// Event to indicate that Cordova is ready
channel.createSticky('onDeviceReady');

// Event to indicate a resume lifecycle event
channel.create('onResume');

// Event to indicate a pause lifecycle event
channel.create('onPause');

// Event to indicate a destroy lifecycle event
channel.createSticky('onDestroy');

// Channels that must fire before "deviceready" is fired.
channel.waitForInitialization('onCordovaReady');
channel.waitForInitialization('onCordovaConnectionReady');
channel.waitForInitialization('onDOMContentLoaded');

module.exports = channel;

});

// file: lib/common/commandProxy.js
define("cordova/commandProxy", function(require, exports, module) {


// internal map of proxy function
var CommandProxyMap = {};

module.exports = {

    // example: cordova.commandProxy.add("Accelerometer",{getCurrentAcceleration: function(successCallback, errorCallback, options) {...},...);
    add:function(id,proxyObj) {
        console.log("adding proxy for " + id);
        CommandProxyMap[id] = proxyObj;
        return proxyObj;
    },

    // cordova.commandProxy.remove("Accelerometer");
    remove:function(id) {
        var proxy = CommandProxyMap[id];
        delete CommandProxyMap[id];
        CommandProxyMap[id] = null;
        return proxy;
    },

    get:function(service,action) {
        return ( CommandProxyMap[service] ? CommandProxyMap[service][action] : null );
    }
};
});

// file: lib/android/exec.js
define("cordova/exec", function(require, exports, module) {

/**
 * Execute a cordova command.  It is up to the native side whether this action
 * is synchronous or asynchronous.  The native side can return:
 *      Synchronous: PluginResult object as a JSON string
 *      Asynchronous: Empty string ""
 * If async, the native side will cordova.callbackSuccess or cordova.callbackError,
 * depending upon the result of the action.
 *
 * @param {Function} success    The success callback
 * @param {Function} fail       The fail callback
 * @param {String} service      The name of the service to use
 * @param {String} action       Action to be run in cordova
 * @param {String[]} [args]     Zero or more arguments to pass to the method
 */
var cordova = require('cordova'),
    nativeApiProvider = require('cordova/plugin/android/nativeapiprovider'),
    utils = require('cordova/utils'),
    jsToNativeModes = {
        PROMPT: 0,
        JS_OBJECT: 1,
        // This mode is currently for benchmarking purposes only. It must be enabled
        // on the native side through the ENABLE_LOCATION_CHANGE_EXEC_MODE
        // constant within CordovaWebViewClient.java before it will work.
        LOCATION_CHANGE: 2
    },
    nativeToJsModes = {
        // Polls for messages using the JS->Native bridge.
        POLLING: 0,
        // For LOAD_URL to be viable, it would need to have a work-around for
        // the bug where the soft-keyboard gets dismissed when a message is sent.
        LOAD_URL: 1,
        // For the ONLINE_EVENT to be viable, it would need to intercept all event
        // listeners (both through addEventListener and window.ononline) as well
        // as set the navigator property itself.
        ONLINE_EVENT: 2,
        // Uses reflection to access private APIs of the WebView that can send JS
        // to be executed.
        // Requires Android 3.2.4 or above.
        PRIVATE_API: 3
    },
    jsToNativeBridgeMode,  // Set lazily.
    nativeToJsBridgeMode = nativeToJsModes.ONLINE_EVENT,
    pollEnabled = false,
    messagesFromNative = [];

function androidExec(success, fail, service, action, args) {
    // Set default bridge modes if they have not already been set.
    // By default, we use the failsafe, since addJavascriptInterface breaks too often
    if (jsToNativeBridgeMode === undefined) {
        androidExec.setJsToNativeBridgeMode(jsToNativeModes.JS_OBJECT);
    }

    // Process any ArrayBuffers in the args into a string.
    for (var i = 0; i < args.length; i++) {
        if (utils.typeName(args[i]) == 'ArrayBuffer') {
            args[i] = window.btoa(String.fromCharCode.apply(null, new Uint8Array(args[i])));
        }
    }

    var callbackId = service + cordova.callbackId++,
        argsJson = JSON.stringify(args);

    if (success || fail) {
        cordova.callbacks[callbackId] = {success:success, fail:fail};
    }

    if (jsToNativeBridgeMode == jsToNativeModes.LOCATION_CHANGE) {
        window.location = 'http://cdv_exec/' + service + '#' + action + '#' + callbackId + '#' + argsJson;
    } else {
        var messages = nativeApiProvider.get().exec(service, action, callbackId, argsJson);
        // If argsJson was received by Java as null, try again with the PROMPT bridge mode.
        // This happens in rare circumstances, such as when certain Unicode characters are passed over the bridge on a Galaxy S2.  See CB-2666.
        if (jsToNativeBridgeMode == jsToNativeModes.JS_OBJECT && messages === "@Null arguments.") {
            androidExec.setJsToNativeBridgeMode(jsToNativeModes.PROMPT);
            androidExec(success, fail, service, action, args);
            androidExec.setJsToNativeBridgeMode(jsToNativeModes.JS_OBJECT);
            return;
        } else {
            androidExec.processMessages(messages);
        }
    }
}

function pollOnce() {
    var msg = nativeApiProvider.get().retrieveJsMessages();
    androidExec.processMessages(msg);
}

function pollingTimerFunc() {
    if (pollEnabled) {
        pollOnce();
        setTimeout(pollingTimerFunc, 50);
    }
}

function hookOnlineApis() {
    function proxyEvent(e) {
        cordova.fireWindowEvent(e.type);
    }
    // The network module takes care of firing online and offline events.
    // It currently fires them only on document though, so we bridge them
    // to window here (while first listening for exec()-releated online/offline
    // events).
    window.addEventListener('online', pollOnce, false);
    window.addEventListener('offline', pollOnce, false);
    cordova.addWindowEventHandler('online');
    cordova.addWindowEventHandler('offline');
    document.addEventListener('online', proxyEvent, false);
    document.addEventListener('offline', proxyEvent, false);
}

hookOnlineApis();

androidExec.jsToNativeModes = jsToNativeModes;
androidExec.nativeToJsModes = nativeToJsModes;

androidExec.setJsToNativeBridgeMode = function(mode) {
    if (mode == jsToNativeModes.JS_OBJECT && !window._cordovaNative) {
        console.log('Falling back on PROMPT mode since _cordovaNative is missing. Expected for Android 3.2 and lower only.');
        mode = jsToNativeModes.PROMPT;
    }
    nativeApiProvider.setPreferPrompt(mode == jsToNativeModes.PROMPT);
    jsToNativeBridgeMode = mode;
};

androidExec.setNativeToJsBridgeMode = function(mode) {
    if (mode == nativeToJsBridgeMode) {
        return;
    }
    if (nativeToJsBridgeMode == nativeToJsModes.POLLING) {
        pollEnabled = false;
    }

    nativeToJsBridgeMode = mode;
    // Tell the native side to switch modes.
    nativeApiProvider.get().setNativeToJsBridgeMode(mode);

    if (mode == nativeToJsModes.POLLING) {
        pollEnabled = true;
        setTimeout(pollingTimerFunc, 1);
    }
};

// Processes a single message, as encoded by NativeToJsMessageQueue.java.
function processMessage(message) {
    try {
        var firstChar = message.charAt(0);
        if (firstChar == 'J') {
            eval(message.slice(1));
        } else if (firstChar == 'S' || firstChar == 'F') {
            var success = firstChar == 'S';
            var keepCallback = message.charAt(1) == '1';
            var spaceIdx = message.indexOf(' ', 2);
            var status = +message.slice(2, spaceIdx);
            var nextSpaceIdx = message.indexOf(' ', spaceIdx + 1);
            var callbackId = message.slice(spaceIdx + 1, nextSpaceIdx);
            var payloadKind = message.charAt(nextSpaceIdx + 1);
            var payload;
            if (payloadKind == 's') {
                payload = message.slice(nextSpaceIdx + 2);
            } else if (payloadKind == 't') {
                payload = true;
            } else if (payloadKind == 'f') {
                payload = false;
            } else if (payloadKind == 'N') {
                payload = null;
            } else if (payloadKind == 'n') {
                payload = +message.slice(nextSpaceIdx + 2);
            } else if (payloadKind == 'A') {
                var data = message.slice(nextSpaceIdx + 2);
                var bytes = window.atob(data);
                var arraybuffer = new Uint8Array(bytes.length);
                for (var i = 0; i < bytes.length; i++) {
                    arraybuffer[i] = bytes.charCodeAt(i);
                }
                payload = arraybuffer.buffer;
            } else if (payloadKind == 'S') {
                payload = window.atob(message.slice(nextSpaceIdx + 2));
            } else {
                payload = JSON.parse(message.slice(nextSpaceIdx + 1));
            }
            cordova.callbackFromNative(callbackId, success, status, [payload], keepCallback);
        } else {
            console.log("processMessage failed: invalid message:" + message);
        }
    } catch (e) {
        console.log("processMessage failed: Message: " + message);
        console.log("processMessage failed: Error: " + e);
        console.log("processMessage failed: Stack: " + e.stack);
    }
}

// This is called from the NativeToJsMessageQueue.java.
androidExec.processMessages = function(messages) {
    if (messages) {
        messagesFromNative.push(messages);
        // Check for the reentrant case, and enqueue the message if that's the case.
        if (messagesFromNative.length > 1) {
            return;
        }
        while (messagesFromNative.length) {
            // Don't unshift until the end so that reentrancy can be detected.
            messages = messagesFromNative[0];
            // The Java side can send a * message to indicate that it
            // still has messages waiting to be retrieved.
            if (messages == '*') {
                messagesFromNative.shift();
                window.setTimeout(pollOnce, 0);
                return;
            }

            var spaceIdx = messages.indexOf(' ');
            var msgLen = +messages.slice(0, spaceIdx);
            var message = messages.substr(spaceIdx + 1, msgLen);
            messages = messages.slice(spaceIdx + msgLen + 1);
            processMessage(message);
            if (messages) {
                messagesFromNative[0] = messages;
            } else {
                messagesFromNative.shift();
            }
        }
    }
};

module.exports = androidExec;

});

// file: lib/common/modulemapper.js
define("cordova/modulemapper", function(require, exports, module) {

var builder = require('cordova/builder'),
    moduleMap = define.moduleMap,
    symbolList,
    deprecationMap;

exports.reset = function() {
    symbolList = [];
    deprecationMap = {};
};

function addEntry(strategy, moduleName, symbolPath, opt_deprecationMessage) {
    if (!(moduleName in moduleMap)) {
        throw new Error('Module ' + moduleName + ' does not exist.');
    }
    symbolList.push(strategy, moduleName, symbolPath);
    if (opt_deprecationMessage) {
        deprecationMap[symbolPath] = opt_deprecationMessage;
    }
}

// Note: Android 2.3 does have Function.bind().
exports.clobbers = function(moduleName, symbolPath, opt_deprecationMessage) {
    addEntry('c', moduleName, symbolPath, opt_deprecationMessage);
};

exports.merges = function(moduleName, symbolPath, opt_deprecationMessage) {
    addEntry('m', moduleName, symbolPath, opt_deprecationMessage);
};

exports.defaults = function(moduleName, symbolPath, opt_deprecationMessage) {
    addEntry('d', moduleName, symbolPath, opt_deprecationMessage);
};

function prepareNamespace(symbolPath, context) {
    if (!symbolPath) {
        return context;
    }
    var parts = symbolPath.split('.');
    var cur = context;
    for (var i = 0, part; part = parts[i]; ++i) {
        cur = cur[part] = cur[part] || {};
    }
    return cur;
}

exports.mapModules = function(context) {
    var origSymbols = {};
    context.CDV_origSymbols = origSymbols;
    for (var i = 0, len = symbolList.length; i < len; i += 3) {
        var strategy = symbolList[i];
        var moduleName = symbolList[i + 1];
        var symbolPath = symbolList[i + 2];
        var lastDot = symbolPath.lastIndexOf('.');
        var namespace = symbolPath.substr(0, lastDot);
        var lastName = symbolPath.substr(lastDot + 1);

        var module = require(moduleName);
        var deprecationMsg = symbolPath in deprecationMap ? 'Access made to deprecated symbol: ' + symbolPath + '. ' + deprecationMsg : null;
        var parentObj = prepareNamespace(namespace, context);
        var target = parentObj[lastName];

        if (strategy == 'm' && target) {
            builder.recursiveMerge(target, module);
        } else if ((strategy == 'd' && !target) || (strategy != 'd')) {
            if (!(symbolPath in origSymbols)) {
                origSymbols[symbolPath] = target;
            }
            builder.assignOrWrapInDeprecateGetter(parentObj, lastName, module, deprecationMsg);
        }
    }
};

exports.getOriginalSymbol = function(context, symbolPath) {
    var origSymbols = context.CDV_origSymbols;
    if (origSymbols && (symbolPath in origSymbols)) {
        return origSymbols[symbolPath];
    }
    var parts = symbolPath.split('.');
    var obj = context;
    for (var i = 0; i < parts.length; ++i) {
        obj = obj && obj[parts[i]];
    }
    return obj;
};

exports.loadMatchingModules = function(matchingRegExp) {
    for (var k in moduleMap) {
        if (matchingRegExp.exec(k)) {
            require(k);
        }
    }
};

exports.reset();


});

// file: lib/android/platform.js
define("cordova/platform", function(require, exports, module) {

module.exports = {
    id: "android",
    initialize:function() {
        var channel = require("cordova/channel"),
            cordova = require('cordova'),
            exec = require('cordova/exec'),
            modulemapper = require('cordova/modulemapper');

        modulemapper.loadMatchingModules(/cordova.*\/symbols$/);
        modulemapper.clobbers('cordova/plugin/android/app', 'navigator.app');

        modulemapper.mapModules(window);

        // Inject a listener for the backbutton on the document.
        var backButtonChannel = cordova.addDocumentEventHandler('backbutton');
        backButtonChannel.onHasSubscribersChange = function() {
            // If we just attached the first handler or detached the last handler,
            // let native know we need to override the back button.
            exec(null, null, "App", "overrideBackbutton", [this.numHandlers == 1]);
        };

        // Add hardware MENU and SEARCH button handlers
        cordova.addDocumentEventHandler('menubutton');
        cordova.addDocumentEventHandler('searchbutton');

        // Let native code know we are all done on the JS side.
        // Native code will then un-hide the WebView.
        channel.join(function() {
            exec(null, null, "App", "show", []);
        }, [channel.onCordovaReady]);
    }
};

});

// file: lib/common/plugin/Acceleration.js
define("cordova/plugin/Acceleration", function(require, exports, module) {

var Acceleration = function(x, y, z, timestamp) {
    this.x = x;
    this.y = y;
    this.z = z;
    this.timestamp = timestamp || (new Date()).getTime();
};

module.exports = Acceleration;

});

// file: lib/common/plugin/Camera.js
define("cordova/plugin/Camera", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    Camera = require('cordova/plugin/CameraConstants'),
    CameraPopoverHandle = require('cordova/plugin/CameraPopoverHandle');

var cameraExport = {};

// Tack on the Camera Constants to the base camera plugin.
for (var key in Camera) {
    cameraExport[key] = Camera[key];
}

/**
 * Gets a picture from source defined by "options.sourceType", and returns the
 * image as defined by the "options.destinationType" option.

 * The defaults are sourceType=CAMERA and destinationType=FILE_URI.
 *
 * @param {Function} successCallback
 * @param {Function} errorCallback
 * @param {Object} options
 */
cameraExport.getPicture = function(successCallback, errorCallback, options) {
    argscheck.checkArgs('fFO', 'Camera.getPicture', arguments);
    options = options || {};
    var getValue = argscheck.getValue;

    var quality = getValue(options.quality, 50);
    var destinationType = getValue(options.destinationType, Camera.DestinationType.FILE_URI);
    var sourceType = getValue(options.sourceType, Camera.PictureSourceType.CAMERA);
    var targetWidth = getValue(options.targetWidth, -1);
    var targetHeight = getValue(options.targetHeight, -1);
    var encodingType = getValue(options.encodingType, Camera.EncodingType.JPEG);
    var mediaType = getValue(options.mediaType, Camera.MediaType.PICTURE);
    var allowEdit = !!options.allowEdit;
    var correctOrientation = !!options.correctOrientation;
    var saveToPhotoAlbum = !!options.saveToPhotoAlbum;
    var popoverOptions = getValue(options.popoverOptions, null);
    var cameraDirection = getValue(options.cameraDirection, Camera.Direction.BACK);

    var args = [quality, destinationType, sourceType, targetWidth, targetHeight, encodingType,
                mediaType, allowEdit, correctOrientation, saveToPhotoAlbum, popoverOptions, cameraDirection];

    exec(successCallback, errorCallback, "Camera", "takePicture", args);
    return new CameraPopoverHandle();
};

cameraExport.cleanup = function(successCallback, errorCallback) {
    exec(successCallback, errorCallback, "Camera", "cleanup", []);
};

module.exports = cameraExport;

});

// file: lib/common/plugin/CameraConstants.js
define("cordova/plugin/CameraConstants", function(require, exports, module) {

module.exports = {
  DestinationType:{
    DATA_URL: 0,         // Return base64 encoded string
    FILE_URI: 1,         // Return file uri (content://media/external/images/media/2 for Android)
    NATIVE_URI: 2        // Return native uri (eg. asset-library://... for iOS)
  },
  EncodingType:{
    JPEG: 0,             // Return JPEG encoded image
    PNG: 1               // Return PNG encoded image
  },
  MediaType:{
    PICTURE: 0,          // allow selection of still pictures only. DEFAULT. Will return format specified via DestinationType
    VIDEO: 1,            // allow selection of video only, ONLY RETURNS URL
    ALLMEDIA : 2         // allow selection from all media types
  },
  PictureSourceType:{
    PHOTOLIBRARY : 0,    // Choose image from picture library (same as SAVEDPHOTOALBUM for Android)
    CAMERA : 1,          // Take picture from camera
    SAVEDPHOTOALBUM : 2  // Choose image from picture library (same as PHOTOLIBRARY for Android)
  },
  PopoverArrowDirection:{
      ARROW_UP : 1,        // matches iOS UIPopoverArrowDirection constants to specify arrow location on popover
      ARROW_DOWN : 2,
      ARROW_LEFT : 4,
      ARROW_RIGHT : 8,
      ARROW_ANY : 15
  },
  Direction:{
      BACK: 0,
      FRONT: 1
  }
};

});

// file: lib/common/plugin/CameraPopoverHandle.js
define("cordova/plugin/CameraPopoverHandle", function(require, exports, module) {

var exec = require('cordova/exec');

/**
 * A handle to an image picker popover.
 */
var CameraPopoverHandle = function() {
    this.setPosition = function(popoverOptions) {
        console.log('CameraPopoverHandle.setPosition is only supported on iOS.');
    };
};

module.exports = CameraPopoverHandle;

});

// file: lib/common/plugin/CameraPopoverOptions.js
define("cordova/plugin/CameraPopoverOptions", function(require, exports, module) {

var Camera = require('cordova/plugin/CameraConstants');

/**
 * Encapsulates options for iOS Popover image picker
 */
var CameraPopoverOptions = function(x,y,width,height,arrowDir){
    // information of rectangle that popover should be anchored to
    this.x = x || 0;
    this.y = y || 32;
    this.width = width || 320;
    this.height = height || 480;
    // The direction of the popover arrow
    this.arrowDir = arrowDir || Camera.PopoverArrowDirection.ARROW_ANY;
};

module.exports = CameraPopoverOptions;

});

// file: lib/common/plugin/CaptureAudioOptions.js
define("cordova/plugin/CaptureAudioOptions", function(require, exports, module) {

/**
 * Encapsulates all audio capture operation configuration options.
 */
var CaptureAudioOptions = function(){
    // Upper limit of sound clips user can record. Value must be equal or greater than 1.
    this.limit = 1;
    // Maximum duration of a single sound clip in seconds.
    this.duration = 0;
    // The selected audio mode. Must match with one of the elements in supportedAudioModes array.
    this.mode = null;
};

module.exports = CaptureAudioOptions;

});

// file: lib/common/plugin/CaptureError.js
define("cordova/plugin/CaptureError", function(require, exports, module) {

/**
 * The CaptureError interface encapsulates all errors in the Capture API.
 */
var CaptureError = function(c) {
   this.code = c || null;
};

// Camera or microphone failed to capture image or sound.
CaptureError.CAPTURE_INTERNAL_ERR = 0;
// Camera application or audio capture application is currently serving other capture request.
CaptureError.CAPTURE_APPLICATION_BUSY = 1;
// Invalid use of the API (e.g. limit parameter has value less than one).
CaptureError.CAPTURE_INVALID_ARGUMENT = 2;
// User exited camera application or audio capture application before capturing anything.
CaptureError.CAPTURE_NO_MEDIA_FILES = 3;
// The requested capture operation is not supported.
CaptureError.CAPTURE_NOT_SUPPORTED = 20;

module.exports = CaptureError;

});

// file: lib/common/plugin/CaptureImageOptions.js
define("cordova/plugin/CaptureImageOptions", function(require, exports, module) {

/**
 * Encapsulates all image capture operation configuration options.
 */
var CaptureImageOptions = function(){
    // Upper limit of images user can take. Value must be equal or greater than 1.
    this.limit = 1;
    // The selected image mode. Must match with one of the elements in supportedImageModes array.
    this.mode = null;
};

module.exports = CaptureImageOptions;

});

// file: lib/common/plugin/CaptureVideoOptions.js
define("cordova/plugin/CaptureVideoOptions", function(require, exports, module) {

/**
 * Encapsulates all video capture operation configuration options.
 */
var CaptureVideoOptions = function(){
    // Upper limit of videos user can record. Value must be equal or greater than 1.
    this.limit = 1;
    // Maximum duration of a single video clip in seconds.
    this.duration = 0;
    // The selected video mode. Must match with one of the elements in supportedVideoModes array.
    this.mode = null;
};

module.exports = CaptureVideoOptions;

});

// file: lib/common/plugin/CompassError.js
define("cordova/plugin/CompassError", function(require, exports, module) {

/**
 *  CompassError.
 *  An error code assigned by an implementation when an error has occurred
 * @constructor
 */
var CompassError = function(err) {
    this.code = (err !== undefined ? err : null);
};

CompassError.COMPASS_INTERNAL_ERR = 0;
CompassError.COMPASS_NOT_SUPPORTED = 20;

module.exports = CompassError;

});

// file: lib/common/plugin/CompassHeading.js
define("cordova/plugin/CompassHeading", function(require, exports, module) {

var CompassHeading = function(magneticHeading, trueHeading, headingAccuracy, timestamp) {
  this.magneticHeading = magneticHeading;
  this.trueHeading = trueHeading;
  this.headingAccuracy = headingAccuracy;
  this.timestamp = timestamp || new Date().getTime();
};

module.exports = CompassHeading;

});

// file: lib/common/plugin/ConfigurationData.js
define("cordova/plugin/ConfigurationData", function(require, exports, module) {

/**
 * Encapsulates a set of parameters that the capture device supports.
 */
function ConfigurationData() {
    // The ASCII-encoded string in lower case representing the media type.
    this.type = null;
    // The height attribute represents height of the image or video in pixels.
    // In the case of a sound clip this attribute has value 0.
    this.height = 0;
    // The width attribute represents width of the image or video in pixels.
    // In the case of a sound clip this attribute has value 0
    this.width = 0;
}

module.exports = ConfigurationData;

});

// file: lib/common/plugin/Connection.js
define("cordova/plugin/Connection", function(require, exports, module) {

/**
 * Network status
 */
module.exports = {
        UNKNOWN: "unknown",
        ETHERNET: "ethernet",
        WIFI: "wifi",
        CELL_2G: "2g",
        CELL_3G: "3g",
        CELL_4G: "4g",
        CELL:"cellular",
        NONE: "none"
};

});

// file: lib/common/plugin/Contact.js
define("cordova/plugin/Contact", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    ContactError = require('cordova/plugin/ContactError'),
    utils = require('cordova/utils');

/**
* Converts primitives into Complex Object
* Currently only used for Date fields
*/
function convertIn(contact) {
    var value = contact.birthday;
    try {
      contact.birthday = new Date(parseFloat(value));
    } catch (exception){
      console.log("Cordova Contact convertIn error: exception creating date.");
    }
    return contact;
}

/**
* Converts Complex objects into primitives
* Only conversion at present is for Dates.
**/

function convertOut(contact) {
    var value = contact.birthday;
    if (value !== null) {
        // try to make it a Date object if it is not already
        if (!utils.isDate(value)){
            try {
                value = new Date(value);
            } catch(exception){
                value = null;
            }
        }
        if (utils.isDate(value)){
            value = value.valueOf(); // convert to milliseconds
        }
        contact.birthday = value;
    }
    return contact;
}

/**
* Contains information about a single contact.
* @constructor
* @param {DOMString} id unique identifier
* @param {DOMString} displayName
* @param {ContactName} name
* @param {DOMString} nickname
* @param {Array.<ContactField>} phoneNumbers array of phone numbers
* @param {Array.<ContactField>} emails array of email addresses
* @param {Array.<ContactAddress>} addresses array of addresses
* @param {Array.<ContactField>} ims instant messaging user ids
* @param {Array.<ContactOrganization>} organizations
* @param {DOMString} birthday contact's birthday
* @param {DOMString} note user notes about contact
* @param {Array.<ContactField>} photos
* @param {Array.<ContactField>} categories
* @param {Array.<ContactField>} urls contact's web sites
*/
var Contact = function (id, displayName, name, nickname, phoneNumbers, emails, addresses,
    ims, organizations, birthday, note, photos, categories, urls) {
    this.id = id || null;
    this.rawId = null;
    this.displayName = displayName || null;
    this.name = name || null; // ContactName
    this.nickname = nickname || null;
    this.phoneNumbers = phoneNumbers || null; // ContactField[]
    this.emails = emails || null; // ContactField[]
    this.addresses = addresses || null; // ContactAddress[]
    this.ims = ims || null; // ContactField[]
    this.organizations = organizations || null; // ContactOrganization[]
    this.birthday = birthday || null;
    this.note = note || null;
    this.photos = photos || null; // ContactField[]
    this.categories = categories || null; // ContactField[]
    this.urls = urls || null; // ContactField[]
};

/**
* Removes contact from device storage.
* @param successCB success callback
* @param errorCB error callback
*/
Contact.prototype.remove = function(successCB, errorCB) {
    argscheck.checkArgs('FF', 'Contact.remove', arguments);
    var fail = errorCB && function(code) {
        errorCB(new ContactError(code));
    };
    if (this.id === null) {
        fail(ContactError.UNKNOWN_ERROR);
    }
    else {
        exec(successCB, fail, "Contacts", "remove", [this.id]);
    }
};

/**
* Creates a deep copy of this Contact.
* With the contact ID set to null.
* @return copy of this Contact
*/
Contact.prototype.clone = function() {
    var clonedContact = utils.clone(this);
    clonedContact.id = null;
    clonedContact.rawId = null;

    function nullIds(arr) {
        if (arr) {
            for (var i = 0; i < arr.length; ++i) {
                arr[i].id = null;
            }
        }
    }

    // Loop through and clear out any id's in phones, emails, etc.
    nullIds(clonedContact.phoneNumbers);
    nullIds(clonedContact.emails);
    nullIds(clonedContact.addresses);
    nullIds(clonedContact.ims);
    nullIds(clonedContact.organizations);
    nullIds(clonedContact.categories);
    nullIds(clonedContact.photos);
    nullIds(clonedContact.urls);
    return clonedContact;
};

/**
* Persists contact to device storage.
* @param successCB success callback
* @param errorCB error callback
*/
Contact.prototype.save = function(successCB, errorCB) {
    argscheck.checkArgs('FFO', 'Contact.save', arguments);
    var fail = errorCB && function(code) {
        errorCB(new ContactError(code));
    };
    var success = function(result) {
        if (result) {
            if (successCB) {
                var fullContact = require('cordova/plugin/contacts').create(result);
                successCB(convertIn(fullContact));
            }
        }
        else {
            // no Entry object returned
            fail(ContactError.UNKNOWN_ERROR);
        }
    };
    var dupContact = convertOut(utils.clone(this));
    exec(success, fail, "Contacts", "save", [dupContact]);
};


module.exports = Contact;

});

// file: lib/common/plugin/ContactAddress.js
define("cordova/plugin/ContactAddress", function(require, exports, module) {

/**
* Contact address.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code
* @param formatted // NOTE: not a W3C standard
* @param streetAddress
* @param locality
* @param region
* @param postalCode
* @param country
*/

var ContactAddress = function(pref, type, formatted, streetAddress, locality, region, postalCode, country) {
    this.id = null;
    this.pref = (typeof pref != 'undefined' ? pref : false);
    this.type = type || null;
    this.formatted = formatted || null;
    this.streetAddress = streetAddress || null;
    this.locality = locality || null;
    this.region = region || null;
    this.postalCode = postalCode || null;
    this.country = country || null;
};

module.exports = ContactAddress;

});

// file: lib/common/plugin/ContactError.js
define("cordova/plugin/ContactError", function(require, exports, module) {

/**
 *  ContactError.
 *  An error code assigned by an implementation when an error has occurred
 * @constructor
 */
var ContactError = function(err) {
    this.code = (typeof err != 'undefined' ? err : null);
};

/**
 * Error codes
 */
ContactError.UNKNOWN_ERROR = 0;
ContactError.INVALID_ARGUMENT_ERROR = 1;
ContactError.TIMEOUT_ERROR = 2;
ContactError.PENDING_OPERATION_ERROR = 3;
ContactError.IO_ERROR = 4;
ContactError.NOT_SUPPORTED_ERROR = 5;
ContactError.PERMISSION_DENIED_ERROR = 20;

module.exports = ContactError;

});

// file: lib/common/plugin/ContactField.js
define("cordova/plugin/ContactField", function(require, exports, module) {

/**
* Generic contact field.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code // NOTE: not a W3C standard
* @param type
* @param value
* @param pref
*/
var ContactField = function(type, value, pref) {
    this.id = null;
    this.type = (type && type.toString()) || null;
    this.value = (value && value.toString()) || null;
    this.pref = (typeof pref != 'undefined' ? pref : false);
};

module.exports = ContactField;

});

// file: lib/common/plugin/ContactFindOptions.js
define("cordova/plugin/ContactFindOptions", function(require, exports, module) {

/**
 * ContactFindOptions.
 * @constructor
 * @param filter used to match contacts against
 * @param multiple boolean used to determine if more than one contact should be returned
 */

var ContactFindOptions = function(filter, multiple) {
    this.filter = filter || '';
    this.multiple = (typeof multiple != 'undefined' ? multiple : false);
};

module.exports = ContactFindOptions;

});

// file: lib/common/plugin/ContactName.js
define("cordova/plugin/ContactName", function(require, exports, module) {

/**
* Contact name.
* @constructor
* @param formatted // NOTE: not part of W3C standard
* @param familyName
* @param givenName
* @param middle
* @param prefix
* @param suffix
*/
var ContactName = function(formatted, familyName, givenName, middle, prefix, suffix) {
    this.formatted = formatted || null;
    this.familyName = familyName || null;
    this.givenName = givenName || null;
    this.middleName = middle || null;
    this.honorificPrefix = prefix || null;
    this.honorificSuffix = suffix || null;
};

module.exports = ContactName;

});

// file: lib/common/plugin/ContactOrganization.js
define("cordova/plugin/ContactOrganization", function(require, exports, module) {

/**
* Contact organization.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code // NOTE: not a W3C standard
* @param name
* @param dept
* @param title
* @param startDate
* @param endDate
* @param location
* @param desc
*/

var ContactOrganization = function(pref, type, name, dept, title) {
    this.id = null;
    this.pref = (typeof pref != 'undefined' ? pref : false);
    this.type = type || null;
    this.name = name || null;
    this.department = dept || null;
    this.title = title || null;
};

module.exports = ContactOrganization;

});

// file: lib/common/plugin/Coordinates.js
define("cordova/plugin/Coordinates", function(require, exports, module) {

/**
 * This class contains position information.
 * @param {Object} lat
 * @param {Object} lng
 * @param {Object} alt
 * @param {Object} acc
 * @param {Object} head
 * @param {Object} vel
 * @param {Object} altacc
 * @constructor
 */
var Coordinates = function(lat, lng, alt, acc, head, vel, altacc) {
    /**
     * The latitude of the position.
     */
    this.latitude = lat;
    /**
     * The longitude of the position,
     */
    this.longitude = lng;
    /**
     * The accuracy of the position.
     */
    this.accuracy = acc;
    /**
     * The altitude of the position.
     */
    this.altitude = (alt !== undefined ? alt : null);
    /**
     * The direction the device is moving at the position.
     */
    this.heading = (head !== undefined ? head : null);
    /**
     * The velocity with which the device is moving at the position.
     */
    this.speed = (vel !== undefined ? vel : null);

    if (this.speed === 0 || this.speed === null) {
        this.heading = NaN;
    }

    /**
     * The altitude accuracy of the position.
     */
    this.altitudeAccuracy = (altacc !== undefined) ? altacc : null;
};

module.exports = Coordinates;

});

// file: lib/common/plugin/DirectoryEntry.js
define("cordova/plugin/DirectoryEntry", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    Entry = require('cordova/plugin/Entry'),
    FileError = require('cordova/plugin/FileError'),
    DirectoryReader = require('cordova/plugin/DirectoryReader');

/**
 * An interface representing a directory on the file system.
 *
 * {boolean} isFile always false (readonly)
 * {boolean} isDirectory always true (readonly)
 * {DOMString} name of the directory, excluding the path leading to it (readonly)
 * {DOMString} fullPath the absolute full path to the directory (readonly)
 * TODO: implement this!!! {FileSystem} filesystem on which the directory resides (readonly)
 */
var DirectoryEntry = function(name, fullPath) {
     DirectoryEntry.__super__.constructor.call(this, false, true, name, fullPath);
};

utils.extend(DirectoryEntry, Entry);

/**
 * Creates a new DirectoryReader to read entries from this directory
 */
DirectoryEntry.prototype.createReader = function() {
    return new DirectoryReader(this.fullPath);
};

/**
 * Creates or looks up a directory
 *
 * @param {DOMString} path either a relative or absolute path from this directory in which to look up or create a directory
 * @param {Flags} options to create or exclusively create the directory
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getDirectory = function(path, options, successCallback, errorCallback) {
    argscheck.checkArgs('sOFF', 'DirectoryEntry.getDirectory', arguments);
    var win = successCallback && function(result) {
        var entry = new DirectoryEntry(result.name, result.fullPath);
        successCallback(entry);
    };
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(win, fail, "File", "getDirectory", [this.fullPath, path, options]);
};

/**
 * Deletes a directory and all of it's contents
 *
 * @param {Function} successCallback is called with no parameters
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.removeRecursively = function(successCallback, errorCallback) {
    argscheck.checkArgs('FF', 'DirectoryEntry.removeRecursively', arguments);
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(successCallback, fail, "File", "removeRecursively", [this.fullPath]);
};

/**
 * Creates or looks up a file
 *
 * @param {DOMString} path either a relative or absolute path from this directory in which to look up or create a file
 * @param {Flags} options to create or exclusively create the file
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getFile = function(path, options, successCallback, errorCallback) {
    argscheck.checkArgs('sOFF', 'DirectoryEntry.getFile', arguments);
    var win = successCallback && function(result) {
        var FileEntry = require('cordova/plugin/FileEntry');
        var entry = new FileEntry(result.name, result.fullPath);
        successCallback(entry);
    };
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(win, fail, "File", "getFile", [this.fullPath, path, options]);
};

module.exports = DirectoryEntry;

});

// file: lib/common/plugin/DirectoryReader.js
define("cordova/plugin/DirectoryReader", function(require, exports, module) {

var exec = require('cordova/exec'),
    FileError = require('cordova/plugin/FileError') ;

/**
 * An interface that lists the files and directories in a directory.
 */
function DirectoryReader(path) {
    this.path = path || null;
}

/**
 * Returns a list of entries from a directory.
 *
 * @param {Function} successCallback is called with a list of entries
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryReader.prototype.readEntries = function(successCallback, errorCallback) {
    var win = typeof successCallback !== 'function' ? null : function(result) {
        var retVal = [];
        for (var i=0; i<result.length; i++) {
            var entry = null;
            if (result[i].isDirectory) {
                entry = new (require('cordova/plugin/DirectoryEntry'))();
            }
            else if (result[i].isFile) {
                entry = new (require('cordova/plugin/FileEntry'))();
            }
            entry.isDirectory = result[i].isDirectory;
            entry.isFile = result[i].isFile;
            entry.name = result[i].name;
            entry.fullPath = result[i].fullPath;
            retVal.push(entry);
        }
        successCallback(retVal);
    };
    var fail = typeof errorCallback !== 'function' ? null : function(code) {
        errorCallback(new FileError(code));
    };
    exec(win, fail, "File", "readEntries", [this.path]);
};

module.exports = DirectoryReader;

});

// file: lib/common/plugin/Entry.js
define("cordova/plugin/Entry", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    FileError = require('cordova/plugin/FileError'),
    Metadata = require('cordova/plugin/Metadata');

/**
 * Represents a file or directory on the local file system.
 *
 * @param isFile
 *            {boolean} true if Entry is a file (readonly)
 * @param isDirectory
 *            {boolean} true if Entry is a directory (readonly)
 * @param name
 *            {DOMString} name of the file or directory, excluding the path
 *            leading to it (readonly)
 * @param fullPath
 *            {DOMString} the absolute full path to the file or directory
 *            (readonly)
 */
function Entry(isFile, isDirectory, name, fullPath, fileSystem) {
    this.isFile = !!isFile;
    this.isDirectory = !!isDirectory;
    this.name = name || '';
    this.fullPath = fullPath || '';
    this.filesystem = fileSystem || null;
}

/**
 * Look up the metadata of the entry.
 *
 * @param successCallback
 *            {Function} is called with a Metadata object
 * @param errorCallback
 *            {Function} is called with a FileError
 */
Entry.prototype.getMetadata = function(successCallback, errorCallback) {
    argscheck.checkArgs('FF', 'Entry.getMetadata', arguments);
    var success = successCallback && function(lastModified) {
        var metadata = new Metadata(lastModified);
        successCallback(metadata);
    };
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };

    exec(success, fail, "File", "getMetadata", [this.fullPath]);
};

/**
 * Set the metadata of the entry.
 *
 * @param successCallback
 *            {Function} is called with a Metadata object
 * @param errorCallback
 *            {Function} is called with a FileError
 * @param metadataObject
 *            {Object} keys and values to set
 */
Entry.prototype.setMetadata = function(successCallback, errorCallback, metadataObject) {
    argscheck.checkArgs('FFO', 'Entry.setMetadata', arguments);
    exec(successCallback, errorCallback, "File", "setMetadata", [this.fullPath, metadataObject]);
};

/**
 * Move a file or directory to a new location.
 *
 * @param parent
 *            {DirectoryEntry} the directory to which to move this entry
 * @param newName
 *            {DOMString} new name of the entry, defaults to the current name
 * @param successCallback
 *            {Function} called with the new DirectoryEntry object
 * @param errorCallback
 *            {Function} called with a FileError
 */
Entry.prototype.moveTo = function(parent, newName, successCallback, errorCallback) {
    argscheck.checkArgs('oSFF', 'Entry.moveTo', arguments);
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    // source path
    var srcPath = this.fullPath,
        // entry name
        name = newName || this.name,
        success = function(entry) {
            if (entry) {
                if (successCallback) {
                    // create appropriate Entry object
                    var result = (entry.isDirectory) ? new (require('cordova/plugin/DirectoryEntry'))(entry.name, entry.fullPath) : new (require('cordova/plugin/FileEntry'))(entry.name, entry.fullPath);
                    successCallback(result);
                }
            }
            else {
                // no Entry object returned
                fail && fail(FileError.NOT_FOUND_ERR);
            }
        };

    // copy
    exec(success, fail, "File", "moveTo", [srcPath, parent.fullPath, name]);
};

/**
 * Copy a directory to a different location.
 *
 * @param parent
 *            {DirectoryEntry} the directory to which to copy the entry
 * @param newName
 *            {DOMString} new name of the entry, defaults to the current name
 * @param successCallback
 *            {Function} called with the new Entry object
 * @param errorCallback
 *            {Function} called with a FileError
 */
Entry.prototype.copyTo = function(parent, newName, successCallback, errorCallback) {
    argscheck.checkArgs('oSFF', 'Entry.copyTo', arguments);
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };

        // source path
    var srcPath = this.fullPath,
        // entry name
        name = newName || this.name,
        // success callback
        success = function(entry) {
            if (entry) {
                if (successCallback) {
                    // create appropriate Entry object
                    var result = (entry.isDirectory) ? new (require('cordova/plugin/DirectoryEntry'))(entry.name, entry.fullPath) : new (require('cordova/plugin/FileEntry'))(entry.name, entry.fullPath);
                    successCallback(result);
                }
            }
            else {
                // no Entry object returned
                fail && fail(FileError.NOT_FOUND_ERR);
            }
        };

    // copy
    exec(success, fail, "File", "copyTo", [srcPath, parent.fullPath, name]);
};

/**
 * Return a URL that can be used to identify this entry.
 */
Entry.prototype.toURL = function() {
    // fullPath attribute contains the full URL
    return this.fullPath;
};

/**
 * Returns a URI that can be used to identify this entry.
 *
 * @param {DOMString} mimeType for a FileEntry, the mime type to be used to interpret the file, when loaded through this URI.
 * @return uri
 */
Entry.prototype.toURI = function(mimeType) {
    console.log("DEPRECATED: Update your code to use 'toURL'");
    // fullPath attribute contains the full URI
    return this.toURL();
};

/**
 * Remove a file or directory. It is an error to attempt to delete a
 * directory that is not empty. It is an error to attempt to delete a
 * root directory of a file system.
 *
 * @param successCallback {Function} called with no parameters
 * @param errorCallback {Function} called with a FileError
 */
Entry.prototype.remove = function(successCallback, errorCallback) {
    argscheck.checkArgs('FF', 'Entry.remove', arguments);
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(successCallback, fail, "File", "remove", [this.fullPath]);
};

/**
 * Look up the parent DirectoryEntry of this entry.
 *
 * @param successCallback {Function} called with the parent DirectoryEntry object
 * @param errorCallback {Function} called with a FileError
 */
Entry.prototype.getParent = function(successCallback, errorCallback) {
    argscheck.checkArgs('FF', 'Entry.getParent', arguments);
    var win = successCallback && function(result) {
        var DirectoryEntry = require('cordova/plugin/DirectoryEntry');
        var entry = new DirectoryEntry(result.name, result.fullPath);
        successCallback(entry);
    };
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(win, fail, "File", "getParent", [this.fullPath]);
};

module.exports = Entry;

});

// file: lib/common/plugin/File.js
define("cordova/plugin/File", function(require, exports, module) {

/**
 * Constructor.
 * name {DOMString} name of the file, without path information
 * fullPath {DOMString} the full path of the file, including the name
 * type {DOMString} mime type
 * lastModifiedDate {Date} last modified date
 * size {Number} size of the file in bytes
 */

var File = function(name, fullPath, type, lastModifiedDate, size){
    this.name = name || '';
    this.fullPath = fullPath || null;
    this.type = type || null;
    this.lastModifiedDate = lastModifiedDate || null;
    this.size = size || 0;

    // These store the absolute start and end for slicing the file.
    this.start = 0;
    this.end = this.size;
};

/**
 * Returns a "slice" of the file. Since Cordova Files don't contain the actual
 * content, this really returns a File with adjusted start and end.
 * Slices of slices are supported.
 * start {Number} The index at which to start the slice (inclusive).
 * end {Number} The index at which to end the slice (exclusive).
 */
File.prototype.slice = function(start, end) {
    var size = this.end - this.start;
    var newStart = 0;
    var newEnd = size;
    if (arguments.length) {
        if (start < 0) {
            newStart = Math.max(size + start, 0);
        } else {
            newStart = Math.min(size, start);
        }
    }

    if (arguments.length >= 2) {
        if (end < 0) {
            newEnd = Math.max(size + end, 0);
        } else {
            newEnd = Math.min(end, size);
        }
    }

    var newFile = new File(this.name, this.fullPath, this.type, this.lastModifiedData, this.size);
    newFile.start = this.start + newStart;
    newFile.end = this.start + newEnd;
    return newFile;
};


module.exports = File;

});

// file: lib/common/plugin/FileEntry.js
define("cordova/plugin/FileEntry", function(require, exports, module) {

var utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    Entry = require('cordova/plugin/Entry'),
    FileWriter = require('cordova/plugin/FileWriter'),
    File = require('cordova/plugin/File'),
    FileError = require('cordova/plugin/FileError');

/**
 * An interface representing a file on the file system.
 *
 * {boolean} isFile always true (readonly)
 * {boolean} isDirectory always false (readonly)
 * {DOMString} name of the file, excluding the path leading to it (readonly)
 * {DOMString} fullPath the absolute full path to the file (readonly)
 * {FileSystem} filesystem on which the file resides (readonly)
 */
var FileEntry = function(name, fullPath) {
     FileEntry.__super__.constructor.apply(this, [true, false, name, fullPath]);
};

utils.extend(FileEntry, Entry);

/**
 * Creates a new FileWriter associated with the file that this FileEntry represents.
 *
 * @param {Function} successCallback is called with the new FileWriter
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.createWriter = function(successCallback, errorCallback) {
    this.file(function(filePointer) {
        var writer = new FileWriter(filePointer);

        if (writer.fileName === null || writer.fileName === "") {
            errorCallback && errorCallback(new FileError(FileError.INVALID_STATE_ERR));
        } else {
            successCallback && successCallback(writer);
        }
    }, errorCallback);
};

/**
 * Returns a File that represents the current state of the file that this FileEntry represents.
 *
 * @param {Function} successCallback is called with the new File object
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.file = function(successCallback, errorCallback) {
    var win = successCallback && function(f) {
        var file = new File(f.name, f.fullPath, f.type, f.lastModifiedDate, f.size);
        successCallback(file);
    };
    var fail = errorCallback && function(code) {
        errorCallback(new FileError(code));
    };
    exec(win, fail, "File", "getFileMetadata", [this.fullPath]);
};


module.exports = FileEntry;

});

// file: lib/common/plugin/FileError.js
define("cordova/plugin/FileError", function(require, exports, module) {

/**
 * FileError
 */
function FileError(error) {
  this.code = error || null;
}

// File error codes
// Found in DOMException
FileError.NOT_FOUND_ERR = 1;
FileError.SECURITY_ERR = 2;
FileError.ABORT_ERR = 3;

// Added by File API specification
FileError.NOT_READABLE_ERR = 4;
FileError.ENCODING_ERR = 5;
FileError.NO_MODIFICATION_ALLOWED_ERR = 6;
FileError.INVALID_STATE_ERR = 7;
FileError.SYNTAX_ERR = 8;
FileError.INVALID_MODIFICATION_ERR = 9;
FileError.QUOTA_EXCEEDED_ERR = 10;
FileError.TYPE_MISMATCH_ERR = 11;
FileError.PATH_EXISTS_ERR = 12;

module.exports = FileError;

});

// file: lib/common/plugin/FileReader.js
define("cordova/plugin/FileReader", function(require, exports, module) {

var exec = require('cordova/exec'),
    modulemapper = require('cordova/modulemapper'),
    utils = require('cordova/utils'),
    File = require('cordova/plugin/File'),
    FileError = require('cordova/plugin/FileError'),
    ProgressEvent = require('cordova/plugin/ProgressEvent'),
    origFileReader = modulemapper.getOriginalSymbol(this, 'FileReader');

/**
 * This class reads the mobile device file system.
 *
 * For Android:
 *      The root directory is the root of the file system.
 *      To read from the SD card, the file name is "sdcard/my_file.txt"
 * @constructor
 */
var FileReader = function() {
    this._readyState = 0;
    this._error = null;
    this._result = null;
    this._fileName = '';
    this._realReader = origFileReader ? new origFileReader() : {};
};

// States
FileReader.EMPTY = 0;
FileReader.LOADING = 1;
FileReader.DONE = 2;

utils.defineGetter(FileReader.prototype, 'readyState', function() {
    return this._fileName ? this._readyState : this._realReader.readyState;
});

utils.defineGetter(FileReader.prototype, 'error', function() {
    return this._fileName ? this._error: this._realReader.error;
});

utils.defineGetter(FileReader.prototype, 'result', function() {
    return this._fileName ? this._result: this._realReader.result;
});

function defineEvent(eventName) {
    utils.defineGetterSetter(FileReader.prototype, eventName, function() {
        return this._realReader[eventName] || null;
    }, function(value) {
        this._realReader[eventName] = value;
    });
}
defineEvent('onloadstart');    // When the read starts.
defineEvent('onprogress');     // While reading (and decoding) file or fileBlob data, and reporting partial file data (progress.loaded/progress.total)
defineEvent('onload');         // When the read has successfully completed.
defineEvent('onerror');        // When the read has failed (see errors).
defineEvent('onloadend');      // When the request has completed (either in success or failure).
defineEvent('onabort');        // When the read has been aborted. For instance, by invoking the abort() method.

function initRead(reader, file) {
    // Already loading something
    if (reader.readyState == FileReader.LOADING) {
      throw new FileError(FileError.INVALID_STATE_ERR);
    }

    reader._result = null;
    reader._error = null;
    reader._readyState = FileReader.LOADING;

    if (typeof file == 'string') {
        // Deprecated in Cordova 2.4.
        console.warn('Using a string argument with FileReader.readAs functions is deprecated.');
        reader._fileName = file;
    } else if (typeof file.fullPath == 'string') {
        reader._fileName = file.fullPath;
    } else {
        reader._fileName = '';
        return true;
    }

    reader.onloadstart && reader.onloadstart(new ProgressEvent("loadstart", {target:reader}));
}

/**
 * Abort reading file.
 */
FileReader.prototype.abort = function() {
    if (origFileReader && !this._fileName) {
        return this._realReader.abort();
    }
    this._result = null;

    if (this._readyState == FileReader.DONE || this._readyState == FileReader.EMPTY) {
      return;
    }

    this._readyState = FileReader.DONE;

    // If abort callback
    if (typeof this.onabort === 'function') {
        this.onabort(new ProgressEvent('abort', {target:this}));
    }
    // If load end callback
    if (typeof this.onloadend === 'function') {
        this.onloadend(new ProgressEvent('loadend', {target:this}));
    }
};

/**
 * Read text file.
 *
 * @param file          {File} File object containing file properties
 * @param encoding      [Optional] (see http://www.iana.org/assignments/character-sets)
 */
FileReader.prototype.readAsText = function(file, encoding) {
    if (initRead(this, file)) {
        return this._realReader.readAsText(file, encoding);
    }

    // Default encoding is UTF-8
    var enc = encoding ? encoding : "UTF-8";
    var me = this;
    var execArgs = [this._fileName, enc, file.start, file.end];

    // Read file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // Save result
            me._result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload(new ProgressEvent("load", {target:me}));
            }

            // DONE state
            me._readyState = FileReader.DONE;

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            // null result
            me._result = null;

            // Save error
            me._error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        }, "File", "readAsText", execArgs);
};


/**
 * Read file and return data as a base64 encoded data url.
 * A data url is of the form:
 *      data:[<mediatype>][;base64],<data>
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsDataURL = function(file) {
    if (initRead(this, file)) {
        return this._realReader.readAsDataURL(file);
    }

    var me = this;
    var execArgs = [this._fileName, file.start, file.end];

    // Read file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            // Save result
            me._result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload(new ProgressEvent("load", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            me._result = null;

            // Save error
            me._error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        }, "File", "readAsDataURL", execArgs);
};

/**
 * Read file and return data as a binary data.
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsBinaryString = function(file) {
    if (initRead(this, file)) {
        return this._realReader.readAsBinaryString(file);
    }

    var me = this;
    var execArgs = [this._fileName, file.start, file.end];

    // Read file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            me._result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload(new ProgressEvent("load", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            me._result = null;

            // Save error
            me._error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        }, "File", "readAsBinaryString", execArgs);
};

/**
 * Read file and return data as a binary data.
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsArrayBuffer = function(file) {
    if (initRead(this, file)) {
        return this._realReader.readAsArrayBuffer(file);
    }

    var me = this;
    var execArgs = [this._fileName, file.start, file.end];

    // Read file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            me._result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload(new ProgressEvent("load", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me._readyState === FileReader.DONE) {
                return;
            }

            // DONE state
            me._readyState = FileReader.DONE;

            me._result = null;

            // Save error
            me._error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {target:me}));
            }

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend(new ProgressEvent("loadend", {target:me}));
            }
        }, "File", "readAsArrayBuffer", execArgs);
};

module.exports = FileReader;

});

// file: lib/common/plugin/FileSystem.js
define("cordova/plugin/FileSystem", function(require, exports, module) {

var DirectoryEntry = require('cordova/plugin/DirectoryEntry');

/**
 * An interface representing a file system
 *
 * @constructor
 * {DOMString} name the unique name of the file system (readonly)
 * {DirectoryEntry} root directory of the file system (readonly)
 */
var FileSystem = function(name, root) {
    this.name = name || null;
    if (root) {
        this.root = new DirectoryEntry(root.name, root.fullPath);
    }
};

module.exports = FileSystem;

});

// file: lib/common/plugin/FileTransfer.js
define("cordova/plugin/FileTransfer", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    FileTransferError = require('cordova/plugin/FileTransferError'),
    ProgressEvent = require('cordova/plugin/ProgressEvent');

function newProgressEvent(result) {
    var pe = new ProgressEvent();
    pe.lengthComputable = result.lengthComputable;
    pe.loaded = result.loaded;
    pe.total = result.total;
    return pe;
}

function getBasicAuthHeader(urlString) {
    var header =  null;

    if (window.btoa) {
        // parse the url using the Location object
        var url = document.createElement('a');
        url.href = urlString;

        var credentials = null;
        var protocol = url.protocol + "//";
        var origin = protocol + url.host;

        // check whether there are the username:password credentials in the url
        if (url.href.indexOf(origin) !== 0) { // credentials found
            var atIndex = url.href.indexOf("@");
            credentials = url.href.substring(protocol.length, atIndex);
        }

        if (credentials) {
            var authHeader = "Authorization";
            var authHeaderValue = "Basic " + window.btoa(credentials);

            header = {
                name : authHeader,
                value : authHeaderValue
            };
        }
    }

    return header;
}

var idCounter = 0;

/**
 * FileTransfer uploads a file to a remote server.
 * @constructor
 */
var FileTransfer = function() {
    this._id = ++idCounter;
    this.onprogress = null; // optional callback
};

/**
* Given an absolute file path, uploads a file on the device to a remote server
* using a multipart HTTP request.
* @param filePath {String}           Full path of the file on the device
* @param server {String}             URL of the server to receive the file
* @param successCallback (Function}  Callback to be invoked when upload has completed
* @param errorCallback {Function}    Callback to be invoked upon error
* @param options {FileUploadOptions} Optional parameters such as file name and mimetype
* @param trustAllHosts {Boolean} Optional trust all hosts (e.g. for self-signed certs), defaults to false
*/
FileTransfer.prototype.upload = function(filePath, server, successCallback, errorCallback, options, trustAllHosts) {
    argscheck.checkArgs('ssFFO*', 'FileTransfer.upload', arguments);
    // check for options
    var fileKey = null;
    var fileName = null;
    var mimeType = null;
    var params = null;
    var chunkedMode = true;
    var headers = null;
    var httpMethod = null;
    var basicAuthHeader = getBasicAuthHeader(server);
    if (basicAuthHeader) {
        options = options || {};
        options.headers = options.headers || {};
        options.headers[basicAuthHeader.name] = basicAuthHeader.value;
    }

    if (options) {
        fileKey = options.fileKey;
        fileName = options.fileName;
        mimeType = options.mimeType;
        headers = options.headers;
        httpMethod = options.httpMethod || "POST";
        if (httpMethod.toUpperCase() == "PUT"){
            httpMethod = "PUT";
        } else {
            httpMethod = "POST";
        }
        if (options.chunkedMode !== null || typeof options.chunkedMode != "undefined") {
            chunkedMode = options.chunkedMode;
        }
        if (options.params) {
            params = options.params;
        }
        else {
            params = {};
        }
    }

    var fail = errorCallback && function(e) {
        var error = new FileTransferError(e.code, e.source, e.target, e.http_status, e.body);
        errorCallback(error);
    };

    var self = this;
    var win = function(result) {
        if (typeof result.lengthComputable != "undefined") {
            if (self.onprogress) {
                self.onprogress(newProgressEvent(result));
            }
        } else {
            successCallback && successCallback(result);
        }
    };
    exec(win, fail, 'FileTransfer', 'upload', [filePath, server, fileKey, fileName, mimeType, params, trustAllHosts, chunkedMode, headers, this._id, httpMethod]);
};

/**
 * Downloads a file form a given URL and saves it to the specified directory.
 * @param source {String}          URL of the server to receive the file
 * @param target {String}         Full path of the file on the device
 * @param successCallback (Function}  Callback to be invoked when upload has completed
 * @param errorCallback {Function}    Callback to be invoked upon error
 * @param trustAllHosts {Boolean} Optional trust all hosts (e.g. for self-signed certs), defaults to false
 * @param options {FileDownloadOptions} Optional parameters such as headers
 */
FileTransfer.prototype.download = function(source, target, successCallback, errorCallback, trustAllHosts, options) {
    argscheck.checkArgs('ssFF*', 'FileTransfer.download', arguments);
    var self = this;

    var basicAuthHeader = getBasicAuthHeader(source);
    if (basicAuthHeader) {
        options = options || {};
        options.headers = options.headers || {};
        options.headers[basicAuthHeader.name] = basicAuthHeader.value;
    }

    var headers = null;
    if (options) {
        headers = options.headers || null;
    }

    var win = function(result) {
        if (typeof result.lengthComputable != "undefined") {
            if (self.onprogress) {
                return self.onprogress(newProgressEvent(result));
            }
        } else if (successCallback) {
            var entry = null;
            if (result.isDirectory) {
                entry = new (require('cordova/plugin/DirectoryEntry'))();
            }
            else if (result.isFile) {
                entry = new (require('cordova/plugin/FileEntry'))();
            }
            entry.isDirectory = result.isDirectory;
            entry.isFile = result.isFile;
            entry.name = result.name;
            entry.fullPath = result.fullPath;
            successCallback(entry);
        }
    };

    var fail = errorCallback && function(e) {
        var error = new FileTransferError(e.code, e.source, e.target, e.http_status, e.body);
        errorCallback(error);
    };

    exec(win, fail, 'FileTransfer', 'download', [source, target, trustAllHosts, this._id, headers]);
};

/**
 * Aborts the ongoing file transfer on this object. The original error
 * callback for the file transfer will be called if necessary.
 */
FileTransfer.prototype.abort = function() {
    exec(null, null, 'FileTransfer', 'abort', [this._id]);
};

module.exports = FileTransfer;

});

// file: lib/common/plugin/FileTransferError.js
define("cordova/plugin/FileTransferError", function(require, exports, module) {

/**
 * FileTransferError
 * @constructor
 */
var FileTransferError = function(code, source, target, status, body) {
    this.code = code || null;
    this.source = source || null;
    this.target = target || null;
    this.http_status = status || null;
    this.body = body || null;
};

FileTransferError.FILE_NOT_FOUND_ERR = 1;
FileTransferError.INVALID_URL_ERR = 2;
FileTransferError.CONNECTION_ERR = 3;
FileTransferError.ABORT_ERR = 4;

module.exports = FileTransferError;

});

// file: lib/common/plugin/FileUploadOptions.js
define("cordova/plugin/FileUploadOptions", function(require, exports, module) {

/**
 * Options to customize the HTTP request used to upload files.
 * @constructor
 * @param fileKey {String}   Name of file request parameter.
 * @param fileName {String}  Filename to be used by the server. Defaults to image.jpg.
 * @param mimeType {String}  Mimetype of the uploaded file. Defaults to image/jpeg.
 * @param params {Object}    Object with key: value params to send to the server.
 * @param headers {Object}   Keys are header names, values are header values. Multiple
 *                           headers of the same name are not supported.
 */
var FileUploadOptions = function(fileKey, fileName, mimeType, params, headers, httpMethod) {
    this.fileKey = fileKey || null;
    this.fileName = fileName || null;
    this.mimeType = mimeType || null;
    this.params = params || null;
    this.headers = headers || null;
    this.httpMethod = httpMethod || null;
};

module.exports = FileUploadOptions;

});

// file: lib/common/plugin/FileUploadResult.js
define("cordova/plugin/FileUploadResult", function(require, exports, module) {

/**
 * FileUploadResult
 * @constructor
 */
var FileUploadResult = function() {
    this.bytesSent = 0;
    this.responseCode = null;
    this.response = null;
};

module.exports = FileUploadResult;

});

// file: lib/common/plugin/FileWriter.js
define("cordova/plugin/FileWriter", function(require, exports, module) {

var exec = require('cordova/exec'),
    FileError = require('cordova/plugin/FileError'),
    ProgressEvent = require('cordova/plugin/ProgressEvent');

/**
 * This class writes to the mobile device file system.
 *
 * For Android:
 *      The root directory is the root of the file system.
 *      To write to the SD card, the file name is "sdcard/my_file.txt"
 *
 * @constructor
 * @param file {File} File object containing file properties
 * @param append if true write to the end of the file, otherwise overwrite the file
 */
var FileWriter = function(file) {
    this.fileName = "";
    this.length = 0;
    if (file) {
        this.fileName = file.fullPath || file;
        this.length = file.size || 0;
    }
    // default is to write at the beginning of the file
    this.position = 0;

    this.readyState = 0; // EMPTY

    this.result = null;

    // Error
    this.error = null;

    // Event handlers
    this.onwritestart = null;   // When writing starts
    this.onprogress = null;     // While writing the file, and reporting partial file data
    this.onwrite = null;        // When the write has successfully completed.
    this.onwriteend = null;     // When the request has completed (either in success or failure).
    this.onabort = null;        // When the write has been aborted. For instance, by invoking the abort() method.
    this.onerror = null;        // When the write has failed (see errors).
};

// States
FileWriter.INIT = 0;
FileWriter.WRITING = 1;
FileWriter.DONE = 2;

/**
 * Abort writing file.
 */
FileWriter.prototype.abort = function() {
    // check for invalid state
    if (this.readyState === FileWriter.DONE || this.readyState === FileWriter.INIT) {
        throw new FileError(FileError.INVALID_STATE_ERR);
    }

    // set error
    this.error = new FileError(FileError.ABORT_ERR);

    this.readyState = FileWriter.DONE;

    // If abort callback
    if (typeof this.onabort === "function") {
        this.onabort(new ProgressEvent("abort", {"target":this}));
    }

    // If write end callback
    if (typeof this.onwriteend === "function") {
        this.onwriteend(new ProgressEvent("writeend", {"target":this}));
    }
};

/**
 * Writes data to the file
 *
 * @param text to be written
 */
FileWriter.prototype.write = function(text) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw new FileError(FileError.INVALID_STATE_ERR);
    }

    // WRITING state
    this.readyState = FileWriter.WRITING;

    var me = this;

    // If onwritestart callback
    if (typeof me.onwritestart === "function") {
        me.onwritestart(new ProgressEvent("writestart", {"target":me}));
    }

    // Write file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // position always increases by bytes written because file would be extended
            me.position += r;
            // The length of the file is now where we are done writing.

            me.length = me.position;

            // DONE state
            me.readyState = FileWriter.DONE;

            // If onwrite callback
            if (typeof me.onwrite === "function") {
                me.onwrite(new ProgressEvent("write", {"target":me}));
            }

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend(new ProgressEvent("writeend", {"target":me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // Save error
            me.error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {"target":me}));
            }

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend(new ProgressEvent("writeend", {"target":me}));
            }
        }, "File", "write", [this.fileName, text, this.position]);
};

/**
 * Moves the file pointer to the location specified.
 *
 * If the offset is a negative number the position of the file
 * pointer is rewound.  If the offset is greater than the file
 * size the position is set to the end of the file.
 *
 * @param offset is the location to move the file pointer to.
 */
FileWriter.prototype.seek = function(offset) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw new FileError(FileError.INVALID_STATE_ERR);
    }

    if (!offset && offset !== 0) {
        return;
    }

    // See back from end of file.
    if (offset < 0) {
        this.position = Math.max(offset + this.length, 0);
    }
    // Offset is bigger than file size so set position
    // to the end of the file.
    else if (offset > this.length) {
        this.position = this.length;
    }
    // Offset is between 0 and file size so set the position
    // to start writing.
    else {
        this.position = offset;
    }
};

/**
 * Truncates the file to the size specified.
 *
 * @param size to chop the file at.
 */
FileWriter.prototype.truncate = function(size) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw new FileError(FileError.INVALID_STATE_ERR);
    }

    // WRITING state
    this.readyState = FileWriter.WRITING;

    var me = this;

    // If onwritestart callback
    if (typeof me.onwritestart === "function") {
        me.onwritestart(new ProgressEvent("writestart", {"target":this}));
    }

    // Write file
    exec(
        // Success callback
        function(r) {
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // Update the length of the file
            me.length = r;
            me.position = Math.min(me.position, r);

            // If onwrite callback
            if (typeof me.onwrite === "function") {
                me.onwrite(new ProgressEvent("write", {"target":me}));
            }

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend(new ProgressEvent("writeend", {"target":me}));
            }
        },
        // Error callback
        function(e) {
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // Save error
            me.error = new FileError(e);

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror(new ProgressEvent("error", {"target":me}));
            }

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend(new ProgressEvent("writeend", {"target":me}));
            }
        }, "File", "truncate", [this.fileName, size]);
};

module.exports = FileWriter;

});

// file: lib/common/plugin/Flags.js
define("cordova/plugin/Flags", function(require, exports, module) {

/**
 * Supplies arguments to methods that lookup or create files and directories.
 *
 * @param create
 *            {boolean} file or directory if it doesn't exist
 * @param exclusive
 *            {boolean} used with create; if true the command will fail if
 *            target path exists
 */
function Flags(create, exclusive) {
    this.create = create || false;
    this.exclusive = exclusive || false;
}

module.exports = Flags;

});

// file: lib/common/plugin/GlobalizationError.js
define("cordova/plugin/GlobalizationError", function(require, exports, module) {


/**
 * Globalization error object
 *
 * @constructor
 * @param code
 * @param message
 */
var GlobalizationError = function(code, message) {
    this.code = code || null;
    this.message = message || '';
};

// Globalization error codes
GlobalizationError.UNKNOWN_ERROR = 0;
GlobalizationError.FORMATTING_ERROR = 1;
GlobalizationError.PARSING_ERROR = 2;
GlobalizationError.PATTERN_ERROR = 3;

module.exports = GlobalizationError;

});

// file: lib/common/plugin/InAppBrowser.js
define("cordova/plugin/InAppBrowser", function(require, exports, module) {

var exec = require('cordova/exec');
var channel = require('cordova/channel');
var modulemapper = require('cordova/modulemapper');

function InAppBrowser() {
   this.channels = {
        'loadstart': channel.create('loadstart'),
        'loadstop' : channel.create('loadstop'),
        'loaderror' : channel.create('loaderror'),
        'exit' : channel.create('exit')
   };
}

InAppBrowser.prototype = {
    _eventHandler: function (event) {
        if (event.type in this.channels) {
            this.channels[event.type].fire(event);
        }
    },
    close: function (eventname) {
        exec(null, null, "InAppBrowser", "close", []);
    },
    addEventListener: function (eventname,f) {
        if (eventname in this.channels) {
            this.channels[eventname].subscribe(f);
        }
    },
    removeEventListener: function(eventname, f) {
        if (eventname in this.channels) {
            this.channels[eventname].unsubscribe(f);
        }
    },

    executeScript: function(injectDetails, cb) {
        if (injectDetails.code) {
            exec(cb, null, "InAppBrowser", "injectScriptCode", [injectDetails.code, !!cb]);
        } else if (injectDetails.file) {
            exec(cb, null, "InAppBrowser", "injectScriptFile", [injectDetails.file, !!cb]);
        } else {
            throw new Error('executeScript requires exactly one of code or file to be specified');
        }
    },

    insertCSS: function(injectDetails, cb) {
        if (injectDetails.code) {
            exec(cb, null, "InAppBrowser", "injectStyleCode", [injectDetails.code, !!cb]);
        } else if (injectDetails.file) {
            exec(cb, null, "InAppBrowser", "injectStyleFile", [injectDetails.file, !!cb]);
        } else {
            throw new Error('insertCSS requires exactly one of code or file to be specified');
        }
    }
};

module.exports = function(strUrl, strWindowName, strWindowFeatures) {
    var iab = new InAppBrowser();
    var cb = function(eventname) {
       iab._eventHandler(eventname);
    };

    // Don't catch calls that write to existing frames (e.g. named iframes).
    if (window.frames && window.frames[strWindowName]) {
        var origOpenFunc = modulemapper.getOriginalSymbol(window, 'open');
        return origOpenFunc.apply(window, arguments);
    }

    exec(cb, cb, "InAppBrowser", "open", [strUrl, strWindowName, strWindowFeatures]);
    return iab;
};


});

// file: lib/common/plugin/LocalFileSystem.js
define("cordova/plugin/LocalFileSystem", function(require, exports, module) {

var exec = require('cordova/exec');

/**
 * Represents a local file system.
 */
var LocalFileSystem = function() {

};

LocalFileSystem.TEMPORARY = 0; //temporary, with no guarantee of persistence
LocalFileSystem.PERSISTENT = 1; //persistent

module.exports = LocalFileSystem;

});

// file: lib/common/plugin/Media.js
define("cordova/plugin/Media", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec');

var mediaObjects = {};

/**
 * This class provides access to the device media, interfaces to both sound and video
 *
 * @constructor
 * @param src                   The file name or url to play
 * @param successCallback       The callback to be called when the file is done playing or recording.
 *                                  successCallback()
 * @param errorCallback         The callback to be called if there is an error.
 *                                  errorCallback(int errorCode) - OPTIONAL
 * @param statusCallback        The callback to be called when media status has changed.
 *                                  statusCallback(int statusCode) - OPTIONAL
 */
var Media = function(src, successCallback, errorCallback, statusCallback) {
    argscheck.checkArgs('SFFF', 'Media', arguments);
    this.id = utils.createUUID();
    mediaObjects[this.id] = this;
    this.src = src;
    this.successCallback = successCallback;
    this.errorCallback = errorCallback;
    this.statusCallback = statusCallback;
    this._duration = -1;
    this._position = -1;
    exec(null, this.errorCallback, "Media", "create", [this.id, this.src]);
};

// Media messages
Media.MEDIA_STATE = 1;
Media.MEDIA_DURATION = 2;
Media.MEDIA_POSITION = 3;
Media.MEDIA_ERROR = 9;

// Media states
Media.MEDIA_NONE = 0;
Media.MEDIA_STARTING = 1;
Media.MEDIA_RUNNING = 2;
Media.MEDIA_PAUSED = 3;
Media.MEDIA_STOPPED = 4;
Media.MEDIA_MSG = ["None", "Starting", "Running", "Paused", "Stopped"];

// "static" function to return existing objs.
Media.get = function(id) {
    return mediaObjects[id];
};

/**
 * Start or resume playing audio file.
 */
Media.prototype.play = function(options) {
    exec(null, null, "Media", "startPlayingAudio", [this.id, this.src, options]);
};

/**
 * Stop playing audio file.
 */
Media.prototype.stop = function() {
    var me = this;
    exec(function() {
        me._position = 0;
    }, this.errorCallback, "Media", "stopPlayingAudio", [this.id]);
};

/**
 * Seek or jump to a new time in the track..
 */
Media.prototype.seekTo = function(milliseconds) {
    var me = this;
    exec(function(p) {
        me._position = p;
    }, this.errorCallback, "Media", "seekToAudio", [this.id, milliseconds]);
};

/**
 * Pause playing audio file.
 */
Media.prototype.pause = function() {
    exec(null, this.errorCallback, "Media", "pausePlayingAudio", [this.id]);
};

/**
 * Get duration of an audio file.
 * The duration is only set for audio that is playing, paused or stopped.
 *
 * @return      duration or -1 if not known.
 */
Media.prototype.getDuration = function() {
    return this._duration;
};

/**
 * Get position of audio.
 */
Media.prototype.getCurrentPosition = function(success, fail) {
    var me = this;
    exec(function(p) {
        me._position = p;
        success(p);
    }, fail, "Media", "getCurrentPositionAudio", [this.id]);
};

/**
 * Start recording audio file.
 */
Media.prototype.startRecord = function() {
    exec(null, this.errorCallback, "Media", "startRecordingAudio", [this.id, this.src]);
};

/**
 * Stop recording audio file.
 */
Media.prototype.stopRecord = function() {
    exec(null, this.errorCallback, "Media", "stopRecordingAudio", [this.id]);
};

/**
 * Release the resources.
 */
Media.prototype.release = function() {
    exec(null, this.errorCallback, "Media", "release", [this.id]);
};

/**
 * Adjust the volume.
 */
Media.prototype.setVolume = function(volume) {
    exec(null, null, "Media", "setVolume", [this.id, volume]);
};

/**
 * Audio has status update.
 * PRIVATE
 *
 * @param id            The media object id (string)
 * @param msgType       The 'type' of update this is
 * @param value         Use of value is determined by the msgType
 */
Media.onStatus = function(id, msgType, value) {

    var media = mediaObjects[id];

    if(media) {
        switch(msgType) {
            case Media.MEDIA_STATE :
                media.statusCallback && media.statusCallback(value);
                if(value == Media.MEDIA_STOPPED) {
                    media.successCallback && media.successCallback();
                }
                break;
            case Media.MEDIA_DURATION :
                media._duration = value;
                break;
            case Media.MEDIA_ERROR :
                media.errorCallback && media.errorCallback(value);
                break;
            case Media.MEDIA_POSITION :
                media._position = Number(value);
                break;
            default :
                console.error && console.error("Unhandled Media.onStatus :: " + msgType);
                break;
        }
    }
    else {
         console.error && console.error("Received Media.onStatus callback for unknown media :: " + id);
    }

};

module.exports = Media;

});

// file: lib/common/plugin/MediaError.js
define("cordova/plugin/MediaError", function(require, exports, module) {

/**
 * This class contains information about any Media errors.
*/
/*
 According to :: http://dev.w3.org/html5/spec-author-view/video.html#mediaerror
 We should never be creating these objects, we should just implement the interface
 which has 1 property for an instance, 'code'

 instead of doing :
    errorCallbackFunction( new MediaError(3,'msg') );
we should simply use a literal :
    errorCallbackFunction( {'code':3} );
 */

 var _MediaError = window.MediaError;


if(!_MediaError) {
    window.MediaError = _MediaError = function(code, msg) {
        this.code = (typeof code != 'undefined') ? code : null;
        this.message = msg || ""; // message is NON-standard! do not use!
    };
}

_MediaError.MEDIA_ERR_NONE_ACTIVE    = _MediaError.MEDIA_ERR_NONE_ACTIVE    || 0;
_MediaError.MEDIA_ERR_ABORTED        = _MediaError.MEDIA_ERR_ABORTED        || 1;
_MediaError.MEDIA_ERR_NETWORK        = _MediaError.MEDIA_ERR_NETWORK        || 2;
_MediaError.MEDIA_ERR_DECODE         = _MediaError.MEDIA_ERR_DECODE         || 3;
_MediaError.MEDIA_ERR_NONE_SUPPORTED = _MediaError.MEDIA_ERR_NONE_SUPPORTED || 4;
// TODO: MediaError.MEDIA_ERR_NONE_SUPPORTED is legacy, the W3 spec now defines it as below.
// as defined by http://dev.w3.org/html5/spec-author-view/video.html#error-codes
_MediaError.MEDIA_ERR_SRC_NOT_SUPPORTED = _MediaError.MEDIA_ERR_SRC_NOT_SUPPORTED || 4;

module.exports = _MediaError;

});

// file: lib/common/plugin/MediaFile.js
define("cordova/plugin/MediaFile", function(require, exports, module) {

var utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    File = require('cordova/plugin/File'),
    CaptureError = require('cordova/plugin/CaptureError');
/**
 * Represents a single file.
 *
 * name {DOMString} name of the file, without path information
 * fullPath {DOMString} the full path of the file, including the name
 * type {DOMString} mime type
 * lastModifiedDate {Date} last modified date
 * size {Number} size of the file in bytes
 */
var MediaFile = function(name, fullPath, type, lastModifiedDate, size){
    MediaFile.__super__.constructor.apply(this, arguments);
};

utils.extend(MediaFile, File);

/**
 * Request capture format data for a specific file and type
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 */
MediaFile.prototype.getFormatData = function(successCallback, errorCallback) {
    if (typeof this.fullPath === "undefined" || this.fullPath === null) {
        errorCallback(new CaptureError(CaptureError.CAPTURE_INVALID_ARGUMENT));
    } else {
        exec(successCallback, errorCallback, "Capture", "getFormatData", [this.fullPath, this.type]);
    }
};

module.exports = MediaFile;

});

// file: lib/common/plugin/MediaFileData.js
define("cordova/plugin/MediaFileData", function(require, exports, module) {

/**
 * MediaFileData encapsulates format information of a media file.
 *
 * @param {DOMString} codecs
 * @param {long} bitrate
 * @param {long} height
 * @param {long} width
 * @param {float} duration
 */
var MediaFileData = function(codecs, bitrate, height, width, duration){
    this.codecs = codecs || null;
    this.bitrate = bitrate || 0;
    this.height = height || 0;
    this.width = width || 0;
    this.duration = duration || 0;
};

module.exports = MediaFileData;

});

// file: lib/common/plugin/Metadata.js
define("cordova/plugin/Metadata", function(require, exports, module) {

/**
 * Information about the state of the file or directory
 *
 * {Date} modificationTime (readonly)
 */
var Metadata = function(time) {
    this.modificationTime = (typeof time != 'undefined'?new Date(time):null);
};

module.exports = Metadata;

});

// file: lib/common/plugin/Position.js
define("cordova/plugin/Position", function(require, exports, module) {

var Coordinates = require('cordova/plugin/Coordinates');

var Position = function(coords, timestamp) {
    if (coords) {
        this.coords = new Coordinates(coords.latitude, coords.longitude, coords.altitude, coords.accuracy, coords.heading, coords.velocity, coords.altitudeAccuracy);
    } else {
        this.coords = new Coordinates();
    }
    this.timestamp = (timestamp !== undefined) ? timestamp : new Date();
};

module.exports = Position;

});

// file: lib/common/plugin/PositionError.js
define("cordova/plugin/PositionError", function(require, exports, module) {

/**
 * Position error object
 *
 * @constructor
 * @param code
 * @param message
 */
var PositionError = function(code, message) {
    this.code = code || null;
    this.message = message || '';
};

PositionError.PERMISSION_DENIED = 1;
PositionError.POSITION_UNAVAILABLE = 2;
PositionError.TIMEOUT = 3;

module.exports = PositionError;

});

// file: lib/common/plugin/ProgressEvent.js
define("cordova/plugin/ProgressEvent", function(require, exports, module) {

// If ProgressEvent exists in global context, use it already, otherwise use our own polyfill
// Feature test: See if we can instantiate a native ProgressEvent;
// if so, use that approach,
// otherwise fill-in with our own implementation.
//
// NOTE: right now we always fill in with our own. Down the road would be nice if we can use whatever is native in the webview.
var ProgressEvent = (function() {
    /*
    var createEvent = function(data) {
        var event = document.createEvent('Events');
        event.initEvent('ProgressEvent', false, false);
        if (data) {
            for (var i in data) {
                if (data.hasOwnProperty(i)) {
                    event[i] = data[i];
                }
            }
            if (data.target) {
                // TODO: cannot call <some_custom_object>.dispatchEvent
                // need to first figure out how to implement EventTarget
            }
        }
        return event;
    };
    try {
        var ev = createEvent({type:"abort",target:document});
        return function ProgressEvent(type, data) {
            data.type = type;
            return createEvent(data);
        };
    } catch(e){
    */
        return function ProgressEvent(type, dict) {
            this.type = type;
            this.bubbles = false;
            this.cancelBubble = false;
            this.cancelable = false;
            this.lengthComputable = false;
            this.loaded = dict && dict.loaded ? dict.loaded : 0;
            this.total = dict && dict.total ? dict.total : 0;
            this.target = dict && dict.target ? dict.target : null;
        };
    //}
})();

module.exports = ProgressEvent;

});

// file: lib/common/plugin/accelerometer.js
define("cordova/plugin/accelerometer", function(require, exports, module) {

/**
 * This class provides access to device accelerometer data.
 * @constructor
 */
var argscheck = require('cordova/argscheck'),
    utils = require("cordova/utils"),
    exec = require("cordova/exec"),
    Acceleration = require('cordova/plugin/Acceleration');

// Is the accel sensor running?
var running = false;

// Keeps reference to watchAcceleration calls.
var timers = {};

// Array of listeners; used to keep track of when we should call start and stop.
var listeners = [];

// Last returned acceleration object from native
var accel = null;

// Tells native to start.
function start() {
    exec(function(a) {
        var tempListeners = listeners.slice(0);
        accel = new Acceleration(a.x, a.y, a.z, a.timestamp);
        for (var i = 0, l = tempListeners.length; i < l; i++) {
            tempListeners[i].win(accel);
        }
    }, function(e) {
        var tempListeners = listeners.slice(0);
        for (var i = 0, l = tempListeners.length; i < l; i++) {
            tempListeners[i].fail(e);
        }
    }, "Accelerometer", "start", []);
    running = true;
}

// Tells native to stop.
function stop() {
    exec(null, null, "Accelerometer", "stop", []);
    running = false;
}

// Adds a callback pair to the listeners array
function createCallbackPair(win, fail) {
    return {win:win, fail:fail};
}

// Removes a win/fail listener pair from the listeners array
function removeListeners(l) {
    var idx = listeners.indexOf(l);
    if (idx > -1) {
        listeners.splice(idx, 1);
        if (listeners.length === 0) {
            stop();
        }
    }
}

var accelerometer = {
    /**
     * Asynchronously acquires the current acceleration.
     *
     * @param {Function} successCallback    The function to call when the acceleration data is available
     * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
     * @param {AccelerationOptions} options The options for getting the accelerometer data such as timeout. (OPTIONAL)
     */
    getCurrentAcceleration: function(successCallback, errorCallback, options) {
        argscheck.checkArgs('fFO', 'accelerometer.getCurrentAcceleration', arguments);

        var p;
        var win = function(a) {
            removeListeners(p);
            successCallback(a);
        };
        var fail = function(e) {
            removeListeners(p);
            errorCallback && errorCallback(e);
        };

        p = createCallbackPair(win, fail);
        listeners.push(p);

        if (!running) {
            start();
        }
    },

    /**
     * Asynchronously acquires the acceleration repeatedly at a given interval.
     *
     * @param {Function} successCallback    The function to call each time the acceleration data is available
     * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
     * @param {AccelerationOptions} options The options for getting the accelerometer data such as timeout. (OPTIONAL)
     * @return String                       The watch id that must be passed to #clearWatch to stop watching.
     */
    watchAcceleration: function(successCallback, errorCallback, options) {
        argscheck.checkArgs('fFO', 'accelerometer.watchAcceleration', arguments);
        // Default interval (10 sec)
        var frequency = (options && options.frequency && typeof options.frequency == 'number') ? options.frequency : 10000;

        // Keep reference to watch id, and report accel readings as often as defined in frequency
        var id = utils.createUUID();

        var p = createCallbackPair(function(){}, function(e) {
            removeListeners(p);
            errorCallback && errorCallback(e);
        });
        listeners.push(p);

        timers[id] = {
            timer:window.setInterval(function() {
                if (accel) {
                    successCallback(accel);
                }
            }, frequency),
            listeners:p
        };

        if (running) {
            // If we're already running then immediately invoke the success callback
            // but only if we have retrieved a value, sample code does not check for null ...
            if (accel) {
                successCallback(accel);
            }
        } else {
            start();
        }

        return id;
    },

    /**
     * Clears the specified accelerometer watch.
     *
     * @param {String} id       The id of the watch returned from #watchAcceleration.
     */
    clearWatch: function(id) {
        // Stop javascript timer & remove from timer list
        if (id && timers[id]) {
            window.clearInterval(timers[id].timer);
            removeListeners(timers[id].listeners);
            delete timers[id];
        }
    }
};

module.exports = accelerometer;

});

// file: lib/common/plugin/accelerometer/symbols.js
define("cordova/plugin/accelerometer/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.defaults('cordova/plugin/Acceleration', 'Acceleration');
modulemapper.defaults('cordova/plugin/accelerometer', 'navigator.accelerometer');

});

// file: lib/android/plugin/android/app.js
define("cordova/plugin/android/app", function(require, exports, module) {

var exec = require('cordova/exec');

module.exports = {
  /**
   * Clear the resource cache.
   */
  clearCache:function() {
    exec(null, null, "App", "clearCache", []);
  },

  /**
   * Load the url into the webview or into new browser instance.
   *
   * @param url           The URL to load
   * @param props         Properties that can be passed in to the activity:
   *      wait: int                           => wait msec before loading URL
   *      loadingDialog: "Title,Message"      => display a native loading dialog
   *      loadUrlTimeoutValue: int            => time in msec to wait before triggering a timeout error
   *      clearHistory: boolean              => clear webview history (default=false)
   *      openExternal: boolean              => open in a new browser (default=false)
   *
   * Example:
   *      navigator.app.loadUrl("http://server/myapp/index.html", {wait:2000, loadingDialog:"Wait,Loading App", loadUrlTimeoutValue: 60000});
   */
  loadUrl:function(url, props) {
    exec(null, null, "App", "loadUrl", [url, props]);
  },

  /**
   * Cancel loadUrl that is waiting to be loaded.
   */
  cancelLoadUrl:function() {
    exec(null, null, "App", "cancelLoadUrl", []);
  },

  /**
   * Clear web history in this web view.
   * Instead of BACK button loading the previous web page, it will exit the app.
   */
  clearHistory:function() {
    exec(null, null, "App", "clearHistory", []);
  },

  /**
   * Go to previous page displayed.
   * This is the same as pressing the backbutton on Android device.
   */
  backHistory:function() {
    exec(null, null, "App", "backHistory", []);
  },

  /**
   * Override the default behavior of the Android back button.
   * If overridden, when the back button is pressed, the "backKeyDown" JavaScript event will be fired.
   *
   * Note: The user should not have to call this method.  Instead, when the user
   *       registers for the "backbutton" event, this is automatically done.
   *
   * @param override        T=override, F=cancel override
   */
  overrideBackbutton:function(override) {
    exec(null, null, "App", "overrideBackbutton", [override]);
  },

  /**
   * Exit and terminate the application.
   */
  exitApp:function() {
    return exec(null, null, "App", "exitApp", []);
  }
};

});

// file: lib/android/plugin/android/device.js
define("cordova/plugin/android/device", function(require, exports, module) {

var channel = require('cordova/channel'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    app = require('cordova/plugin/android/app');

module.exports = {
    /*
     * DEPRECATED
     * This is only for Android.
     *
     * You must explicitly override the back button.
     */
    overrideBackButton:function() {
        console.log("Device.overrideBackButton() is deprecated.  Use App.overrideBackbutton(true).");
        app.overrideBackbutton(true);
    },

    /*
     * DEPRECATED
     * This is only for Android.
     *
     * This resets the back button to the default behavior
     */
    resetBackButton:function() {
        console.log("Device.resetBackButton() is deprecated.  Use App.overrideBackbutton(false).");
        app.overrideBackbutton(false);
    },

    /*
     * DEPRECATED
     * This is only for Android.
     *
     * This terminates the activity!
     */
    exitApp:function() {
        console.log("Device.exitApp() is deprecated.  Use App.exitApp().");
        app.exitApp();
    }
};

});

// file: lib/android/plugin/android/nativeapiprovider.js
define("cordova/plugin/android/nativeapiprovider", function(require, exports, module) {

var nativeApi = this._cordovaNative || require('cordova/plugin/android/promptbasednativeapi');
var currentApi = nativeApi;

module.exports = {
    get: function() { return currentApi; },
    setPreferPrompt: function(value) {
        currentApi = value ? require('cordova/plugin/android/promptbasednativeapi') : nativeApi;
    },
    // Used only by tests.
    set: function(value) {
        currentApi = value;
    }
};

});

// file: lib/android/plugin/android/notification.js
define("cordova/plugin/android/notification", function(require, exports, module) {

var exec = require('cordova/exec');

/**
 * Provides Android enhanced notification API.
 */
module.exports = {
    activityStart : function(title, message) {
        // If title and message not specified then mimic Android behavior of
        // using default strings.
        if (typeof title === "undefined" && typeof message == "undefined") {
            title = "Busy";
            message = 'Please wait...';
        }

        exec(null, null, 'Notification', 'activityStart', [ title, message ]);
    },

    /**
     * Close an activity dialog
     */
    activityStop : function() {
        exec(null, null, 'Notification', 'activityStop', []);
    },

    /**
     * Display a progress dialog with progress bar that goes from 0 to 100.
     *
     * @param {String}
     *            title Title of the progress dialog.
     * @param {String}
     *            message Message to display in the dialog.
     */
    progressStart : function(title, message) {
        exec(null, null, 'Notification', 'progressStart', [ title, message ]);
    },

    /**
     * Close the progress dialog.
     */
    progressStop : function() {
        exec(null, null, 'Notification', 'progressStop', []);
    },

    /**
     * Set the progress dialog value.
     *
     * @param {Number}
     *            value 0-100
     */
    progressValue : function(value) {
        exec(null, null, 'Notification', 'progressValue', [ value ]);
    }
};

});

// file: lib/android/plugin/android/promptbasednativeapi.js
define("cordova/plugin/android/promptbasednativeapi", function(require, exports, module) {

module.exports = {
    exec: function(service, action, callbackId, argsJson) {
        return prompt(argsJson, 'gap:'+JSON.stringify([service, action, callbackId]));
    },
    setNativeToJsBridgeMode: function(value) {
        prompt(value, 'gap_bridge_mode:');
    },
    retrieveJsMessages: function() {
        return prompt('', 'gap_poll:');
    }
};

});

// file: lib/android/plugin/android/storage.js
define("cordova/plugin/android/storage", function(require, exports, module) {

var utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    channel = require('cordova/channel');

var queryQueue = {};

/**
 * SQL result set object
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Rows = function() {
    this.resultSet = [];    // results array
    this.length = 0;        // number of rows
};

/**
 * Get item from SQL result set
 *
 * @param row           The row number to return
 * @return              The row object
 */
DroidDB_Rows.prototype.item = function(row) {
    return this.resultSet[row];
};

/**
 * SQL result set that is returned to user.
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Result = function() {
    this.rows = new DroidDB_Rows();
};

/**
 * Callback from native code when query is complete.
 * PRIVATE METHOD
 *
 * @param id   Query id
 */
function completeQuery(id, data) {
    var query = queryQueue[id];
    if (query) {
        try {
            delete queryQueue[id];

            // Get transaction
            var tx = query.tx;

            // If transaction hasn't failed
            // Note: We ignore all query results if previous query
            //       in the same transaction failed.
            if (tx && tx.queryList[id]) {

                // Save query results
                var r = new DroidDB_Result();
                r.rows.resultSet = data;
                r.rows.length = data.length;
                try {
                    if (typeof query.successCallback === 'function') {
                        query.successCallback(query.tx, r);
                    }
                } catch (ex) {
                    console.log("executeSql error calling user success callback: "+ex);
                }

                tx.queryComplete(id);
            }
        } catch (e) {
            console.log("executeSql error: "+e);
        }
    }
}

/**
 * Callback from native code when query fails
 * PRIVATE METHOD
 *
 * @param reason            Error message
 * @param id                Query id
 */
function failQuery(reason, id) {
    var query = queryQueue[id];
    if (query) {
        try {
            delete queryQueue[id];

            // Get transaction
            var tx = query.tx;

            // If transaction hasn't failed
            // Note: We ignore all query results if previous query
            //       in the same transaction failed.
            if (tx && tx.queryList[id]) {
                tx.queryList = {};

                try {
                    if (typeof query.errorCallback === 'function') {
                        query.errorCallback(query.tx, reason);
                    }
                } catch (ex) {
                    console.log("executeSql error calling user error callback: "+ex);
                }

                tx.queryFailed(id, reason);
            }

        } catch (e) {
            console.log("executeSql error: "+e);
        }
    }
}

/**
 * SQL query object
 * PRIVATE METHOD
 *
 * @constructor
 * @param tx                The transaction object that this query belongs to
 */
var DroidDB_Query = function(tx) {

    // Set the id of the query
    this.id = utils.createUUID();

    // Add this query to the queue
    queryQueue[this.id] = this;

    // Init result
    this.resultSet = [];

    // Set transaction that this query belongs to
    this.tx = tx;

    // Add this query to transaction list
    this.tx.queryList[this.id] = this;

    // Callbacks
    this.successCallback = null;
    this.errorCallback = null;

};

/**
 * Transaction object
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Tx = function() {

    // Set the id of the transaction
    this.id = utils.createUUID();

    // Callbacks
    this.successCallback = null;
    this.errorCallback = null;

    // Query list
    this.queryList = {};
};

/**
 * Mark query in transaction as complete.
 * If all queries are complete, call the user's transaction success callback.
 *
 * @param id                Query id
 */
DroidDB_Tx.prototype.queryComplete = function(id) {
    delete this.queryList[id];

    // If no more outstanding queries, then fire transaction success
    if (this.successCallback) {
        var count = 0;
        var i;
        for (i in this.queryList) {
            if (this.queryList.hasOwnProperty(i)) {
                count++;
            }
        }
        if (count === 0) {
            try {
                this.successCallback();
            } catch(e) {
                console.log("Transaction error calling user success callback: " + e);
            }
        }
    }
};

/**
 * Mark query in transaction as failed.
 *
 * @param id                Query id
 * @param reason            Error message
 */
DroidDB_Tx.prototype.queryFailed = function(id, reason) {

    // The sql queries in this transaction have already been run, since
    // we really don't have a real transaction implemented in native code.
    // However, the user callbacks for the remaining sql queries in transaction
    // will not be called.
    this.queryList = {};

    if (this.errorCallback) {
        try {
            this.errorCallback(reason);
        } catch(e) {
            console.log("Transaction error calling user error callback: " + e);
        }
    }
};

/**
 * Execute SQL statement
 *
 * @param sql                   SQL statement to execute
 * @param params                Statement parameters
 * @param successCallback       Success callback
 * @param errorCallback         Error callback
 */
DroidDB_Tx.prototype.executeSql = function(sql, params, successCallback, errorCallback) {

    // Init params array
    if (typeof params === 'undefined') {
        params = [];
    }

    // Create query and add to queue
    var query = new DroidDB_Query(this);
    queryQueue[query.id] = query;

    // Save callbacks
    query.successCallback = successCallback;
    query.errorCallback = errorCallback;

    // Call native code
    exec(null, null, "Storage", "executeSql", [sql, params, query.id]);
};

var DatabaseShell = function() {
};

/**
 * Start a transaction.
 * Does not support rollback in event of failure.
 *
 * @param process {Function}            The transaction function
 * @param successCallback {Function}
 * @param errorCallback {Function}
 */
DatabaseShell.prototype.transaction = function(process, errorCallback, successCallback) {
    var tx = new DroidDB_Tx();
    tx.successCallback = successCallback;
    tx.errorCallback = errorCallback;
    try {
        process(tx);
    } catch (e) {
        console.log("Transaction error: "+e);
        if (tx.errorCallback) {
            try {
                tx.errorCallback(e);
            } catch (ex) {
                console.log("Transaction error calling user error callback: "+e);
            }
        }
    }
};

/**
 * Open database
 *
 * @param name              Database name
 * @param version           Database version
 * @param display_name      Database display name
 * @param size              Database size in bytes
 * @return                  Database object
 */
var DroidDB_openDatabase = function(name, version, display_name, size) {
    exec(null, null, "Storage", "openDatabase", [name, version, display_name, size]);
    var db = new DatabaseShell();
    return db;
};


module.exports = {
  openDatabase:DroidDB_openDatabase,
  failQuery:failQuery,
  completeQuery:completeQuery
};

});

// file: lib/android/plugin/android/storage/openDatabase.js
define("cordova/plugin/android/storage/openDatabase", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper'),
    storage = require('cordova/plugin/android/storage');

var originalOpenDatabase = modulemapper.getOriginalSymbol(window, 'openDatabase');

module.exports = function(name, version, desc, size) {
    // First patch WebSQL if necessary
    if (!originalOpenDatabase) {
        // Not defined, create an openDatabase function for all to use!
        return storage.openDatabase.apply(this, arguments);
    }

    // Defined, but some Android devices will throw a SECURITY_ERR -
    // so we wrap the whole thing in a try-catch and shim in our own
    // if the device has Android bug 16175.
    try {
        return originalOpenDatabase(name, version, desc, size);
    } catch (ex) {
        if (ex.code !== 18) {
            throw ex;
        }
    }
    return storage.openDatabase(name, version, desc, size);
};



});

// file: lib/android/plugin/android/storage/symbols.js
define("cordova/plugin/android/storage/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/android/storage/openDatabase', 'openDatabase');


});

// file: lib/common/plugin/battery.js
define("cordova/plugin/battery", function(require, exports, module) {

/**
 * This class contains information about the current battery status.
 * @constructor
 */
var cordova = require('cordova'),
    exec = require('cordova/exec');

function handlers() {
  return battery.channels.batterystatus.numHandlers +
         battery.channels.batterylow.numHandlers +
         battery.channels.batterycritical.numHandlers;
}

var Battery = function() {
    this._level = null;
    this._isPlugged = null;
    // Create new event handlers on the window (returns a channel instance)
    this.channels = {
      batterystatus:cordova.addWindowEventHandler("batterystatus"),
      batterylow:cordova.addWindowEventHandler("batterylow"),
      batterycritical:cordova.addWindowEventHandler("batterycritical")
    };
    for (var key in this.channels) {
        this.channels[key].onHasSubscribersChange = Battery.onHasSubscribersChange;
    }
};
/**
 * Event handlers for when callbacks get registered for the battery.
 * Keep track of how many handlers we have so we can start and stop the native battery listener
 * appropriately (and hopefully save on battery life!).
 */
Battery.onHasSubscribersChange = function() {
  // If we just registered the first handler, make sure native listener is started.
  if (this.numHandlers === 1 && handlers() === 1) {
      exec(battery._status, battery._error, "Battery", "start", []);
  } else if (handlers() === 0) {
      exec(null, null, "Battery", "stop", []);
  }
};

/**
 * Callback for battery status
 *
 * @param {Object} info            keys: level, isPlugged
 */
Battery.prototype._status = function(info) {
    if (info) {
        var me = battery;
    var level = info.level;
        if (me._level !== level || me._isPlugged !== info.isPlugged) {
            // Fire batterystatus event
            cordova.fireWindowEvent("batterystatus", info);

            // Fire low battery event
            if (level === 20 || level === 5) {
                if (level === 20) {
                    cordova.fireWindowEvent("batterylow", info);
                }
                else {
                    cordova.fireWindowEvent("batterycritical", info);
                }
            }
        }
        me._level = level;
        me._isPlugged = info.isPlugged;
    }
};

/**
 * Error callback for battery start
 */
Battery.prototype._error = function(e) {
    console.log("Error initializing Battery: " + e);
};

var battery = new Battery();

module.exports = battery;

});

// file: lib/common/plugin/battery/symbols.js
define("cordova/plugin/battery/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.defaults('cordova/plugin/battery', 'navigator.battery');

});

// file: lib/common/plugin/camera/symbols.js
define("cordova/plugin/camera/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.defaults('cordova/plugin/Camera', 'navigator.camera');
modulemapper.defaults('cordova/plugin/CameraConstants', 'Camera');
modulemapper.defaults('cordova/plugin/CameraPopoverOptions', 'CameraPopoverOptions');

});

// file: lib/common/plugin/capture.js
define("cordova/plugin/capture", function(require, exports, module) {

var exec = require('cordova/exec'),
    MediaFile = require('cordova/plugin/MediaFile');

/**
 * Launches a capture of different types.
 *
 * @param (DOMString} type
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureVideoOptions} options
 */
function _capture(type, successCallback, errorCallback, options) {
    var win = function(pluginResult) {
        var mediaFiles = [];
        var i;
        for (i = 0; i < pluginResult.length; i++) {
            var mediaFile = new MediaFile();
            mediaFile.name = pluginResult[i].name;
            mediaFile.fullPath = pluginResult[i].fullPath;
            mediaFile.type = pluginResult[i].type;
            mediaFile.lastModifiedDate = pluginResult[i].lastModifiedDate;
            mediaFile.size = pluginResult[i].size;
            mediaFiles.push(mediaFile);
        }
        successCallback(mediaFiles);
    };
    exec(win, errorCallback, "Capture", type, [options]);
}
/**
 * The Capture interface exposes an interface to the camera and microphone of the hosting device.
 */
function Capture() {
    this.supportedAudioModes = [];
    this.supportedImageModes = [];
    this.supportedVideoModes = [];
}

/**
 * Launch audio recorder application for recording audio clip(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureAudioOptions} options
 */
Capture.prototype.captureAudio = function(successCallback, errorCallback, options){
    _capture("captureAudio", successCallback, errorCallback, options);
};

/**
 * Launch camera application for taking image(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureImageOptions} options
 */
Capture.prototype.captureImage = function(successCallback, errorCallback, options){
    _capture("captureImage", successCallback, errorCallback, options);
};

/**
 * Launch device camera application for recording video(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureVideoOptions} options
 */
Capture.prototype.captureVideo = function(successCallback, errorCallback, options){
    _capture("captureVideo", successCallback, errorCallback, options);
};


module.exports = new Capture();

});

// file: lib/common/plugin/capture/symbols.js
define("cordova/plugin/capture/symbols", function(require, exports, module) {

var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/CaptureError', 'CaptureError');
modulemapper.clobbers('cordova/plugin/CaptureAudioOptions', 'CaptureAudioOptions');
modulemapper.clobbers('cordova/plugin/CaptureImageOptions', 'CaptureImageOptions');
modulemapper.clobbers('cordova/plugin/CaptureVideoOptions', 'CaptureVideoOptions');
modulemapper.clobbers('cordova/plugin/ConfigurationData', 'ConfigurationData');
modulemapper.clobbers('cordova/plugin/MediaFile', 'MediaFile');
modulemapper.clobbers('cordova/plugin/MediaFileData', 'MediaFileData');
modulemapper.clobbers('cordova/plugin/capture', 'navigator.device.capture');

});

// file: lib/common/plugin/compass.js
define("cordova/plugin/compass", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    utils = require('cordova/utils'),
    CompassHeading = require('cordova/plugin/CompassHeading'),
    CompassError = require('cordova/plugin/CompassError'),
    timers = {},
    compass = {
        /**
         * Asynchronously acquires the current heading.
         * @param {Function} successCallback The function to call when the heading
         * data is available
         * @param {Function} errorCallback The function to call when there is an error
         * getting the heading data.
         * @param {CompassOptions} options The options for getting the heading data (not used).
         */
        getCurrentHeading:function(successCallback, errorCallback, options) {
            argscheck.checkArgs('fFO', 'compass.getCurrentHeading', arguments);

            var win = function(result) {
                var ch = new CompassHeading(result.magneticHeading, result.trueHeading, result.headingAccuracy, result.timestamp);
                successCallback(ch);
            };
            var fail = errorCallback && function(code) {
                var ce = new CompassError(code);
                errorCallback(ce);
            };

            // Get heading
            exec(win, fail, "Compass", "getHeading", [options]);
        },

        /**
         * Asynchronously acquires the heading repeatedly at a given interval.
         * @param {Function} successCallback The function to call each time the heading
         * data is available
         * @param {Function} errorCallback The function to call when there is an error
         * getting the heading data.
         * @param {HeadingOptions} options The options for getting the heading data
         * such as timeout and the frequency of the watch. For iOS, filter parameter
         * specifies to watch via a distance filter rather than time.
         */
        watchHeading:function(successCallback, errorCallback, options) {
            argscheck.checkArgs('fFO', 'compass.watchHeading', arguments);
            // Default interval (100 msec)
            var frequency = (options !== undefined && options.frequency !== undefined) ? options.frequency : 100;
            var filter = (options !== undefined && options.filter !== undefined) ? options.filter : 0;

            var id = utils.createUUID();
            if (filter > 0) {
                // is an iOS request for watch by filter, no timer needed
                timers[id] = "iOS";
                compass.getCurrentHeading(successCallback, errorCallback, options);
            } else {
                // Start watch timer to get headings
                timers[id] = window.setInterval(function() {
                    compass.getCurrentHeading(successCallback, errorCallback);
                }, frequency);
            }

            return id;
        },

        /**
         * Clears the specified heading watch.
         * @param {String} watchId The ID of the watch returned from #watchHeading.
         */
        clearWatch:function(id) {
            // Stop javascript timer & remove from timer list
            if (id && timers[id]) {
                if (timers[id] != "iOS") {
                    clearInterval(timers[id]);
                } else {
                    // is iOS watch by filter so call into device to stop
                    exec(null, null, "Compass", "stopHeading", []);
                }
                delete timers[id];
            }
        }
    };

module.exports = compass;

});

// file: lib/common/plugin/compass/symbols.js
define("cordova/plugin/compass/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/CompassHeading', 'CompassHeading');
modulemapper.clobbers('cordova/plugin/CompassError', 'CompassError');
modulemapper.clobbers('cordova/plugin/compass', 'navigator.compass');

});

// file: lib/common/plugin/console-via-logger.js
define("cordova/plugin/console-via-logger", function(require, exports, module) {

//------------------------------------------------------------------------------

var logger = require("cordova/plugin/logger");
var utils  = require("cordova/utils");

//------------------------------------------------------------------------------
// object that we're exporting
//------------------------------------------------------------------------------
var console = module.exports;

//------------------------------------------------------------------------------
// copy of the original console object
//------------------------------------------------------------------------------
var WinConsole = window.console;

//------------------------------------------------------------------------------
// whether to use the logger
//------------------------------------------------------------------------------
var UseLogger = false;

//------------------------------------------------------------------------------
// Timers
//------------------------------------------------------------------------------
var Timers = {};

//------------------------------------------------------------------------------
// used for unimplemented methods
//------------------------------------------------------------------------------
function noop() {}

//------------------------------------------------------------------------------
// used for unimplemented methods
//------------------------------------------------------------------------------
console.useLogger = function (value) {
    if (arguments.length) UseLogger = !!value;

    if (UseLogger) {
        if (logger.useConsole()) {
            throw new Error("console and logger are too intertwingly");
        }
    }

    return UseLogger;
};

//------------------------------------------------------------------------------
console.log = function() {
    if (logger.useConsole()) return;
    logger.log.apply(logger, [].slice.call(arguments));
};

//------------------------------------------------------------------------------
console.error = function() {
    if (logger.useConsole()) return;
    logger.error.apply(logger, [].slice.call(arguments));
};

//------------------------------------------------------------------------------
console.warn = function() {
    if (logger.useConsole()) return;
    logger.warn.apply(logger, [].slice.call(arguments));
};

//------------------------------------------------------------------------------
console.info = function() {
    if (logger.useConsole()) return;
    logger.info.apply(logger, [].slice.call(arguments));
};

//------------------------------------------------------------------------------
console.debug = function() {
    if (logger.useConsole()) return;
    logger.debug.apply(logger, [].slice.call(arguments));
};

//------------------------------------------------------------------------------
console.assert = function(expression) {
    if (expression) return;

    var message = logger.format.apply(logger.format, [].slice.call(arguments, 1));
    console.log("ASSERT: " + message);
};

//------------------------------------------------------------------------------
console.clear = function() {};

//------------------------------------------------------------------------------
console.dir = function(object) {
    console.log("%o", object);
};

//------------------------------------------------------------------------------
console.dirxml = function(node) {
    console.log(node.innerHTML);
};

//------------------------------------------------------------------------------
console.trace = noop;

//------------------------------------------------------------------------------
console.group = console.log;

//------------------------------------------------------------------------------
console.groupCollapsed = console.log;

//------------------------------------------------------------------------------
console.groupEnd = noop;

//------------------------------------------------------------------------------
console.time = function(name) {
    Timers[name] = new Date().valueOf();
};

//------------------------------------------------------------------------------
console.timeEnd = function(name) {
    var timeStart = Timers[name];
    if (!timeStart) {
        console.warn("unknown timer: " + name);
        return;
    }

    var timeElapsed = new Date().valueOf() - timeStart;
    console.log(name + ": " + timeElapsed + "ms");
};

//------------------------------------------------------------------------------
console.timeStamp = noop;

//------------------------------------------------------------------------------
console.profile = noop;

//------------------------------------------------------------------------------
console.profileEnd = noop;

//------------------------------------------------------------------------------
console.count = noop;

//------------------------------------------------------------------------------
console.exception = console.log;

//------------------------------------------------------------------------------
console.table = function(data, columns) {
    console.log("%o", data);
};

//------------------------------------------------------------------------------
// return a new function that calls both functions passed as args
//------------------------------------------------------------------------------
function wrappedOrigCall(orgFunc, newFunc) {
    return function() {
        var args = [].slice.call(arguments);
        try { orgFunc.apply(WinConsole, args); } catch (e) {}
        try { newFunc.apply(console,    args); } catch (e) {}
    };
}

//------------------------------------------------------------------------------
// For every function that exists in the original console object, that
// also exists in the new console object, wrap the new console method
// with one that calls both
//------------------------------------------------------------------------------
for (var key in console) {
    if (typeof WinConsole[key] == "function") {
        console[key] = wrappedOrigCall(WinConsole[key], console[key]);
    }
}

});

// file: lib/common/plugin/contacts.js
define("cordova/plugin/contacts", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    ContactError = require('cordova/plugin/ContactError'),
    utils = require('cordova/utils'),
    Contact = require('cordova/plugin/Contact');

/**
* Represents a group of Contacts.
* @constructor
*/
var contacts = {
    /**
     * Returns an array of Contacts matching the search criteria.
     * @param fields that should be searched
     * @param successCB success callback
     * @param errorCB error callback
     * @param {ContactFindOptions} options that can be applied to contact searching
     * @return array of Contacts matching search criteria
     */
    find:function(fields, successCB, errorCB, options) {
        argscheck.checkArgs('afFO', 'contacts.find', arguments);
        if (!fields.length) {
            errorCB && errorCB(new ContactError(ContactError.INVALID_ARGUMENT_ERROR));
        } else {
            var win = function(result) {
                var cs = [];
                for (var i = 0, l = result.length; i < l; i++) {
                    cs.push(contacts.create(result[i]));
                }
                successCB(cs);
            };
            exec(win, errorCB, "Contacts", "search", [fields, options]);
        }
    },

    /**
     * This function creates a new contact, but it does not persist the contact
     * to device storage. To persist the contact to device storage, invoke
     * contact.save().
     * @param properties an object whose properties will be examined to create a new Contact
     * @returns new Contact object
     */
    create:function(properties) {
        argscheck.checkArgs('O', 'contacts.create', arguments);
        var contact = new Contact();
        for (var i in properties) {
            if (typeof contact[i] !== 'undefined' && properties.hasOwnProperty(i)) {
                contact[i] = properties[i];
            }
        }
        return contact;
    }
};

module.exports = contacts;

});

// file: lib/common/plugin/contacts/symbols.js
define("cordova/plugin/contacts/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/contacts', 'navigator.contacts');
modulemapper.clobbers('cordova/plugin/Contact', 'Contact');
modulemapper.clobbers('cordova/plugin/ContactAddress', 'ContactAddress');
modulemapper.clobbers('cordova/plugin/ContactError', 'ContactError');
modulemapper.clobbers('cordova/plugin/ContactField', 'ContactField');
modulemapper.clobbers('cordova/plugin/ContactFindOptions', 'ContactFindOptions');
modulemapper.clobbers('cordova/plugin/ContactName', 'ContactName');
modulemapper.clobbers('cordova/plugin/ContactOrganization', 'ContactOrganization');

});

// file: lib/common/plugin/device.js
define("cordova/plugin/device", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    channel = require('cordova/channel'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec');

// Tell cordova channel to wait on the CordovaInfoReady event
channel.waitForInitialization('onCordovaInfoReady');

/**
 * This represents the mobile device, and provides properties for inspecting the model, version, UUID of the
 * phone, etc.
 * @constructor
 */
function Device() {
    this.available = false;
    this.platform = null;
    this.version = null;
    this.name = null;
    this.uuid = null;
    this.cordova = null;
    this.model = null;

    var me = this;

    channel.onCordovaReady.subscribe(function() {
        me.getInfo(function(info) {
            me.available = true;
            me.platform = info.platform;
            me.version = info.version;
            me.name = info.name;
            me.uuid = info.uuid;
            me.cordova = info.cordova;
            me.model = info.model;
            channel.onCordovaInfoReady.fire();
        },function(e) {
            me.available = false;
            utils.alert("[ERROR] Error initializing Cordova: " + e);
        });
    });
}

/**
 * Get device info
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 */
Device.prototype.getInfo = function(successCallback, errorCallback) {
    argscheck.checkArgs('fF', 'Device.getInfo', arguments);
    exec(successCallback, errorCallback, "Device", "getDeviceInfo", []);
};

module.exports = new Device();

});

// file: lib/android/plugin/device/symbols.js
define("cordova/plugin/device/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/device', 'device');
modulemapper.merges('cordova/plugin/android/device', 'device');

});

// file: lib/common/plugin/echo.js
define("cordova/plugin/echo", function(require, exports, module) {

var exec = require('cordova/exec'),
    utils = require('cordova/utils');

/**
 * Sends the given message through exec() to the Echo plugin, which sends it back to the successCallback.
 * @param successCallback  invoked with a FileSystem object
 * @param errorCallback  invoked if error occurs retrieving file system
 * @param message  The string to be echoed.
 * @param forceAsync  Whether to force an async return value (for testing native->js bridge).
 */
module.exports = function(successCallback, errorCallback, message, forceAsync) {
    var action = 'echo';
    var messageIsMultipart = (utils.typeName(message) == "Array");
    var args = messageIsMultipart ? message : [message];

    if (utils.typeName(message) == 'ArrayBuffer') {
        if (forceAsync) {
            console.warn('Cannot echo ArrayBuffer with forced async, falling back to sync.');
        }
        action += 'ArrayBuffer';
    } else if (messageIsMultipart) {
        if (forceAsync) {
            console.warn('Cannot echo MultiPart Array with forced async, falling back to sync.');
        }
        action += 'MultiPart';
    } else if (forceAsync) {
        action += 'Async';
    }

    exec(successCallback, errorCallback, "Echo", action, args);
};


});

// file: lib/android/plugin/file/symbols.js
define("cordova/plugin/file/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper'),
    symbolshelper = require('cordova/plugin/file/symbolshelper');

symbolshelper(modulemapper.clobbers);

});

// file: lib/common/plugin/file/symbolshelper.js
define("cordova/plugin/file/symbolshelper", function(require, exports, module) {

module.exports = function(exportFunc) {
    exportFunc('cordova/plugin/DirectoryEntry', 'DirectoryEntry');
    exportFunc('cordova/plugin/DirectoryReader', 'DirectoryReader');
    exportFunc('cordova/plugin/Entry', 'Entry');
    exportFunc('cordova/plugin/File', 'File');
    exportFunc('cordova/plugin/FileEntry', 'FileEntry');
    exportFunc('cordova/plugin/FileError', 'FileError');
    exportFunc('cordova/plugin/FileReader', 'FileReader');
    exportFunc('cordova/plugin/FileSystem', 'FileSystem');
    exportFunc('cordova/plugin/FileUploadOptions', 'FileUploadOptions');
    exportFunc('cordova/plugin/FileUploadResult', 'FileUploadResult');
    exportFunc('cordova/plugin/FileWriter', 'FileWriter');
    exportFunc('cordova/plugin/Flags', 'Flags');
    exportFunc('cordova/plugin/LocalFileSystem', 'LocalFileSystem');
    exportFunc('cordova/plugin/Metadata', 'Metadata');
    exportFunc('cordova/plugin/ProgressEvent', 'ProgressEvent');
    exportFunc('cordova/plugin/requestFileSystem', 'requestFileSystem');
    exportFunc('cordova/plugin/resolveLocalFileSystemURI', 'resolveLocalFileSystemURI');
};

});

// file: lib/common/plugin/filetransfer/symbols.js
define("cordova/plugin/filetransfer/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/FileTransfer', 'FileTransfer');
modulemapper.clobbers('cordova/plugin/FileTransferError', 'FileTransferError');

});

// file: lib/common/plugin/geolocation.js
define("cordova/plugin/geolocation", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    PositionError = require('cordova/plugin/PositionError'),
    Position = require('cordova/plugin/Position');

var timers = {};   // list of timers in use

// Returns default params, overrides if provided with values
function parseParameters(options) {
    var opt = {
        maximumAge: 0,
        enableHighAccuracy: false,
        timeout: Infinity
    };

    if (options) {
        if (options.maximumAge !== undefined && !isNaN(options.maximumAge) && options.maximumAge > 0) {
            opt.maximumAge = options.maximumAge;
        }
        if (options.enableHighAccuracy !== undefined) {
            opt.enableHighAccuracy = options.enableHighAccuracy;
        }
        if (options.timeout !== undefined && !isNaN(options.timeout)) {
            if (options.timeout < 0) {
                opt.timeout = 0;
            } else {
                opt.timeout = options.timeout;
            }
        }
    }

    return opt;
}

// Returns a timeout failure, closed over a specified timeout value and error callback.
function createTimeout(errorCallback, timeout) {
    var t = setTimeout(function() {
        clearTimeout(t);
        t = null;
        errorCallback({
            code:PositionError.TIMEOUT,
            message:"Position retrieval timed out."
        });
    }, timeout);
    return t;
}

var geolocation = {
    lastPosition:null, // reference to last known (cached) position returned
    /**
   * Asynchronously acquires the current position.
   *
   * @param {Function} successCallback    The function to call when the position data is available
   * @param {Function} errorCallback      The function to call when there is an error getting the heading position. (OPTIONAL)
   * @param {PositionOptions} options     The options for getting the position data. (OPTIONAL)
   */
    getCurrentPosition:function(successCallback, errorCallback, options) {
        argscheck.checkArgs('fFO', 'geolocation.getCurrentPosition', arguments);
        options = parseParameters(options);

        // Timer var that will fire an error callback if no position is retrieved from native
        // before the "timeout" param provided expires
        var timeoutTimer = {timer:null};

        var win = function(p) {
            clearTimeout(timeoutTimer.timer);
            if (!(timeoutTimer.timer)) {
                // Timeout already happened, or native fired error callback for
                // this geo request.
                // Don't continue with success callback.
                return;
            }
            var pos = new Position(
                {
                    latitude:p.latitude,
                    longitude:p.longitude,
                    altitude:p.altitude,
                    accuracy:p.accuracy,
                    heading:p.heading,
                    velocity:p.velocity,
                    altitudeAccuracy:p.altitudeAccuracy
                },
                (p.timestamp === undefined ? new Date() : ((p.timestamp instanceof Date) ? p.timestamp : new Date(p.timestamp)))
            );
            geolocation.lastPosition = pos;
            successCallback(pos);
        };
        var fail = function(e) {
            clearTimeout(timeoutTimer.timer);
            timeoutTimer.timer = null;
            var err = new PositionError(e.code, e.message);
            if (errorCallback) {
                errorCallback(err);
            }
        };

        // Check our cached position, if its timestamp difference with current time is less than the maximumAge, then just
        // fire the success callback with the cached position.
        if (geolocation.lastPosition && options.maximumAge && (((new Date()).getTime() - geolocation.lastPosition.timestamp.getTime()) <= options.maximumAge)) {
            successCallback(geolocation.lastPosition);
        // If the cached position check failed and the timeout was set to 0, error out with a TIMEOUT error object.
        } else if (options.timeout === 0) {
            fail({
                code:PositionError.TIMEOUT,
                message:"timeout value in PositionOptions set to 0 and no cached Position object available, or cached Position object's age exceeds provided PositionOptions' maximumAge parameter."
            });
        // Otherwise we have to call into native to retrieve a position.
        } else {
            if (options.timeout !== Infinity) {
                // If the timeout value was not set to Infinity (default), then
                // set up a timeout function that will fire the error callback
                // if no successful position was retrieved before timeout expired.
                timeoutTimer.timer = createTimeout(fail, options.timeout);
            } else {
                // This is here so the check in the win function doesn't mess stuff up
                // may seem weird but this guarantees timeoutTimer is
                // always truthy before we call into native
                timeoutTimer.timer = true;
            }
            exec(win, fail, "Geolocation", "getLocation", [options.enableHighAccuracy, options.maximumAge]);
        }
        return timeoutTimer;
    },
    /**
     * Asynchronously watches the geolocation for changes to geolocation.  When a change occurs,
     * the successCallback is called with the new location.
     *
     * @param {Function} successCallback    The function to call each time the location data is available
     * @param {Function} errorCallback      The function to call when there is an error getting the location data. (OPTIONAL)
     * @param {PositionOptions} options     The options for getting the location data such as frequency. (OPTIONAL)
     * @return String                       The watch id that must be passed to #clearWatch to stop watching.
     */
    watchPosition:function(successCallback, errorCallback, options) {
        argscheck.checkArgs('fFO', 'geolocation.getCurrentPosition', arguments);
        options = parseParameters(options);

        var id = utils.createUUID();

        // Tell device to get a position ASAP, and also retrieve a reference to the timeout timer generated in getCurrentPosition
        timers[id] = geolocation.getCurrentPosition(successCallback, errorCallback, options);

        var fail = function(e) {
            clearTimeout(timers[id].timer);
            var err = new PositionError(e.code, e.message);
            if (errorCallback) {
                errorCallback(err);
            }
        };

        var win = function(p) {
            clearTimeout(timers[id].timer);
            if (options.timeout !== Infinity) {
                timers[id].timer = createTimeout(fail, options.timeout);
            }
            var pos = new Position(
                {
                    latitude:p.latitude,
                    longitude:p.longitude,
                    altitude:p.altitude,
                    accuracy:p.accuracy,
                    heading:p.heading,
                    velocity:p.velocity,
                    altitudeAccuracy:p.altitudeAccuracy
                },
                (p.timestamp === undefined ? new Date() : ((p.timestamp instanceof Date) ? p.timestamp : new Date(p.timestamp)))
            );
            geolocation.lastPosition = pos;
            successCallback(pos);
        };

        exec(win, fail, "Geolocation", "addWatch", [id, options.enableHighAccuracy]);

        return id;
    },
    /**
     * Clears the specified heading watch.
     *
     * @param {String} id       The ID of the watch returned from #watchPosition
     */
    clearWatch:function(id) {
        if (id && timers[id] !== undefined) {
            clearTimeout(timers[id].timer);
            timers[id].timer = false;
            exec(null, null, "Geolocation", "clearWatch", [id]);
        }
    }
};

module.exports = geolocation;

});

// file: lib/common/plugin/geolocation/symbols.js
define("cordova/plugin/geolocation/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.defaults('cordova/plugin/geolocation', 'navigator.geolocation');
modulemapper.clobbers('cordova/plugin/PositionError', 'PositionError');
modulemapper.clobbers('cordova/plugin/Position', 'Position');
modulemapper.clobbers('cordova/plugin/Coordinates', 'Coordinates');

});

// file: lib/common/plugin/globalization.js
define("cordova/plugin/globalization", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    exec = require('cordova/exec'),
    GlobalizationError = require('cordova/plugin/GlobalizationError');

var globalization = {

/**
* Returns the string identifier for the client's current language.
* It returns the language identifier string to the successCB callback with a
* properties object as a parameter. If there is an error getting the language,
* then the errorCB callback is invoked.
*
* @param {Function} successCB
* @param {Function} errorCB
*
* @return Object.value {String}: The language identifier
*
* @error GlobalizationError.UNKNOWN_ERROR
*
* Example
*    globalization.getPreferredLanguage(function (language) {alert('language:' + language.value + '\n');},
*                                function () {});
*/
getPreferredLanguage:function(successCB, failureCB) {
    argscheck.checkArgs('fF', 'Globalization.getPreferredLanguage', arguments);
    exec(successCB, failureCB, "Globalization","getPreferredLanguage", []);
},

/**
* Returns the string identifier for the client's current locale setting.
* It returns the locale identifier string to the successCB callback with a
* properties object as a parameter. If there is an error getting the locale,
* then the errorCB callback is invoked.
*
* @param {Function} successCB
* @param {Function} errorCB
*
* @return Object.value {String}: The locale identifier
*
* @error GlobalizationError.UNKNOWN_ERROR
*
* Example
*    globalization.getLocaleName(function (locale) {alert('locale:' + locale.value + '\n');},
*                                function () {});
*/
getLocaleName:function(successCB, failureCB) {
    argscheck.checkArgs('fF', 'Globalization.getLocaleName', arguments);
    exec(successCB, failureCB, "Globalization","getLocaleName", []);
},


/**
* Returns a date formatted as a string according to the client's user preferences and
* calendar using the time zone of the client. It returns the formatted date string to the
* successCB callback with a properties object as a parameter. If there is an error
* formatting the date, then the errorCB callback is invoked.
*
* The defaults are: formatLenght="short" and selector="date and time"
*
* @param {Date} date
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            formatLength {String}: 'short', 'medium', 'long', or 'full'
*            selector {String}: 'date', 'time', or 'date and time'
*
* @return Object.value {String}: The localized date string
*
* @error GlobalizationError.FORMATTING_ERROR
*
* Example
*    globalization.dateToString(new Date(),
*                function (date) {alert('date:' + date.value + '\n');},
*                function (errorCode) {alert(errorCode);},
*                {formatLength:'short'});
*/
dateToString:function(date, successCB, failureCB, options) {
    argscheck.checkArgs('dfFO', 'Globalization.dateToString', arguments);
    var dateValue = date.valueOf();
    exec(successCB, failureCB, "Globalization", "dateToString", [{"date": dateValue, "options": options}]);
},


/**
* Parses a date formatted as a string according to the client's user
* preferences and calendar using the time zone of the client and returns
* the corresponding date object. It returns the date to the successCB
* callback with a properties object as a parameter. If there is an error
* parsing the date string, then the errorCB callback is invoked.
*
* The defaults are: formatLength="short" and selector="date and time"
*
* @param {String} dateString
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            formatLength {String}: 'short', 'medium', 'long', or 'full'
*            selector {String}: 'date', 'time', or 'date and time'
*
* @return    Object.year {Number}: The four digit year
*            Object.month {Number}: The month from (0 - 11)
*            Object.day {Number}: The day from (1 - 31)
*            Object.hour {Number}: The hour from (0 - 23)
*            Object.minute {Number}: The minute from (0 - 59)
*            Object.second {Number}: The second from (0 - 59)
*            Object.millisecond {Number}: The milliseconds (from 0 - 999),
*                                        not available on all platforms
*
* @error GlobalizationError.PARSING_ERROR
*
* Example
*    globalization.stringToDate('4/11/2011',
*                function (date) { alert('Month:' + date.month + '\n' +
*                    'Day:' + date.day + '\n' +
*                    'Year:' + date.year + '\n');},
*                function (errorCode) {alert(errorCode);},
*                {selector:'date'});
*/
stringToDate:function(dateString, successCB, failureCB, options) {
    argscheck.checkArgs('sfFO', 'Globalization.stringToDate', arguments);
    exec(successCB, failureCB, "Globalization", "stringToDate", [{"dateString": dateString, "options": options}]);
},


/**
* Returns a pattern string for formatting and parsing dates according to the client's
* user preferences. It returns the pattern to the successCB callback with a
* properties object as a parameter. If there is an error obtaining the pattern,
* then the errorCB callback is invoked.
*
* The defaults are: formatLength="short" and selector="date and time"
*
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            formatLength {String}: 'short', 'medium', 'long', or 'full'
*            selector {String}: 'date', 'time', or 'date and time'
*
* @return    Object.pattern {String}: The date and time pattern for formatting and parsing dates.
*                                    The patterns follow Unicode Technical Standard #35
*                                    http://unicode.org/reports/tr35/tr35-4.html
*            Object.timezone {String}: The abbreviated name of the time zone on the client
*            Object.utc_offset {Number}: The current difference in seconds between the client's
*                                        time zone and coordinated universal time.
*            Object.dst_offset {Number}: The current daylight saving time offset in seconds
*                                        between the client's non-daylight saving's time zone
*                                        and the client's daylight saving's time zone.
*
* @error GlobalizationError.PATTERN_ERROR
*
* Example
*    globalization.getDatePattern(
*                function (date) {alert('pattern:' + date.pattern + '\n');},
*                function () {},
*                {formatLength:'short'});
*/
getDatePattern:function(successCB, failureCB, options) {
    argscheck.checkArgs('fFO', 'Globalization.getDatePattern', arguments);
    exec(successCB, failureCB, "Globalization", "getDatePattern", [{"options": options}]);
},


/**
* Returns an array of either the names of the months or days of the week
* according to the client's user preferences and calendar. It returns the array of names to the
* successCB callback with a properties object as a parameter. If there is an error obtaining the
* names, then the errorCB callback is invoked.
*
* The defaults are: type="wide" and item="months"
*
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            type {String}: 'narrow' or 'wide'
*            item {String}: 'months', or 'days'
*
* @return Object.value {Array{String}}: The array of names starting from either
*                                        the first month in the year or the
*                                        first day of the week.
* @error GlobalizationError.UNKNOWN_ERROR
*
* Example
*    globalization.getDateNames(function (names) {
*        for(var i = 0; i < names.value.length; i++) {
*            alert('Month:' + names.value[i] + '\n');}},
*        function () {});
*/
getDateNames:function(successCB, failureCB, options) {
    argscheck.checkArgs('fFO', 'Globalization.getDateNames', arguments);
    exec(successCB, failureCB, "Globalization", "getDateNames", [{"options": options}]);
},

/**
* Returns whether daylight savings time is in effect for a given date using the client's
* time zone and calendar. It returns whether or not daylight savings time is in effect
* to the successCB callback with a properties object as a parameter. If there is an error
* reading the date, then the errorCB callback is invoked.
*
* @param {Date} date
* @param {Function} successCB
* @param {Function} errorCB
*
* @return Object.dst {Boolean}: The value "true" indicates that daylight savings time is
*                                in effect for the given date and "false" indicate that it is not.
*
* @error GlobalizationError.UNKNOWN_ERROR
*
* Example
*    globalization.isDayLightSavingsTime(new Date(),
*                function (date) {alert('dst:' + date.dst + '\n');}
*                function () {});
*/
isDayLightSavingsTime:function(date, successCB, failureCB) {
    argscheck.checkArgs('dfF', 'Globalization.isDayLightSavingsTime', arguments);
    var dateValue = date.valueOf();
    exec(successCB, failureCB, "Globalization", "isDayLightSavingsTime", [{"date": dateValue}]);
},

/**
* Returns the first day of the week according to the client's user preferences and calendar.
* The days of the week are numbered starting from 1 where 1 is considered to be Sunday.
* It returns the day to the successCB callback with a properties object as a parameter.
* If there is an error obtaining the pattern, then the errorCB callback is invoked.
*
* @param {Function} successCB
* @param {Function} errorCB
*
* @return Object.value {Number}: The number of the first day of the week.
*
* @error GlobalizationError.UNKNOWN_ERROR
*
* Example
*    globalization.getFirstDayOfWeek(function (day)
*                { alert('Day:' + day.value + '\n');},
*                function () {});
*/
getFirstDayOfWeek:function(successCB, failureCB) {
    argscheck.checkArgs('fF', 'Globalization.getFirstDayOfWeek', arguments);
    exec(successCB, failureCB, "Globalization", "getFirstDayOfWeek", []);
},


/**
* Returns a number formatted as a string according to the client's user preferences.
* It returns the formatted number string to the successCB callback with a properties object as a
* parameter. If there is an error formatting the number, then the errorCB callback is invoked.
*
* The defaults are: type="decimal"
*
* @param {Number} number
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            type {String}: 'decimal', "percent", or 'currency'
*
* @return Object.value {String}: The formatted number string.
*
* @error GlobalizationError.FORMATTING_ERROR
*
* Example
*    globalization.numberToString(3.25,
*                function (number) {alert('number:' + number.value + '\n');},
*                function () {},
*                {type:'decimal'});
*/
numberToString:function(number, successCB, failureCB, options) {
    argscheck.checkArgs('nfFO', 'Globalization.numberToString', arguments);
    exec(successCB, failureCB, "Globalization", "numberToString", [{"number": number, "options": options}]);
},

/**
* Parses a number formatted as a string according to the client's user preferences and
* returns the corresponding number. It returns the number to the successCB callback with a
* properties object as a parameter. If there is an error parsing the number string, then
* the errorCB callback is invoked.
*
* The defaults are: type="decimal"
*
* @param {String} numberString
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            type {String}: 'decimal', "percent", or 'currency'
*
* @return Object.value {Number}: The parsed number.
*
* @error GlobalizationError.PARSING_ERROR
*
* Example
*    globalization.stringToNumber('1234.56',
*                function (number) {alert('Number:' + number.value + '\n');},
*                function () { alert('Error parsing number');});
*/
stringToNumber:function(numberString, successCB, failureCB, options) {
    argscheck.checkArgs('sfFO', 'Globalization.stringToNumber', arguments);
    exec(successCB, failureCB, "Globalization", "stringToNumber", [{"numberString": numberString, "options": options}]);
},

/**
* Returns a pattern string for formatting and parsing numbers according to the client's user
* preferences. It returns the pattern to the successCB callback with a properties object as a
* parameter. If there is an error obtaining the pattern, then the errorCB callback is invoked.
*
* The defaults are: type="decimal"
*
* @param {Function} successCB
* @param {Function} errorCB
* @param {Object} options {optional}
*            type {String}: 'decimal', "percent", or 'currency'
*
* @return    Object.pattern {String}: The number pattern for formatting and parsing numbers.
*                                    The patterns follow Unicode Technical Standard #35.
*                                    http://unicode.org/reports/tr35/tr35-4.html
*            Object.symbol {String}: The symbol to be used when formatting and parsing
*                                    e.g., percent or currency symbol.
*            Object.fraction {Number}: The number of fractional digits to use when parsing and
*                                    formatting numbers.
*            Object.rounding {Number}: The rounding increment to use when parsing and formatting.
*            Object.positive {String}: The symbol to use for positive numbers when parsing and formatting.
*            Object.negative: {String}: The symbol to use for negative numbers when parsing and formatting.
*            Object.decimal: {String}: The decimal symbol to use for parsing and formatting.
*            Object.grouping: {String}: The grouping symbol to use for parsing and formatting.
*
* @error GlobalizationError.PATTERN_ERROR
*
* Example
*    globalization.getNumberPattern(
*                function (pattern) {alert('Pattern:' + pattern.pattern + '\n');},
*                function () {});
*/
getNumberPattern:function(successCB, failureCB, options) {
    argscheck.checkArgs('fFO', 'Globalization.getNumberPattern', arguments);
    exec(successCB, failureCB, "Globalization", "getNumberPattern", [{"options": options}]);
},

/**
* Returns a pattern string for formatting and parsing currency values according to the client's
* user preferences and ISO 4217 currency code. It returns the pattern to the successCB callback with a
* properties object as a parameter. If there is an error obtaining the pattern, then the errorCB
* callback is invoked.
*
* @param {String} currencyCode
* @param {Function} successCB
* @param {Function} errorCB
*
* @return    Object.pattern {String}: The currency pattern for formatting and parsing currency values.
*                                    The patterns follow Unicode Technical Standard #35
*                                    http://unicode.org/reports/tr35/tr35-4.html
*            Object.code {String}: The ISO 4217 currency code for the pattern.
*            Object.fraction {Number}: The number of fractional digits to use when parsing and
*                                    formatting currency.
*            Object.rounding {Number}: The rounding increment to use when parsing and formatting.
*            Object.decimal: {String}: The decimal symbol to use for parsing and formatting.
*            Object.grouping: {String}: The grouping symbol to use for parsing and formatting.
*
* @error GlobalizationError.FORMATTING_ERROR
*
* Example
*    globalization.getCurrencyPattern('EUR',
*                function (currency) {alert('Pattern:' + currency.pattern + '\n');}
*                function () {});
*/
getCurrencyPattern:function(currencyCode, successCB, failureCB) {
    argscheck.checkArgs('sfF', 'Globalization.getCurrencyPattern', arguments);
    exec(successCB, failureCB, "Globalization", "getCurrencyPattern", [{"currencyCode": currencyCode}]);
}

};

module.exports = globalization;

});

// file: lib/common/plugin/globalization/symbols.js
define("cordova/plugin/globalization/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/globalization', 'navigator.globalization');
modulemapper.clobbers('cordova/plugin/GlobalizationError', 'GlobalizationError');

});

// file: lib/android/plugin/inappbrowser/symbols.js
define("cordova/plugin/inappbrowser/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/InAppBrowser', 'open');

});

// file: lib/common/plugin/logger.js
define("cordova/plugin/logger", function(require, exports, module) {

//------------------------------------------------------------------------------
// The logger module exports the following properties/functions:
//
// LOG                          - constant for the level LOG
// ERROR                        - constant for the level ERROR
// WARN                         - constant for the level WARN
// INFO                         - constant for the level INFO
// DEBUG                        - constant for the level DEBUG
// logLevel()                   - returns current log level
// logLevel(value)              - sets and returns a new log level
// useConsole()                 - returns whether logger is using console
// useConsole(value)            - sets and returns whether logger is using console
// log(message,...)             - logs a message at level LOG
// error(message,...)           - logs a message at level ERROR
// warn(message,...)            - logs a message at level WARN
// info(message,...)            - logs a message at level INFO
// debug(message,...)           - logs a message at level DEBUG
// logLevel(level,message,...)  - logs a message specified level
//
//------------------------------------------------------------------------------

var logger = exports;

var exec    = require('cordova/exec');
var utils   = require('cordova/utils');

var UseConsole   = true;
var Queued       = [];
var DeviceReady  = false;
var CurrentLevel;

/**
 * Logging levels
 */

var Levels = [
    "LOG",
    "ERROR",
    "WARN",
    "INFO",
    "DEBUG"
];

/*
 * add the logging levels to the logger object and
 * to a separate levelsMap object for testing
 */

var LevelsMap = {};
for (var i=0; i<Levels.length; i++) {
    var level = Levels[i];
    LevelsMap[level] = i;
    logger[level]    = level;
}

CurrentLevel = LevelsMap.WARN;

/**
 * Getter/Setter for the logging level
 *
 * Returns the current logging level.
 *
 * When a value is passed, sets the logging level to that value.
 * The values should be one of the following constants:
 *    logger.LOG
 *    logger.ERROR
 *    logger.WARN
 *    logger.INFO
 *    logger.DEBUG
 *
 * The value used determines which messages get printed.  The logging
 * values above are in order, and only messages logged at the logging
 * level or above will actually be displayed to the user.  E.g., the
 * default level is WARN, so only messages logged with LOG, ERROR, or
 * WARN will be displayed; INFO and DEBUG messages will be ignored.
 */
logger.level = function (value) {
    if (arguments.length) {
        if (LevelsMap[value] === null) {
            throw new Error("invalid logging level: " + value);
        }
        CurrentLevel = LevelsMap[value];
    }

    return Levels[CurrentLevel];
};

/**
 * Getter/Setter for the useConsole functionality
 *
 * When useConsole is true, the logger will log via the
 * browser 'console' object.  Otherwise, it will use the
 * native Logger plugin.
 */
logger.useConsole = function (value) {
    if (arguments.length) UseConsole = !!value;

    if (UseConsole) {
        if (typeof console == "undefined") {
            throw new Error("global console object is not defined");
        }

        if (typeof console.log != "function") {
            throw new Error("global console object does not have a log function");
        }

        if (typeof console.useLogger == "function") {
            if (console.useLogger()) {
                throw new Error("console and logger are too intertwingly");
            }
        }
    }

    return UseConsole;
};

/**
 * Logs a message at the LOG level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.log   = function(message) { logWithArgs("LOG",   arguments); };

/**
 * Logs a message at the ERROR level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.error = function(message) { logWithArgs("ERROR", arguments); };

/**
 * Logs a message at the WARN level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.warn  = function(message) { logWithArgs("WARN",  arguments); };

/**
 * Logs a message at the INFO level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.info  = function(message) { logWithArgs("INFO",  arguments); };

/**
 * Logs a message at the DEBUG level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.debug = function(message) { logWithArgs("DEBUG", arguments); };

// log at the specified level with args
function logWithArgs(level, args) {
    args = [level].concat([].slice.call(args));
    logger.logLevel.apply(logger, args);
}

/**
 * Logs a message at the specified level.
 *
 * Parameters passed after message are used applied to
 * the message with utils.format()
 */
logger.logLevel = function(level /* , ... */) {
    // format the message with the parameters
    var formatArgs = [].slice.call(arguments, 1);
    var message    = logger.format.apply(logger.format, formatArgs);

    if (LevelsMap[level] === null) {
        throw new Error("invalid logging level: " + level);
    }

    if (LevelsMap[level] > CurrentLevel) return;

    // queue the message if not yet at deviceready
    if (!DeviceReady && !UseConsole) {
        Queued.push([level, message]);
        return;
    }

    // if not using the console, use the native logger
    if (!UseConsole) {
        exec(null, null, "Logger", "logLevel", [level, message]);
        return;
    }

    // make sure console is not using logger
    if (console.__usingCordovaLogger) {
        throw new Error("console and logger are too intertwingly");
    }

    // log to the console
    switch (level) {
        case logger.LOG:   console.log(message); break;
        case logger.ERROR: console.log("ERROR: " + message); break;
        case logger.WARN:  console.log("WARN: "  + message); break;
        case logger.INFO:  console.log("INFO: "  + message); break;
        case logger.DEBUG: console.log("DEBUG: " + message); break;
    }
};


/**
 * Formats a string and arguments following it ala console.log()
 *
 * Any remaining arguments will be appended to the formatted string.
 *
 * for rationale, see FireBug's Console API:
 *    http://getfirebug.com/wiki/index.php/Console_API
 */
logger.format = function(formatString, args) {
    return __format(arguments[0], [].slice.call(arguments,1)).join(' ');
};


//------------------------------------------------------------------------------
/**
 * Formats a string and arguments following it ala vsprintf()
 *
 * format chars:
 *   %j - format arg as JSON
 *   %o - format arg as JSON
 *   %c - format arg as ''
 *   %% - replace with '%'
 * any other char following % will format it's
 * arg via toString().
 *
 * Returns an array containing the formatted string and any remaining
 * arguments.
 */
function __format(formatString, args) {
    if (formatString === null || formatString === undefined) return [""];
    if (arguments.length == 1) return [formatString.toString()];

    if (typeof formatString != "string")
        formatString = formatString.toString();

    var pattern = /(.*?)%(.)(.*)/;
    var rest    = formatString;
    var result  = [];

    while (args.length) {
        var match = pattern.exec(rest);
        if (!match) break;

        var arg   = args.shift();
        rest = match[3];
        result.push(match[1]);

        if (match[2] == '%') {
            result.push('%');
            args.unshift(arg);
            continue;
        }

        result.push(__formatted(arg, match[2]));
    }

    result.push(rest);

    var remainingArgs = [].slice.call(args);
    remainingArgs.unshift(result.join(''));
    return remainingArgs;
}

function __formatted(object, formatChar) {

    try {
        switch(formatChar) {
            case 'j':
            case 'o': return JSON.stringify(object);
            case 'c': return '';
        }
    }
    catch (e) {
        return "error JSON.stringify()ing argument: " + e;
    }

    if ((object === null) || (object === undefined)) {
        return Object.prototype.toString.call(object);
    }

    return object.toString();
}


//------------------------------------------------------------------------------
// when deviceready fires, log queued messages
logger.__onDeviceReady = function() {
    if (DeviceReady) return;

    DeviceReady = true;

    for (var i=0; i<Queued.length; i++) {
        var messageArgs = Queued[i];
        logger.logLevel(messageArgs[0], messageArgs[1]);
    }

    Queued = null;
};

// add a deviceready event to log queued messages
document.addEventListener("deviceready", logger.__onDeviceReady, false);

});

// file: lib/common/plugin/logger/symbols.js
define("cordova/plugin/logger/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/logger', 'cordova.logger');

});

// file: lib/android/plugin/media/symbols.js
define("cordova/plugin/media/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.defaults('cordova/plugin/Media', 'Media');
modulemapper.clobbers('cordova/plugin/MediaError', 'MediaError');

});

// file: lib/common/plugin/network.js
define("cordova/plugin/network", function(require, exports, module) {

var exec = require('cordova/exec'),
    cordova = require('cordova'),
    channel = require('cordova/channel'),
    utils = require('cordova/utils');

// Link the onLine property with the Cordova-supplied network info.
// This works because we clobber the naviagtor object with our own
// object in bootstrap.js.
if (typeof navigator != 'undefined') {
    utils.defineGetter(navigator, 'onLine', function() {
        return this.connection.type != 'none';
    });
}

function NetworkConnection() {
    this.type = 'unknown';
}

/**
 * Get connection info
 *
 * @param {Function} successCallback The function to call when the Connection data is available
 * @param {Function} errorCallback The function to call when there is an error getting the Connection data. (OPTIONAL)
 */
NetworkConnection.prototype.getInfo = function(successCallback, errorCallback) {
    exec(successCallback, errorCallback, "NetworkStatus", "getConnectionInfo", []);
};

var me = new NetworkConnection();
var timerId = null;
var timeout = 500;

channel.onCordovaReady.subscribe(function() {
    me.getInfo(function(info) {
        me.type = info;
        if (info === "none") {
            // set a timer if still offline at the end of timer send the offline event
            timerId = setTimeout(function(){
                cordova.fireDocumentEvent("offline");
                timerId = null;
            }, timeout);
        } else {
            // If there is a current offline event pending clear it
            if (timerId !== null) {
                clearTimeout(timerId);
                timerId = null;
            }
            cordova.fireDocumentEvent("online");
        }

        // should only fire this once
        if (channel.onCordovaConnectionReady.state !== 2) {
            channel.onCordovaConnectionReady.fire();
        }
    },
    function (e) {
        // If we can't get the network info we should still tell Cordova
        // to fire the deviceready event.
        if (channel.onCordovaConnectionReady.state !== 2) {
            channel.onCordovaConnectionReady.fire();
        }
        console.log("Error initializing Network Connection: " + e);
    });
});

module.exports = me;

});

// file: lib/common/plugin/networkstatus/symbols.js
define("cordova/plugin/networkstatus/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/network', 'navigator.network.connection', 'navigator.network.connection is deprecated. Use navigator.connection instead.');
modulemapper.clobbers('cordova/plugin/network', 'navigator.connection');
modulemapper.defaults('cordova/plugin/Connection', 'Connection');

});

// file: lib/common/plugin/notification.js
define("cordova/plugin/notification", function(require, exports, module) {

var exec = require('cordova/exec');
var platform = require('cordova/platform');

/**
 * Provides access to notifications on the device.
 */

module.exports = {

    /**
     * Open a native alert dialog, with a customizable title and button text.
     *
     * @param {String} message              Message to print in the body of the alert
     * @param {Function} completeCallback   The callback that is called when user clicks on a button.
     * @param {String} title                Title of the alert dialog (default: Alert)
     * @param {String} buttonLabel          Label of the close button (default: OK)
     */
    alert: function(message, completeCallback, title, buttonLabel) {
        var _title = (title || "Alert");
        var _buttonLabel = (buttonLabel || "OK");
        exec(completeCallback, null, "Notification", "alert", [message, _title, _buttonLabel]);
    },

    /**
     * Open a native confirm dialog, with a customizable title and button text.
     * The result that the user selects is returned to the result callback.
     *
     * @param {String} message              Message to print in the body of the alert
     * @param {Function} resultCallback     The callback that is called when user clicks on a button.
     * @param {String} title                Title of the alert dialog (default: Confirm)
     * @param {Array} buttonLabels          Array of the labels of the buttons (default: ['OK', 'Cancel'])
     */
    confirm: function(message, resultCallback, title, buttonLabels) {
        var _title = (title || "Confirm");
        var _buttonLabels = (buttonLabels || ["OK", "Cancel"]);

        // Strings are deprecated!
        if (typeof _buttonLabels === 'string') {
            console.log("Notification.confirm(string, function, string, string) is deprecated.  Use Notification.confirm(string, function, string, array).");
        }

        // Some platforms take an array of button label names.
        // Other platforms take a comma separated list.
        // For compatibility, we convert to the desired type based on the platform.
        if (platform.id == "android" || platform.id == "ios" || platform.id == "windowsphone") {
            if (typeof _buttonLabels === 'string') {
                var buttonLabelString = _buttonLabels;
                _buttonLabels = _buttonLabels.split(","); // not crazy about changing the var type here
            }
        } else {
            if (Array.isArray(_buttonLabels)) {
                var buttonLabelArray = _buttonLabels;
                _buttonLabels = buttonLabelArray.toString();
            }
        }
        exec(resultCallback, null, "Notification", "confirm", [message, _title, _buttonLabels]);
    },

    /**
     * Open a native prompt dialog, with a customizable title and button text.
     * The following results are returned to the result callback:
     *  buttonIndex     Index number of the button selected.
     *  input1          The text entered in the prompt dialog box.
     *
     * @param {String} message              Dialog message to display (default: "Prompt message")
     * @param {Function} resultCallback     The callback that is called when user clicks on a button.
     * @param {String} title                Title of the dialog (default: "Prompt")
     * @param {Array} buttonLabels          Array of strings for the button labels (default: ["OK","Cancel"])
     */
    prompt: function(message, resultCallback, title, buttonLabels) {
        var _message = (message || "Prompt message");
        var _title = (title || "Prompt");
        var _buttonLabels = (buttonLabels || ["OK","Cancel"]);
        exec(resultCallback, null, "Notification", "prompt", [_message, _title, _buttonLabels]);
    },

    /**
     * Causes the device to vibrate.
     *
     * @param {Integer} mills       The number of milliseconds to vibrate for.
     */
    vibrate: function(mills) {
        exec(null, null, "Notification", "vibrate", [mills]);
    },

    /**
     * Causes the device to beep.
     * On Android, the default notification ringtone is played "count" times.
     *
     * @param {Integer} count       The number of beeps.
     */
    beep: function(count) {
        exec(null, null, "Notification", "beep", [count]);
    }
};

});

// file: lib/android/plugin/notification/symbols.js
define("cordova/plugin/notification/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/notification', 'navigator.notification');
modulemapper.merges('cordova/plugin/android/notification', 'navigator.notification');

});

// file: lib/common/plugin/requestFileSystem.js
define("cordova/plugin/requestFileSystem", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    FileError = require('cordova/plugin/FileError'),
    FileSystem = require('cordova/plugin/FileSystem'),
    exec = require('cordova/exec');

/**
 * Request a file system in which to store application data.
 * @param type  local file system type
 * @param size  indicates how much storage space, in bytes, the application expects to need
 * @param successCallback  invoked with a FileSystem object
 * @param errorCallback  invoked if error occurs retrieving file system
 */
var requestFileSystem = function(type, size, successCallback, errorCallback) {
    argscheck.checkArgs('nnFF', 'requestFileSystem', arguments);
    var fail = function(code) {
        errorCallback && errorCallback(new FileError(code));
    };

    if (type < 0 || type > 3) {
        fail(FileError.SYNTAX_ERR);
    } else {
        // if successful, return a FileSystem object
        var success = function(file_system) {
            if (file_system) {
                if (successCallback) {
                    // grab the name and root from the file system object
                    var result = new FileSystem(file_system.name, file_system.root);
                    successCallback(result);
                }
            }
            else {
                // no FileSystem object returned
                fail(FileError.NOT_FOUND_ERR);
            }
        };
        exec(success, fail, "File", "requestFileSystem", [type, size]);
    }
};

module.exports = requestFileSystem;

});

// file: lib/common/plugin/resolveLocalFileSystemURI.js
define("cordova/plugin/resolveLocalFileSystemURI", function(require, exports, module) {

var argscheck = require('cordova/argscheck'),
    DirectoryEntry = require('cordova/plugin/DirectoryEntry'),
    FileEntry = require('cordova/plugin/FileEntry'),
    FileError = require('cordova/plugin/FileError'),
    exec = require('cordova/exec');

/**
 * Look up file system Entry referred to by local URI.
 * @param {DOMString} uri  URI referring to a local file or directory
 * @param successCallback  invoked with Entry object corresponding to URI
 * @param errorCallback    invoked if error occurs retrieving file system entry
 */
module.exports = function(uri, successCallback, errorCallback) {
    argscheck.checkArgs('sFF', 'resolveLocalFileSystemURI', arguments);
    // error callback
    var fail = function(error) {
        errorCallback && errorCallback(new FileError(error));
    };
    // sanity check for 'not:valid:filename'
    if(!uri || uri.split(":").length > 2) {
        setTimeout( function() {
            fail(FileError.ENCODING_ERR);
        },0);
        return;
    }
    // if successful, return either a file or directory entry
    var success = function(entry) {
        var result;
        if (entry) {
            if (successCallback) {
                // create appropriate Entry object
                result = (entry.isDirectory) ? new DirectoryEntry(entry.name, entry.fullPath) : new FileEntry(entry.name, entry.fullPath);
                successCallback(result);
            }
        }
        else {
            // no Entry object returned
            fail(FileError.NOT_FOUND_ERR);
        }
    };

    exec(success, fail, "File", "resolveLocalFileSystemURI", [uri]);
};

});

// file: lib/common/plugin/splashscreen.js
define("cordova/plugin/splashscreen", function(require, exports, module) {

var exec = require('cordova/exec');

var splashscreen = {
    show:function() {
        exec(null, null, "SplashScreen", "show", []);
    },
    hide:function() {
        exec(null, null, "SplashScreen", "hide", []);
    }
};

module.exports = splashscreen;

});

// file: lib/common/plugin/splashscreen/symbols.js
define("cordova/plugin/splashscreen/symbols", function(require, exports, module) {


var modulemapper = require('cordova/modulemapper');

modulemapper.clobbers('cordova/plugin/splashscreen', 'navigator.splashscreen');

});

// file: lib/common/symbols.js
define("cordova/symbols", function(require, exports, module) {

var modulemapper = require('cordova/modulemapper');

// Use merges here in case others symbols files depend on this running first,
// but fail to declare the dependency with a require().
modulemapper.merges('cordova', 'cordova');
modulemapper.clobbers('cordova/exec', 'cordova.exec');
modulemapper.clobbers('cordova/exec', 'Cordova.exec');

});

// file: lib/common/utils.js
define("cordova/utils", function(require, exports, module) {

var utils = exports;

/**
 * Defines a property getter / setter for obj[key].
 */
utils.defineGetterSetter = function(obj, key, getFunc, opt_setFunc) {
    if (Object.defineProperty) {
        var desc = {
            get: getFunc,
            configurable: true
        };
        if (opt_setFunc) {
            desc.set = opt_setFunc;
        }
        Object.defineProperty(obj, key, desc);
    } else {
        obj.__defineGetter__(key, getFunc);
        if (opt_setFunc) {
            obj.__defineSetter__(key, opt_setFunc);
        }
    }
};

/**
 * Defines a property getter for obj[key].
 */
utils.defineGetter = utils.defineGetterSetter;

utils.arrayIndexOf = function(a, item) {
    if (a.indexOf) {
        return a.indexOf(item);
    }
    var len = a.length;
    for (var i = 0; i < len; ++i) {
        if (a[i] == item) {
            return i;
        }
    }
    return -1;
};

/**
 * Returns whether the item was found in the array.
 */
utils.arrayRemove = function(a, item) {
    var index = utils.arrayIndexOf(a, item);
    if (index != -1) {
        a.splice(index, 1);
    }
    return index != -1;
};

utils.typeName = function(val) {
    return Object.prototype.toString.call(val).slice(8, -1);
};

/**
 * Returns an indication of whether the argument is an array or not
 */
utils.isArray = function(a) {
    return utils.typeName(a) == 'Array';
};

/**
 * Returns an indication of whether the argument is a Date or not
 */
utils.isDate = function(d) {
    return utils.typeName(d) == 'Date';
};

/**
 * Does a deep clone of the object.
 */
utils.clone = function(obj) {
    if(!obj || typeof obj == 'function' || utils.isDate(obj) || typeof obj != 'object') {
        return obj;
    }

    var retVal, i;

    if(utils.isArray(obj)){
        retVal = [];
        for(i = 0; i < obj.length; ++i){
            retVal.push(utils.clone(obj[i]));
        }
        return retVal;
    }

    retVal = {};
    for(i in obj){
        if(!(i in retVal) || retVal[i] != obj[i]) {
            retVal[i] = utils.clone(obj[i]);
        }
    }
    return retVal;
};

/**
 * Returns a wrapped version of the function
 */
utils.close = function(context, func, params) {
    if (typeof params == 'undefined') {
        return function() {
            return func.apply(context, arguments);
        };
    } else {
        return function() {
            return func.apply(context, params);
        };
    }
};

/**
 * Create a UUID
 */
utils.createUUID = function() {
    return UUIDcreatePart(4) + '-' +
        UUIDcreatePart(2) + '-' +
        UUIDcreatePart(2) + '-' +
        UUIDcreatePart(2) + '-' +
        UUIDcreatePart(6);
};

/**
 * Extends a child object from a parent object using classical inheritance
 * pattern.
 */
utils.extend = (function() {
    // proxy used to establish prototype chain
    var F = function() {};
    // extend Child from Parent
    return function(Child, Parent) {
        F.prototype = Parent.prototype;
        Child.prototype = new F();
        Child.__super__ = Parent.prototype;
        Child.prototype.constructor = Child;
    };
}());

/**
 * Alerts a message in any available way: alert or console.log.
 */
utils.alert = function(msg) {
    if (window.alert) {
        window.alert(msg);
    } else if (console && console.log) {
        console.log(msg);
    }
};


//------------------------------------------------------------------------------
function UUIDcreatePart(length) {
    var uuidpart = "";
    for (var i=0; i<length; i++) {
        var uuidchar = parseInt((Math.random() * 256), 10).toString(16);
        if (uuidchar.length == 1) {
            uuidchar = "0" + uuidchar;
        }
        uuidpart += uuidchar;
    }
    return uuidpart;
}


});


window.cordova = require('cordova');

// file: lib/scripts/bootstrap.js

(function (context) {
    var channel = require('cordova/channel');
    var platformInitChannelsArray = [channel.onNativeReady, channel.onPluginsReady];

    function logUnfiredChannels(arr) {
        for (var i = 0; i < arr.length; ++i) {
            if (arr[i].state != 2) {
                console.log('Channel not fired: ' + arr[i].type);
            }
        }
    }

    window.setTimeout(function() {
        if (channel.onDeviceReady.state != 2) {
            console.log('deviceready has not fired after 5 seconds.');
            logUnfiredChannels(platformInitChannelsArray);
            logUnfiredChannels(channel.deviceReadyChannelsArray);
        }
    }, 5000);

    // Replace navigator before any modules are required(), to ensure it happens as soon as possible.
    // We replace it so that properties that can't be clobbered can instead be overridden.
    function replaceNavigator(origNavigator) {
        var CordovaNavigator = function() {};
        CordovaNavigator.prototype = origNavigator;
        var newNavigator = new CordovaNavigator();
        // This work-around really only applies to new APIs that are newer than Function.bind.
        // Without it, APIs such as getGamepads() break.
        if (CordovaNavigator.bind) {
            for (var key in origNavigator) {
                if (typeof origNavigator[key] == 'function') {
                    newNavigator[key] = origNavigator[key].bind(origNavigator);
                }
            }
        }
        return newNavigator;
    }
    if (context.navigator) {
        context.navigator = replaceNavigator(context.navigator);
    }

    // _nativeReady is global variable that the native side can set
    // to signify that the native code is ready. It is a global since
    // it may be called before any cordova JS is ready.
    if (window._nativeReady) {
        channel.onNativeReady.fire();
    }

    /**
     * Create all cordova objects once native side is ready.
     */
    channel.join(function() {
        // Call the platform-specific initialization
        require('cordova/platform').initialize();

        // Fire event to notify that all objects are created
        channel.onCordovaReady.fire();

        // Fire onDeviceReady event once page has fully loaded, all
        // constructors have run and cordova info has been received from native
        // side.
        // This join call is deliberately made after platform.initialize() in
        // order that plugins may manipulate channel.deviceReadyChannelsArray
        // if necessary.
        channel.join(function() {
            require('cordova').fireDocumentEvent('deviceready');
        }, channel.deviceReadyChannelsArray);

    }, platformInitChannelsArray);

}(window));

// file: lib/scripts/bootstrap-android.js

require('cordova/channel').onNativeReady.fire();

// file: lib/scripts/plugin_loader.js

// Tries to load all plugins' js-modules.
// This is an async process, but onDeviceReady is blocked on onPluginsReady.
// onPluginsReady is fired when there are no plugins to load, or they are all done.
(function (context) {
    // To be populated with the handler by handlePluginsObject.
    var onScriptLoadingComplete;

    var scriptCounter = 0;
    function scriptLoadedCallback() {
        scriptCounter--;
        if (scriptCounter === 0) {
            onScriptLoadingComplete && onScriptLoadingComplete();
        }
    }

    // Helper function to inject a <script> tag.
    function injectScript(path) {
        scriptCounter++;
        var script = document.createElement("script");
        script.onload = scriptLoadedCallback;
        script.src = path;
        document.head.appendChild(script);
    }

    // Called when:
    // * There are plugins defined and all plugins are finished loading.
    // * There are no plugins to load.
    function finishPluginLoading() {
        context.cordova.require('cordova/channel').onPluginsReady.fire();
    }

    // Handler for the cordova_plugins.json content.
    // See plugman's plugin_loader.js for the details of this object.
    // This function is only called if the really is a plugins array that isn't empty.
    // Otherwise the XHR response handler will just call finishPluginLoading().
    function handlePluginsObject(modules) {
        // First create the callback for when all plugins are loaded.
        var mapper = context.cordova.require('cordova/modulemapper');
        onScriptLoadingComplete = function() {
            // Loop through all the plugins and then through their clobbers and merges.
            for (var i = 0; i < modules.length; i++) {
                var module = modules[i];
                if (!module) continue;

                if (module.clobbers && module.clobbers.length) {
                    for (var j = 0; j < module.clobbers.length; j++) {
                        mapper.clobbers(module.id, module.clobbers[j]);
                    }
                }

                if (module.merges && module.merges.length) {
                    for (var k = 0; k < module.merges.length; k++) {
                        mapper.merges(module.id, module.merges[k]);
                    }
                }

                // Finally, if runs is truthy we want to simply require() the module.
                // This can be skipped if it had any merges or clobbers, though,
                // since the mapper will already have required the module.
                if (module.runs && !(module.clobbers && module.clobbers.length) && !(module.merges && module.merges.length)) {
                    context.cordova.require(module.id);
                }
            }

            finishPluginLoading();
        };

        // Now inject the scripts.
        for (var i = 0; i < modules.length; i++) {
            injectScript(modules[i].file);
        }
    }


    // Try to XHR the cordova_plugins.json file asynchronously.
    try { // we commented we were going to try, so let us actually try and catch
        var xhr = new context.XMLHttpRequest();
        xhr.onload = function() {
            // If the response is a JSON string which composes an array, call handlePluginsObject.
            // If the request fails, or the response is not a JSON array, just call finishPluginLoading.
            var obj = this.responseText && JSON.parse(this.responseText);
            if (obj && obj instanceof Array && obj.length > 0) {
                handlePluginsObject(obj);
            } else {
                finishPluginLoading();
            }
        };
        xhr.onerror = function() {
            finishPluginLoading();
        };
        xhr.open('GET', 'cordova_plugins.json', true); // Async
        xhr.send();
    }
    catch(err){
        finishPluginLoading();
    }
}(window));



})();var PhoneGap = cordova;


(function() {
//
//  ShareKitPlugin.js
//
//
//  Created by Erick Camacho on 28/07/11.
//  MIT Licensed
//

function ShareKitPlugin()
{
	console.log('creating plugin');
};

ShareKitPlugin.prototype.share = function(message, url)
{
	cordova.exec(null, null, "ShareKitPlugin", "share", [message, url]);

};


ShareKitPlugin.prototype.isLoggedToTwitter = function( callback )
{

    cordova.exec(callback, null, "ShareKitPlugin", "isLoggedToTwitter", [] );
};

ShareKitPlugin.prototype.isLoggedToFacebook = function( callback )
{

    cordova.exec(callback, null, "ShareKitPlugin", "isLoggedToFacebook", [] );

};

ShareKitPlugin.prototype.logoutFromTwitter = function()
{

    cordova.exec(null, null, "ShareKitPlugin", "logoutFromTwitter", [] );

};

ShareKitPlugin.prototype.logoutFromFacebook = function()
{

    cordova.exec(null, null, "ShareKitPlugin", "logoutFromFacebook", [] );

};


ShareKitPlugin.prototype.facebookConnect = function()
{

    cordova.exec(null, null, "ShareKitPlugin", "facebookConnect", [] );

};

ShareKitPlugin.prototype.shareToFacebook = function( message, url)
{

    cordova.exec(null, null, "ShareKitPlugin", "shareToFacebook", [message, url] );

};

ShareKitPlugin.prototype.shareToTwitter = function( message, url)
{

    cordova.exec(null, null, "ShareKitPlugin", "shareToTwitter", [message, url] );

};

ShareKitPlugin.prototype.shareToMail = function( subject, message)
{

    cordova.exec(null, null, "ShareKitPlugin", "shareToMail", [subject, message] );

};




ShareKitPlugin.install = function()
{
    if (!window.plugins) {
        window.plugins = {};
    }

    window.plugins.shareKit = new ShareKitPlugin();
    window.plugins.share = window.plugins.shareKit.share;

    return;
};

cordova.addConstructor(ShareKitPlugin.install);
})();
(function() {
  /* This bootstrap script is documented at http://developer.joshfire.com/ */
  var Joshfire = window.Joshfire || {};
  Joshfire.factory = {
    globalConfig: {"DATAVERSION":"1","IMGPROXYURL":"http://imgschemaio.herokuapp.com/proxy","HOSTPORT":"factory.joshfire.com","DATAHOSTPORT":"api.datajs.com","STATSHOSTPORT":"stats.joshfirefactory.com"},
    config: {"app":{"id":"51f641f0e59a690200000055","icon":{"height":512,"width":512,"encodingFormat":"png","contentSize":211982,"contentURL":"http://static.us-east-1.prod.factory.joshfire.com.s3.amazonaws.com/99/d18456cf5c75f5508562a04951d012/archiappicon2.png","itemType":"ImageObject","name":"archiappicon2.png","url":"/transloadit/99d18456cf5c75f5508562a04951d012"},"logo":null,"name":"World Architecture Newsfeed","integration":"factory","version":"1.0"},"template":{"name":"sleek","version":"1.3.11","options":{"language":"auto","color":"blue","tabs":["World Architec…"]},"hooks":["loaded"]}},
    device: {"type":"phone-android"},
    addons: {}
  };
  Joshfire.factory.config.deploy = {"flags":{"android":true,"cordova":"2.7.0"},"type":"androidbuild","id":"51fbc2c1cdb3090200000027","env":"live"};
  Joshfire.factory.config.datasources = {"main":[{"name":"World Architectural News","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"World Architectural News","db":"graph","col":"union","query":{"filter":{"field":"-special!date","datasources":{"main":[{"name":"All Architect Projects","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"All Architect Projects","id":"51fbc56e74450d0200000005","db":"graph","col":"grabbing","runatclient":false,"runtime":"hosted","apikey":"SF8CEi2n9jgqelCoMXjlsT1mcf2Qez03rpNbd0Ijs0H8dCeOu/4MCK4O9+vOwLn9MSonymoEJmOo2M3sSzB3UA=="}},"action":"addtarget"}},"runatclient":true,"runtime":"browser"},{"name":"Most Recent Features","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Most Recent Features","id":"51fbc56ee8b3dd0200000005","db":"graph","col":"grabbing","runatclient":false,"runtime":"hosted","apikey":"SF8CEi2n9jgqelCoMXjlsT1mcf2Qez03rpNbd0Ijs0H8dCeOu/4MCK4O9+vOwLn9MSonymoEJmOo2M3sSzB3UA=="}},"action":"addtarget"}},"runatclient":true,"runtime":"browser"},{"name":"Most Recent Designs","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Most Recent Designs","id":"51fbc56e59cb810200000005","db":"graph","col":"grabbing","runatclient":false,"runtime":"hosted","apikey":"SF8CEi2n9jgqelCoMXjlsT1mcf2Qez03rpNbd0Ijs0EtgPz3nB8fASKBiH3SwE0QlHXpOG2IeU/js+uqAhOaow=="}},"action":"addtarget"}},"runatclient":true,"runtime":"browser"},{"name":"Most Recent News","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Most Recent News","id":"51fbc56e74450d0200000006","db":"graph","col":"grabbing","runatclient":false,"runtime":"hosted","apikey":"SF8CEi2n9jgqelCoMXjlsT1mcf2Qez03rpNbd0Ijs0EtgPz3nB8fASKBiH3SwE0QlHXpOG2IeU/js+uqAhOaow=="}},"action":"addtarget"}},"runatclient":true,"runtime":"browser"},{"name":"Bulding news","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Bulding news","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://www.architectmagazine.com/queries/rss-queries.aspx?view=rss&id=Query_tcm20790602"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"RIBA feeds","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"RIBA feeds","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://www.architecture.com/syndication.riba?feed_type=PressReleases"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"1888 press release feeds","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"1888 press release feeds","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://www.1888pressrelease.com/rss/architecture.xml"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"Green Magazine Feeds","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Green Magazine Feeds","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://www.greenmagazine.com.au/rss_projects.php"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"New York Times Architect's Feeds","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"New York Times Architect's Feeds","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://topics.nytimes.com/top/reference/timestopics/subjects/a/architecture/index.html?rss=1"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"Architect Newspaper feed","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Architect Newspaper feed","db":"feed","col":"rss","query":{"filter":{"disableThumbnailExtraction":false,"url":"http://architectureau.com/rss.xml"}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"},{"name":"Architectural Digest News Feed","db":"operator","col":"updatelinks","query":{"filter":{"datasources":{"main":{"name":"Architectural Digest News Feed","id":"51fbc56e74450d0200000004","db":"graph","col":"grabbing","runatclient":false,"runtime":"hosted","apikey":"SF8CEi2n9jgqelCoMXjlsT1mcf2Qez03rpNbd0Ijs0EtgPz3nB8fASKBiH3SwE0QlHXpOG2IeU/js+uqAhOaow=="}},"action":"addtarget"}},"runatclient":true,"runtime":"browser"}]}}},"runatclient":true,"missingKeys":[],"outputType":"BlogPosting","runtime":"browser"}},"action":"addtarget"}},"outputType":"BlogPosting","runatclient":true,"runtime":"browser"}]};
  Joshfire.factory.config.addons = [{"name":"splashscreen","hooks":["loaded"]}];
  window.Joshfire = Joshfire;

})();
(function () { (function(a){a.factory.ready=!1;var b=[];a.factory.onReady=function(c){if(a.factory.ready)return c();b.push(c)},a.factory.onDeviceReady=a.factory.onReady;var c=function(a){if(typeof blackberry!="undefined")document.addEventListener("webworksready",a,!1);else if(typeof cordova!="undefined")document.addEventListener("deviceready",a,!1);else if(window.addEventListener)window.addEventListener("load",a,!1);else if(document.addEventListener)document.addEventListener("load",a,!1);else if(window.attachEvent)window.attachEvent("onload",a);else if(typeof window.onload!="function")window.onload=a;else{var b=window.onload;window.onload=function(){b(),a()}}};c(function(){a.factory.ready=!0;for(var c=0,d=b.length;c<d;c++)try{b.pop()()}catch(e){}var f=function(){var b=0;if(a.factory.config&&a.factory.config.template&&a.factory.config.template.hooks){b=a.factory.config.template.hooks;for(c=0;c<b.length;c++)if(b[c]==="loaded")return!1}return!0}();try{f&&a.factory.getAddOns("loaded").run()}catch(e){}typeof cordova!="undefined"&&cordova.exec(null,null,"SplashScreen","hide",[])},!1)})(Joshfire);
(function(a,b,c){var d="http://"+c.globalConfig.STATSHOSTPORT,e=function(){return},f=function(a){var b=[];for(var c in a)a.hasOwnProperty(c)&&b.push(c+"="+encodeURIComponent(a[c]));var f=new Image(1,1);f.src=d+"/pixel.gif?"+b.join("&"),f.onload=function(){e()}},g=function(){return{tz:(new Date).getTimezoneOffset(),url:b.location.toString(),ref:a.referrer,app:c.config.app.id,appv:c.config.app.version,tpl:c.config.template.id,dev:c.device.type,env:(c.config.deploy||{}).env||"preview",dp:(c.config.deploy||{}).id,dpr:(c.config.deploy||{}).type,id:""}};c.stats={},c.stats.event=function(a){var b=g();b.type=a,f(b)},c.stats.event("open"),a.addEventListener&&a.addEventListener("resume",function(){c.stats.event("resume")},!1)})(document,window,Joshfire.factory);
/**
 * almond 0.0.3 Copyright (c) 2011, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/almond for details
 */

// IMPORTANT: Same code as the original Underscore lib except code gets
// exported as an AMD module and lines 58 and following have been commented
// out not to leak '_' to the global scope
//     Underscore.js 1.3.3
//     (c) 2009-2012 Jeremy Ashkenas, DocumentCloud Inc.
//     Underscore is freely distributable under the MIT license.
//     Portions of Underscore are inspired or borrowed from Prototype,
//     Oliver Steele's Functional, and John Resig's Micro-Templating.
//     For all details and documentation:
//     http://documentcloud.github.com/underscore

/*!
 * jQuery JavaScript Library v1.9.1
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2012 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2013-2-4
 */

/*!
 * Sizzle CSS Selector Engine
 * Copyright 2012 jQuery Foundation and other contributors
 * Released under the MIT license
 * http://sizzlejs.com/
 */

/**
 * @fileoverview A forgiving HTML/XML/RSS parser written in JS for both
 * the browser and NodeJS.
 *
 * Original code:
 * https://github.com/tautologistics/node-htmlparser
 * v1.7.2
 *
 * Updates done by tidoust for Joshfire:
 * - code wrapped in a "define" module
 * - HTML named entities correctly supported
 * - CDATA sections handled correctly
 *
 * License
 * -------
 * Copyright 2010, Chris Winberry <chris@winberry.net>. All rights reserved.
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * @fileOverview
 *
 * URI support as per rfc3986
 * - Client and Serverside ECMA-262 v3 compatible, node.js friendly
 * - All classes extend the native String implementation
 * - Full test suite can be launched by running URI.Test();
 *
 * This fork from original repo fixes path resolution issues
 * and client-side execution in browser environments, c.f.:
 * - issue #1: https://github.com/webr3/URI/issues/1
 * - issue #2: https://github.com/webr3/URI/pull/2
 * - issue #4: https://github.com/webr3/URI/issues/4
 * - also ensured that the class does not leak to global scope
 * - updated code not to use the non-standard __proto__
 * (not supported by Opera and Internet Explorer)
 * - updated code to match style guidelines
 *
 * @author Nathan <http://webr3.org/nathan#me>
 * @author Francois Daoust <francois@joshfire.com>
 * @version 2012-02-09T21:55:00Z
 * @license http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Initial source: <http://github.com/webr3/URI>
 * Source of the fork: <http://github.com/joshfire/URI>
 *
 * To the extent possible under law, <http://webr3.org/nathan#me>
 * has waived all copyright and related or neighboring rights to
 * this work.
 */

var requirejs,require,define;(function(e){function o(e,t){if(e&&e.charAt(0)==="."&&t){t=t.split("/"),t=t.slice(0,t.length-1),e=t.concat(e.split("/"));var n,r;for(n=0;r=e[n];n++)if(r===".")e.splice(n,1),n-=1;else if(r===".."){if(n===1&&(e[2]===".."||e[0]===".."))break;n>0&&(e.splice(n-1,2),n-=2)}e=e.join("/")}return e}function u(t,n){return function(){return s.apply(e,r.call(arguments,0).concat([t,n]))}}function a(e){return function(t){return o(t,e)}}function f(e){return function(n){t[e]=n}}function l(r){if(n.hasOwnProperty(r)){var s=n[r];delete n[r],i.apply(e,s)}return t[r]}function c(e,t){var n,r,i=e.indexOf("!");return i!==-1?(n=o(e.slice(0,i),t),e=e.slice(i+1),r=l(n),r&&r.normalize?e=r.normalize(e,a(t)):e=o(e,t)):e=o(e,t),{f:n?n+"!"+e:e,n:e,p:r}}var t={},n={},r=[].slice,i,s;if(typeof define=="function")return;i=function(r,i,s,o){var a=[],h,p,d,v,m,g;o||(o=r);if(typeof s=="function"){!i.length&&s.length&&(i=["require","exports","module"]);for(v=0;v<i.length;v++){g=c(i[v],o),d=g.f;if(d==="require")a[v]=u(r);else if(d==="exports")a[v]=t[r]={},h=!0;else if(d==="module")p=a[v]={id:r,uri:"",exports:t[r]};else if(t.hasOwnProperty(d)||n.hasOwnProperty(d))a[v]=l(d);else{if(!g.p)throw r+" missing "+d;g.p.load(g.n,u(o,!0),f(d),{}),a[v]=t[d]}}m=s.apply(t[r],a),r&&(p&&p.exports!==e?t[r]=p.exports:h||(t[r]=m))}else r&&(t[r]=s)},requirejs=s=function(t,n,r,o){return typeof t=="string"?l(c(t,n).f):(t.splice||(n.splice?(t=n,n=arguments[2]):t=[]),o?i(e,t,n,r):setTimeout(function(){i(e,t,n,r)},15),s)},s.config=function(){return s},require||(require=s),define=function(e,t,r){t.splice||(r=t,t=[]),define.unordered?n[e]=[e,t,r]:i(e,t,r)},define.amd={jQuery:!0}})(),define("runtime/hosted/almond",function(){}),define("datajslib",{normalize:function(e,t){return e.match(/^runtime\-browser/)?e:t("runtime-browser/"+e)},load:function(e,t,n,r){t([e],function(e){n(e)})}}),define("runtime-browser/utils/getthumbnail",["require"],function(e){var t=Array.isArray||function(e){return Object.prototype.toString.call(e)==="[object Array]"},n=function(e,n,r){if(!e)return null;var i=function(e){if(!e)return!1;var t=e.width||0,i=e.height||0;return(!t||!n||t>=n)&&(!i||!r||i>=r)},s=function(e,t){if(!t)return!0;if(!e)return!1;var n=e.width||0,r=e.height||0,i=n&&r&&(n<=t.width||!t.width)&&(r<=t.height||!t.height);return i},o=function(e,t){return!s(e,t)},u=[];e["@type"]==="ImageObject"&&u.push(e),e.thumbnail&&(t(e.thumbnail)?u=u.concat(e.thumbnail):u.push(e.thumbnail)),e.image&&(u.push(e.image),e.image.thumbnail&&(t(e.image.thumbnail)?u=u.concat(e.image.thumbnail):u.push(e.image.thumbnail)));if(u.length===0)return null;var a=0,f=null,l=null;for(a=0;a<u.length;a++)f=u[a],i(f)&&s(f,l)&&(l=f);if(l)return l;l=u[0];for(a=0;a<u.length;a++)f=u[a],o(f,l)&&(l=f);return l};return n}),define("runtime-browser/utils/getthumbnailurl",["datajslib!utils/getthumbnail"],function(e){var t=function(t,n,r){var i=e(t,n,r)||{};return i.contentURL||i.contentUrl||""};return t}),define("runtime-browser/runtime",{name:"browser"}),define("runtime-browser/errors/error",[],function(){var e=function(e,t){this.name="error",this.message=e,this.err=t,Error.call(this),Error.captureStackTrace&&Error.captureStackTrace(this,arguments.callee)};return e.prototype=new Error,e}),define("runtime-browser/errors/internalerror",["./error"],function(e){var t=function(t,n){e.call(this,t,n),this.name="internal error"};return t.prototype=new e,t}),define("runtime-browser/errors/notfounderror",["./error"],function(e){var t=function(t,n){e.call(this,t,n),this.name="not found"};return t.prototype=new e,t}),define("runtime-browser/collection",["require","require","datajslib!runtime","datajslib!errors/internalerror","datajslib!errors/notfounderror"],function(e,t,n,r,i){var s=function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e},o={getCollection:function(e,t){return e?new u(e,t):{desc:{},name:"",find:function(e,t){return t(null,{entries:[]})},onLoaded:function(e){e()},getDesc:function(e){e(null,{})},getOutputType:function(){return"Thing"}}}},u=function(e,o){var u=e.query,a=this,f=[],l=!1,c=null,h="datajslib!collection.",p=null,d=function(e){var t=s({},u);return t=s(t,e),t.filter||(t.filter={}),c.setDefaultValues&&c.setDefaultValues(t.filter),t};a.onLoaded=function(e){l?e():f.push(e)},a.find=function(t,s){a.onLoaded(function(){if(p)return s(p);if(!c)return s(new i("Collection does not exist.",e));var o=d(t),u=10800;n.mongodb&&typeof n.mongodb.cacheMaxAge!="undefined"&&(u=n.mongodb.cacheMaxAge),c.desc&&typeof c.desc.maxAge!="undefined"&&(u=c.desc.maxAge);var f=u;c.getMaxAge?f=c.getMaxAge(o):t.nocache?f=0:t.maxAge&&t.maxAge>f&&f>0&&(f=t.maxAge);var l=null;c.cache&&f>0?(l={db:e.db,col:e.col,query:JSON.parse(JSON.stringify(o))},l.query.nocache&&(l.query.nocache=!1),c.cache.get(l,f,function(e,t){if(e)return s(new r("No way to query the cache",e));if(t)return s(null,t);a._find(o,function(e,t){return e?s(e):(c.cache.set(l,t,f),s(null,t))})})):a._find(o,function(e,t){return e?s(e):s(null,t)})})},a._find=function(t,n){if(c.find)c.find(t,function(r,i){return i&&!i.name&&e.name&&(i.name=e.name),i&&!i.maxAge&&(c.getMaxAge?i.maxAge=c.getMaxAge(t):c.desc&&c.desc.maxAge&&(i.maxAge=c.desc.maxAge)),n(r,i)});else{if(!c.fetch||!c.process)return n(new r('Invalid collection (no "fetch", "process", and/or "find")',e));c.fetch(t,function(r,i){if(r)return n(r,null);c.process(i,t,function(r,i){return i&&!i.name&&e.name&&(i.name=e.name),i&&!i.maxAge&&(c.getMaxAge?i.maxAge=c.getMaxAge(t):c.desc&&c.desc.maxAge&&(i.maxAge=c.desc.maxAge)),n(r,i)})})}},a.process=function(e,t,n){a.onLoaded(function(){if(p)return n(p);if(!c)return n(new i("Collection does not exist."));c.process?c.process(e,d(t),n):n(new r('Collection does not have a "process" method.'))})},a.fetch=function(e,t){a.onLoaded(function(){if(p)return t(p);if(!c)return t(new i("Collection does not exist."));c.fetch?c.fetch(d(e),t):t(null,null)})},a.updateInputOptions=function(e,t){a.onLoaded(function(){if(p)return t(p);if(!c)return t(new i("Collection does not exist."));var n={};for(var r in e)r!=="filter"&&r!=="graph"&&e.hasOwnProperty(r)&&(n[r]=e[r]);if(!c.updateInputOptions)return t(null,n);c.updateInputOptions(n,t)})},a.canBeFetched=function(){return c&&!p&&c.fetch},a.canBeProcessed=function(){return c&&!p&&c.process},a.getDesc=function(e){a.onLoaded(function(){return p?e(p):c?(c.setDefaultValues&&c.setDefaultValues(u.filter),c.getDesc?e(null,c.getDesc(u)):e(null,c.desc)):e(new i("Collection does not exist."))})},a.getOutputType=function(){return e.outputType?e.outputType:"Thing"},u=u||{},u.filter||(u.filter={}),!o&&e.db==="operator"?n.name!=="hosted"&&!e.runatclient&&(!e.runtime||e.runtime==="hosted")?h+="proxy":h+="graph":e.runatclient||e.runtime!=="hosted"?h+="client":h+="proxy",t([h],function(t){t.getCollection(e,function(e,t){p=e,c=t,a.cache&&(c.cache=a.cache),l=!0;for(var n=0;n<f.length;n++)f[n]();f=[]})})};return o}),define("runtime-browser/collection.graph",["require","datajslib!collection","datajslib!runtime"],function(e,t,n){var r={},i=Object.prototype.hasOwnProperty,s=Object.prototype.toString,o=Object.keys||function(e){if(e!==Object(e))throw new TypeError("Invalid object");var t=[];for(var n in e)i.call(e,n)&&(t[t.length]=n);return t},u=Array.isArray||function(e){return s.call(e)==="[object Array]"},a=function(e,t,n){if(e==null)return;if(Array.prototype.forEach&&e.forEach===Array.prototype.forEach)e.forEach(t,n);else if(e.length===+e.length){for(var s=0,o=e.length;s<o;s++)if(s in e&&t.call(n,e[s],s,e)===r)return}else for(var u in e)if(i.call(e,u)&&t.call(n,e[u],u,e)===r)return},f=function(e){return e===Object(e)},l=function(){var e=function(e,t){if(e.forEach)return e.forEach(t);for(var n=0;n<e.length;n+=1)t(e[n],n,e)},t=function(t,n){if(t.map)return t.map(n);var r=[];return e(t,function(e,t,i){r.push(n(e,t,i))}),r},n=function(t,n,r){if(!t.length)return r();var i=0;e(t,function(e){n(e,function(e){e?(r(e),r=function(){}):(i+=1,i===t.length&&r())})})},r=function(e){return function(){var t=Array.prototype.slice.call(arguments);return e.apply(null,[n].concat(t))}},i=function(e,n,r,i){var s=[];n=t(n,function(e,t){return{index:t,value:e}}),e(n,function(e,t){r(e.value,function(n,r){s[e.index]=r,t(n)})},function(e){i(e,s)})};return r(i)}(),c=function(e){var t=this;this.counter=0,this.nodes={};var r=function(i,s){if(t.nodes[i])return t.nodes[i];var o=new h(i,s,t.nodes);return t.nodes[i]=o,n.name!=="hosted"&&!s.runatclient&&(!s.runtime||s.runtime==="hosted")?o:!s.query||!s.query.filter?o:(a(s.query.filter.datasources,function(n,i){var l=null,c=null;if(u(n))o.inputNodes[i]=[],a(n,function(u,a){var l="_:"+ ++t.counter,c=null;if(f(u))c=r(l,u),o.inputNodes[i].push(c);else{l=u,c=t.nodes[l];if(c)u=c.datasource;else{u=s.query.filter.datasources[l]||e.query.filter.datasources[l]||(e.query.graph?e.query.graph[l]:null);if(!u)throw new Error('Missing datasource "'+l+'" detected');c=r(l,u)}n[a]=u,o.inputNodes[i].push(c)}});else if(f(n))l="_:"+ ++t.counter,o.inputNodes[i]=r(l,n);else{l=n,c=t.nodes[l];if(!c){n=s.query.filter.datasources[l]||e.query.filter.datasources[l]||(e.query.graph?e.query.graph[l]:null);if(!n)throw new Error('Missing datasource "'+l+'" detected');s.query.filter.datasources[i]=n,c=r(l,n)}o.inputNodes[i]=c}}),o)};this.finalNode=r("_:final",JSON.parse(JSON.stringify(e)))};c.prototype.find=function(e,t){var n=this;if(!this.finalNode)return t("No data source to retrieve");a(this.nodes,function(e){e.reset()});var r={};a(e,function(e,t){if(t==="graph"||t==="filter")return;r[t]=e}),this.finalNode.prepare(),this.finalNode.find(r,function(e,r){return a(n.nodes,function(e){e.reset()}),e,t(e,r)})};var h=function(e,t,n){this.id=e,this.graphNodes=n,this.datasource=t,this.inputNodes={},this.reset()};return h.prototype.reset=function(){this.err=null,this.data=null,this.running=!1,this.occurrences=0,this.doneCallbacks=[],this.collection=null},h.prototype.prepare=function(){this.occurrences+=1,this.occurrences===1&&(this.collection=t.getCollection(this.datasource,!0));if(this.occurrences>1e3)throw new Error("Infinite loop detected while preparing datasource execution");a(this.inputNodes,function(e){u(e)?a(e,function(e){e.prepare()}):e.prepare()})},h.prototype.onDone=function(e){this.doneCallbacks.push(e)},h.prototype.find=function(e,t){var n=this,r=function(e,t){n.running=!1,n.err=e,n.data=t,a(n.doneCallbacks,function(e){var t=null;n.occurrences>1?t=JSON.parse(JSON.stringify(n.data)):t=n.data,e(n.err,t)}),n.doneCallbacks=[],n.occurrences===0&&(n.err=null,n.data=null)},i=function(t,r){var i=n.inputNodes[t];u(i)?l(i,function(t,r){n.collection.updateInputOptions(e,function(e,n){if(e)return r(e,null);t.find(n,r)})},r):n.collection.updateInputOptions(e,function(e,t){if(e)return r(e);i.find(t,r)})};this.onDone(function(e,r){return n.occurrences-=1,t(e,r)});if(this.running){if(this.occurrences<=1)throw new Error("Node is executed more than expected");return}if(this.err||this.data)return r(this.err,this.data);this.running=!0;if(this.datasource.db==="operator"){if(!this.inputNodes)return r(null,{entries:[]});this.collection.fetch(e,function(t,s){if(t)return r(t);s!==null?n.collection.process(s,e,function(e,t){return t&&!t.maxAge&&a(s,function(e){e.maxAge&&(!t.maxAge||e.maxAge<t.maxAge)&&(t.maxAge=e.maxAge)}),r(e,t)}):l(o(n.inputNodes),i,function(t,i){if(t)return r(t);var s={};a(o(n.inputNodes),function(e,t){s[e]=i[t]}),n.collection.process(s,e,function(e,t){return t&&!t.maxAge&&a(s,function(e){e&&e.maxAge&&(!t.maxAge||e.maxAge<t.maxAge)&&(t.maxAge=e.maxAge)}),r(e,t)})})})}else this.collection.find(e,r)},{getCollection:function(e,t){var n=new c(e);return t(null,n)}}}),define("runtime-browser/collection.client",["require"],function(e){var t="databases/";return{getCollection:function(n,r){var i=n.db,s=n.col;e([t+i+"/"+s,"datajslib!jsonform-defaults"],function(e,t){return e=e||{},e.setDefaultValues=function(n){var r=(e.GetDesc?e.getDesc():e.desc)||{},i=r.options||{},s=i.schema||{};s.properties||(s={properties:s}),t.setDefaultValues(n,s)},r(null,e)})}}}),define("databases/operator/updatelinks",[],function(){var e=/<a\s[^>]*>/gi,t=/<\/a\s*>/gi;return{desc:{datasources:{main:{title:"Input",multiple:!1}},options:{action:{type:"string",title:"Requested action","default":"addtarget","enum":["addtarget","addtargetnojs","remove"]}},form:["datasources.main",{key:"action",titleMap:{addtarget:"Force links to open in a new window (with a location bar)",addtargetnojs:"Force links to open in a new window (no location bar)",remove:"Remove links (keeping link titles)"}}],runtimes:["hosted","browser","nodejs","win8"]},process:function(e,t,n){var r=null,i=0;t=t||{},e&&e.main&&(e=e.main);if(!e||!e.entries||!e.entries.length)return n(null,{entries:[]});r=this.desc.options.action["default"],t.filter&&t.filter.action&&(r=t.filter.action);for(i=0;i<e.entries.length;i++)this.updateItemLinks(e.entries[i],r);n(null,e)},updateItemLinks:function(e,t){if(!e)return;e.articleBody&&(e.articleBody=this.updateHtml(e.articleBody,t)),e.content&&(e.content=this.updateHtml(e.content,t))},updateHtml:function(n,r){return r==="remove"?(n=n.replace(e,""),n=n.replace(t,"")):r==="addtarget"?n=n.replace(e,function(e,t,n,r){return"<a target=\"_blank\" onclick=\"if(!window.IAB){window.IAB = window.open(this.href, this.target, 'location=yes');window.IAB.addEventListener('exit', function() {window.IAB=null;});}return false;\" "+e.substring(3)}):n=n.replace(e,function(e,t,n,r){return'<a target="_blank" '+e.substring(3)}),n}}}),define("runtime-browser/utils/schematype",[],function(){var e={types:{AboutPage:{supertypes:["WebPage"]},AccountingService:{supertypes:["ProfessionalService","FinancialService"]},AdministrativeArea:{supertypes:["Place"]},AdultEntertainment:{supertypes:["EntertainmentBusiness"]},AggregateOffer:{supertypes:["Offer"]},AggregateRating:{supertypes:["Rating"]},Airport:{supertypes:["CivicStructure"]},AmusementPark:{supertypes:["EntertainmentBusiness"]},AnimalShelter:{supertypes:["LocalBusiness"]},ApartmentComplex:{supertypes:["Residence"]},Aquarium:{supertypes:["CivicStructure"]},ArtGallery:{supertypes:["EntertainmentBusiness"]},Article:{supertypes:["CreativeWork"]},Attorney:{supertypes:["ProfessionalService"]},AudioObject:{supertypes:["MediaObject"]},AutoBodyShop:{supertypes:["AutomotiveBusiness"]},AutoDealer:{supertypes:["AutomotiveBusiness"]},AutoPartsStore:{supertypes:["AutomotiveBusiness","Store"]},AutoRental:{supertypes:["AutomotiveBusiness"]},AutoRepair:{supertypes:["AutomotiveBusiness"]},AutoWash:{supertypes:["AutomotiveBusiness"]},AutomatedTeller:{supertypes:["FinancialService"]},AutomotiveBusiness:{supertypes:["LocalBusiness"]},Bakery:{supertypes:["FoodEstablishment"]},BankOrCreditUnion:{supertypes:["FinancialService"]},BarOrPub:{supertypes:["FoodEstablishment"]},Beach:{supertypes:["CivicStructure"]},BeautySalon:{supertypes:["HealthAndBeautyBusiness"]},BedAndBreakfast:{supertypes:["LodgingBusiness"]},BikeStore:{supertypes:["Store"]},Blog:{supertypes:["CreativeWork"]},BlogPosting:{supertypes:["Article"]},BodyOfWater:{supertypes:["Landform"]},Book:{supertypes:["CreativeWork"]},BookFormatType:{supertypes:["Enumeration"]},BookStore:{supertypes:["Store"]},BowlingAlley:{supertypes:["SportsActivityLocation"]},Brewery:{supertypes:["FoodEstablishment"]},BuddhistTemple:{supertypes:["PlaceOfWorship"]},BusStation:{supertypes:["CivicStructure"]},BusStop:{supertypes:["CivicStructure"]},BusinessEvent:{supertypes:["Event"]},CafeOrCoffeeShop:{supertypes:["FoodEstablishment"]},Campground:{supertypes:["CivicStructure"]},Canal:{supertypes:["BodyOfWater"]},Casino:{supertypes:["EntertainmentBusiness"]},CatholicChurch:{supertypes:["PlaceOfWorship"]},Cemetery:{supertypes:["CivicStructure"]},CheckoutPage:{supertypes:["WebPage"]},ChildCare:{supertypes:["LocalBusiness"]},ChildrensEvent:{supertypes:["Event"]},Church:{supertypes:["PlaceOfWorship"]},City:{supertypes:["AdministrativeArea"]},CityHall:{supertypes:["GovernmentBuilding"]},CivicStructure:{supertypes:["Place"]},ClothingStore:{supertypes:["Store"]},CollectionPage:{supertypes:["WebPage"]},CollegeOrUniversity:{supertypes:["EducationalOrganization"]},ComedyClub:{supertypes:["EntertainmentBusiness"]},ComedyEvent:{supertypes:["Event"]},ComputerStore:{supertypes:["Store"]},ContactPage:{supertypes:["WebPage"]},ContactPoint:{supertypes:["StructuredValue"]},Continent:{supertypes:["Landform"]},ConvenienceStore:{supertypes:["Store"]},Corporation:{supertypes:["Organization"]},Country:{supertypes:["AdministrativeArea"]},Courthouse:{supertypes:["GovernmentBuilding"]},CreativeWork:{supertypes:["Thing"]},Crematorium:{supertypes:["CivicStructure"]},DanceEvent:{supertypes:["Event"]},DanceGroup:{supertypes:["PerformingGroup"]},DaySpa:{supertypes:["HealthAndBeautyBusiness"]},DefenceEstablishment:{supertypes:["GovernmentBuilding"]},Dentist:{supertypes:["MedicalOrganization","ProfessionalService"]},DepartmentStore:{supertypes:["Store"]},Distance:{supertypes:["Quantity"]},DryCleaningOrLaundry:{supertypes:["LocalBusiness"]},Duration:{supertypes:["Quantity"]},EducationEvent:{supertypes:["Event"]},EducationalOrganization:{supertypes:["Organization"]},Electrician:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},ElectronicsStore:{supertypes:["Store"]},ElementarySchool:{supertypes:["EducationalOrganization"]},Embassy:{supertypes:["GovernmentBuilding"]},EmergencyService:{supertypes:["LocalBusiness"]},EmploymentAgency:{supertypes:["LocalBusiness"]},Energy:{supertypes:["Quantity"]},EntertainmentBusiness:{supertypes:["LocalBusiness"]},Enumeration:{supertypes:["Intangible"]},Event:{supertypes:["Thing"]},EventVenue:{supertypes:["CivicStructure"]},ExerciseGym:{supertypes:["SportsActivityLocation"]},FastFoodRestaurant:{supertypes:["FoodEstablishment"]},Festival:{supertypes:["Event"]},FinancialService:{supertypes:["LocalBusiness"]},FireStation:{supertypes:["CivicStructure","EmergencyService"]},Florist:{supertypes:["Store"]},FoodEstablishment:{supertypes:["LocalBusiness"]},FoodEvent:{supertypes:["Event"]},FurnitureStore:{supertypes:["Store"]},GardenStore:{supertypes:["Store"]},GasStation:{supertypes:["AutomotiveBusiness"]},GatedResidenceCommunity:{supertypes:["Residence"]},GeneralContractor:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},GeoCoordinates:{supertypes:["StructuredValue"]},GeoShape:{supertypes:["StructuredValue"]},GolfCourse:{supertypes:["SportsActivityLocation"]},GovernmentBuilding:{supertypes:["CivicStructure"]},GovernmentOffice:{supertypes:["LocalBusiness"]},GovernmentOrganization:{supertypes:["Organization"]},GroceryStore:{supertypes:["Store"]},HVACBusiness:{supertypes:["HomeAndConstructionBusiness"]},HairSalon:{supertypes:["HealthAndBeautyBusiness"]},HardwareStore:{supertypes:["Store"]},HealthAndBeautyBusiness:{supertypes:["LocalBusiness"]},HealthClub:{supertypes:["SportsActivityLocation","HealthAndBeautyBusiness"]},HighSchool:{supertypes:["EducationalOrganization"]},HinduTemple:{supertypes:["PlaceOfWorship"]},HobbyShop:{supertypes:["Store"]},HomeAndConstructionBusiness:{supertypes:["LocalBusiness"]},HomeGoodsStore:{supertypes:["Store"]},Hospital:{supertypes:["CivicStructure","MedicalOrganization","EmergencyService"]},Hostel:{supertypes:["LodgingBusiness"]},Hotel:{supertypes:["LodgingBusiness"]},HousePainter:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},IceCreamShop:{supertypes:["FoodEstablishment"]},ImageGallery:{supertypes:["CollectionPage"]},ImageObject:{supertypes:["MediaObject"]},InsuranceAgency:{supertypes:["FinancialService"]},Intangible:{supertypes:["Thing"]},InternetCafe:{supertypes:["LocalBusiness"]},ItemAvailability:{supertypes:["Enumeration"]},ItemList:{supertypes:["CreativeWork"]},ItemPage:{supertypes:["WebPage"]},JewelryStore:{supertypes:["Store"]},JobPosting:{supertypes:["Intangible"]},LakeBodyOfWater:{supertypes:["BodyOfWater"]},Landform:{supertypes:["Place"]},LandmarksOrHistoricalBuildings:{supertypes:["Place"]},Language:{supertypes:["Intangible"]},LegislativeBuilding:{supertypes:["GovernmentBuilding"]},Library:{supertypes:["LocalBusiness"]},LiquorStore:{supertypes:["Store"]},LiteraryEvent:{supertypes:["Event"]},LocalBusiness:{supertypes:["Place","Organization"]},Locksmith:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},LodgingBusiness:{supertypes:["LocalBusiness"]},Map:{supertypes:["CreativeWork"]},Mass:{supertypes:["Quantity"]},MediaObject:{supertypes:["CreativeWork"]},MedicalClinic:{supertypes:["MedicalOrganization"]},MedicalOrganization:{supertypes:["LocalBusiness"]},MensClothingStore:{supertypes:["Store"]},MiddleSchool:{supertypes:["EducationalOrganization"]},MobilePhoneStore:{supertypes:["Store"]},Mosque:{supertypes:["PlaceOfWorship"]},Motel:{supertypes:["LodgingBusiness"]},MotorcycleDealer:{supertypes:["AutomotiveBusiness"]},MotorcycleRepair:{supertypes:["AutomotiveBusiness"]},Mountain:{supertypes:["Landform"]},Movie:{supertypes:["CreativeWork"]},MovieRentalStore:{supertypes:["Store"]},MovieTheater:{supertypes:["CivicStructure","EntertainmentBusiness"]},MovingCompany:{supertypes:["HomeAndConstructionBusiness"]},Museum:{supertypes:["CivicStructure"]},MusicAlbum:{supertypes:["MusicPlaylist"]},MusicEvent:{supertypes:["Event"]},MusicGroup:{supertypes:["PerformingGroup"]},MusicPlaylist:{supertypes:["CreativeWork"]},MusicRecording:{supertypes:["CreativeWork"]},MusicStore:{supertypes:["Store"]},MusicVenue:{supertypes:["CivicStructure"]},MusicVideoObject:{supertypes:["MediaObject"]},NGO:{supertypes:["Organization"]},NailSalon:{supertypes:["HealthAndBeautyBusiness"]},NewsArticle:{supertypes:["Article"]},NightClub:{supertypes:["EntertainmentBusiness"]},Notary:{supertypes:["ProfessionalService"]},NutritionInformation:{supertypes:["StructuredValue"]},OceanBodyOfWater:{supertypes:["BodyOfWater"]},Offer:{supertypes:["Intangible"]},OfferItemCondition:{supertypes:["Enumeration"]},OfficeEquipmentStore:{supertypes:["Store"]},Optician:{supertypes:["MedicalOrganization"]},Organization:{supertypes:["Thing"]},OutletStore:{supertypes:["Store"]},Painting:{supertypes:["CreativeWork"]},Park:{supertypes:["CivicStructure"]},ParkingFacility:{supertypes:["CivicStructure"]},PawnShop:{supertypes:["Store"]},PerformingArtsTheater:{supertypes:["CivicStructure"]},PerformingGroup:{supertypes:["Organization"]},Person:{supertypes:["Thing"]},PetStore:{supertypes:["Store"]},Pharmacy:{supertypes:["MedicalOrganization"]},Photograph:{supertypes:["CreativeWork"]},Physician:{supertypes:["MedicalOrganization"]},Place:{supertypes:["Thing"]},PlaceOfWorship:{supertypes:["CivicStructure"]},Playground:{supertypes:["CivicStructure"]},Plumber:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},PoliceStation:{supertypes:["CivicStructure","EmergencyService"]},Pond:{supertypes:["BodyOfWater"]},PostOffice:{supertypes:["GovernmentOffice"]},PostalAddress:{supertypes:["ContactPoint"]},Preschool:{supertypes:["EducationalOrganization"]},Product:{supertypes:["Thing"]},ProfessionalService:{supertypes:["LocalBusiness"]},ProfilePage:{supertypes:["WebPage"]},PublicSwimmingPool:{supertypes:["SportsActivityLocation"]},Quantity:{supertypes:["Intangible"]},RVPark:{supertypes:["CivicStructure"]},RadioStation:{supertypes:["LocalBusiness"]},Rating:{supertypes:["Intangible"]},RealEstateAgent:{supertypes:["LocalBusiness"]},Recipe:{supertypes:["CreativeWork"]},RecyclingCenter:{supertypes:["LocalBusiness"]},Reservoir:{supertypes:["BodyOfWater"]},Residence:{supertypes:["Place"]},Restaurant:{supertypes:["FoodEstablishment"]},Review:{supertypes:["CreativeWork"]},RiverBodyOfWater:{supertypes:["BodyOfWater"]},RoofingContractor:{supertypes:["HomeAndConstructionBusiness","ProfessionalService"]},SaleEvent:{supertypes:["Event"]},ScholarlyArticle:{supertypes:["Article"]},School:{supertypes:["EducationalOrganization"]},Sculpture:{supertypes:["CreativeWork"]},SeaBodyOfWater:{supertypes:["BodyOfWater"]},SearchResultsPage:{supertypes:["WebPage"]},SelfStorage:{supertypes:["LocalBusiness"]},ShoeStore:{supertypes:["Store"]},ShoppingCenter:{supertypes:["LocalBusiness"]},SingleFamilyResidence:{supertypes:["Residence"]},SiteNavigationElement:{supertypes:["WebPageElement"]},SkiResort:{supertypes:["SportsActivityLocation"]},SocialEvent:{supertypes:["Event"]},SportingGoodsStore:{supertypes:["Store"]},SportsActivityLocation:{supertypes:["LocalBusiness"]},SportsClub:{supertypes:["SportsActivityLocation"]},SportsEvent:{supertypes:["Event"]},SportsTeam:{supertypes:["Organization"]},StadiumOrArena:{supertypes:["CivicStructure","SportsActivityLocation"]},State:{supertypes:["AdministrativeArea"]},Store:{supertypes:["LocalBusiness"]},StructuredValue:{supertypes:["Intangible"]},SubwayStation:{supertypes:["CivicStructure"]},Synagogue:{supertypes:["PlaceOfWorship"]},TVEpisode:{supertypes:["CreativeWork"]},TVSeason:{supertypes:["CreativeWork"]},TVSeries:{supertypes:["CreativeWork"]},Table:{supertypes:["WebPageElement"]},TattooParlor:{supertypes:["HealthAndBeautyBusiness"]},TaxiStand:{supertypes:["CivicStructure"]},TelevisionStation:{supertypes:["LocalBusiness"]},TennisComplex:{supertypes:["SportsActivityLocation"]},TheaterEvent:{supertypes:["Event"]},TheaterGroup:{supertypes:["PerformingGroup"]},Thing:{supertypes:[]},TireShop:{supertypes:["Store"]},TouristAttraction:{supertypes:["Place"]},TouristInformationCenter:{supertypes:["LocalBusiness"]},ToyStore:{supertypes:["Store"]},TrainStation:{supertypes:["CivicStructure"]},TravelAgency:{supertypes:["LocalBusiness"]},UserBlocks:{supertypes:["UserInteraction"]},UserCheckins:{supertypes:["UserInteraction"]},UserComments:{supertypes:["UserInteraction"]},UserDownloads:{supertypes:["UserInteraction"]},UserInteraction:{supertypes:["Event"]},UserLikes:{supertypes:["UserInteraction"]},UserPageVisits:{supertypes:["UserInteraction"]},UserPlays:{supertypes:["UserInteraction"]},UserPlusOnes:{supertypes:["UserInteraction"]},UserTweets:{supertypes:["UserInteraction"]},VeterinaryCare:{supertypes:["MedicalOrganization"]},VideoGallery:{supertypes:["CollectionPage"]},VideoObject:{supertypes:["MediaObject"]},VisualArtsEvent:{supertypes:["Event"]},Volcano:{supertypes:["Landform"]},WPAdBlock:{supertypes:["WebPageElement"]},WPFooter:{supertypes:["WebPageElement"]},WPHeader:{supertypes:["WebPageElement"]},WPSideBar:{supertypes:["WebPageElement"]},Waterfall:{supertypes:["BodyOfWater"]},WebPage:{supertypes:["CreativeWork"]},WebPageElement:{supertypes:["CreativeWork"]},WholesaleStore:{supertypes:["Store"]},Winery:{supertypes:["FoodEstablishment"]},Zoo:{supertypes:["CivicStructure"]}}},t=function(n,r){if(n===r)return!0;var i=e.types[n];if(!i)return!1;if(!i.supertypes)return!1;for(var s=0;s<i.supertypes.length;s++)if(t(e.types[n].supertypes[s],r))return!0;return!1},n=function(t){var r=[t],i=[],s=!1,o=0,u=0;t.indexOf("/")!==-1&&(t=t.substring(0,t.indexOf("/")),r.push(t));var a=e.types[t];if(!a||!a.supertypes)return r;for(o=0;o<a.supertypes.length;o++)r=r.concat(n(a.supertypes[o]));i=[];for(o=0;o<r.length;o++){s=!1;for(u=0;u<i.length;u++)if(r[o]===i[u]){s=!0;break}s||i.push(r[o])}return i},r=function(e,t){if(!e||!t)return null;e=e["@type"]||e.itemType||e,t=t["@type"]||t.itemType||t;if(!e||!t)return null;if(e===t)return e;var r=n(e),i=n(t),s=[];for(var o=0;o<r.length;o++)for(var u=0;u<i.length;u++)if(r[o]===i[u]){s.push(r[o]);break}if(s.length===0)return null;if(s.length===1)return s[0];var a=null;for(var f=0;f<s.length;f++){a=s[f];if(a&&a!=="Thing")return a}return s[0]};return{isCompatible:function(e,n){if(!n)return!0;for(var r=0;r<n.length;r++)if(t(e,n[r]))return!0;return!1},getCommonAncestor:function(e){var t=[],n=null,i=null,s=0;if(!e||e.length===0)return null;if(e.length===1)return i=e[0],i?i["@type"]||i.itemType||i:null;for(s=0;s<e.length;s++){i=e[s],i=i["@type"]||i.itemType||i;if(!i)return null;t.push(i)}i=t[0];for(s=1;s<t.length;s++){i=r(i,t[s]);if(!i)return i}return i}}}),define("runtime-browser/underscore",[],function(){function C(e,t,n){if(e===t)return e!==0||1/e==1/t;if(e==null||t==null)return e===t;e._chain&&(e=e._wrapped),t._chain&&(t=t._wrapped);if(e.isEqual&&S.isFunction(e.isEqual))return e.isEqual(t);if(t.isEqual&&S.isFunction(t.isEqual))return t.isEqual(e);var r=a.call(e);if(r!=a.call(t))return!1;switch(r){case"[object String]":return e==String(t);case"[object Number]":return e!=+e?t!=+t:e==0?1/e==1/t:e==+t;case"[object Date]":case"[object Boolean]":return+e==+t;case"[object RegExp]":return e.source==t.source&&e.global==t.global&&e.multiline==t.multiline&&e.ignoreCase==t.ignoreCase}if(typeof e!="object"||typeof t!="object")return!1;var i=n.length;while(i--)if(n[i]==e)return!0;n.push(e);var s=0,o=!0;if(r=="[object Array]"){s=e.length,o=s==t.length;if(o)while(s--)if(!(o=s in e==s in t&&C(e[s],t[s],n)))break}else{if("constructor"in e!="constructor"in t||e.constructor!=t.constructor)return!1;for(var u in e)if(S.has(e,u)){s++;if(!(o=S.has(t,u)&&C(e[u],t[u],n)))break}if(o){for(u in t)if(S.has(t,u)&&!(s--))break;o=!s}}return n.pop(),o}var e=this,t=e._,n={},r=Array.prototype,i=Object.prototype,s=Function.prototype,o=r.slice,u=r.unshift,a=i.toString,f=i.hasOwnProperty,l=r.forEach,c=r.map,h=r.reduce,p=r.reduceRight,d=r.filter,v=r.every,m=r.some,g=r.indexOf,y=r.lastIndexOf,b=Array.isArray,w=Object.keys,E=s.bind,S=function(e){return new P(e)};S.VERSION="1.3.3";var x=S.each=S.forEach=function(e,t,r){if(e==null)return;if(l&&e.forEach===l)e.forEach(t,r);else if(e.length===+e.length){for(var i=0,s=e.length;i<s;i++)if(i in e&&t.call(r,e[i],i,e)===n)return}else for(var o in e)if(S.has(e,o)&&t.call(r,e[o],o,e)===n)return};S.map=S.collect=function(e,t,n){var r=[];return e==null?r:c&&e.map===c?e.map(t,n):(x(e,function(e,i,s){r[r.length]=t.call(n,e,i,s)}),e.length===+e.length&&(r.length=e.length),r)},S.reduce=S.foldl=S.inject=function(e,t,n,r){var i=arguments.length>2;e==null&&(e=[]);if(h&&e.reduce===h)return r&&(t=S.bind(t,r)),i?e.reduce(t,n):e.reduce(t);x(e,function(e,s,o){i?n=t.call(r,n,e,s,o):(n=e,i=!0)});if(!i)throw new TypeError("Reduce of empty array with no initial value");return n},S.reduceRight=S.foldr=function(e,t,n,r){var i=arguments.length>2;e==null&&(e=[]);if(p&&e.reduceRight===p)return r&&(t=S.bind(t,r)),i?e.reduceRight(t,n):e.reduceRight(t);var s=S.toArray(e).reverse();return r&&!i&&(t=S.bind(t,r)),i?S.reduce(s,t,n,r):S.reduce(s,t)},S.find=S.detect=function(e,t,n){var r;return T(e,function(e,i,s){if(t.call(n,e,i,s))return r=e,!0}),r},S.filter=S.select=function(e,t,n){var r=[];return e==null?r:d&&e.filter===d?e.filter(t,n):(x(e,function(e,i,s){t.call(n,e,i,s)&&(r[r.length]=e)}),r)},S.reject=function(e,t,n){var r=[];return e==null?r:(x(e,function(e,i,s){t.call(n,e,i,s)||(r[r.length]=e)}),r)},S.every=S.all=function(e,t,r){var i=!0;return e==null?i:v&&e.every===v?e.every(t,r):(x(e,function(e,s,o){if(!(i=i&&t.call(r,e,s,o)))return n}),!!i)};var T=S.some=S.any=function(e,t,r){t||(t=S.identity);var i=!1;return e==null?i:m&&e.some===m?e.some(t,r):(x(e,function(e,s,o){if(i||(i=t.call(r,e,s,o)))return n}),!!i)};S.include=S.contains=function(e,t){var n=!1;return e==null?n:g&&e.indexOf===g?e.indexOf(t)!=-1:(n=T(e,function(e){return e===t}),n)},S.invoke=function(e,t){var n=o.call(arguments,2);return S.map(e,function(e){return(S.isFunction(t)?t||e:e[t]).apply(e,n)})},S.pluck=function(e,t){return S.map(e,function(e){return e[t]})},S.max=function(e,t,n){if(!t&&S.isArray(e)&&e[0]===+e[0])return Math.max.apply(Math,e);if(!t&&S.isEmpty(e))return-Infinity;var r={computed:-Infinity};return x(e,function(e,i,s){var o=t?t.call(n,e,i,s):e;o>=r.computed&&(r={value:e,computed:o})}),r.value},S.min=function(e,t,n){if(!t&&S.isArray(e)&&e[0]===+e[0])return Math.min.apply(Math,e);if(!t&&S.isEmpty(e))return Infinity;var r={computed:Infinity};return x(e,function(e,i,s){var o=t?t.call(n,e,i,s):e;o<r.computed&&(r={value:e,computed:o})}),r.value},S.shuffle=function(e){var t=[],n;return x(e,function(e,r,i){n=Math.floor(Math.random()*(r+1)),t[r]=t[n],t[n]=e}),t},S.sortBy=function(e,t,n){var r=S.isFunction(t)?t:function(e){return e[t]};return S.pluck(S.map(e,function(e,t,i){return{value:e,criteria:r.call(n,e,t,i)}}).sort(function(e,t){var n=e.criteria,r=t.criteria;return n===void 0?1:r===void 0?-1:n<r?-1:n>r?1:0}),"value")},S.groupBy=function(e,t){var n={},r=S.isFunction(t)?t:function(e){return e[t]};return x(e,function(e,t){var i=r(e,t);(n[i]||(n[i]=[])).push(e)}),n},S.sortedIndex=function(e,t,n){n||(n=S.identity);var r=0,i=e.length;while(r<i){var s=r+i>>1;n(e[s])<n(t)?r=s+1:i=s}return r},S.toArray=function(e){return e?S.isArray(e)?o.call(e):S.isArguments(e)?o.call(e):e.toArray&&S.isFunction(e.toArray)?e.toArray():S.values(e):[]},S.size=function(e){return S.isArray(e)?e.length:S.keys(e).length},S.first=S.head=S.take=function(e,t,n){return t!=null&&!n?o.call(e,0,t):e[0]},S.initial=function(e,t,n){return o.call(e,0,e.length-(t==null||n?1:t))},S.last=function(e,t,n){return t!=null&&!n?o.call(e,Math.max(e.length-t,0)):e[e.length-1]},S.rest=S.tail=function(e,t,n){return o.call(e,t==null||n?1:t)},S.compact=function(e){return S.filter(e,function(e){return!!e})},S.flatten=function(e,t){return S.reduce(e,function(e,n){return S.isArray(n)?e.concat(t?n:S.flatten(n)):(e[e.length]=n,e)},[])},S.without=function(e){return S.difference(e,o.call(arguments,1))},S.uniq=S.unique=function(e,t,n){var r=n?S.map(e,n):e,i=[];return e.length<3&&(t=!0),S.reduce(r,function(n,r,s){if(t?S.last(n)!==r||!n.length:!S.include(n,r))n.push(r),i.push(e[s]);return n},[]),i},S.union=function(){return S.uniq(S.flatten(arguments,!0))},S.intersection=S.intersect=function(e){var t=o.call(arguments,1);return S.filter(S.uniq(e),function(e){return S.every(t,function(t){return S.indexOf(t,e)>=0})})},S.difference=function(e){var t=S.flatten(o.call(arguments,1),!0);return S.filter(e,function(e){return!S.include(t,e)})},S.zip=function(){var e=o.call(arguments),t=S.max(S.pluck(e,"length")),n=new Array(t);for(var r=0;r<t;r++)n[r]=S.pluck(e,""+r);return n},S.indexOf=function(e,t,n){if(e==null)return-1;var r,i;if(n)return r=S.sortedIndex(e,t),e[r]===t?r:-1;if(g&&e.indexOf===g)return e.indexOf(t);for(r=0,i=e.length;r<i;r++)if(r in e&&e[r]===t)return r;return-1},S.lastIndexOf=function(e,t){if(e==null)return-1;if(y&&e.lastIndexOf===y)return e.lastIndexOf(t);var n=e.length;while(n--)if(n in e&&e[n]===t)return n;return-1},S.range=function(e,t,n){arguments.length<=1&&(t=e||0,e=0),n=arguments[2]||1;var r=Math.max(Math.ceil((t-e)/n),0),i=0,s=new Array(r);while(i<r)s[i++]=e,e+=n;return s};var N=function(){};S.bind=function(t,n){var r,i;if(t.bind===E&&E)return E.apply(t,o.call(arguments,1));if(!S.isFunction(t))throw new TypeError;return i=o.call(arguments,2),r=function(){if(this instanceof r){N.prototype=t.prototype;var e=new N,s=t.apply(e,i.concat(o.call(arguments)));return Object(s)===s?s:e}return t.apply(n,i.concat(o.call(arguments)))}},S.bindAll=function(e){var t=o.call(arguments,1);return t.length==0&&(t=S.functions(e)),x(t,function(t){e[t]=S.bind(e[t],e)}),e},S.memoize=function(e,t){var n={};return t||(t=S.identity),function(){var r=t.apply(this,arguments);return S.has(n,r)?n[r]:n[r]=e.apply(this,arguments)}},S.delay=function(e,t){var n=o.call(arguments,2);return setTimeout(function(){return e.apply(null,n)},t)},S.defer=function(e){return S.delay.apply(S,[e,1].concat(o.call(arguments,1)))},S.throttle=function(e,t){var n,r,i,s,o,u,a=S.debounce(function(){o=s=!1},t);return function(){n=this,r=arguments;var f=function(){i=null,o&&e.apply(n,r),a()};return i||(i=setTimeout(f,t)),s?o=!0:u=e.apply(n,r),a(),s=!0,u}},S.debounce=function(e,t,n){var r;return function(){var i=this,s=arguments,o=function(){r=null,n||e.apply(i,s)};n&&!r&&e.apply(i,s),clearTimeout(r),r=setTimeout(o,t)}},S.once=function(e){var t=!1,n;return function(){return t?n:(t=!0,n=e.apply(this,arguments))}},S.wrap=function(e,t){return function(){var n=[e].concat(o.call(arguments,0));return t.apply(this,n)}},S.compose=function(){var e=arguments;return function(){var t=arguments;for(var n=e.length-1;n>=0;n--)t=[e[n].apply(this,t)];return t[0]}},S.after=function(e,t){return e<=0?t():function(){if(--e<1)return t.apply(this,arguments)}},S.keys=w||function(e){if(e!==Object(e))throw new TypeError("Invalid object");var t=[];for(var n in e)S.has(e,n)&&(t[t.length]=n);return t},S.values=function(e){return S.map(e,S.identity)},S.functions=S.methods=function(e){var t=[];for(var n in e)S.isFunction(e[n])&&t.push(n);return t.sort()},S.extend=function(e){return x(o.call(arguments,1),function(t){for(var n in t)e[n]=t[n]}),e},S.pick=function(e){var t={};return x(S.flatten(o.call(arguments,1)),function(n){n in e&&(t[n]=e[n])}),t},S.defaults=function(e){return x(o.call(arguments,1),function(t){for(var n in t)e[n]==null&&(e[n]=t[n])}),e},S.clone=function(e){return S.isObject(e)?S.isArray(e)?e.slice():S.extend({},e):e},S.tap=function(e,t){return t(e),e},S.isEqual=function(e,t){return C(e,t,[])},S.isEmpty=function(e){if(e==null)return!0;if(S.isArray(e)||S.isString(e))return e.length===0;for(var t in e)if(S.has(e,t))return!1;return!0},S.isElement=function(e){return!!e&&e.nodeType==1},S.isArray=b||function(e){return a.call(e)=="[object Array]"},S.isObject=function(e){return e===Object(e)},S.isArguments=function(e){return a.call(e)=="[object Arguments]"},S.isArguments(arguments)||(S.isArguments=function(e){return!!e&&!!S.has(e,"callee")}),S.isFunction=function(e){return a.call(e)=="[object Function]"},S.isString=function(e){return a.call(e)=="[object String]"},S.isNumber=function(e){return a.call(e)=="[object Number]"},S.isFinite=function(e){return S.isNumber(e)&&isFinite(e)},S.isNaN=function(e){return e!==e},S.isBoolean=function(e){return e===!0||e===!1||a.call(e)=="[object Boolean]"},S.isDate=function(e){return a.call(e)=="[object Date]"},S.isRegExp=function(e){return a.call(e)=="[object RegExp]"},S.isNull=function(e){return e===null},S.isUndefined=function(e){return e===void 0},S.has=function(e,t){return f.call(e,t)},S.noConflict=function(){return e._=t,this},S.identity=function(e){return e},S.times=function(e,t,n){for(var r=0;r<e;r++)t.call(n,r)},S.escape=function(e){return(""+e).replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#x27;").replace(/\//g,"&#x2F;")},S.result=function(e,t){if(e==null)return null;var n=e[t];return S.isFunction(n)?n.call(e):n},S.mixin=function(e){x(S.functions(e),function(t){B(t,S[t]=e[t])})};var k=0;S.uniqueId=function(e){var t=k++;return e?e+t:t},S.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g};var L=/.^/,A={"\\":"\\","'":"'",r:"\r",n:"\n",t:"	",u2028:"\u2028",u2029:"\u2029"};for(var O in A)A[A[O]]=O;var M=/\\|'|\r|\n|\t|\u2028|\u2029/g,_=/\\(\\|'|r|n|t|u2028|u2029)/g,D=function(e){return e.replace(_,function(e,t){return A[t]})};S.template=function(e,t,n){n=S.defaults(n||{},S.templateSettings);var r="__p+='"+e.replace(M,function(e){return"\\"+A[e]}).replace(n.escape||L,function(e,t){return"'+\n_.escape("+D(t)+")+\n'"}).replace(n.interpolate||L,function(e,t){return"'+\n("+D(t)+")+\n'"}).replace(n.evaluate||L,function(e,t){return"';\n"+D(t)+"\n;__p+='"})+"';\n";n.variable||(r="with(obj||{}){\n"+r+"}\n"),r="var __p='';var print=function(){__p+=Array.prototype.join.call(arguments, '')};\n"+r+"return __p;\n";var i=new Function(n.variable||"obj","_",r);if(t)return i(t,S);var s=function(e){return i.call(this,e,S)};return s.source="function("+(n.variable||"obj")+"){\n"+r+"}",s},S.chain=function(e){return S(e).chain()};var P=function(e){this._wrapped=e};S.prototype=P.prototype;var H=function(e,t){return t?S(e).chain():e},B=function(e,t){P.prototype[e]=function(){var e=o.call(arguments);return u.call(e,this._wrapped),H(t.apply(S,e),this._chain)}};return S.mixin(S),x(["pop","push","reverse","shift","sort","splice","unshift"],function(e){var t=r[e];P.prototype[e]=function(){var n=this._wrapped;t.apply(n,arguments);var r=n.length;return(e=="shift"||e=="splice")&&r===0&&delete n[0],H(n,this._chain)}}),x(["concat","join","slice"],function(e){var t=r[e];P.prototype[e]=function(){return H(t.apply(this._wrapped,arguments),this._chain)}}),P.prototype.chain=function(){return this._chain=!0,this},P.prototype.value=function(){return this._wrapped},S}),define("runtime-browser/getobjkey",[],function(){var e=/\[([0-9]*)\](?:\.|$)/;return function(t,n){var r=t,i=n.split("."),s=null,o=null,u=Object.prototype.toString;for(var a=0;a<i.length;a++){if(r===null||typeof r!="object")return null;s=i[a],o=s.match(e);if(o){s=s.replace(e,"");if(u.call(r[s])!=="[object Array]")return null;r=r[s][parseInt(o[1],10)]}else r=r[s]}return r}}),define("databases/operator/union",["require","datajslib!underscore","datajslib!getobjkey"],function(e,t,n){return{desc:{datasources:{main:{title:"Feeds to merge",multiple:!0}},options:{schema:{field:{title:"Sort by",type:"string","default":"-special!date"},ignoreCase:{title:"Case-insensitive sort",type:"boolean","default":!0}},form:["datasources.main","field",{template:"NaNitems in descending order"},{key:"ignoreCase",notitle:!0,inlinetitle:"Ignore case when sorting"}]},runtimes:["hosted","browser","nodejs","win8"]},updateInputOptions:function(e,t){var n=e;if(e.skip||e.limit){n={};for(var r in e)r!=="skip"&&r!=="limit"&&(n[r]=e[r]);n.limit=(e.limit||20)+(e.skip||0)}return t(null,n)},process:function(e,n,r){var i=null,s={entries:[]};if(!e)return r(null,s);e.main?i=e.main:i=e;if(!i[0])return r(null,s);n=n||{},n.filter=n.filter||{};var o=0,u=20;n.skip>0&&(o=n.skip),n.limit>0&&(u=n.limit);var a=n.filter.field||"-special!date",f=Math.ceil((o+u)/u),l=t.map(t.range(f),function(e){var n=[];return t.each(i,function(t){t=t||{},t=t.entries||[],t=t.slice(u*e,u*(e+1)),n=n.concat(t)}),n}),c=0,h=[];t.each(l,function(e){if(c+e.length<=o){c+=e.length;return}if(c>=o+u){c+=e.length;return}e=this.sort(e,a,n.filter.ignoreCase);var t=o-c;t<0&&(t=0);var r=o+u-c;h=h.concat(e.slice(t,r)),c+=e.length},this),s.entries=h;if(a==="special!shuffle"||a==="-special!shuffle")s.maxAge=0;return r(null,s)},sort:function(e,r,i){var s=r.replace(/^\-/,""),o=null;return s==="special!date"?o=t.sortBy(e,function(e){return e.datePublished||e.dateModified||e.dateCreated||e.uploadDate||e.startDate||e.endDate||e.datePosted||e.foundingDate||e.birthDate||e.deathDate}):s==="special!shuffle"?o=t.shuffle(e):o=t.sortBy(e,function(e){var r=n(e,s)||"";return i&&t.isString(r)?r.toLowerCase():r}),r.substring(0,1)==="-"&&(o=o.reverse()),o}}}),define("databases/graph/union",["datajslib!collection.graph","datajslib!utils/schematype","../operator/union"],function(e,t){return{desc:{datasources:{main:{title:"Datasources to merge",multiple:!0}},options:{schema:{field:{title:"Sort result by",type:"string","default":"-special!date","enum":["none","-special!date","special!date","name","special!shuffle"]}},form:["datasources.main",{key:"field",titleMap:{none:"Keep initial order","-special!date":"Most recent first","special!date":"Least recent first",name:"Sort by title","special!shuffle":"Shuffle items"}}]},runtimes:["hosted","browser","nodejs","win8"]},find:function(t,n){t=t||{};var r=t.filter||{},i=r.datasources||{},s={db:"operator",col:"union",query:{filter:{datasources:{main:i.main},field:r.field||"-special!date",ignoreCase:!0}},runtime:"browser"},o={};for(var u in t)u!=="filter"&&u!=="runtime"&&u!=="db"&&u!=="col"&&u!=="query"&&(o[u]=t[u]);e.getCollection(s,function(e,t){if(e)return n(e);t.find(o,n)})},getDesc:function(e){var n=[],r=0;this.desc.outputType="Thing";if(e&&e.filter&&e.filter.datasources&&e.filter.datasources.main){for(r=0;r<e.filter.datasources.main.length;r++)n.push(e.filter.datasources.main[r].outputType);this.desc.outputType=t.getCommonAncestor(n)}return this.desc}}}),define("runtime-browser/collection.proxy",[],function(){var e=Joshfire.factory.globalConfig,t=function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e},n=function(){var e=0,t=document.getElementsByTagName("head")[0];return{get:function(n,r,i,s){var o=null,u=null,a=null,f=!1,l=!1,c=null,h="json"+ ++e,p=function(){if(l)return;l=!0;try{a.onload=null,a.onreadystatechange=null,a.onerror=null,a.parentNode&&a.parentNode.removeChild(a)}catch(e){}},d=function(e,t){if(f)return;f=!0,c&&clearTimeout(c),s(e,t);try{delete Joshfire.datajs[h]}catch(n){}Joshfire.datajs[h]=null};if(!n)return s({status:400,message:"No url to request"});o=n.indexOf("?")!==-1?"&":"?",r=r||{};for(u in r)r.hasOwnProperty(u)&&(o+=encodeURIComponent(u)+"="+encodeURIComponent(r[u])+"&");return o+="callback=Joshfire.datajs."+h,Joshfire.datajs=Joshfire.datajs||{},Joshfire.datajs[h]=function(e){d(null,e)},a=document.createElement("script"),a.src=n+o,a.async=!0,a.onload=a.onreadystatechange=function(){(!this.readyState||this.readyState==="loaded"||this.readyState==="complete"||this.readyState===4)&&p()},a.onerror=function(e){p(),d({status:500,message:"Invalid script received"},null)},t.appendChild(a),c=setTimeout(function(){p(),d({status:500,message:"Timeout exceeded"})},i*1e3),h}}}(),r=function(r){var s=r.query||{},o=r.db,u=r.col,a=r.id,f=r.apikey;this.find=function(r,i){var l=this,c={},h=null;t(c,s),t(c,r),c.apikey=f,a?h="http://"+e.DATAHOSTPORT+"/api/"+e.DATAVERSION+"/feed/ds/"+a:(c.filter&&(c.filter=JSON.stringify(c.filter)),h="http://"+e.DATAHOSTPORT+"/api/"+e.DATAVERSION+"/feed/"+o+"/"+u),n.get(h,c,30,function(e,t){return t&&!t.name&&l.name&&(t.name=l.name),e?i(e,t):t&&t.error?i(t.error):i(null,t)})},this.getDesc=function(e){i.getCollectionDesc(o,u,s,e)}},i={getCollection:function(e,t){var n=new r(e);return t(null,n)},getCollectionDesc:function(r,i,s,o){var u={},a=null;s&&t(u,s),u.filter&&(u.filter=JSON.stringify(u.filter)),a="http://"+e.DATAHOSTPORT+"/api/"+e.DATAVERSION+"/collection/"+r+"/"+i,n.get(a,u,30,o)}};return i}),define("runtime-browser/jquery",[],function(){function _(e){var t=e.length,n=g.type(e);return g.isWindow(e)?!1:e.nodeType===1&&t?!0:n==="array"||n!=="function"&&(t===0||typeof t=="number"&&t>0&&t-1 in e)}function P(e){var t=D[e]={};return g.each(e.match(b)||[],function(e,n){t[n]=!0}),t}function j(e,t,n,r){if(!g.acceptData(e))return;var i,s,o=g.expando,u=typeof t=="string",f=e.nodeType,l=f?g.cache:e,c=f?e[o]:e[o]&&o;if((!c||!l[c]||!r&&!l[c].data)&&u&&n===undefined)return;c||(f?e[o]=c=a.pop()||g.guid++:c=o),l[c]||(l[c]={},f||(l[c].toJSON=g.noop));if(typeof t=="object"||typeof t=="function")r?l[c]=g.extend(l[c],t):l[c].data=g.extend(l[c].data,t);return i=l[c],r||(i.data||(i.data={}),i=i.data),n!==undefined&&(i[g.camelCase(t)]=n),u?(s=i[t],s==null&&(s=i[g.camelCase(t)])):s=i,s}function F(e,t,n){if(!g.acceptData(e))return;var r,i,s,o=e.nodeType,u=o?g.cache:e,a=o?e[g.expando]:g.expando;if(!u[a])return;if(t){s=n?u[a]:u[a].data;if(s){g.isArray(t)?t=t.concat(g.map(t,g.camelCase)):t in s?t=[t]:(t=g.camelCase(t),t in s?t=[t]:t=t.split(" "));for(r=0,i=t.length;r<i;r++)delete s[t[r]];if(!(n?q:g.isEmptyObject)(s))return}}if(!n){delete u[a].data;if(!q(u[a]))return}o?g.cleanData([e],!0):g.support.deleteExpando||u!=u.window?delete u[a]:u[a]=null}function I(e,t,n){if(n===undefined&&e.nodeType===1){var r="data-"+t.replace(B,"-$1").toLowerCase();n=e.getAttribute(r);if(typeof n=="string"){try{n=n==="true"?!0:n==="false"?!1:n==="null"?null:+n+""===n?+n:H.test(n)?g.parseJSON(n):n}catch(i){}g.data(e,t,n)}else n=undefined}return n}function q(e){var t;for(t in e){if(t==="data"&&g.isEmptyObject(e[t]))continue;if(t!=="toJSON")return!1}return!0}function nt(){return!0}function rt(){return!1}function ft(e,t){do e=e[t];while(e&&e.nodeType!==1);return e}function lt(e,t,n){t=t||0;if(g.isFunction(t))return g.grep(e,function(e,r){var i=!!t.call(e,r,e);return i===n});if(t.nodeType)return g.grep(e,function(e){return e===t===n});if(typeof t=="string"){var r=g.grep(e,function(e){return e.nodeType===1});if(ot.test(t))return g.filter(t,r,!n);t=g.filter(t,r)}return g.grep(e,function(e){return g.inArray(e,t)>=0===n})}function ct(e){var t=ht.split("|"),n=e.createDocumentFragment();if(n.createElement)while(t.length)n.createElement(t.pop());return n}function At(e,t){return e.getElementsByTagName(t)[0]||e.appendChild(e.ownerDocument.createElement(t))}function Ot(e){var t=e.getAttributeNode("type");return e.type=(t&&t.specified)+"/"+e.type,e}function Mt(e){var t=Tt.exec(e.type);return t?e.type=t[1]:e.removeAttribute("type"),e}function _t(e,t){var n,r=0;for(;(n=e[r])!=null;r++)g._data(n,"globalEval",!t||g._data(t[r],"globalEval"))}function Dt(e,t){if(t.nodeType!==1||!g.hasData(e))return;var n,r,i,s=g._data(e),o=g._data(t,s),u=s.events;if(u){delete o.handle,o.events={};for(n in u)for(r=0,i=u[n].length;r<i;r++)g.event.add(t,n,u[n][r])}o.data&&(o.data=g.extend({},o.data))}function Pt(e,t){var n,r,i;if(t.nodeType!==1)return;n=t.nodeName.toLowerCase();if(!g.support.noCloneEvent&&t[g.expando]){i=g._data(t);for(r in i.events)g.removeEvent(t,r,i.handle);t.removeAttribute(g.expando)}if(n==="script"&&t.text!==e.text)Ot(t).text=e.text,Mt(t);else if(n==="object")t.parentNode&&(t.outerHTML=e.outerHTML),g.support.html5Clone&&e.innerHTML&&!g.trim(t.innerHTML)&&(t.innerHTML=e.innerHTML);else if(n==="input"&&Et.test(e.type))t.defaultChecked=t.checked=e.checked,t.value!==e.value&&(t.value=e.value);else if(n==="option")t.defaultSelected=t.selected=e.defaultSelected;else if(n==="input"||n==="textarea")t.defaultValue=e.defaultValue}function Ht(e,t){var r,i,s=0,o=typeof e.getElementsByTagName!==n?e.getElementsByTagName(t||"*"):typeof e.querySelectorAll!==n?e.querySelectorAll(t||"*"):undefined;if(!o)for(o=[],r=e.childNodes||e;(i=r[s])!=null;s++)!t||g.nodeName(i,t)?o.push(i):g.merge(o,Ht(i,t));return t===undefined||t&&g.nodeName(e,t)?g.merge([e],o):o}function Bt(e){Et.test(e.type)&&(e.defaultChecked=e.checked)}function Zt(e,t){if(t in e)return t;var n=t.charAt(0).toUpperCase()+t.slice(1),r=t,i=Yt.length;while(i--){t=Yt[i]+n;if(t in e)return t}return r}function en(e,t){return e=t||e,g.css(e,"display")==="none"||!g.contains(e.ownerDocument,e)}function tn(e,t){var n,r,i,s=[],o=0,u=e.length;for(;o<u;o++){r=e[o];if(!r.style)continue;s[o]=g._data(r,"olddisplay"),n=r.style.display,t?(!s[o]&&n==="none"&&(r.style.display=""),r.style.display===""&&en(r)&&(s[o]=g._data(r,"olddisplay",on(r.nodeName)))):s[o]||(i=en(r),(n&&n!=="none"||!i)&&g._data(r,"olddisplay",i?n:g.css(r,"display")))}for(o=0;o<u;o++){r=e[o];if(!r.style)continue;if(!t||r.style.display==="none"||r.style.display==="")r.style.display=t?s[o]||"":"none"}return e}function nn(e,t,n){var r=Xt.exec(t);return r?Math.max(0,r[1]-(n||0))+(r[2]||"px"):t}function rn(e,t,n,r,i){var s=n===(r?"border":"content")?4:t==="width"?1:0,o=0;for(;s<4;s+=2)n==="margin"&&(o+=g.css(e,n+Gt[s],!0,i)),r?(n==="content"&&(o-=g.css(e,"padding"+Gt[s],!0,i)),n!=="margin"&&(o-=g.css(e,"border"+Gt[s]+"Width",!0,i))):(o+=g.css(e,"padding"+Gt[s],!0,i),n!=="padding"&&(o+=g.css(e,"border"+Gt[s]+"Width",!0,i)));return o}function sn(e,t,n){var r=!0,i=t==="width"?e.offsetWidth:e.offsetHeight,s=Ft(e),o=g.support.boxSizing&&g.css(e,"boxSizing",!1,s)==="border-box";if(i<=0||i==null){i=It(e,t,s);if(i<0||i==null)i=e.style[t];if(Vt.test(i))return i;r=o&&(g.support.boxSizingReliable||i===e.style[t]),i=parseFloat(i)||0}return i+rn(e,t,n||(o?"border":"content"),r,s)+"px"}function on(e){var t=r,n=Jt[e];if(!n){n=un(e,t);if(n==="none"||!n)jt=(jt||g("<iframe frameborder='0' width='0' height='0'/>").css("cssText","display:block !important")).appendTo(t.documentElement),t=(jt[0].contentWindow||jt[0].contentDocument).document,t.write("<!doctype html><html><body>"),t.close(),n=un(e,t),jt.detach();Jt[e]=n}return n}function un(e,t){var n=g(t.createElement(e)).appendTo(t.body),r=g.css(n[0],"display");return n.remove(),r}function pn(e,t,n,r){var i;if(g.isArray(t))g.each(t,function(t,i){n||fn.test(e)?r(e,i):pn(e+"["+(typeof i=="object"?t:"")+"]",i,n,r)});else if(!n&&g.type(t)==="object")for(i in t)pn(e+"["+i+"]",t[i],n,r);else r(e,t)}function On(e){return function(t,n){typeof t!="string"&&(n=t,t="*");var r,i=0,s=t.toLowerCase().match(b)||[];if(g.isFunction(n))while(r=s[i++])r[0]==="+"?(r=r.slice(1)||"*",(e[r]=e[r]||[]).unshift(n)):(e[r]=e[r]||[]).push(n)}}function Mn(e,t,n,r){function o(u){var a;return i[u]=!0,g.each(e[u]||[],function(e,u){var f=u(t,n,r);if(typeof f=="string"&&!s&&!i[f])return t.dataTypes.unshift(f),o(f),!1;if(s)return!(a=f)}),a}var i={},s=e===kn;return o(t.dataTypes[0])||!i["*"]&&o("*")}function _n(e,t){var n,r,i=g.ajaxSettings.flatOptions||{};for(r in t)t[r]!==undefined&&((i[r]?e:n||(n={}))[r]=t[r]);return n&&g.extend(!0,e,n),e}function Dn(e,t,n){var r,i,s,o,u=e.contents,a=e.dataTypes,f=e.responseFields;for(o in f)o in n&&(t[f[o]]=n[o]);while(a[0]==="*")a.shift(),i===undefined&&(i=e.mimeType||t.getResponseHeader("Content-Type"));if(i)for(o in u)if(u[o]&&u[o].test(i)){a.unshift(o);break}if(a[0]in n)s=a[0];else{for(o in n){if(!a[0]||e.converters[o+" "+a[0]]){s=o;break}r||(r=o)}s=s||r}if(s)return s!==a[0]&&a.unshift(s),n[s]}function Pn(e,t){var n,r,i,s,o={},u=0,a=e.dataTypes.slice(),f=a[0];e.dataFilter&&(t=e.dataFilter(t,e.dataType));if(a[1])for(i in e.converters)o[i.toLowerCase()]=e.converters[i];for(;r=a[++u];)if(r!=="*"){if(f!=="*"&&f!==r){i=o[f+" "+r]||o["* "+r];if(!i)for(n in o){s=n.split(" ");if(s[1]===r){i=o[f+" "+s[0]]||o["* "+s[0]];if(i){i===!0?i=o[n]:o[n]!==!0&&(r=s[0],a.splice(u--,0,r));break}}}if(i!==!0)if(i&&e["throws"])t=i(t);else try{t=i(t)}catch(l){return{state:"parsererror",error:i?l:"No conversion from "+f+" to "+r}}}f=r}return{state:"success",data:t}}function Rn(){try{return new window.XMLHttpRequest}catch(e){}}function Un(){try{return new window.ActiveXObject("Microsoft.XMLHTTP")}catch(e){}}function Qn(){return setTimeout(function(){zn=undefined}),zn=g.now()}function Gn(e,t){g.each(t,function(t,n){var r=(Kn[t]||[]).concat(Kn["*"]),i=0,s=r.length;for(;i<s;i++)if(r[i].call(e,t,n))return})}function Yn(e,t,n){var r,i,s=0,o=Jn.length,u=g.Deferred().always(function(){delete a.elem}),a=function(){if(i)return!1;var t=zn||Qn(),n=Math.max(0,f.startTime+f.duration-t),r=n/f.duration||0,s=1-r,o=0,a=f.tweens.length;for(;o<a;o++)f.tweens[o].run(s);return u.notifyWith(e,[f,s,n]),s<1&&a?n:(u.resolveWith(e,[f]),!1)},f=u.promise({elem:e,props:g.extend({},t),opts:g.extend(!0,{specialEasing:{}},n),originalProperties:t,originalOptions:n,startTime:zn||Qn(),duration:n.duration,tweens:[],createTween:function(t,n){var r=g.Tween(e,f.opts,t,n,f.opts.specialEasing[t]||f.opts.easing);return f.tweens.push(r),r},stop:function(t){var n=0,r=t?f.tweens.length:0;if(i)return this;i=!0;for(;n<r;n++)f.tweens[n].run(1);return t?u.resolveWith(e,[f,t]):u.rejectWith(e,[f,t]),this}}),l=f.props;Zn(l,f.opts.specialEasing);for(;s<o;s++){r=Jn[s].call(f,e,l,f.opts);if(r)return r}return Gn(f,l),g.isFunction(f.opts.start)&&f.opts.start.call(e,f),g.fx.timer(g.extend(a,{elem:e,anim:f,queue:f.opts.queue})),f.progress(f.opts.progress).done(f.opts.done,f.opts.complete).fail(f.opts.fail).always(f.opts.always)}function Zn(e,t){var n,r,i,s,o;for(i in e){r=g.camelCase(i),s=t[r],n=e[i],g.isArray(n)&&(s=n[1],n=e[i]=n[0]),i!==r&&(e[r]=n,delete e[i]),o=g.cssHooks[r];if(o&&"expand"in o){n=o.expand(n),delete e[r];for(i in n)i in e||(e[i]=n[i],t[i]=s)}else t[r]=s}}function er(e,t,n){var r,i,s,o,u,a,f,l,c,h=this,p=e.style,d={},v=[],m=e.nodeType&&en(e);n.queue||(l=g._queueHooks(e,"fx"),l.unqueued==null&&(l.unqueued=0,c=l.empty.fire,l.empty.fire=function(){l.unqueued||c()}),l.unqueued++,h.always(function(){h.always(function(){l.unqueued--,g.queue(e,"fx").length||l.empty.fire()})})),e.nodeType===1&&("height"in t||"width"in t)&&(n.overflow=[p.overflow,p.overflowX,p.overflowY],g.css(e,"display")==="inline"&&g.css(e,"float")==="none"&&(!g.support.inlineBlockNeedsLayout||on(e.nodeName)==="inline"?p.display="inline-block":p.zoom=1)),n.overflow&&(p.overflow="hidden",g.support.shrinkWrapBlocks||h.always(function(){p.overflow=n.overflow[0],p.overflowX=n.overflow[1],p.overflowY=n.overflow[2]}));for(i in t){o=t[i];if(Xn.exec(o)){delete t[i],a=a||o==="toggle";if(o===(m?"hide":"show"))continue;v.push(i)}}s=v.length;if(s){u=g._data(e,"fxshow")||g._data(e,"fxshow",{}),"hidden"in u&&(m=u.hidden),a&&(u.hidden=!m),m?g(e).show():h.done(function(){g(e).hide()}),h.done(function(){var t;g._removeData(e,"fxshow");for(t in d)g.style(e,t,d[t])});for(i=0;i<s;i++)r=v[i],f=h.createTween(r,m?u[r]:0),d[r]=u[r]||g.style(e,r),r in u||(u[r]=f.start,m&&(f.end=f.start,f.start=r==="width"||r==="height"?1:0))}}function tr(e,t,n,r,i){return new tr.prototype.init(e,t,n,r,i)}function nr(e,t){var n,r={height:e},i=0;t=t?1:0;for(;i<4;i+=2-t)n=Gt[i],r["margin"+n]=r["padding"+n]=e;return t&&(r.opacity=r.width=e),r}function rr(e){return g.isWindow(e)?e:e.nodeType===9?e.defaultView||e.parentWindow:!1}var e,t,n=typeof undefined,r=window.document,i=window.location,s=window.jQuery,o=window.$,u={},a=[],f="1.9.1",l=a.concat,c=a.push,h=a.slice,p=a.indexOf,d=u.toString,v=u.hasOwnProperty,m=f.trim,g=function(e,n){return new g.fn.init(e,n,t)},y=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,b=/\S+/g,w=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,E=/^(?:(<[\w\W]+>)[^>]*|#([\w-]*))$/,S=/^<(\w+)\s*\/?>(?:<\/\1>|)$/,x=/^[\],:{}\s]*$/,T=/(?:^|:|,)(?:\s*\[)+/g,N=/\\(?:["\\\/bfnrt]|u[\da-fA-F]{4})/g,C=/"[^"\\\r\n]*"|true|false|null|-?(?:\d+\.|)\d+(?:[eE][+-]?\d+|)/g,k=/^-ms-/,L=/-([\da-z])/gi,A=function(e,t){return t.toUpperCase()},O=function(e){if(r.addEventListener||e.type==="load"||r.readyState==="complete")M(),g.ready()},M=function(){r.addEventListener?(r.removeEventListener("DOMContentLoaded",O,!1),window.removeEventListener("load",O,!1)):(r.detachEvent("onreadystatechange",O),window.detachEvent("onload",O))};g.fn=g.prototype={jquery:f,constructor:g,init:function(e,t,n){var i,s;if(!e)return this;if(typeof e=="string"){e.charAt(0)==="<"&&e.charAt(e.length-1)===">"&&e.length>=3?i=[null,e,null]:i=E.exec(e);if(i&&(i[1]||!t)){if(i[1]){t=t instanceof g?t[0]:t,g.merge(this,g.parseHTML(i[1],t&&t.nodeType?t.ownerDocument||t:r,!0));if(S.test(i[1])&&g.isPlainObject(t))for(i in t)g.isFunction(this[i])?this[i](t[i]):this.attr(i,t[i]);return this}s=r.getElementById(i[2]);if(s&&s.parentNode){if(s.id!==i[2])return n.find(e);this.length=1,this[0]=s}return this.context=r,this.selector=e,this}return!t||t.jquery?(t||n).find(e):this.constructor(t).find(e)}return e.nodeType?(this.context=this[0]=e,this.length=1,this):g.isFunction(e)?n.ready(e):(e.selector!==undefined&&(this.selector=e.selector,this.context=e.context),g.makeArray(e,this))},selector:"",length:0,size:function(){return this.length},toArray:function(){return h.call(this)},get:function(e){return e==null?this.toArray():e<0?this[this.length+e]:this[e]},pushStack:function(e){var t=g.merge(this.constructor(),e);return t.prevObject=this,t.context=this.context,t},each:function(e,t){return g.each(this,e,t)},ready:function(e){return g.ready.promise().done(e),this},slice:function(){return this.pushStack(h.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(e){var t=this.length,n=+e+(e<0?t:0);return this.pushStack(n>=0&&n<t?[this[n]]:[])},map:function(e){return this.pushStack(g.map(this,function(t,n){return e.call(t,n,t)}))},end:function(){return this.prevObject||this.constructor(null)},push:c,sort:[].sort,splice:[].splice},g.fn.init.prototype=g.fn,g.extend=g.fn.extend=function(){var e,t,n,r,i,s,o=arguments[0]||{},u=1,a=arguments.length,f=!1;typeof o=="boolean"&&(f=o,o=arguments[1]||{},u=2),typeof o!="object"&&!g.isFunction(o)&&(o={}),a===u&&(o=this,--u);for(;u<a;u++)if((i=arguments[u])!=null)for(r in i){e=o[r],n=i[r];if(o===n)continue;f&&n&&(g.isPlainObject(n)||(t=g.isArray(n)))?(t?(t=!1,s=e&&g.isArray(e)?e:[]):s=e&&g.isPlainObject(e)?e:{},o[r]=g.extend(f,s,n)):n!==undefined&&(o[r]=n)}return o},g.extend({noConflict:function(e){return window.$===g&&(window.$=o),e&&window.jQuery===g&&(window.jQuery=s),g},isReady:!1,readyWait:1,holdReady:function(e){e?g.readyWait++:g.ready(!0)},ready:function(t){if(t===!0?--g.readyWait:g.isReady)return;if(!r.body)return setTimeout(g.ready);g.isReady=!0;if(t!==!0&&--g.readyWait>0)return;e.resolveWith(r,[g]),g.fn.trigger&&g(r).trigger("ready").off("ready")},isFunction:function(e){return g.type(e)==="function"},isArray:Array.isArray||function(e){return g.type(e)==="array"},isWindow:function(e){return e!=null&&e==e.window},isNumeric:function(e){return!isNaN(parseFloat(e))&&isFinite(e)},type:function(e){return e==null?String(e):typeof e=="object"||typeof e=="function"?u[d.call(e)]||"object":typeof e},isPlainObject:function(e){if(!e||g.type(e)!=="object"||e.nodeType||g.isWindow(e))return!1;try{if(e.constructor&&!v.call(e,"constructor")&&!v.call(e.constructor.prototype,"isPrototypeOf"))return!1}catch(t){return!1}var n;for(n in e);return n===undefined||v.call(e,n)},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},error:function(e){throw new Error(e)},parseHTML:function(e,t,n){if(!e||typeof e!="string")return null;typeof t=="boolean"&&(n=t,t=!1),t=t||r;var i=S.exec(e),s=!n&&[];return i?[t.createElement(i[1])]:(i=g.buildFragment([e],t,s),s&&g(s).remove(),g.merge([],i.childNodes))},parseJSON:function(e){if(window.JSON&&window.JSON.parse)return window.JSON.parse(e);if(e===null)return e;if(typeof e=="string"){e=g.trim(e);if(e&&x.test(e.replace(N,"@").replace(C,"]").replace(T,"")))return(new Function("return "+e))()}g.error("Invalid JSON: "+e)},parseXML:function(e){var t,n;if(!e||typeof e!="string")return null;try{window.DOMParser?(n=new DOMParser,t=n.parseFromString(e,"text/xml")):(t=new ActiveXObject("Microsoft.XMLDOM"),t.async="false",t.loadXML(e))}catch(r){t=undefined}return(!t||!t.documentElement||t.getElementsByTagName("parsererror").length)&&g.error("Invalid XML: "+e),t},noop:function(){},globalEval:function(e){e&&g.trim(e)&&(window.execScript||function(e){window.eval.call(window,e)})(e)},camelCase:function(e){return e.replace(k,"ms-").replace(L,A)},nodeName:function(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()},each:function(e,t,n){var r,i=0,s=e.length,o=_(e);if(n)if(o)for(;i<s;i++){r=t.apply(e[i],n);if(r===!1)break}else for(i in e){r=t.apply(e[i],n);if(r===!1)break}else if(o)for(;i<s;i++){r=t.call(e[i],i,e[i]);if(r===!1)break}else for(i in e){r=t.call(e[i],i,e[i]);if(r===!1)break}return e},trim:m&&!m.call("﻿ ")?function(e){return e==null?"":m.call(e)}:function(e){return e==null?"":(e+"").replace(w,"")},makeArray:function(e,t){var n=t||[];return e!=null&&(_(Object(e))?g.merge(n,typeof e=="string"?[e]:e):c.call(n,e)),n},inArray:function(e,t,n){var r;if(t){if(p)return p.call(t,e,n);r=t.length,n=n?n<0?Math.max(0,r+n):n:0;for(;n<r;n++)if(n in t&&t[n]===e)return n}return-1},merge:function(e,t){var n=t.length,r=e.length,i=0;if(typeof n=="number")for(;i<n;i++)e[r++]=t[i];else while(t[i]!==undefined)e[r++]=t[i++];return e.length=r,e},grep:function(e,t,n){var r,i=[],s=0,o=e.length;n=!!n;for(;s<o;s++)r=!!t(e[s],s),n!==r&&i.push(e[s]);return i},map:function(e,t,n){var r,i=0,s=e.length,o=_(e),u=[];if(o)for(;i<s;i++)r=t(e[i],i,n),r!=null&&(u[u.length]=r);else for(i in e)r=t(e[i],i,n),r!=null&&(u[u.length]=r);return l.apply([],u)},guid:1,proxy:function(e,t){var n,r,i;return typeof t=="string"&&(i=e[t],t=e,e=i),g.isFunction(e)?(n=h.call(arguments,2),r=function(){return e.apply(t||this,n.concat(h.call(arguments)))},r.guid=e.guid=e.guid||g.guid++,r):undefined},access:function(e,t,n,r,i,s,o){var u=0,a=e.length,f=n==null;if(g.type(n)==="object"){i=!0;for(u in n)g.access(e,t,u,n[u],!0,s,o)}else if(r!==undefined){i=!0,g.isFunction(r)||(o=!0),f&&(o?(t.call(e,r),t=null):(f=t,t=function(e,t,n){return f.call(g(e),n)}));if(t)for(;u<a;u++)t(e[u],n,o?r:r.call(e[u],u,t(e[u],n)))}return i?e:f?t.call(e):a?t(e[0],n):s},now:function(){return(new Date).getTime()}}),g.ready.promise=function(t){if(!e){e=g.Deferred();if(r.readyState==="complete")setTimeout(g.ready);else if(r.addEventListener)r.addEventListener("DOMContentLoaded",O,!1),window.addEventListener("load",O,!1);else{r.attachEvent("onreadystatechange",O),window.attachEvent("onload",O);var n=!1;try{n=window.frameElement==null&&r.documentElement}catch(i){}n&&n.doScroll&&function s(){if(!g.isReady){try{n.doScroll("left")}catch(e){return setTimeout(s,50)}M(),g.ready()}}()}}return e.promise(t)},g.each("Boolean Number String Function Array Date RegExp Object Error".split(" "),function(e,t){u["[object "+t+"]"]=t.toLowerCase()}),t=g(r);var D={};g.Callbacks=function(e){e=typeof e=="string"?D[e]||P(e):g.extend({},e);var t,n,r,i,s,o,u=[],a=!e.once&&[],f=function(c){n=e.memory&&c,r=!0,s=o||0,o=0,i=u.length,t=!0;for(;u&&s<i;s++)if(u[s].apply(c[0],c[1])===!1&&e.stopOnFalse){n=!1;break}t=!1,u&&(a?a.length&&f(a.shift()):n?u=[]:l.disable())},l={add:function(){if(u){var r=u.length;(function s(t){g.each(t,function(t,n){var r=g.type(n);r==="function"?(!e.unique||!l.has(n))&&u.push(n):n&&n.length&&r!=="string"&&s(n)})})(arguments),t?i=u.length:n&&(o=r,f(n))}return this},remove:function(){return u&&g.each(arguments,function(e,n){var r;while((r=g.inArray(n,u,r))>-1)u.splice(r,1),t&&(r<=i&&i--,r<=s&&s--)}),this},has:function(e){return e?g.inArray(e,u)>-1:!!u&&!!u.length},empty:function(){return u=[],this},disable:function(){return u=a=n=undefined,this},disabled:function(){return!u},lock:function(){return a=undefined,n||l.disable(),this},locked:function(){return!a},fireWith:function(e,n){return n=n||[],n=[e,n.slice?n.slice():n],u&&(!r||a)&&(t?a.push(n):f(n)),this},fire:function(){return l.fireWith(this,arguments),this},fired:function(){return!!r}};return l},g.extend({Deferred:function(e){var t=[["resolve","done",g.Callbacks("once memory"),"resolved"],["reject","fail",g.Callbacks("once memory"),"rejected"],["notify","progress",g.Callbacks("memory")]],n="pending",r={state:function(){return n},always:function(){return i.done(arguments).fail(arguments),this},then:function(){var e=arguments;return g.Deferred(function(n){g.each(t,function(t,s){var o=s[0],u=g.isFunction(e[t])&&e[t];i[s[1]](function(){var e=u&&u.apply(this,arguments);e&&g.isFunction(e.promise)?e.promise().done(n.resolve).fail(n.reject).progress(n.notify):n[o+"With"](this===r?n.promise():this,u?[e]:arguments)})}),e=null}).promise()},promise:function(e){return e!=null?g.extend(e,r):r}},i={};return r.pipe=r.then,g.each(t,function(e,s){var o=s[2],u=s[3];r[s[1]]=o.add,u&&o.add(function(){n=u},t[e^1][2].disable,t[2][2].lock),i[s[0]]=function(){return i[s[0]+"With"](this===i?r:this,arguments),this},i[s[0]+"With"]=o.fireWith}),r.promise(i),e&&e.call(i,i),i},when:function(e){var t=0,n=h.call(arguments),r=n.length,i=r!==1||e&&g.isFunction(e.promise)?r:0,s=i===1?e:g.Deferred(),o=function(e,t,n){return function(r){t[e]=this,n[e]=arguments.length>1?h.call(arguments):r,n===u?s.notifyWith(t,n):--i||s.resolveWith(t,n)}},u,a,f;if(r>1){u=new Array(r),a=new Array(r),f=new Array(r);for(;t<r;t++)n[t]&&g.isFunction(n[t].promise)?n[t].promise().done(o(t,f,n)).fail(s.reject).progress(o(t,a,u)):--i}return i||s.resolveWith(f,n),s.promise()}}),g.support=function(){var e,t,i,s,o,u,a,f,l,c,h=r.createElement("div");h.setAttribute("className","t"),h.innerHTML="  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>",t=h.getElementsByTagName("*"),i=h.getElementsByTagName("a")[0];if(!t||!i||!t.length)return{};o=r.createElement("select"),a=o.appendChild(r.createElement("option")),s=h.getElementsByTagName("input")[0],i.style.cssText="top:1px;float:left;opacity:.5",e={getSetAttribute:h.className!=="t",leadingWhitespace:h.firstChild.nodeType===3,tbody:!h.getElementsByTagName("tbody").length,htmlSerialize:!!h.getElementsByTagName("link").length,style:/top/.test(i.getAttribute("style")),hrefNormalized:i.getAttribute("href")==="/a",opacity:/^0.5/.test(i.style.opacity),cssFloat:!!i.style.cssFloat,checkOn:!!s.value,optSelected:a.selected,enctype:!!r.createElement("form").enctype,html5Clone:r.createElement("nav").cloneNode(!0).outerHTML!=="<:nav></:nav>",boxModel:r.compatMode==="CSS1Compat",deleteExpando:!0,noCloneEvent:!0,inlineBlockNeedsLayout:!1,shrinkWrapBlocks:!1,reliableMarginRight:!0,boxSizingReliable:!0,pixelPosition:!1},s.checked=!0,e.noCloneChecked=s.cloneNode(!0).checked,o.disabled=!0,e.optDisabled=!a.disabled;try{delete h.test}catch(p){e.deleteExpando=!1}s=r.createElement("input"),s.setAttribute("value",""),e.input=s.getAttribute("value")==="",s.value="t",s.setAttribute("type","radio"),e.radioValue=s.value==="t",s.setAttribute("checked","t"),s.setAttribute("name","t"),u=r.createDocumentFragment(),u.appendChild(s),e.appendChecked=s.checked,e.checkClone=u.cloneNode(!0).cloneNode(!0).lastChild.checked,h.attachEvent&&(h.attachEvent("onclick",function(){e.noCloneEvent=!1}),h.cloneNode(!0).click());for(c in{submit:!0,change:!0,focusin:!0})h.setAttribute(f="on"+c,"t"),e[c+"Bubbles"]=f in window||h.attributes[f].expando===!1;return h.style.backgroundClip="content-box",h.cloneNode(!0).style.backgroundClip="",e.clearCloneStyle=h.style.backgroundClip==="content-box",g(function(){var t,i,s,o="padding:0;margin:0;border:0;display:block;box-sizing:content-box;-moz-box-sizing:content-box;-webkit-box-sizing:content-box;",u=r.getElementsByTagName("body")[0];if(!u)return;t=r.createElement("div"),t.style.cssText="border:0;width:0;height:0;position:absolute;top:0;left:-9999px;margin-top:1px",u.appendChild(t).appendChild(h),h.innerHTML="<table><tr><td></td><td>t</td></tr></table>",s=h.getElementsByTagName("td"),s[0].style.cssText="padding:0;margin:0;border:0;display:none",l=s[0].offsetHeight===0,s[0].style.display="",s[1].style.display="none",e.reliableHiddenOffsets=l&&s[0].offsetHeight===0,h.innerHTML="",h.style.cssText="box-sizing:border-box;-moz-box-sizing:border-box;-webkit-box-sizing:border-box;padding:1px;border:1px;display:block;width:4px;margin-top:1%;position:absolute;top:1%;",e.boxSizing=h.offsetWidth===4,e.doesNotIncludeMarginInBodyOffset=u.offsetTop!==1,window.getComputedStyle&&(e.pixelPosition=(window.getComputedStyle(h,null)||{}).top!=="1%",e.boxSizingReliable=(window.getComputedStyle(h,null)||{width:"4px"}).width==="4px",i=h.appendChild(r.createElement("div")),i.style.cssText=h.style.cssText=o,i.style.marginRight=i.style.width="0",h.style.width="1px",e.reliableMarginRight=!parseFloat((window.getComputedStyle(i,null)||{}).marginRight)),typeof h.style.zoom!==n&&(h.innerHTML="",h.style.cssText=o+"width:1px;padding:1px;display:inline;zoom:1",e.inlineBlockNeedsLayout=h.offsetWidth===3,h.style.display="block",h.innerHTML="<div></div>",h.firstChild.style.width="5px",e.shrinkWrapBlocks=h.offsetWidth!==3,e.inlineBlockNeedsLayout&&(u.style.zoom=1)),u.removeChild(t),t=h=s=i=null}),t=o=u=a=i=s=null,e}();var H=/(?:\{[\s\S]*\}|\[[\s\S]*\])$/,B=/([A-Z])/g;g.extend({cache:{},expando:"jQuery"+(f+Math.random()).replace(/\D/g,""),noData:{embed:!0,object:"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000",applet:!0},hasData:function(e){return e=e.nodeType?g.cache[e[g.expando]]:e[g.expando],!!e&&!q(e)},data:function(e,t,n){return j(e,t,n)},removeData:function(e,t){return F(e,t)},_data:function(e,t,n){return j(e,t,n,!0)},_removeData:function(e,t){return F(e,t,!0)},acceptData:function(e){if(e.nodeType&&e.nodeType!==1&&e.nodeType!==9)return!1;var t=e.nodeName&&g.noData[e.nodeName.toLowerCase()];return!t||t!==!0&&e.getAttribute("classid")===t}}),g.fn.extend({data:function(e,t){var n,r,i=this[0],s=0,o=null;if(e===undefined){if(this.length){o=g.data(i);if(i.nodeType===1&&!g._data(i,"parsedAttrs")){n=i.attributes;for(;s<n.length;s++)r=n[s].name,r.indexOf("data-")||(r=g.camelCase(r.slice(5)),I(i,r,o[r]));g._data(i,"parsedAttrs",!0)}}return o}return typeof e=="object"?this.each(function(){g.data(this,e)}):g.access(this,function(t){if(t===undefined)return i?I(i,e,g.data(i,e)):null;this.each(function(){g.data(this,e,t)})},null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each(function(){g.removeData(this,e)})}}),g.extend({queue:function(e,t,n){var r;if(e)return t=(t||"fx")+"queue",r=g._data(e,t),n&&(!r||g.isArray(n)?r=g._data(e,t,g.makeArray(n)):r.push(n)),r||[]},dequeue:function(e,t){t=t||"fx";var n=g.queue(e,t),r=n.length,i=n.shift(),s=g._queueHooks(e,t),o=function(){g.dequeue(e,t)};i==="inprogress"&&(i=n.shift(),r--),s.cur=i,i&&(t==="fx"&&n.unshift("inprogress"),delete s.stop,i.call(e,o,s)),!r&&s&&s.empty.fire()},_queueHooks:function(e,t){var n=t+"queueHooks";return g._data(e,n)||g._data(e,n,{empty:g.Callbacks("once memory").add(function(){g._removeData(e,t+"queue"),g._removeData(e,n)})})}}),g.fn.extend({queue:function(e,t){var n=2;return typeof e!="string"&&(t=e,e="fx",n--),arguments.length<n?g.queue(this[0],e):t===undefined?this:this.each(function(){var n=g.queue(this,e,t);g._queueHooks(this,e),e==="fx"&&n[0]!=="inprogress"&&g.dequeue(this,e)})},dequeue:function(e){return this.each(function(){g.dequeue(this,e)})},delay:function(e,t){return e=g.fx?g.fx.speeds[e]||e:e,t=t||"fx",this.queue(t,function(t,n){var r=setTimeout(t,e);n.stop=function(){clearTimeout(r)}})},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var n,r=1,i=g.Deferred(),s=this,o=this.length,u=function(){--r||i.resolveWith(s,[s])};typeof e!="string"&&(t=e,e=undefined),e=e||"fx";while(o--)n=g._data(s[o],e+"queueHooks"),n&&n.empty&&(r++,n.empty.add(u));return u(),i.promise(t)}});var R,U,z=/[\t\r\n]/g,W=/\r/g,X=/^(?:input|select|textarea|button|object)$/i,V=/^(?:a|area)$/i,$=/^(?:checked|selected|autofocus|autoplay|async|controls|defer|disabled|hidden|loop|multiple|open|readonly|required|scoped)$/i,J=/^(?:checked|selected)$/i,K=g.support.getSetAttribute,Q=g.support.input;g.fn.extend({attr:function(e,t){return g.access(this,g.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each(function(){g.removeAttr(this,e)})},prop:function(e,t){return g.access(this,g.prop,e,t,arguments.length>1)},removeProp:function(e){return e=g.propFix[e]||e,this.each(function(){try{this[e]=undefined,delete this[e]}catch(t){}})},addClass:function(e){var t,n,r,i,s,o=0,u=this.length,a=typeof e=="string"&&e;if(g.isFunction(e))return this.each(function(t){g(this).addClass(e.call(this,t,this.className))});if(a){t=(e||"").match(b)||[];for(;o<u;o++){n=this[o],r=n.nodeType===1&&(n.className?(" "+n.className+" ").replace(z," "):" ");if(r){s=0;while(i=t[s++])r.indexOf(" "+i+" ")<0&&(r+=i+" ");n.className=g.trim(r)}}}return this},removeClass:function(e){var t,n,r,i,s,o=0,u=this.length,a=arguments.length===0||typeof e=="string"&&e;if(g.isFunction(e))return this.each(function(t){g(this).removeClass(e.call(this,t,this.className))});if(a){t=(e||"").match(b)||[];for(;o<u;o++){n=this[o],r=n.nodeType===1&&(n.className?(" "+n.className+" ").replace(z," "):"");if(r){s=0;while(i=t[s++])while(r.indexOf(" "+i+" ")>=0)r=r.replace(" "+i+" "," ");n.className=e?g.trim(r):""}}}return this},toggleClass:function(e,t){var r=typeof e,i=typeof t=="boolean";return g.isFunction(e)?this.each(function(n){g(this).toggleClass(e.call(this,n,this.className,t),t)}):this.each(function(){if(r==="string"){var s,o=0,u=g(this),a=t,f=e.match(b)||[];while(s=f[o++])a=i?a:!u.hasClass(s),u[a?"addClass":"removeClass"](s)}else if(r===n||r==="boolean")this.className&&g._data(this,"__className__",this.className),this.className=this.className||e===!1?"":g._data(this,"__className__")||""})},hasClass:function(e){var t=" "+e+" ",n=0,r=this.length;for(;n<r;n++)if(this[n].nodeType===1&&(" "+this[n].className+" ").replace(z," ").indexOf(t)>=0)return!0;return!1},val:function(e){var t,n,r,i=this[0];if(!arguments.length){if(i)return n=g.valHooks[i.type]||g.valHooks[i.nodeName.toLowerCase()],n&&"get"in n&&(t=n.get(i,"value"))!==undefined?t:(t=i.value,typeof t=="string"?t.replace(W,""):t==null?"":t);return}return r=g.isFunction(e),this.each(function(t){var i,s=g(this);if(this.nodeType!==1)return;r?i=e.call(this,t,s.val()):i=e,i==null?i="":typeof i=="number"?i+="":g.isArray(i)&&(i=g.map(i,function(e){return e==null?"":e+""})),n=g.valHooks[this.type]||g.valHooks[this.nodeName.toLowerCase()];if(!n||!("set"in n)||n.set(this,i,"value")===undefined)this.value=i})}}),g.extend({valHooks:{option:{get:function(e){var t=e.attributes.value;return!t||t.specified?e.value:e.text}},select:{get:function(e){var t,n,r=e.options,i=e.selectedIndex,s=e.type==="select-one"||i<0,o=s?null:[],u=s?i+1:r.length,a=i<0?u:s?i:0;for(;a<u;a++){n=r[a];if((n.selected||a===i)&&(g.support.optDisabled?!n.disabled:n.getAttribute("disabled")===null)&&(!n.parentNode.disabled||!g.nodeName(n.parentNode,"optgroup"))){t=g(n).val();if(s)return t;o.push(t)}}return o},set:function(e,t){var n=g.makeArray(t);return g(e).find("option").each(function(){this.selected=g.inArray(g(this).val(),n)>=0}),n.length||(e.selectedIndex=-1),n}}},attr:function(e,t,r){var i,s,o,u=e.nodeType;if(!e||u===3||u===8||u===2)return;if(typeof e.getAttribute===n)return g.prop(e,t,r);s=u!==1||!g.isXMLDoc(e),s&&(t=t.toLowerCase(),i=g.attrHooks[t]||($.test(t)?U:R));if(r===undefined)return i&&s&&"get"in i&&(o=i.get(e,t))!==null?o:(typeof e.getAttribute!==n&&(o=e.getAttribute(t)),o==null?undefined:o);if(r!==null)return i&&s&&"set"in i&&(o=i.set(e,r,t))!==undefined?o:(e.setAttribute(t,r+""),r);g.removeAttr(e,t)},removeAttr:function(e,t){var n,r,i=0,s=t&&t.match(b);if(s&&e.nodeType===1)while(n=s[i++])r=g.propFix[n]||n,$.test(n)?!K&&J.test(n)?e[g.camelCase("default-"+n)]=e[r]=!1:e[r]=!1:g.attr(e,n,""),e.removeAttribute(K?n:r)},attrHooks:{type:{set:function(e,t){if(!g.support.radioValue&&t==="radio"&&g.nodeName(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},propFix:{tabindex:"tabIndex",readonly:"readOnly","for":"htmlFor","class":"className",maxlength:"maxLength",cellspacing:"cellSpacing",cellpadding:"cellPadding",rowspan:"rowSpan",colspan:"colSpan",usemap:"useMap",frameborder:"frameBorder",contenteditable:"contentEditable"},prop:function(e,t,n){var r,i,s,o=e.nodeType;if(!e||o===3||o===8||o===2)return;return s=o!==1||!g.isXMLDoc(e),s&&(t=g.propFix[t]||t,i=g.propHooks[t]),n!==undefined?i&&"set"in i&&(r=i.set(e,n,t))!==undefined?r:e[t]=n:i&&"get"in i&&(r=i.get(e,t))!==null?r:e[t]},propHooks:{tabIndex:{get:function(e){var t=e.getAttributeNode("tabindex");return t&&t.specified?parseInt(t.value,10):X.test(e.nodeName)||V.test(e.nodeName)&&e.href?0:undefined}}}}),U={get:function(e,t){var n=g.prop(e,t),r=typeof n=="boolean"&&e.getAttribute(t),i=typeof n=="boolean"?Q&&K?r!=null:J.test(t)?e[g.camelCase("default-"+t)]:!!r:e.getAttributeNode(t);return i&&i.value!==!1?t.toLowerCase():undefined},set:function(e,t,n){return t===!1?g.removeAttr(e,n):Q&&K||!J.test(n)?e.setAttribute(!K&&g.propFix[n]||n,n):e[g.camelCase("default-"+n)]=e[n]=!0,n}};if(!Q||!K)g.attrHooks.value={get:function(e,t){var n=e.getAttributeNode(t);return g.nodeName(e,"input")?e.defaultValue:n&&n.specified?n.value:undefined},set:function(e,t,n){if(!g.nodeName(e,"input"))return R&&R.set(e,t,n);e.defaultValue=t}};K||(R=g.valHooks.button={get:function(e,t){var n=e.getAttributeNode(t);return n&&(t==="id"||t==="name"||t==="coords"?n.value!=="":n.specified)?n.value:undefined},set:function(e,t,n){var r=e.getAttributeNode(n);return r||e.setAttributeNode(r=e.ownerDocument.createAttribute(n)),r.value=t+="",n==="value"||t===e.getAttribute(n)?t:undefined}},g.attrHooks.contenteditable={get:R.get,set:function(e,t,n){R.set(e,t===""?!1:t,n)}},g.each(["width","height"],function(e,t){g.attrHooks[t]=g.extend(g.attrHooks[t],{set:function(e,n){if(n==="")return e.setAttribute(t,"auto"),n}})})),g.support.hrefNormalized||(g.each(["href","src","width","height"],function(e,t){g.attrHooks[t]=g.extend(g.attrHooks[t],{get:function(e){var n=e.getAttribute(t,2);return n==null?undefined:n}})}),g.each(["href","src"],function(e,t){g.propHooks[t]={get:function(e){return e.getAttribute(t,4)}}})),g.support.style||(g.attrHooks.style={get:function(e){return e.style.cssText||undefined},set:function(e,t){return e.style.cssText=t+""}}),g.support.optSelected||(g.propHooks.selected=g.extend(g.propHooks.selected,{get:function(e){var t=e.parentNode;return t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex),null}})),g.support.enctype||(g.propFix.enctype="encoding"),g.support.checkOn||g.each(["radio","checkbox"],function(){g.valHooks[this]={get:function(e){return e.getAttribute("value")===null?"on":e.value}}}),g.each(["radio","checkbox"],function(){g.valHooks[this]=g.extend(g.valHooks[this],{set:function(e,t){if(g.isArray(t))return e.checked=g.inArray(g(e).val(),t)>=0}})});var G=/^(?:input|select|textarea)$/i,Y=/^key/,Z=/^(?:mouse|contextmenu)|click/,et=/^(?:focusinfocus|focusoutblur)$/,tt=/^([^.]*)(?:\.(.+)|)$/;g.event={global:{},add:function(e,t,r,i,s){var o,u,a,f,l,c,h,p,d,v,m,y=g._data(e);if(!y)return;r.handler&&(f=r,r=f.handler,s=f.selector),r.guid||(r.guid=g.guid++),(u=y.events)||(u=y.events={}),(c=y.handle)||(c=y.handle=function(e){return typeof g===n||!!e&&g.event.triggered===e.type?undefined:g.event.dispatch.apply(c.elem,arguments)},c.elem=e),t=(t||"").match(b)||[""],a=t.length;while(a--){o=tt.exec(t[a])||[],d=m=o[1],v=(o[2]||"").split(".").sort(),l=g.event.special[d]||{},d=(s?l.delegateType:l.bindType)||d,l=g.event.special[d]||{},h=g.extend({type:d,origType:m,data:i,handler:r,guid:r.guid,selector:s,needsContext:s&&g.expr.match.needsContext.test(s),namespace:v.join(".")},f);if(!(p=u[d])){p=u[d]=[],p.delegateCount=0;if(!l.setup||l.setup.call(e,i,v,c)===!1)e.addEventListener?e.addEventListener(d,c,!1):e.attachEvent&&e.attachEvent("on"+d,c)}l.add&&(l.add.call(e,h),h.handler.guid||(h.handler.guid=r.guid)),s?p.splice(p.delegateCount++,0,h):p.push(h),g.event.global[d]=!0}e=null},remove:function(e,t,n,r,i){var s,o,u,a,f,l,c,h,p,d,v,m=g.hasData(e)&&g._data(e);if(!m||!(l=m.events))return;t=(t||"").match(b)||[""],f=t.length;while(f--){u=tt.exec(t[f])||[],p=v=u[1],d=(u[2]||"").split(".").sort();if(!p){for(p in l)g.event.remove(e,p+t[f],n,r,!0);continue}c=g.event.special[p]||{},p=(r?c.delegateType:c.bindType)||p,h=l[p]||[],u=u[2]&&new RegExp("(^|\\.)"+d.join("\\.(?:.*\\.|)")+"(\\.|$)"),a=s=h.length;while(s--)o=h[s],(i||v===o.origType)&&(!n||n.guid===o.guid)&&(!u||u.test(o.namespace))&&(!r||r===o.selector||r==="**"&&o.selector)&&(h.splice(s,1),o.selector&&h.delegateCount--,c.remove&&c.remove.call(e,o));a&&!h.length&&((!c.teardown||c.teardown.call(e,d,m.handle)===!1)&&g.removeEvent(e,p,m.handle),delete l[p])}g.isEmptyObject(l)&&(delete m.handle,g._removeData(e,"events"))},trigger:function(e,t,n,i){var s,o,u,a,f,l,c,h=[n||r],p=v.call(e,"type")?e.type:e,d=v.call(e,"namespace")?e.namespace.split("."):[];u=l=n=n||r;if(n.nodeType===3||n.nodeType===8)return;if(et.test(p+g.event.triggered))return;p.indexOf(".")>=0&&(d=p.split("."),p=d.shift(),d.sort()),o=p.indexOf(":")<0&&"on"+p,e=e[g.expando]?e:new g.Event(p,typeof e=="object"&&e),e.isTrigger=!0,e.namespace=d.join("."),e.namespace_re=e.namespace?new RegExp("(^|\\.)"+d.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=undefined,e.target||(e.target=n),t=t==null?[e]:g.makeArray(t,[e]),f=g.event.special[p]||{};if(!i&&f.trigger&&f.trigger.apply(n,t)===!1)return;if(!i&&!f.noBubble&&!g.isWindow(n)){a=f.delegateType||p,et.test(a+p)||(u=u.parentNode);for(;u;u=u.parentNode)h.push(u),l=u;l===(n.ownerDocument||r)&&h.push(l.defaultView||l.parentWindow||window)}c=0;while((u=h[c++])&&!e.isPropagationStopped())e.type=c>1?a:f.bindType||p,s=(g._data(u,"events")||{})[e.type]&&g._data(u,"handle"),s&&s.apply(u,t),s=o&&u[o],s&&g.acceptData(u)&&s.apply&&s.apply(u,t)===!1&&e.preventDefault();e.type=p;if(!i&&!e.isDefaultPrevented()&&(!f._default||f._default.apply(n.ownerDocument,t)===!1)&&(p!=="click"||!g.nodeName(n,"a"))&&g.acceptData(n)&&o&&n[p]&&!g.isWindow(n)){l=n[o],l&&(n[o]=null),g.event.triggered=p;try{n[p]()}catch(m){}g.event.triggered=undefined,l&&(n[o]=l)}return e.result},dispatch:function(e){e=g.event.fix(e);var t,n,r,i,s,o=[],u=h.call(arguments),a=(g._data(this,"events")||{})[e.type]||[],f=g.event.special[e.type]||{};u[0]=e,e.delegateTarget=this;if(f.preDispatch&&f.preDispatch.call(this,e)===!1)return;o=g.event.handlers.call(this,e,a),t=0;while((i=o[t++])&&!e.isPropagationStopped()){e.currentTarget=i.elem,s=0;while((r=i.handlers[s++])&&!e.isImmediatePropagationStopped())if(!e.namespace_re||e.namespace_re.test(r.namespace))e.handleObj=r,e.data=r.data,n=((g.event.special[r.origType]||{}).handle||r.handler).apply(i.elem,u),n!==undefined&&(e.result=n)===!1&&(e.preventDefault(),e.stopPropagation())}return f.postDispatch&&f.postDispatch.call(this,e),e.result},handlers:function(e,t){var n,r,i,s,o=[],u=t.delegateCount,a=e.target;if(u&&a.nodeType&&(!e.button||e.type!=="click"))for(;a!=this;a=a.parentNode||this)if(a.nodeType===1&&(a.disabled!==!0||e.type!=="click")){i=[];for(s=0;s<u;s++)r=t[s],n=r.selector+" ",i[n]===undefined&&(i[n]=r.needsContext?g(n,this).index(a)>=0:g.find(n,this,null,[a]).length),i[n]&&i.push(r);i.length&&o.push({elem:a,handlers:i})}return u<t.length&&o.push({elem:this,handlers:t.slice(u)}),o},fix:function(e){if(e[g.expando])return e;var t,n,i,s=e.type,o=e,u=this.fixHooks[s];u||(this.fixHooks[s]=u=Z.test(s)?this.mouseHooks:Y.test(s)?this.keyHooks:{}),i=u.props?this.props.concat(u.props):this.props,e=new g.Event(o),t=i.length;while(t--)n=i[t],e[n]=o[n];return e.target||(e.target=o.srcElement||r),e.target.nodeType===3&&(e.target=e.target.parentNode),e.metaKey=!!e.metaKey,u.filter?u.filter(e,o):e},props:"altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),fixHooks:{},keyHooks:{props:"char charCode key keyCode".split(" "),filter:function(e,t){return e.which==null&&(e.which=t.charCode!=null?t.charCode:t.keyCode),e}},mouseHooks:{props:"button buttons clientX clientY fromElement offsetX offsetY pageX pageY screenX screenY toElement".split(" "),filter:function(e,t){var n,i,s,o=t.button,u=t.fromElement;return e.pageX==null&&t.clientX!=null&&(i=e.target.ownerDocument||r,s=i.documentElement,n=i.body,e.pageX=t.clientX+(s&&s.scrollLeft||n&&n.scrollLeft||0)-(s&&s.clientLeft||n&&n.clientLeft||0),e.pageY=t.clientY+(s&&s.scrollTop||n&&n.scrollTop||0)-(s&&s.clientTop||n&&n.clientTop||0)),!e.relatedTarget&&u&&(e.relatedTarget=u===e.target?t.toElement:u),!e.which&&o!==undefined&&(e.which=o&1?1:o&2?3:o&4?2:0),e}},special:{load:{noBubble:!0},click:{trigger:function(){if(g.nodeName(this,"input")&&this.type==="checkbox"&&this.click)return this.click(),!1}},focus:{trigger:function(){if(this!==r.activeElement&&this.focus)try{return this.focus(),!1}catch(e){}},delegateType:"focusin"},blur:{trigger:function(){if(this===r.activeElement&&this.blur)return this.blur(),!1},delegateType:"focusout"},beforeunload:{postDispatch:function(e){e.result!==undefined&&(e.originalEvent.returnValue=e.result)}}},simulate:function(e,t,n,r){var i=g.extend(new g.Event,n,{type:e,isSimulated:!0,originalEvent:{}});r?g.event.trigger(i,null,t):g.event.dispatch.call(t,i),i.isDefaultPrevented()&&n.preventDefault()}},g.removeEvent=r.removeEventListener?function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n,!1)}:function(e,t,r){var i="on"+t;e.detachEvent&&(typeof e[i]===n&&(e[i]=null),e.detachEvent(i,r))},g.Event=function(e,t){if(!(this instanceof g.Event))return new g.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||e.returnValue===!1||e.getPreventDefault&&e.getPreventDefault()?nt:rt):this.type=e,t&&g.extend(this,t),this.timeStamp=e&&e.timeStamp||g.now(),this[g.expando]=!0},g.Event.prototype={isDefaultPrevented:rt,isPropagationStopped:rt,isImmediatePropagationStopped:rt,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=nt;if(!e)return;e.preventDefault?e.preventDefault():e.returnValue=!1},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=nt;if(!e)return;e.stopPropagation&&e.stopPropagation(),e.cancelBubble=!0},stopImmediatePropagation:function(){this.isImmediatePropagationStopped=nt,this.stopPropagation()}},g.each({mouseenter:"mouseover",mouseleave:"mouseout"},function(e,t){g.event.special[e]={delegateType:t,bindType:t,handle:function(e){var n,r=this,i=e.relatedTarget,s=e.handleObj;if(!i||i!==r&&!g.contains(r,i))e.type=s.origType,n=s.handler.apply(this,arguments),e.type=t;return n}}}),g.support.submitBubbles||(g.event.special.submit={setup:function(){if(g.nodeName(this,"form"))return!1;g.event.add(this,"click._submit keypress._submit",function(e){var t=e.target,n=g.nodeName(t,"input")||g.nodeName(t,"button")?t.form:undefined;n&&!g._data(n,"submitBubbles")&&(g.event.add(n,"submit._submit",function(e){e._submit_bubble=!0}),g._data(n,"submitBubbles",!0))})},postDispatch:function(e){e._submit_bubble&&(delete e._submit_bubble,this.parentNode&&!e.isTrigger&&g.event.simulate("submit",this.parentNode,e,!0))},teardown:function(){if(g.nodeName(this,"form"))return!1;g.event.remove(this,"._submit")}}),g.support.changeBubbles||(g.event.special.change={setup:function(){if(G.test(this.nodeName)){if(this.type==="checkbox"||this.type==="radio")g.event.add(this,"propertychange._change",function(e){e.originalEvent.propertyName==="checked"&&(this._just_changed=!0)}),g.event.add(this,"click._change",function(e){this._just_changed&&!e.isTrigger&&(this._just_changed=!1),g.event.simulate("change",this,e,!0)});return!1}g.event.add(this,"beforeactivate._change",function(e){var t=e.target;G.test(t.nodeName)&&!g._data(t,"changeBubbles")&&(g.event.add(t,"change._change",function(e){this.parentNode&&!e.isSimulated&&!e.isTrigger&&g.event.simulate("change",this.parentNode,e,!0)}),g._data(t,"changeBubbles",!0))})},handle:function(e){var t=e.target;if(this!==t||e.isSimulated||e.isTrigger||t.type!=="radio"&&t.type!=="checkbox")return e.handleObj.handler.apply(this,arguments)},teardown:function(){return g.event.remove(this,"._change"),!G.test(this.nodeName)}}),g.support.focusinBubbles||g.each({focus:"focusin",blur:"focusout"},function(e,t){var n=0,i=function(e){g.event.simulate(t,e.target,g.event.fix(e),!0)};g.event.special[t]={setup:function(){n++===0&&r.addEventListener(e,i,!0)},teardown:function(){--n===0&&r.removeEventListener(e,i,!0)}}}),g.fn.extend({on:function(e,t,n,r,i){var s,o;if(typeof e=="object"){typeof t!="string"&&(n=n||t,t=undefined);for(s in e)this.on(s,t,n,e[s],i);return this}n==null&&r==null?(r=t,n=t=undefined):r==null&&(typeof t=="string"?(r=n,n=undefined):(r=n,n=t,t=undefined));if(r===!1)r=rt;else if(!r)return this;return i===1&&(o=r,r=function(e){return g().off(e),o.apply(this,arguments)},r.guid=o.guid||(o.guid=g.guid++)),this.each(function(){g.event.add(this,e,r,n,t)})},one:function(e,t,n,r){return this.on(e,t,n,r,1)},off:function(e,t,n){var r,i;if(e&&e.preventDefault&&e.handleObj)return r=e.handleObj,g(e.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if(typeof e=="object"){for(i in e)this.off(i,t,e[i]);return this}if(t===!1||typeof t=="function")n=t,t=undefined;return n===!1&&(n=rt),this.each(function(){g.event.remove(this,e,n,t)})},bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,r){return this.on(t,e,n,r)},undelegate:function(e,t,n){return arguments.length===1?this.off(e,"**"):this.off(t,e||"**",n)},trigger:function(e,t){return this.each(function(){g.event.trigger(e,t,this)})},triggerHandler:function(e,t){var n=this[0];if(n)return g.event.trigger(e,t,n,!0)}}),function(e,t){function rt(e){return J.test(e+"")}function it(){var e,t=[];return e=function(n,r){return t.push(n+=" ")>i.cacheLength&&delete e[t.shift()],e[n]=r}}function st(e){return e[w]=!0,e}function ot(e){var t=c.createElement("div");try{return e(t)}catch(n){return!1}finally{t=null}}function ut(e,t,n,r){var i,s,o,u,a,f,h,v,m,g;(t?t.ownerDocument||t:E)!==c&&l(t),t=t||c,n=n||[];if(!e||typeof e!="string")return n;if((u=t.nodeType)!==1&&u!==9)return[];if(!p&&!r){if(i=K.exec(e))if(o=i[1]){if(u===9){s=t.getElementById(o);if(!s||!s.parentNode)return n;if(s.id===o)return n.push(s),n}else if(t.ownerDocument&&(s=t.ownerDocument.getElementById(o))&&y(t,s)&&s.id===o)return n.push(s),n}else{if(i[2])return _.apply(n,D.call(t.getElementsByTagName(e),0)),n;if((o=i[3])&&S.getByClassName&&t.getElementsByClassName)return _.apply(n,D.call(t.getElementsByClassName(o),0)),n}if(S.qsa&&!d.test(e)){h=!0,v=w,m=t,g=u===9&&e;if(u===1&&t.nodeName.toLowerCase()!=="object"){f=ht(e),(h=t.getAttribute("id"))?v=h.replace(Y,"\\$&"):t.setAttribute("id",v),v="[id='"+v+"'] ",a=f.length;while(a--)f[a]=v+pt(f[a]);m=$.test(e)&&t.parentNode||t,g=f.join(",")}if(g)try{return _.apply(n,D.call(m.querySelectorAll(g),0)),n}catch(b){}finally{h||t.removeAttribute("id")}}}return Et(e.replace(R,"$1"),t,n,r)}function at(e,t){var n=t&&e,r=n&&(~t.sourceIndex||A)-(~e.sourceIndex||A);if(r)return r;if(n)while(n=n.nextSibling)if(n===t)return-1;return e?1:-1}function ft(e){return function(t){var n=t.nodeName.toLowerCase();return n==="input"&&t.type===e}}function lt(e){return function(t){var n=t.nodeName.toLowerCase();return(n==="input"||n==="button")&&t.type===e}}function ct(e){return st(function(t){return t=+t,st(function(n,r){var i,s=e([],n.length,t),o=s.length;while(o--)n[i=s[o]]&&(n[i]=!(r[i]=n[i]))})})}function ht(e,t){var n,r,s,o,u,a,f,l=C[e+" "];if(l)return t?0:l.slice(0);u=e,a=[],f=i.preFilter;while(u){if(!n||(r=U.exec(u)))r&&(u=u.slice(r[0].length)||u),a.push(s=[]);n=!1;if(r=z.exec(u))n=r.shift(),s.push({value:n,type:r[0].replace(R," ")}),u=u.slice(n.length);for(o in i.filter)(r=V[o].exec(u))&&(!f[o]||(r=f[o](r)))&&(n=r.shift(),s.push({value:n,type:o,matches:r}),u=u.slice(n.length));if(!n)break}return t?u.length:u?ut.error(e):C(e,a).slice(0)}function pt(e){var t=0,n=e.length,r="";for(;t<n;t++)r+=e[t].value;return r}function dt(e,t,n){var i=t.dir,s=n&&i==="parentNode",o=T++;return t.first?function(t,n,r){while(t=t[i])if(t.nodeType===1||s)return e(t,n,r)}:function(t,n,u){var a,f,l,c=x+" "+o;if(u){while(t=t[i])if(t.nodeType===1||s)if(e(t,n,u))return!0}else while(t=t[i])if(t.nodeType===1||s){l=t[w]||(t[w]={});if((f=l[i])&&f[0]===c){if((a=f[1])===!0||a===r)return a===!0}else{f=l[i]=[c],f[1]=e(t,n,u)||r;if(f[1]===!0)return!0}}}}function vt(e){return e.length>1?function(t,n,r){var i=e.length;while(i--)if(!e[i](t,n,r))return!1;return!0}:e[0]}function mt(e,t,n,r,i){var s,o=[],u=0,a=e.length,f=t!=null;for(;u<a;u++)if(s=e[u])if(!n||n(s,r,i))o.push(s),f&&t.push(u);return o}function gt(e,t,n,r,i,s){return r&&!r[w]&&(r=gt(r)),i&&!i[w]&&(i=gt(i,s)),st(function(s,o,u,a){var f,l,c,h=[],p=[],d=o.length,v=s||wt(t||"*",u.nodeType?[u]:u,[]),m=e&&(s||!t)?mt(v,h,e,u,a):v,g=n?i||(s?e:d||r)?[]:o:m;n&&n(m,g,u,a);if(r){f=mt(g,p),r(f,[],u,a),l=f.length;while(l--)if(c=f[l])g[p[l]]=!(m[p[l]]=c)}if(s){if(i||e){if(i){f=[],l=g.length;while(l--)(c=g[l])&&f.push(m[l]=c);i(null,g=[],f,a)}l=g.length;while(l--)(c=g[l])&&(f=i?P.call(s,c):h[l])>-1&&(s[f]=!(o[f]=c))}}else g=mt(g===o?g.splice(d,g.length):g),i?i(null,o,g,a):_.apply(o,g)})}function yt(e){var t,n,r,s=e.length,o=i.relative[e[0].type],u=o||i.relative[" "],a=o?1:0,l=dt(function(e){return e===t},u,!0),c=dt(function(e){return P.call(t,e)>-1},u,!0),h=[function(e,n,r){return!o&&(r||n!==f)||((t=n).nodeType?l(e,n,r):c(e,n,r))}];for(;a<s;a++)if(n=i.relative[e[a].type])h=[dt(vt(h),n)];else{n=i.filter[e[a].type].apply(null,e[a].matches);if(n[w]){r=++a;for(;r<s;r++)if(i.relative[e[r].type])break;return gt(a>1&&vt(h),a>1&&pt(e.slice(0,a-1)).replace(R,"$1"),n,a<r&&yt(e.slice(a,r)),r<s&&yt(e=e.slice(r)),r<s&&pt(e))}h.push(n)}return vt(h)}function bt(e,t){var n=0,s=t.length>0,o=e.length>0,u=function(u,a,l,h,p){var d,v,m,g=[],y=0,b="0",w=u&&[],E=p!=null,S=f,T=u||o&&i.find.TAG("*",p&&a.parentNode||a),N=x+=S==null?1:Math.random()||.1;E&&(f=a!==c&&a,r=n);for(;(d=T[b])!=null;b++){if(o&&d){v=0;while(m=e[v++])if(m(d,a,l)){h.push(d);break}E&&(x=N,r=++n)}s&&((d=!m&&d)&&y--,u&&w.push(d))}y+=b;if(s&&b!==y){v=0;while(m=t[v++])m(w,g,a,l);if(u){if(y>0)while(b--)!w[b]&&!g[b]&&(g[b]=M.call(h));g=mt(g)}_.apply(h,g),E&&!u&&g.length>0&&y+t.length>1&&ut.uniqueSort(h)}return E&&(x=N,f=S),w};return s?st(u):u}function wt(e,t,n){var r=0,i=t.length;for(;r<i;r++)ut(e,t[r],n);return n}function Et(e,t,n,r){var s,o,a,f,l,c=ht(e);if(!r&&c.length===1){o=c[0]=c[0].slice(0);if(o.length>2&&(a=o[0]).type==="ID"&&t.nodeType===9&&!p&&i.relative[o[1].type]){t=i.find.ID(a.matches[0].replace(et,tt),t)[0];if(!t)return n;e=e.slice(o.shift().value.length)}s=V.needsContext.test(e)?0:o.length;while(s--){a=o[s];if(i.relative[f=a.type])break;if(l=i.find[f])if(r=l(a.matches[0].replace(et,tt),$.test(o[0].type)&&t.parentNode||t)){o.splice(s,1),e=r.length&&pt(o);if(!e)return _.apply(n,D.call(r,0)),n;break}}}return u(e,c)(r,t,p,n,$.test(e)),n}function St(){}var n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,y,b,w="sizzle"+ -(new Date),E=e.document,S={},x=0,T=0,N=it(),C=it(),k=it(),L=typeof t,A=1<<31,O=[],M=O.pop,_=O.push,D=O.slice,P=O.indexOf||function(e){var t=0,n=this.length;for(;t<n;t++)if(this[t]===e)return t;return-1},H="[\\x20\\t\\r\\n\\f]",B="(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",j=B.replace("w","w#"),F="([*^$|!~]?=)",I="\\["+H+"*("+B+")"+H+"*(?:"+F+H+"*(?:(['\"])((?:\\\\.|[^\\\\])*?)\\3|("+j+")|)|)"+H+"*\\]",q=":("+B+")(?:\\(((['\"])((?:\\\\.|[^\\\\])*?)\\3|((?:\\\\.|[^\\\\()[\\]]|"+I.replace(3,8)+")*)|.*)\\)|)",R=new RegExp("^"+H+"+|((?:^|[^\\\\])(?:\\\\.)*)"+H+"+$","g"),U=new RegExp("^"+H+"*,"+H+"*"),z=new RegExp("^"+H+"*([\\x20\\t\\r\\n\\f>+~])"+H+"*"),W=new RegExp(q),X=new RegExp("^"+j+"$"),V={ID:new RegExp("^#("+B+")"),CLASS:new RegExp("^\\.("+B+")"),NAME:new RegExp("^\\[name=['\"]?("+B+")['\"]?\\]"),TAG:new RegExp("^("+B.replace("w","w*")+")"),ATTR:new RegExp("^"+I),PSEUDO:new RegExp("^"+q),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+H+"*(even|odd|(([+-]|)(\\d*)n|)"+H+"*(?:([+-]|)"+H+"*(\\d+)|))"+H+"*\\)|)","i"),needsContext:new RegExp("^"+H+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+H+"*((?:-\\d)?\\d*)"+H+"*\\)|)(?=[^-]|$)","i")},$=/[\x20\t\r\n\f]*[+~]/,J=/^[^{]+\{\s*\[native code/,K=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,Q=/^(?:input|select|textarea|button)$/i,G=/^h\d$/i,Y=/'|\\/g,Z=/\=[\x20\t\r\n\f]*([^'"\]]*)[\x20\t\r\n\f]*\]/g,et=/\\([\da-fA-F]{1,6}[\x20\t\r\n\f]?|.)/g,tt=function(e,t){var n="0x"+t-65536;return n!==n?t:n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,n&1023|56320)};try{D.call(E.documentElement.childNodes,0)[0].nodeType}catch(nt){D=function(e){var t,n=[];while(t=this[e++])n.push(t);return n}}o=ut.isXML=function(e){var t=e&&(e.ownerDocument||e).documentElement;return t?t.nodeName!=="HTML":!1},l=ut.setDocument=function(e){var n=e?e.ownerDocument||e:E;if(n===c||n.nodeType!==9||!n.documentElement)return c;c=n,h=n.documentElement,p=o(n),S.tagNameNoComments=ot(function(e){return e.appendChild(n.createComment("")),!e.getElementsByTagName("*").length}),S.attributes=ot(function(e){e.innerHTML="<select></select>";var t=typeof e.lastChild.getAttribute("multiple");return t!=="boolean"&&t!=="string"}),S.getByClassName=ot(function(e){return e.innerHTML="<div class='hidden e'></div><div class='hidden'></div>",!e.getElementsByClassName||!e.getElementsByClassName("e").length?!1:(e.lastChild.className="e",e.getElementsByClassName("e").length===2)}),S.getByName=ot(function(e){e.id=w+0,e.innerHTML="<a name='"+w+"'></a><div name='"+w+"'></div>",h.insertBefore(e,h.firstChild);var t=n.getElementsByName&&n.getElementsByName(w).length===2+n.getElementsByName(w+0).length;return S.getIdNotName=!n.getElementById(w),h.removeChild(e),t}),i.attrHandle=ot(function(e){return e.innerHTML="<a href='#'></a>",e.firstChild&&typeof e.firstChild.getAttribute!==L&&e.firstChild.getAttribute("href")==="#"})?{}:{href:function(e){return e.getAttribute("href",2)},type:function(e){return e.getAttribute("type")}},S.getIdNotName?(i.find.ID=function(e,t){if(typeof t.getElementById!==L&&!p){var n=t.getElementById(e);return n&&n.parentNode?[n]:[]}},i.filter.ID=function(e){var t=e.replace(et,tt);return function(e){return e.getAttribute("id")===t}}):(i.find.ID=function(e,n){if(typeof n.getElementById!==L&&!p){var r=n.getElementById(e);return r?r.id===e||typeof r.getAttributeNode!==L&&r.getAttributeNode("id").value===e?[r]:t:[]}},i.filter.ID=function(e){var t=e.replace(et,tt);return function(e){var n=typeof e.getAttributeNode!==L&&e.getAttributeNode("id");return n&&n.value===t}}),i.find.TAG=S.tagNameNoComments?function(e,t){if(typeof t.getElementsByTagName!==L)return t.getElementsByTagName(e)}:function(e,t){var n,r=[],i=0,s=t.getElementsByTagName(e);if(e==="*"){while(n=s[i++])n.nodeType===1&&r.push(n);return r}return s},i.find.NAME=S.getByName&&function(e,t){if(typeof t.getElementsByName!==L)return t.getElementsByName(name)},i.find.CLASS=S.getByClassName&&function(e,t){if(typeof t.getElementsByClassName!==L&&!p)return t.getElementsByClassName(e)},v=[],d=[":focus"];if(S.qsa=rt(n.querySelectorAll))ot(function(e){e.innerHTML="<select><option selected=''></option></select>",e.querySelectorAll("[selected]").length||d.push("\\["+H+"*(?:checked|disabled|ismap|multiple|readonly|selected|value)"),e.querySelectorAll(":checked").length||d.push(":checked")}),ot(function(e){e.innerHTML="<input type='hidden' i=''/>",e.querySelectorAll("[i^='']").length&&d.push("[*^$]="+H+"*(?:\"\"|'')"),e.querySelectorAll(":enabled").length||d.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),d.push(",.*:")});return(S.matchesSelector=rt(m=h.matchesSelector||h.mozMatchesSelector||h.webkitMatchesSelector||h.oMatchesSelector||h.msMatchesSelector))&&ot(function(e){S.disconnectedMatch=m.call(e,"div"),m.call(e,"[s!='']:x"),v.push("!=",q)}),d=new RegExp(d.join("|")),v=new RegExp(v.join("|")),y=rt(h.contains)||h.compareDocumentPosition?function(e,t){var n=e.nodeType===9?e.documentElement:e,r=t&&t.parentNode;return e===r||!!r&&r.nodeType===1&&!!(n.contains?n.contains(r):e.compareDocumentPosition&&e.compareDocumentPosition(r)&16)}:function(e,t){if(t)while(t=t.parentNode)if(t===e)return!0;return!1},b=h.compareDocumentPosition?function(e,t){var r;if(e===t)return a=!0,0;if(r=t.compareDocumentPosition&&e.compareDocumentPosition&&e.compareDocumentPosition(t))return r&1||e.parentNode&&e.parentNode.nodeType===11?e===n||y(E,e)?-1:t===n||y(E,t)?1:0:r&4?-1:1;return e.compareDocumentPosition?-1:1}:function(e,t){var r,i=0,s=e.parentNode,o=t.parentNode,u=[e],f=[t];if(e===t)return a=!0,0;if(!s||!o)return e===n?-1:t===n?1:s?-1:o?1:0;if(s===o)return at(e,t);r=e;while(r=r.parentNode)u.unshift(r);r=t;while(r=r.parentNode)f.unshift(r);while(u[i]===f[i])i++;return i?at(u[i],f[i]):u[i]===E?-1:f[i]===E?1:0},a=!1,[0,0].sort(b),S.detectDuplicates=a,c},ut.matches=function(e,t){return ut(e,null,null,t)},ut.matchesSelector=function(e,t){(e.ownerDocument||e)!==c&&l(e),t=t.replace(Z,"='$1']");if(S.matchesSelector&&!p&&(!v||!v.test(t))&&!d.test(t))try{var n=m.call(e,t);if(n||S.disconnectedMatch||e.document&&e.document.nodeType!==11)return n}catch(r){}return ut(t,c,null,[e]).length>0},ut.contains=function(e,t){return(e.ownerDocument||e)!==c&&l(e),y(e,t)},ut.attr=function(e,t){var n;return(e.ownerDocument||e)!==c&&l(e),p||(t=t.toLowerCase()),(n=i.attrHandle[t])?n(e):p||S.attributes?e.getAttribute(t):((n=e.getAttributeNode(t))||e.getAttribute(t))&&e[t]===!0?t:n&&n.specified?n.value:null},ut.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},ut.uniqueSort=function(e){var t,n=[],r=1,i=0;a=!S.detectDuplicates,e.sort(b);if(a){for(;t=e[r];r++)t===e[r-1]&&(i=n.push(r));while(i--)e.splice(n[i],1)}return e},s=ut.getText=function(e){var t,n="",r=0,i=e.nodeType;if(!i)for(;t=e[r];r++)n+=s(t);else if(i===1||i===9||i===11){if(typeof e.textContent=="string")return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=s(e)}else if(i===3||i===4)return e.nodeValue;return n},i=ut.selectors={cacheLength:50,createPseudo:st,match:V,find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(et,tt),e[3]=(e[4]||e[5]||"").replace(et,tt),e[2]==="~="&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),e[1].slice(0,3)==="nth"?(e[3]||ut.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*(e[3]==="even"||e[3]==="odd")),e[5]=+(e[7]+e[8]||e[3]==="odd")):e[3]&&ut.error(e[0]),e},PSEUDO:function(e){var t,n=!e[5]&&e[2];return V.CHILD.test(e[0])?null:(e[4]?e[2]=e[4]:n&&W.test(n)&&(t=ht(n,!0))&&(t=n.indexOf(")",n.length-t)-n.length)&&(e[0]=e[0].slice(0,t),e[2]=n.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){return e==="*"?function(){return!0}:(e=e.replace(et,tt).toLowerCase(),function(t){return t.nodeName&&t.nodeName.toLowerCase()===e})},CLASS:function(e){var t=N[e+" "];return t||(t=new RegExp("(^|"+H+")"+e+"("+H+"|$)"))&&N(e,function(e){return t.test(e.className||typeof e.getAttribute!==L&&e.getAttribute("class")||"")})},ATTR:function(e,t,n){return function(r){var i=ut.attr(r,e);return i==null?t==="!=":t?(i+="",t==="="?i===n:t==="!="?i!==n:t==="^="?n&&i.indexOf(n)===0:t==="*="?n&&i.indexOf(n)>-1:t==="$="?n&&i.slice(-n.length)===n:t==="~="?(" "+i+" ").indexOf(n)>-1:t==="|="?i===n||i.slice(0,n.length+1)===n+"-":!1):!0}},CHILD:function(e,t,n,r,i){var s=e.slice(0,3)!=="nth",o=e.slice(-4)!=="last",u=t==="of-type";return r===1&&i===0?function(e){return!!e.parentNode}:function(t,n,a){var f,l,c,h,p,d,v=s!==o?"nextSibling":"previousSibling",m=t.parentNode,g=u&&t.nodeName.toLowerCase(),y=!a&&!u;if(m){if(s){while(v){c=t;while(c=c[v])if(u?c.nodeName.toLowerCase()===g:c.nodeType===1)return!1;d=v=e==="only"&&!d&&"nextSibling"}return!0}d=[o?m.firstChild:m.lastChild];if(o&&y){l=m[w]||(m[w]={}),f=l[e]||[],p=f[0]===x&&f[1],h=f[0]===x&&f[2],c=p&&m.childNodes[p];while(c=++p&&c&&c[v]||(h=p=0)||d.pop())if(c.nodeType===1&&++h&&c===t){l[e]=[x,p,h];break}}else if(y&&(f=(t[w]||(t[w]={}))[e])&&f[0]===x)h=f[1];else while(c=++p&&c&&c[v]||(h=p=0)||d.pop())if((u?c.nodeName.toLowerCase()===g:c.nodeType===1)&&++h){y&&((c[w]||(c[w]={}))[e]=[x,h]);if(c===t)break}return h-=i,h===r||h%r===0&&h/r>=0}}},PSEUDO:function(e,t){var n,r=i.pseudos[e]||i.setFilters[e.toLowerCase()]||ut.error("unsupported pseudo: "+e);return r[w]?r(t):r.length>1?(n=[e,e,"",t],i.setFilters.hasOwnProperty(e.toLowerCase())?st(function(e,n){var i,s=r(e,t),o=s.length;while(o--)i=P.call(e,s[o]),e[i]=!(n[i]=s[o])}):function(e){return r(e,0,n)}):r}},pseudos:{not:st(function(e){var t=[],n=[],r=u(e.replace(R,"$1"));return r[w]?st(function(e,t,n,i){var s,o=r(e,null,i,[]),u=e.length;while(u--)if(s=o[u])e[u]=!(t[u]=s)}):function(e,i,s){return t[0]=e,r(t,null,s,n),!n.pop()}}),has:st(function(e){return function(t){return ut(e,t).length>0}}),contains:st(function(e){return function(t){return(t.textContent||t.innerText||s(t)).indexOf(e)>-1}}),lang:st(function(e){return X.test(e||"")||ut.error("unsupported lang: "+e),e=e.replace(et,tt).toLowerCase(),function(t){var n;do if(n=p?t.getAttribute("xml:lang")||t.getAttribute("lang"):t.lang)return n=n.toLowerCase(),n===e||n.indexOf(e+"-")===0;while((t=t.parentNode)&&t.nodeType===1);return!1}}),target:function(t){var n=e.location&&e.location.hash;return n&&n.slice(1)===t.id},root:function(e){return e===h},focus:function(e){return e===c.activeElement&&(!c.hasFocus||c.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:function(e){return e.disabled===!1},disabled:function(e){return e.disabled===!0},checked:function(e){var t=e.nodeName.toLowerCase();return t==="input"&&!!e.checked||t==="option"&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,e.selected===!0},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeName>"@"||e.nodeType===3||e.nodeType===4)return!1;return!0},parent:function(e){return!i.pseudos.empty(e)},header:function(e){return G.test(e.nodeName)},input:function(e){return Q.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return t==="input"&&e.type==="button"||t==="button"},text:function(e){var t;return e.nodeName.toLowerCase()==="input"&&e.type==="text"&&((t=e.getAttribute("type"))==null||t.toLowerCase()===e.type)},first:ct(function(){return[0]}),last:ct(function(e,t){return[t-1]}),eq:ct(function(e,t,n){return[n<0?n+t:n]}),even:ct(function(e,t){var n=0;for(;n<t;n+=2)e.push(n);return e}),odd:ct(function(e,t){var n=1;for(;n<t;n+=2)e.push(n);return e}),lt:ct(function(e,t,n){var r=n<0?n+t:n;for(;--r>=0;)e.push(r);return e}),gt:ct(function(e,t,n){var r=n<0?n+t:n;for(;++r<t;)e.push(r);return e})}};for(n in{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})i.pseudos[n]=ft(n);for(n in{submit:!0,reset:!0})i.pseudos[n]=lt(n);u=ut.compile=function(e,t){var n,r=[],i=[],s=k[e+" "];if(!s){t||(t=ht(e)),n=t.length;while(n--)s=yt(t[n]),s[w]?r.push(s):i.push(s);s=k(e,bt(i,r))}return s},i.pseudos.nth=i.pseudos.eq,i.filters=St.prototype=i.pseudos,i.setFilters=new St,l(),ut.attr=g.attr,g.find=ut,g.expr=ut.selectors,g.expr[":"]=g.expr.pseudos,g.unique=ut.uniqueSort,g.text=ut.getText,g.isXMLDoc=ut.isXML,g.contains=ut.contains}(window);var it=/Until$/,st=/^(?:parents|prev(?:Until|All))/,ot=/^.[^:#\[\.,]*$/,ut=g.expr.match.needsContext,at={children:!0,contents:!0,next:!0,prev:!0};g.fn.extend({find:function(e){var t,n,r,i=this.length;if(typeof e!="string")return r=this,this.pushStack(g(e).filter(function(){for(t=0;t<i;t++)if(g.contains(r[t],this))return!0}));n=[];for(t=0;t<i;t++)g.find(e,this[t],n);return n=this.pushStack(i>1?g.unique(n):n),n.selector=(this.selector?this.selector+" ":"")+e,n},has:function(e){var t,n=g(e,this),r=n.length;return this.filter(function(){for(t=0;t<r;t++)if(g.contains(this,n[t]))return!0})},not:function(e){return this.pushStack(lt(this,e,!1))},filter:function(e){return this.pushStack(lt(this,e,!0))},is:function(e){return!!e&&(typeof e=="string"?ut.test(e)?g(e,this.context).index(this[0])>=0:g.filter(e,this).length>0:this.filter(e).length>0)},closest:function(e,t){var n,r=0,i=this.length,s=[],o=ut.test(e)||typeof e!="string"?g(e,t||this.context):0;for(;r<i;r++){n=this[r];while(n&&n.ownerDocument&&n!==t&&n.nodeType!==11){if(o?o.index(n)>-1:g.find.matchesSelector(n,e)){s.push(n);break}n=n.parentNode}}return this.pushStack(s.length>1?g.unique(s):s)},index:function(e){return e?typeof e=="string"?g.inArray(this[0],g(e)):g.inArray(e.jquery?e[0]:e,this):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){var n=typeof e=="string"?g(e,t):g.makeArray(e&&e.nodeType?[e]:e),r=g.merge(this.get(),n);return this.pushStack(g.unique(r))},addBack:function(e){return this.add(e==null?this.prevObject:this.prevObject.filter(e))}}),g.fn.andSelf=g.fn.addBack,g.each({parent:function(e){var t=e.parentNode;return t&&t.nodeType!==11?t:null},parents:function(e){return g.dir(e,"parentNode")},parentsUntil:function(e,t,n){return g.dir(e,"parentNode",n)},next:function(e){return ft(e,"nextSibling")},prev:function(e){return ft(e,"previousSibling")},nextAll:function(e){return g.dir(e,"nextSibling")},prevAll:function(e){return g.dir(e,"previousSibling")},nextUntil:function(e,t,n){return g.dir(e,"nextSibling",n)},prevUntil:function(e,t,n){return g.dir(e,"previousSibling",n)},siblings:function(e){return g.sibling((e.parentNode||{}).firstChild,e)},children:function(e){return g.sibling(e.firstChild)},contents:function(e){return g.nodeName(e,"iframe")?e.contentDocument||e.contentWindow.document:g.merge([],e.childNodes)}},function(e,t){g.fn[e]=function(n,r){var i=g.map(this,t,n);return it.test(e)||(r=n),r&&typeof r=="string"&&(i=g.filter(r,i)),i=this.length>1&&!at[e]?g.unique(i):i,this.length>1&&st.test(e)&&(i=i.reverse()),this.pushStack(i)}}),g.extend({filter:function(e,t,n){return n&&(e=":not("+e+")"),t.length===1?g.find.matchesSelector(t[0],e)?[t[0]]:[]:g.find.matches(e,t)},dir:function(e,t,n){var r=[],i=e[t];while(i&&i.nodeType!==9&&(n===undefined||i.nodeType!==1||!g(i).is(n)))i.nodeType===1&&r.push(i),i=i[t];return r},sibling:function(e,t){var n=[];for(;e;e=e.nextSibling)e.nodeType===1&&e!==t&&n.push(e);return n}});var ht="abbr|article|aside|audio|bdi|canvas|data|datalist|details|figcaption|figure|footer|header|hgroup|mark|meter|nav|output|progress|section|summary|time|video",pt=/ jQuery\d+="(?:null|\d+)"/g,dt=new RegExp("<(?:"+ht+")[\\s/>]","i"),vt=/^\s+/,mt=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi,gt=/<([\w:]+)/,yt=/<tbody/i,bt=/<|&#?\w+;/,wt=/<(?:script|style|link)/i,Et=/^(?:checkbox|radio)$/i,St=/checked\s*(?:[^=]|=\s*.checked.)/i,xt=/^$|\/(?:java|ecma)script/i,Tt=/^true\/(.*)/,Nt=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g,Ct={option:[1,"<select multiple='multiple'>","</select>"],legend:[1,"<fieldset>","</fieldset>"],area:[1,"<map>","</map>"],param:[1,"<object>","</object>"],thead:[1,"<table>","</table>"],tr:[2,"<table><tbody>","</tbody></table>"],col:[2,"<table><tbody></tbody><colgroup>","</colgroup></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:g.support.htmlSerialize?[0,"",""]:[1,"X<div>","</div>"]},kt=ct(r),Lt=kt.appendChild(r.createElement("div"));Ct.optgroup=Ct.option,Ct.tbody=Ct.tfoot=Ct.colgroup=Ct.caption=Ct.thead,Ct.th=Ct.td,g.fn.extend({text:function(e){return g.access(this,function(e){return e===undefined?g.text(this):this.empty().append((this[0]&&this[0].ownerDocument||r).createTextNode(e))},null,e,arguments.length)},wrapAll:function(e){if(g.isFunction(e))return this.each(function(t){g(this).wrapAll(e.call(this,t))});if(this[0]){var t=g(e,this[0].ownerDocument).eq(0).clone(!0);this[0].parentNode&&t.insertBefore(this[0]),t.map(function(){var e=this;while(e.firstChild&&e.firstChild.nodeType===1)e=e.firstChild;return e}).append(this)}return this},wrapInner:function(e){return g.isFunction(e)?this.each(function(t){g(this).wrapInner(e.call(this,t))}):this.each(function(){var t=g(this),n=t.contents();n.length?n.wrapAll(e):t.append(e)})},wrap:function(e){var t=g.isFunction(e);return this.each(function(n){g(this).wrapAll(t?e.call(this,n):e)})},unwrap:function(){return this.parent().each(function(){g.nodeName(this,"body")||g(this).replaceWith(this.childNodes)}).end()},append:function(){return this.domManip(arguments,!0,function(e){(this.nodeType===1||this.nodeType===11||this.nodeType===9)&&this.appendChild(e)})},prepend:function(){return this.domManip(arguments,!0,function(e){(this.nodeType===1||this.nodeType===11||this.nodeType===9)&&this.insertBefore(e,this.firstChild)})},before:function(){return this.domManip(arguments,!1,function(e){this.parentNode&&this.parentNode.insertBefore(e,this)})},after:function(){return this.domManip(arguments,!1,function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)})},remove:function(e,t){var n,r=0;for(;(n=this[r])!=null;r++)if(!e||g.filter(e,[n]).length>0)!t&&n.nodeType===1&&g.cleanData(Ht(n)),n.parentNode&&(t&&g.contains(n.ownerDocument,n)&&_t(Ht(n,"script")),n.parentNode.removeChild(n));return this},empty:function(){var e,t=0;for(;(e=this[t])!=null;t++){e.nodeType===1&&g.cleanData(Ht(e,!1));while(e.firstChild)e.removeChild(e.firstChild);e.options&&g.nodeName(e,"select")&&(e.options.length=0)}return this},clone:function(e,t){return e=e==null?!1:e,t=t==null?e:t,this.map(function(){return g.clone(this,e,t)})},html:function(e){return g.access(this,function(e){var t=this[0]||{},n=0,r=this.length;if(e===undefined)return t.nodeType===1?t.innerHTML.replace(pt,""):undefined;if(typeof e=="string"&&!wt.test(e)&&(g.support.htmlSerialize||!dt.test(e))&&(g.support.leadingWhitespace||!vt.test(e))&&!Ct[(gt.exec(e)||["",""])[1].toLowerCase()]){e=e.replace(mt,"<$1></$2>");try{for(;n<r;n++)t=this[n]||{},t.nodeType===1&&(g.cleanData(Ht(t,!1)),t.innerHTML=e);t=0}catch(i){}}t&&this.empty().append(e)},null,e,arguments.length)},replaceWith:function(e){var t=g.isFunction(e);return!t&&typeof e!="string"&&(e=g(e).not(this).detach()),this.domManip([e],!0,function(e){var t=this.nextSibling,n=this.parentNode;n&&(g(this).remove(),n.insertBefore(e,t))})},detach:function(e){return this.remove(e,!0)},domManip:function(e,t,n){e=l.apply([],e);var r,i,s,o,u,a,f=0,c=this.length,h=this,p=c-1,d=e[0],v=g.isFunction(d);if(v||!(c<=1||typeof d!="string"||g.support.checkClone||!St.test(d)))return this.each(function(r){var i=h.eq(r);v&&(e[0]=d.call(this,r,t?i.html():undefined)),i.domManip(e,t,n)});if(c){a=g.buildFragment(e,this[0].ownerDocument,!1,this),r=a.firstChild,a.childNodes.length===1&&(a=r);if(r){t=t&&g.nodeName(r,"tr"),o=g.map(Ht(a,"script"),Ot),s=o.length;for(;f<c;f++)i=a,f!==p&&(i=g.clone(i,!0,!0),s&&g.merge(o,Ht(i,"script"))),n.call(t&&g.nodeName(this[f],"table")?At(this[f],"tbody"):this[f],i,f);if(s){u=o[o.length-1].ownerDocument,g.map(o,Mt);for(f=0;f<s;f++)i=o[f],xt.test(i.type||"")&&!g._data(i,"globalEval")&&g.contains(u,i)&&(i.src?g.ajax({url:i.src,type:"GET",dataType:"script",async:!1,global:!1,"throws":!0}):g.globalEval((i.text||i.textContent||i.innerHTML||"").replace(Nt,"")))}a=r=null}}return this}}),g.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(e,t){g.fn[e]=function(e){var n,r=0,i=[],s=g(e),o=s.length-1;for(;r<=o;r++)n=r===o?this:this.clone(!0),g(s[r])[t](n),c.apply(i,n.get());return this.pushStack(i)}}),g.extend({clone:function(e,t,n){var r,i,s,o,u,a=g.contains(e.ownerDocument,e);g.support.html5Clone||g.isXMLDoc(e)||!dt.test("<"+e.nodeName+">")?s=e.cloneNode(!0):(Lt.innerHTML=e.outerHTML,Lt.removeChild(s=Lt.firstChild));if((!g.support.noCloneEvent||!g.support.noCloneChecked)&&(e.nodeType===1||e.nodeType===11)&&!g.isXMLDoc(e)){r=Ht(s),u=Ht(e);for(o=0;(i=u[o])!=null;++o)r[o]&&Pt(i,r[o])}if(t)if(n){u=u||Ht(e),r=r||Ht(s);for(o=0;(i=u[o])!=null;o++)Dt(i,r[o])}else Dt(e,s);return r=Ht(s,"script"),r.length>0&&_t(r,!a&&Ht(e,"script")),r=u=i=null,s},buildFragment:function(e,t,n,r){var i,s,o,u,a,f,l,c=e.length,h=ct(t),p=[],d=0;for(;d<c;d++){s=e[d];if(s||s===0)if(g.type(s)==="object")g.merge(p,s.nodeType?[s]:s);else if(!bt.test(s))p.push(t.createTextNode(s));else{u=u||h.appendChild(t.createElement("div")),a=(gt.exec(s)||["",""])[1].toLowerCase(),l=Ct[a]||Ct._default,u.innerHTML=l[1]+s.replace(mt,"<$1></$2>")+l[2],i=l[0];while(i--)u=u.lastChild;!g.support.leadingWhitespace&&vt.test(s)&&p.push(t.createTextNode(vt.exec(s)[0]));if(!g.support.tbody){s=a==="table"&&!yt.test(s)?u.firstChild:l[1]==="<table>"&&!yt.test(s)?u:0,i=s&&s.childNodes.length;while(i--)g.nodeName(f=s.childNodes[i],"tbody")&&!f.childNodes.length&&s.removeChild(f)}g.merge(p,u.childNodes),u.textContent="";while(u.firstChild)u.removeChild(u.firstChild);u=h.lastChild}}u&&h.removeChild(u),g.support.appendChecked||g.grep(Ht(p,"input"),Bt),d=0;while(s=p[d++]){if(r&&g.inArray(s,r)!==-1)continue;o=g.contains(s.ownerDocument,s),u=Ht(h.appendChild(s),"script"),o&&_t(u);if(n){i=0;while(s=u[i++])xt.test(s.type||"")&&n.push(s)}}return u=null,h},cleanData:function(e,t){var r,i,s,o,u=0,f=g.expando,l=g.cache,c=g.support.deleteExpando,h=g.event.special;for(;(r=e[u])!=null;u++)if(t||g.acceptData(r)){s=r[f],o=s&&l[s];if(o){if(o.events)for(i in o.events)h[i]?g.event.remove(r,i):g.removeEvent(r,i,o.handle);l[s]&&(delete l[s],c?delete r[f]:typeof r.removeAttribute!==n?r.removeAttribute(f):r[f]=null,a.push(s))}}}});var jt,Ft,It,qt=/alpha\([^)]*\)/i,Rt=/opacity\s*=\s*([^)]*)/,Ut=/^(top|right|bottom|left)$/,zt=/^(none|table(?!-c[ea]).+)/,Wt=/^margin/,Xt=new RegExp("^("+y+")(.*)$","i"),Vt=new RegExp("^("+y+")(?!px)[a-z%]+$","i"),$t=new RegExp("^([+-])=("+y+")","i"),Jt={BODY:"block"},Kt={position:"absolute",visibility:"hidden",display:"block"},Qt={letterSpacing:0,fontWeight:400},Gt=["Top","Right","Bottom","Left"],Yt=["Webkit","O","Moz","ms"];g.fn.extend({css:function(e,t){return g.access(this,function(e,t,n){var r,i,s={},o=0;if(g.isArray(t)){i=Ft(e),r=t.length;for(;o<r;o++)s[t[o]]=g.css(e,t[o],!1,i);return s}return n!==undefined?g.style(e,t,n):g.css(e,t)},e,t,arguments.length>1)},show:function(){return tn(this,!0)},hide:function(){return tn(this)},toggle:function(e){var t=typeof e=="boolean";return this.each(function(){(t?e:en(this))?g(this).show():g(this).hide()})}}),g.extend({cssHooks:{opacity:{get:function(e,t){if(t){var n=It(e,"opacity");return n===""?"1":n}}}},cssNumber:{columnCount:!0,fillOpacity:!0,fontWeight:!0,lineHeight:!0,opacity:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{"float":g.support.cssFloat?"cssFloat":"styleFloat"},style:function(e,t,n,r){if(!e||e.nodeType===3||e.nodeType===8||!e.style)return;var i,s,o,u=g.camelCase(t),a=e.style;t=g.cssProps[u]||(g.cssProps[u]=Zt(a,u)),o=g.cssHooks[t]||g.cssHooks[u];if(n===undefined)return o&&"get"in o&&(i=o.get(e,!1,r))!==undefined?i:a[t];s=typeof n,s==="string"&&(i=$t.exec(n))&&(n=(i[1]+1)*i[2]+parseFloat(g.css(e,t)),s="number");if(n==null||s==="number"&&isNaN(n))return;s==="number"&&!g.cssNumber[u]&&(n+="px"),!g.support.clearCloneStyle&&n===""&&t.indexOf("background")===0&&(a[t]="inherit");if(!o||!("set"in o)||(n=o.set(e,n,r))!==undefined)try{a[t]=n}catch(f){}},css:function(e,t,n,r){var i,s,o,u=g.camelCase(t);return t=g.cssProps[u]||(g.cssProps[u]=Zt(e.style,u)),o=g.cssHooks[t]||g.cssHooks[u],o&&"get"in o&&(s=o.get(e,!0,n)),s===undefined&&(s=It(e,t,r)),s==="normal"&&t in Qt&&(s=Qt[t]),n===""||n?(i=parseFloat(s),n===!0||g.isNumeric(i)?i||0:s):s},swap:function(e,t,n,r){var i,s,o={};for(s in t)o[s]=e.style[s],e.style[s]=t[s];i=n.apply(e,r||[]);for(s in t)e.style[s]=o[s];return i}}),window.getComputedStyle?(Ft=function(e){return window.getComputedStyle(e,null)},It=function(e,t,n){var r,i,s,o=n||Ft(e),u=o?o.getPropertyValue(t)||o[t]:undefined,a=e.style;return o&&(u===""&&!g.contains(e.ownerDocument,e)&&(u=g.style(e,t)),Vt.test(u)&&Wt.test(t)&&(r=a.width,i=a.minWidth,s=a.maxWidth,a.minWidth=a.maxWidth=a.width=u,u=o.width,a.width=r,a.minWidth=i,a.maxWidth=s)),u}):r.documentElement.currentStyle&&(Ft=function(e){return e.currentStyle},It=function(e,t,n){var r,i,s,o=n||Ft(e),u=o?o[t]:undefined,a=e.style;return u==null&&a&&a[t]&&(u=a[t]),Vt.test(u)&&!Ut.test(t)&&(r=a.left,i=e.runtimeStyle,s=i&&i.left,s&&(i.left=e.currentStyle.left),a.left=t==="fontSize"?"1em":u,u=a.pixelLeft+"px",a.left=r,s&&(i.left=s)),u===""?"auto":u}),g.each(["height","width"],function(e,t){g.cssHooks[t]={get:function(e,n,r){if(n)return e.offsetWidth===0&&zt.test(g.css(e,"display"))?g.swap(e,Kt,function(){return sn(e,t,r)}):sn(e,t,r)},set:function(e,n,r){var i=r&&Ft(e);return nn(e,n,r?rn(e,t,r,g.support.boxSizing&&g.css(e,"boxSizing",!1,i)==="border-box",i):0)}}}),g.support.opacity||(g.cssHooks.opacity={get:function(e,t){return Rt.test((t&&e.currentStyle?e.currentStyle.filter:e.style.filter)||"")?.01*parseFloat(RegExp.$1)+"":t?"1":""},set:function(e,t){var n=e.style,r=e.currentStyle,i=g.isNumeric(t)?"alpha(opacity="+t*100+")":"",s=r&&r.filter||n.filter||"";n.zoom=1;if((t>=1||t==="")&&g.trim(s.replace(qt,""))===""&&n.removeAttribute){n.removeAttribute("filter");if(t===""||r&&!r.filter)return}n.filter=qt.test(s)?s.replace(qt,i):s+" "+i}}),g(function(){g.support.reliableMarginRight||(g.cssHooks.marginRight={get:function(e,t){if(t)return g.swap(e,{display:"inline-block"},It,[e,"marginRight"])}}),!g.support.pixelPosition&&g.fn.position&&g.each(["top","left"],function(e,t){g.cssHooks[t]={get:function(e,n){if(n)return n=It(e,t),Vt.test(n)?g(e).position()[t]+"px":n}}})}),g.expr&&g.expr.filters&&(g.expr.filters.hidden=function(e){return e.offsetWidth<=0&&e.offsetHeight<=0||!g.support.reliableHiddenOffsets&&(e.style&&e.style.display||g.css(e,"display"))==="none"},g.expr.filters.visible=function(e){return!g.expr.filters.hidden(e)}),g.each({margin:"",padding:"",border:"Width"},function(e,t){g.cssHooks[e+t]={expand:function(n){var r=0,i={},s=typeof n=="string"?n.split(" "):[n];for(;r<4;r++)i[e+Gt[r]+t]=s[r]||s[r-2]||s[0];return i}},Wt.test(e)||(g.cssHooks[e+t].set=nn)});var an=/%20/g,fn=/\[\]$/,ln=/\r?\n/g,cn=/^(?:submit|button|image|reset|file)$/i,hn=/^(?:input|select|textarea|keygen)/i;g.fn.extend({serialize:function(){return g.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var e=g.prop(this,"elements");return e?g.makeArray(e):this}).filter(function(){var e=this.type;return this.name&&!g(this).is(":disabled")&&hn.test(this.nodeName)&&!cn.test(e)&&(this.checked||!Et.test(e))}).map(function(e,t){var n=g(this).val();return n==null?null:g.isArray(n)?g.map(n,function(e){return{name:t.name,value:e.replace(ln,"\r\n")}}):{name:t.name,value:n.replace(ln,"\r\n")}}).get()}}),g.param=function(e,t){var n,r=[],i=function(e,t){t=g.isFunction(t)?t():t==null?"":t,r[r.length]=encodeURIComponent(e)+"="+encodeURIComponent(t)};t===undefined&&(t=g.ajaxSettings&&g.ajaxSettings.traditional);if(g.isArray(e)||e.jquery&&!g.isPlainObject(e))g.each(e,function(){i(this.name,this.value)});else for(n in e)pn(n,e[n],t,i);return r.join("&").replace(an,"+")},g.each("blur focus focusin focusout load resize scroll unload click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup error contextmenu".split(" "),function(e,t){g.fn[t]=function(e,n){return arguments.length>0?this.on(t,null,e,n):this.trigger(t)}}),g.fn.hover=function(e,t){return this.mouseenter(e).mouseleave(t||e)};var dn,vn,mn=g.now(),gn=/\?/,yn=/#.*$/,bn=/([?&])_=[^&]*/,wn=/^(.*?):[ \t]*([^\r\n]*)\r?$/mg,En=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,Sn=/^(?:GET|HEAD)$/,xn=/^\/\//,Tn=/^([\w.+-]+:)(?:\/\/([^\/?#:]*)(?::(\d+)|)|)/,Nn=g.fn.load,Cn={},kn={},Ln="*/".concat("*");try{vn=i.href}catch(An){vn=r.createElement("a"),vn.href="",vn=vn.href}dn=Tn.exec(vn.toLowerCase())||[],g.fn.load=function(e,t,n){if(typeof e!="string"&&Nn)return Nn.apply(this,arguments);var r,i,s,o=this,u=e.indexOf(" ");return u>=0&&(r=e.slice(u,e.length),e=e.slice(0,u)),g.isFunction(t)?(n=t,t=undefined):t&&typeof t=="object"&&(s="POST"),o.length>0&&g.ajax({url:e,type:s,dataType:"html",data:t}).done(function(e){i=arguments,o.html(r?g("<div>").append(g.parseHTML(e)).find(r):e)}).complete(n&&function(e,t){o.each(n,i||[e.responseText,t,e])}),this},g.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(e,t){g.fn[t]=function(e){return this.on(t,e)}}),g.each(["get","post"],function(e,t){g[t]=function(e,n,r,i){return g.isFunction(n)&&(i=i||r,r=n,n=undefined),g.ajax({url:e,type:t,dataType:i,data:n,success:r})}}),g.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:vn,type:"GET",isLocal:En.test(dn[1]),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Ln,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/xml/,html:/html/,json:/json/},responseFields:{xml:"responseXML",text:"responseText"},converters:{"* text":window.String,"text html":!0,"text json":g.parseJSON,"text xml":g.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?_n(_n(e,g.ajaxSettings),t):_n(g.ajaxSettings,e)},ajaxPrefilter:On(Cn),ajaxTransport:On(kn),ajax:function(e,t){function T(e,t,n,r){var f,m,y,b,E,x=t;if(w===2)return;w=2,o&&clearTimeout(o),a=undefined,s=r||"",S.readyState=e>0?4:0,n&&(b=Dn(l,S,n));if(e>=200&&e<300||e===304)l.ifModified&&(E=S.getResponseHeader("Last-Modified"),E&&(g.lastModified[i]=E),E=S.getResponseHeader("etag"),E&&(g.etag[i]=E)),e===204?(f=!0,x="nocontent"):e===304?(f=!0,x="notmodified"):(f=Pn(l,b),x=f.state,m=f.data,y=f.error,f=!y);else{y=x;if(e||!x)x="error",e<0&&(e=0)}S.status=e,S.statusText=(t||x)+"",f?p.resolveWith(c,[m,x,S]):p.rejectWith(c,[S,x,y]),S.statusCode(v),v=undefined,u&&h.trigger(f?"ajaxSuccess":"ajaxError",[S,l,f?m:y]),d.fireWith(c,[S,x]),u&&(h.trigger("ajaxComplete",[S,l]),--g.active||g.event.trigger("ajaxStop"))}typeof e=="object"&&(t=e,e=undefined),t=t||{};var n,r,i,s,o,u,a,f,l=g.ajaxSetup({},t),c=l.context||l,h=l.context&&(c.nodeType||c.jquery)?g(c):g.event,p=g.Deferred(),d=g.Callbacks("once memory"),v=l.statusCode||{},m={},y={},w=0,E="canceled",S={readyState:0,getResponseHeader:function(e){var t;if(w===2){if(!f){f={};while(t=wn.exec(s))f[t[1].toLowerCase()]=t[2]}t=f[e.toLowerCase()]}return t==null?null:t},getAllResponseHeaders:function(){return w===2?s:null},setRequestHeader:function(e,t){var n=e.toLowerCase();return w||(e=y[n]=y[n]||e,m[e]=t),this},overrideMimeType:function(e){return w||(l.mimeType=e),this},statusCode:function(e){var t;if(e)if(w<2)for(t in e)v[t]=[v[t],e[t]];else S.always(e[S.status]);return this},abort:function(e){var t=e||E;return a&&a.abort(t),T(0,t),this}};p.promise(S).complete=d.add,S.success=S.done,S.error=S.fail,l.url=((e||l.url||vn)+"").replace(yn,"").replace(xn,dn[1]+"//"),l.type=t.method||t.type||l.method||l.type,l.dataTypes=g.trim(l.dataType||"*").toLowerCase().match(b)||[""],l.crossDomain==null&&(n=Tn.exec(l.url.toLowerCase()),l.crossDomain=!(!n||n[1]===dn[1]&&n[2]===dn[2]&&(n[3]||(n[1]==="http:"?80:443))==(dn[3]||(dn[1]==="http:"?80:443)))),l.data&&l.processData&&typeof l.data!="string"&&(l.data=g.param(l.data,l.traditional)),Mn(Cn,l,t,S);if(w===2)return S;u=l.global,u&&g.active++===0&&g.event.trigger("ajaxStart"),l.type=l.type.toUpperCase(),l.hasContent=!Sn.test(l.type),i=l.url,l.hasContent||(l.data&&(i=l.url+=(gn.test(i)?"&":"?")+l.data,delete l.data),l.cache===!1&&(l.url=bn.test(i)?i.replace(bn,"$1_="+mn++):i+(gn.test(i)?"&":"?")+"_="+mn++)),l.ifModified&&(g.lastModified[i]&&S.setRequestHeader("If-Modified-Since",g.lastModified[i]),g.etag[i]&&S.setRequestHeader("If-None-Match",g.etag[i])),(l.data&&l.hasContent&&l.contentType!==!1||t.contentType)&&S.setRequestHeader("Content-Type",l.contentType),S.setRequestHeader("Accept",l.dataTypes[0]&&l.accepts[l.dataTypes[0]]?l.accepts[l.dataTypes[0]]+(l.dataTypes[0]!=="*"?", "+Ln+"; q=0.01":""):l.accepts["*"]);for(r in l.headers)S.setRequestHeader(r,l.headers[r]);if(!l.beforeSend||l.beforeSend.call(c,S,l)!==!1&&w!==2){E="abort";for(r in{success:1,error:1,complete:1})S[r](l[r]);a=Mn(kn,l,t,S);if(!a)T(-1,"No Transport");else{S.readyState=1,u&&h.trigger("ajaxSend",[S,l]),l.async&&l.timeout>0&&(o=setTimeout(function(){S.abort("timeout")},l.timeout));try{w=1,a.send(m,T)}catch(x){if(!(w<2))throw x;T(-1,x)}}return S}return S.abort()},getScript:function(e,t){return g.get(e,undefined,t,"script")},getJSON:function(e,t,n){return g.get(e,t,n,"json")}}),g.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/(?:java|ecma)script/},converters:{"text script":function(e){return g.globalEval(e),e}}}),g.ajaxPrefilter("script",function(e){e.cache===undefined&&(e.cache=!1),e.crossDomain&&(e.type="GET",e.global=!1)}),g.ajaxTransport("script",function(e){if(e.crossDomain){var t,n=r.head||g("head")[0]||r.documentElement;return{send:function(i,s){t=r.createElement("script"),t.async=!0,e.scriptCharset&&(t.charset=e.scriptCharset),t.src=e.url,t.onload=t.onreadystatechange=function(e,n){if(n||!t.readyState||/loaded|complete/.test(t.readyState))t.onload=t.onreadystatechange=null,t.parentNode&&t.parentNode.removeChild(t),t=null,n||s(200,"success")},n.insertBefore(t,n.firstChild)},abort:function(){t&&t.onload(undefined,!0)}}}});var Hn=[],Bn=/(=)\?(?=&|$)|\?\?/;g.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=Hn.pop()||g.expando+"_"+mn++;return this[e]=!0,e}}),g.ajaxPrefilter("json jsonp",function(e,t,n){var r,i,s,o=e.jsonp!==!1&&(Bn.test(e.url)?"url":typeof e.data=="string"&&!(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Bn.test(e.data)&&"data");if(o||e.dataTypes[0]==="jsonp")return r=e.jsonpCallback=g.isFunction(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,o?e[o]=e[o].replace(Bn,"$1"+r):e.jsonp!==!1&&(e.url+=(gn.test(e.url)?"&":"?")+e.jsonp+"="+r),e.converters["script json"]=function(){return s||g.error(r+" was not called"),s[0]},e.dataTypes[0]="json",i=window[r],window[r]=function(){s=arguments},n.always(function(){window[r]=i,e[r]&&(e.jsonpCallback=t.jsonpCallback,Hn.push(r)),s&&g.isFunction(i)&&i(s[0]),s=i=undefined}),"script"});var jn,Fn,In=0,qn=window.ActiveXObject&&function(){var e;for(e in jn)jn[e](undefined,!0)};g.ajaxSettings.xhr=window.ActiveXObject?function(){return!this.isLocal&&Rn()||Un()}:Rn,Fn=g.ajaxSettings.xhr(),g.support.cors=!!Fn&&"withCredentials"in Fn,Fn=g.support.ajax=!!Fn,Fn&&g.ajaxTransport(function(e){if(!e.crossDomain||g.support.cors){var t;return{send:function(n,r){var i,s,o=e.xhr();e.username?o.open(e.type,e.url,e.async,e.username,e.password):o.open(e.type,e.url,e.async);if(e.xhrFields)for(s in e.xhrFields)o[s]=e.xhrFields[s];e.mimeType&&o.overrideMimeType&&o.overrideMimeType(e.mimeType),!e.crossDomain&&!n["X-Requested-With"]&&(n["X-Requested-With"]="XMLHttpRequest");try{for(s in n)o.setRequestHeader(s,n[s])}catch(u){}o.send(e.hasContent&&e.data||null),t=function(n,s){var u,a,f,l;try{if(t&&(s||o.readyState===4)){t=undefined,i&&(o.onreadystatechange=g.noop,qn&&delete jn[i]);if(s)o.readyState!==4&&o.abort();else{l={},u=o.status,a=o.getAllResponseHeaders(),typeof o.responseText=="string"&&(l.text=o.responseText);try{f=o.statusText}catch(c){f=""}!u&&e.isLocal&&!e.crossDomain?u=l.text?200:404:u===1223&&(u=204)}}}catch(h){s||r(-1,h)}l&&r(u,f,l,a)},e.async?o.readyState===4?setTimeout(t):(i=++In,qn&&(jn||(jn={},g(window).unload(qn)),jn[i]=t),o.onreadystatechange=t):t()},abort:function(){t&&t(undefined,!0)}}}});var zn,Wn,Xn=/^(?:toggle|show|hide)$/,Vn=new RegExp("^(?:([+-])=|)("+y+")([a-z%]*)$","i"),$n=/queueHooks$/,Jn=[er],Kn={"*":[function(e,t){var n,r,i=this.createTween(e,t),s=Vn.exec(t),o=i.cur(),u=+o||0,a=1,f=20;if(s){n=+s[2],r=s[3]||(g.cssNumber[e]?"":"px");if(r!=="px"&&u){u=g.css(i.elem,e,!0)||n||1;do a=a||".5",u/=a,g.style(i.elem,e,u+r);while(a!==(a=i.cur()/o)&&a!==1&&--f)}i.unit=r,i.start=u,i.end=s[1]?u+(s[1]+1)*n:n}return i}]};return g.Animation=g.extend(Yn,{tweener:function(e,t){g.isFunction(e)?(t=e,e=["*"]):e=e.split(" ");var n,r=0,i=e.length;for(;r<i;r++)n=e[r],Kn[n]=Kn[n]||[],Kn[n].unshift(t)},prefilter:function(e,t){t?Jn.unshift(e):Jn.push(e)}}),g.Tween=tr,tr.prototype={constructor:tr,init:function(e,t,n,r,i,s){this.elem=e,this.prop=n,this.easing=i||"swing",this.options=t,this.start=this.now=this.cur(),this.end=r,this.unit=s||(g.cssNumber[n]?"":"px")},cur:function(){var e=tr.propHooks[this.prop];return e&&e.get?e.get(this):tr.propHooks._default.get(this)},run:function(e){var t,n=tr.propHooks[this.prop];return this.options.duration?this.pos=t=g.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):tr.propHooks._default.set(this),this}},tr.prototype.init.prototype=tr.prototype,tr.propHooks={_default:{get:function(e){var t;return e.elem[e.prop]==null||!!e.elem.style&&e.elem.style[e.prop]!=null?(t=g.css(e.elem,e.prop,""),!t||t==="auto"?0:t):e.elem[e.prop]},set:function(e){g.fx.step[e.prop]?g.fx.step[e.prop](e):e.elem.style&&(e.elem.style[g.cssProps[e.prop]]!=null||g.cssHooks[e.prop])?g.style(e.elem,e.prop,e.now+e.unit):e.elem[e.prop]=e.now}}},tr.propHooks.scrollTop=tr.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},g.each(["toggle","show","hide"],function(e,t){var n=g.fn[t];g.fn[t]=function(e,r,i){return e==null||typeof e=="boolean"?n.apply(this,arguments):this.animate(nr(t,!0),e,r,i)}}),g.fn.extend({fadeTo:function(e,t,n,r){return this.filter(en).css("opacity",0).show().end().animate({opacity:t},e,n,r)},animate:function(e,t,n,r){var i=g.isEmptyObject(e),s=g.speed(t,n,r),o=function(){var t=Yn(this,g.extend({},e),s);o.finish=function(){t.stop(!0)},(i||g._data(this,"finish"))&&t.stop(!0)};return o.finish=o,i||s.queue===!1?this.each(o):this.queue(s.queue,o)},stop:function(e,t,n){var r=function(e){var t=e.stop;delete e.stop,t(n)};return typeof e!="string"&&(n=t,t=e,e=undefined),t&&e!==!1&&this.queue(e||"fx",[]),this.each(function(){var t=!0,i=e!=null&&e+"queueHooks",s=g.timers,o=g._data(this);if(i)o[i]&&o[i].stop&&r(o[i]);else for(i in o)o[i]&&o[i].stop&&$n.test(i)&&r(o[i]);for(i=s.length;i--;)s[i].elem===this&&(e==null||s[i].queue===e)&&(s[i].anim.stop(n),t=!1,s.splice(i,1));(t||!n)&&g.dequeue(this,e)})},finish:function(e){return e!==!1&&(e=e||"fx"),this.each(function(){var t,n=g._data(this),r=n[e+"queue"],i=n[e+"queueHooks"],s=g.timers,o=r?r.length:0;n.finish=!0,g.queue(this,e,[]),i&&i.cur&&i.cur.finish&&i.cur.finish.call(this);for(t=s.length;t--;)s[t].elem===this&&s[t].queue===e&&(s[t].anim.stop(!0),s.splice(t,1));for(t=0;t<o;t++)r[t]&&r[t].finish&&r[t].finish.call(this);delete n.finish})}}),g.each({slideDown:nr("show"),slideUp:nr("hide"),slideToggle:nr("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(e,t){g.fn[e]=function(e,n,r){return this.animate(t,e,n,r)}}),g.speed=function(e,t,n){var r=e&&typeof e=="object"?g.extend({},e):{complete:n||!n&&t||g.isFunction(e)&&e,duration:e,easing:n&&t||t&&!g.isFunction(t)&&t};r.duration=g.fx.off?0:typeof r.duration=="number"?r.duration:r.duration in g.fx.speeds?g.fx.speeds[r.duration]:g.fx.speeds._default;if(r.queue==null||r.queue===!0)r.queue="fx";return r.old=r.complete,r.complete=function(){g.isFunction(r.old)&&r.old.call(this),r.queue&&g.dequeue(this,r.queue)},r},g.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2}},g.timers=[],g.fx=tr.prototype.init,g.fx.tick=function(){var e,t=g.timers,n=0;zn=g.now();for(;n<t.length;n++)e=t[n],!e()&&t[n]===e&&t.splice(n--,1);t.length||g.fx.stop(),zn=undefined},g.fx.timer=function(e){e()&&g.timers.push(e)&&g.fx.start()},g.fx.interval=13,g.fx.start=function(){Wn||(Wn=setInterval(g.fx.tick,g.fx.interval))},g.fx.stop=function(){clearInterval(Wn),Wn=null},g.fx.speeds={slow:600,fast:200,_default:400},g.fx.step={},g.expr&&g.expr.filters&&(g.expr.filters.animated=function(e){return g.grep(g.timers,function(t){return e===t.elem}).length}),g.fn.offset=function(e){if(arguments.length)return e===undefined?this:this.each(function(t){g.offset.setOffset(this,e,t)});var t,r,i={top:0,left:0},s=this[0],o=s&&s.ownerDocument;if(!o)return;return t=o.documentElement,g.contains(t,s)?(typeof s.getBoundingClientRect!==n&&(i=s.getBoundingClientRect()),r=rr(o),{top:i.top+(r.pageYOffset||t.scrollTop)-(t.clientTop||0),left:i.left+(r.pageXOffset||t.scrollLeft)-(t.clientLeft||0)}):i},g.offset={setOffset:function(e,t,n){var r=g.css(e,"position");r==="static"&&(e.style.position="relative");var i=g(e),s=i.offset(),o=g.css(e,"top"),u=g.css(e,"left"),a=(r==="absolute"||r==="fixed")&&g.inArray("auto",[o,u])>-1,f={},l={},c,h;a?(l=i.position(),c=l.top,h=l.left):(c=parseFloat(o)||0,h=parseFloat(u)||0),g.isFunction(t)&&(t=t.call(e,n,s)),t.top!=null&&(f.top=t.top-s.top+c),t.left!=null&&(f.left=t.left-s.left+h),"using"in t?t.using.call(e,f):i.css(f)}},g.fn.extend({position:function(){if(!this[0])return;var e,t,n={top:0,left:0},r=this[0];return g.css(r,"position")==="fixed"?t=r.getBoundingClientRect():(e=this.offsetParent(),t=this.offset(),g.nodeName(e[0],"html")||(n=e.offset()),n.top+=g.css(e[0],"borderTopWidth",!0),n.left+=g.css(e[0],"borderLeftWidth",!0)),{top:t.top-n.top-g.css(r,"marginTop",!0),left:t.left-n.left-g.css(r,"marginLeft",!0)}},offsetParent:function(){return this.map(function(){var e=this.offsetParent||r.documentElement;while(e&&!g.nodeName(e,"html")&&g.css(e,"position")==="static")e=e.offsetParent;return e||r.documentElement})}}),g.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(e,t){var n=/Y/.test(t);g.fn[e]=function(r){return g.access(this,function(e,r,i){var s=rr(e);if(i===undefined)return s?t in s?s[t]:s.document.documentElement[r]:e[r];s?s.scrollTo(n?g(s).scrollLeft():i,n?i:g(s).scrollTop()):e[r]=i},e,r,arguments.length,null)}}),g.each({Height:"height",Width:"width"},function(e,t){g.each({padding:"inner"+e,content:t,"":"outer"+e},function(n,r){g.fn[r]=function(r,i){var s=arguments.length&&(n||typeof r!="boolean"),o=n||(r===!0||i===!0?"margin":"border");return g.access(this,function(t,n,r){var i;return g.isWindow(t)?t.document.documentElement["client"+e]:t.nodeType===9?(i=t.documentElement,Math.max(t.body["scroll"+e],i["scroll"+e],t.body["offset"+e],i["offset"+e],i["client"+e])):r===undefined?g.css(t,n,o):g.style(t,n,r,o)},t,s?r:undefined,s,null)}})}),g}),define("runtime-browser/errors/proxyerror",["./error"],function(e){var t=function(t,n,r){e.call(this,t,n),this.name="gateway error",this.statusCode=r};return t.prototype=new e,t}),define("runtime-browser/http",["datajslib!jquery","datajslib!errors/proxyerror"],function(e,t){var n=0,r=function(e,n,r){return n>=500?r(new t("Server returned an internal error","status="+n,n)):n>=400?r(new t("Server returned a client error","status="+n,n)):r(null,e,n)};return{request:function(i,s){return i.dataType==="jsonp"&&!i.jsonpCallback&&(i.jsonpCallback="datajslibjsonp"+ ++n),i.success=function(e,t,n){r(e,n.status,s)},i.error=function(e,n,i){if(!e)return s(new t("Network error",i));if(e.status===200)return s(new t("Invalid reponse received from the server",i));r(n,e.status,s)},e.ajax(i)}}}),define("runtime-browser/htmlentities",[],function(){var e=/&(#?)(\d{1,5}|\w{1,8});/gm,t={"&nbsp;":" ","&iexcl;":"¡","&cent;":"¢","&pound;":"£","&curren;":"¤","&yen;":"¥","&brvbar;":"¦","&sect;":"§","&uml;":"¨","&copy;":"©","&ordf;":"ª","&laquo;":"«","&not;":"¬","&shy;":"","&reg;":"®","&macr;":"¯","&deg;":"°","&plusmn;":"±","&sup2;":"²","&sup3;":"³","&acute;":"´","&micro;":"µ","&para;":"¶","&middot;":"·","&cedil;":"¸","&sup1;":"¹","&ordm;":"º","&raquo;":"»","&frac14;":"¼","&frac12;":"½","&frac34;":"¾","&iquest;":"¿","&Agrave;":"À","&Aacute;":"Á","&Acirc;":"Â","&Atilde;":"Ã","&Auml;":"Ä","&Aring;":"Å","&AElig;":"Æ","&Ccedil;":"Ç","&Egrave;":"È","&Eacute;":"É","&Ecirc;":"Ê","&Euml;":"Ë","&Igrave;":"Ì","&Iacute;":"Í","&Icirc;":"Î","&Iuml;":"Ï","&ETH;":"Ð","&Ntilde;":"Ñ","&Ograve;":"Ò","&Oacute;":"Ó","&Ocirc;":"Ô","&Otilde;":"Õ","&Ouml;":"Ö","&times;":"×","&Oslash;":"Ø","&Ugrave;":"Ù","&Uacute;":"Ú","&Ucirc;":"Û","&Uuml;":"Ü","&Yacute;":"Ý","&THORN;":"Þ","&szlig;":"ß","&agrave;":"à","&aacute;":"á","&acirc;":"â","&atilde;":"ã","&auml;":"ä","&aring;":"å","&aelig;":"æ","&ccedil;":"ç","&egrave;":"è","&eacute;":"é","&ecirc;":"ê","&euml;":"ë","&igrave;":"ì","&iacute;":"í","&icirc;":"î","&iuml;":"ï","&eth;":"ð","&ntilde;":"ñ","&ograve;":"ò","&oacute;":"ó","&ocirc;":"ô","&otilde;":"õ","&ouml;":"ö","&divide;":"÷","&oslash;":"ø","&ugrave;":"ù","&uacute;":"ú","&ucirc;":"û","&uuml;":"ü","&yacute;":"ý","&thorn;":"þ","&yuml;":"ÿ","&fnof;":"ƒ","&Alpha;":"Α","&Beta;":"Β","&Gamma;":"Γ","&Delta;":"Δ","&Epsilon;":"Ε","&Zeta;":"Ζ","&Eta;":"Η","&Theta;":"Θ","&Iota;":"Ι","&Kappa;":"Κ","&Lambda;":"Λ","&Mu;":"Μ","&Nu;":"Ν","&Xi;":"Ξ","&Omicron;":"Ο","&Pi;":"Π","&Rho;":"Ρ","&Sigma;":"Σ","&Tau;":"Τ","&Upsilon;":"Υ","&Phi;":"Φ","&Chi;":"Χ","&Psi;":"Ψ","&Omega;":"Ω","&alpha;":"α","&beta;":"β","&gamma;":"γ","&delta;":"δ","&epsilon;":"ε","&zeta;":"ζ","&eta;":"η","&theta;":"θ","&iota;":"ι","&kappa;":"κ","&lambda;":"λ","&mu;":"μ","&nu;":"ν","&xi;":"ξ","&omicron;":"ο","&pi;":"π","&rho;":"ρ","&sigmaf;":"ς","&sigma;":"σ","&tau;":"τ","&upsilon;":"υ","&phi;":"φ","&chi;":"χ","&psi;":"ψ","&omega;":"ω","&thetasym;":"ϑ","&upsih;":"ϒ","&piv;":"ϖ","&bull;":"•","&hellip;":"…","&prime;":"′","&Prime;":"″","&oline;":"‾","&frasl;":"⁄","&weierp;":"℘","&image;":"ℑ","&real;":"ℜ","&trade;":"™","&alefsym;":"ℵ","&larr;":"←","&uarr;":"↑","&rarr;":"→","&darr;":"↓","&harr;":"↔","&crarr;":"↵","&lArr;":"⇐","&uArr;":"⇑","&rArr;":"⇒","&dArr;":"⇓","&hArr;":"⇔","&forall;":"∀","&part;":"∂","&exist;":"∃","&empty;":"∅","&nabla;":"∇","&isin;":"∈","&notin;":"∉","&ni;":"∋","&prod;":"∏","&sum;":"∑","&minus;":"−","&lowast;":"∗","&radic;":"√","&prop;":"∝","&infin;":"∞","&ang;":"∠","&and;":"∧","&or;":"∨","&cap;":"∩","&cup;":"∪","&int;":"∫","&there4;":"∴","&sim;":"∼","&cong;":"≅","&asymp;":"≈","&ne;":"≠","&equiv;":"≡","&le;":"≤","&ge;":"≥","&sub;":"⊂","&sup;":"⊃","&nsub;":"⊄","&sube;":"⊆","&supe;":"⊇","&oplus;":"⊕","&otimes;":"⊗","&perp;":"⊥","&sdot;":"⋅","&lceil;":"⌈","&rceil;":"⌉","&lfloor;":"⌊","&rfloor;":"⌋","&lang;":"〈","&rang;":"〉","&loz;":"◊","&spades;":"♠","&clubs;":"♣","&hearts;":"♥","&diams;":"♦","&quot;":'"',"&amp;":"&","&lt;":"<","&gt;":">","&apos;":"'","&OElig;":"Œ","&oelig;":"œ","&Scaron;":"Š","&scaron;":"š","&Yuml;":"Ÿ","&circ;":"ˆ","&tilde;":"˜","&ensp;":" ","&emsp;":" ","&thinsp;":" ","&zwnj;":"‌","&zwj;":"‍","&lrm;":"‎","&rlm;":"‏","&ndash;":"–","&mdash;":"—","&lsquo;":"‘","&rsquo;":"’","&sbquo;":"‚","&ldquo;":"“","&rdquo;":"”","&bdquo;":"„","&dagger;":"†","&Dagger;":"‡","&permil;":"‰","&lsaquo;":"‹","&rsaquo;":"›","&euro;":"€"};return{decode:function(n){return n?n.replace(e,function(e){var n=null;return e.charAt(1)==="#"?e.charAt(2)==="x"?String.fromCharCode(parseInt(e.substring(3),16)):String.fromCharCode(parseInt(e.substring(2),10)):t[e]||""}):null}}}),define("runtime-browser/htmlparser",["datajslib!htmlentities"],function(e){return function(){function r(e,t){this._options=t?t:{},this._options.includeLocation==undefined&&(this._options.includeLocation=!1),this.validateHandler(e),this._handler=e,this.reset()}function i(e){i.super_.call(this,e,{ignoreWhitespace:!0,verbose:!1,enforceEmptyTags:!1})}function s(e,t){this.reset(),this._options=t?t:{},this._options.ignoreWhitespace==undefined&&(this._options.ignoreWhitespace=!1),this._options.verbose==undefined&&(this._options.verbose=!0),this._options.enforceEmptyTags==undefined&&(this._options.enforceEmptyTags=!0),typeof e=="function"&&(this._callback=e)}function u(e,t){var n=function(){};n.prototype=t.prototype,e.super_=t,e.prototype=new n,e.prototype.constructor=e}var t={},n={Text:"text",Directive:"directive",Comment:"comment",Script:"script",Style:"style",Tag:"tag"};r._reTrim=/(^\s+|\s+$)/g,r._reTrimComment=/(^\!--|--$)/g,r._reWhitespace=/\s/g,r._reTagName=/^\s*(\/?)\s*([^\s\/]+)/,r._reAttrib=/([^=<>\"\'\s]+)\s*=\s*"([^"]*)"|([^=<>\"\'\s]+)\s*=\s*'([^']*)'|([^=<>\"\'\s]+)\s*=\s*([^'"\s]+)|([^=<>\"\'\s\/]+)/g,r._reTags=/[\<\>]/g,r._reCDEnd=/\]\]\>/g,r.prototype.parseComplete=function(t){this.reset(),this.parseChunk(t),this.done()},r.prototype.parseChunk=function(t){this._done&&this.handleError(new Error("Attempted to parse chunk after parsing already done")),this._buffer+=t,this.parseTags()},r.prototype.done=function(){if(this._done)return;this._done=!0;if(this._buffer.length){var i=this._buffer;this._buffer="";var s={raw:i,data:this._parseState==n.Text?e.decode(i):i.replace(r._reTrim,""),type:this._parseState};if(this._parseState==n.Tag||this._parseState==n.Script||this._parseState==n.Style)s.name=this.parseTagName(s.data);this.parseAttribs(s),this._elements.push(s)}this.writeHandler(),this._handler.done()},r.prototype.reset=function(){this._buffer="",this._done=!1,this._elements=[],this._elementsCurrent=0,this._current=0,this._next=0,this._location={row:0,col:0,charOffset:0,inBuffer:0},this._parseState=n.Text,this._prevTagSep="",this._tagStack=[],this._handler.reset()},r.prototype._options=null,r.prototype._handler=null,r.prototype._buffer=null,r.prototype._done=!1,r.prototype._elements=null,r.prototype._elementsCurrent=0,r.prototype._current=0,r.prototype._next=0,r.prototype._location=null,r.prototype._parseState=n.Text,r.prototype._prevTagSep="",r.prototype._tagStack=null,r.prototype.parseTagAttribs=function(t){var r=t.length,i=0;while(i<r){var s=t[i++];(s.type==n.Tag||s.type==n.Script||s.type==n.style)&&this.parseAttribs(s)}return t},r.prototype.parseAttribs=function(i){if(i.type!=n.Script&&i.type!=n.Style&&i.type!=n.Tag)return;var s=i.data.split(r._reWhitespace,1)[0],o=i.data.substring(s.length);if(o.length<1)return;var u;r._reAttrib.lastIndex=0;while(u=r._reAttrib.exec(o))i.attribs==undefined&&(i.attribs={}),typeof u[1]=="string"&&u[1].length?i.attribs[u[1]]=e.decode(u[2]):typeof u[3]=="string"&&u[3].length?i.attribs[u[3].toString()]=e.decode(u[4].toString()):typeof u[5]=="string"&&u[5].length?i.attribs[u[5]]=e.decode(u[6]):typeof u[7]=="string"&&u[7].length&&(i.attribs[u[7]]=u[7])},r.prototype.parseTagName=function(t){if(t==null||t=="")return"";var n=r._reTagName.exec(t);return n?(n[1]?"/":"")+n[2]:""},r.prototype.parseTags=function(){var i=this._buffer.length-1;while(r._reTags.test(this._buffer)){this._next=r._reTags.lastIndex-1;var s=this._buffer.charAt(this._next),o=this._buffer.substring(this._current,this._next),u={raw:o,data:this._parseState==n.Text?o:o.replace(r._reTrim,""),type:this._parseState},a=this.parseTagName(u.data);if(this._tagStack.length)if(this._tagStack[this._tagStack.length-1]==n.Script){if(a=="/script")this._tagStack.pop();else if(u.raw.indexOf("!--")!=0){u.type=n.Text;if(this._elements.length&&this._elements[this._elements.length-1].type==n.Text){var f=this._elements[this._elements.length-1];f.raw=f.data=f.raw+this._prevTagSep+u.raw,u.raw=u.data=""}}}else if(this._tagStack[this._tagStack.length-1]==n.Style){if(a=="/style")this._tagStack.pop();else if(u.raw.indexOf("!--")!=0){u.type=n.Text;if(this._elements.length&&this._elements[this._elements.length-1].type==n.Text){var f=this._elements[this._elements.length-1];u.raw!=""?(f.raw=f.data=f.raw+this._prevTagSep+u.raw,u.raw=u.data=""):f.raw=f.data=f.raw+this._prevTagSep}else u.raw!=""&&(u.raw=u.data=u.raw)}}else if(this._tagStack[this._tagStack.length-1]==n.Comment){var l=u.raw.length;if(u.raw.charAt(l-2)=="-"&&u.raw.charAt(l-1)=="-"&&s==">"){this._tagStack.pop();if(this._elements.length&&this._elements[this._elements.length-1].type==n.Comment){var f=this._elements[this._elements.length-1];f.raw=f.data=(f.raw+u.raw).replace(r._reTrimComment,""),u.raw=u.data="",u.type=n.Text}else u.type=n.Comment}else{u.type=n.Comment;if(this._elements.length&&this._elements[this._elements.length-1].type==n.Comment){var f=this._elements[this._elements.length-1];f.raw=f.data=f.raw+u.raw+s,u.raw=u.data="",u.type=n.Text}else u.raw=u.data=u.raw+s}}if(u.type==n.Tag){u.name=a;if(u.raw.indexOf("!--")==0){u.type=n.Comment,delete u.name;var l=u.raw.length;u.raw.charAt(l-1)=="-"&&u.raw.charAt(l-2)=="-"&&s==">"?u.raw=u.data=u.raw.replace(r._reTrimComment,""):(u.raw+=s,u.decoded=!0,this._tagStack.push(n.Comment))}else u.raw.indexOf("!")==0||u.raw.indexOf("?")==0?u.raw.indexOf("![CDATA[")===0?(this._current+="![CDATA[".length,r._reCDEnd.lastIndex=this._current,r._reCDEnd.test(this._buffer)?(this._next=r._reCDEnd.lastIndex-1,s=this._buffer.charAt(this._next),o=this._buffer.substring(this._current,this._next-2)):o="",delete u.name,u.type=n.Text,u.raw=o,u.data=u.raw,u.decoded=!0,r._reTags.lastIndex=r._reCDEnd.lastIndex,this._next=r._reTags.lastIndex-1,s=this._buffer.charAt(this._next)):u.type=n.Directive:u.name=="script"?(u.type=n.Script,u.data.charAt(u.data.length-1)!="/"&&this._tagStack.push(n.Script)):u.name=="/script"?u.type=n.Script:u.name=="style"?(u.type=n.Style,u.data.charAt(u.data.length-1)!="/"&&this._tagStack.push(n.Style)):u.name=="/style"&&(u.type=n.Style);u.name&&u.name.charAt(0)=="/"&&(u.data=u.name)}if(u.raw!=""||u.type!=n.Text){this._options.includeLocation&&!u.location&&(u.location=this.getLocation(u.type==n.Tag));if(u.type===n.Text||u.type===n.Comment)u.decoded||(u.data=e.decode(u.data),u.decoded=!0);this.parseAttribs(u),this._elements.push(u),u.type!=n.Text&&u.type!=n.Comment&&u.type!=n.Directive&&u.data.charAt(u.data.length-1)=="/"&&this._elements.push({raw:"/"+u.name,data:"/"+u.name,name:"/"+u.name,type:u.type})}this._parseState=s=="<"?n.Tag:n.Text,this._current=this._next+1,this._prevTagSep=s}this._options.includeLocation&&(this.getLocation(),this._location.row+=this._location.inBuffer,this._location.inBuffer=0,this._location.charOffset=0),this._buffer=this._current<=i?this._buffer.substring(this._current):"",this._current=0,this.writeHandler()},r.prototype.getLocation=function(t){var n,r=this._location,i=this._current-(t?1:0),s=t&&r.charOffset==0&&this._current==0;for(;r.charOffset<i;r.charOffset++)n=this._buffer.charAt(r.charOffset),n=="\n"?(r.inBuffer++,r.col=0):n!="\r"&&r.col++;return{line:r.row+r.inBuffer+1,col:r.col+(s?0:1)}},r.prototype.validateHandler=function(t){if(typeof t!="object")throw new Error("Handler is not an object");if(typeof t.reset!="function")throw new Error("Handler method 'reset' is invalid");if(typeof t.done!="function")throw new Error("Handler method 'done' is invalid");if(typeof t.writeTag!="function")throw new Error("Handler method 'writeTag' is invalid");if(typeof t.writeText!="function")throw new Error("Handler method 'writeText' is invalid");if(typeof t.writeComment!="function")throw new Error("Handler method 'writeComment' is invalid");if(typeof t.writeDirective!="function")throw new Error("Handler method 'writeDirective' is invalid")},r.prototype.writeHandler=function(t){t=!!t;if(this._tagStack.length&&!t)return;while(this._elements.length){var r=this._elements.shift();switch(r.type){case n.Comment:this._handler.writeComment(r);break;case n.Directive:this._handler.writeDirective(r);break;case n.Text:this._handler.writeText(r);break;default:this._handler.writeTag(r)}}},r.prototype.handleError=function(t){if(typeof this._handler.error!="function")throw t;this._handler.error(t)},u(i,s),i.prototype.done=function(){var t={},n,r=o.getElementsByTagName(function(e){return e=="rss"||e=="feed"},this.dom,!1);r.length&&(n=r[0]);if(n){if(n.name=="rss"){t.type="rss",n=n.children[0],t.id="";try{t.title=o.getElementsByTagName("title",n.children,!1)[0].children[0].data}catch(s){}try{t.link=o.getElementsByTagName("link",n.children,!1)[0].children[0].data}catch(s){}try{t.description=o.getElementsByTagName("description",n.children,!1)[0].children[0].data}catch(s){}try{t.updated=new Date(o.getElementsByTagName("lastBuildDate",n.children,!1)[0].children[0].data)}catch(s){}try{t.author=o.getElementsByTagName("managingEditor",n.children,!1)[0].children[0].data}catch(s){}t.items=[],o.getElementsByTagName("item",n.children).forEach(function(e,n,r){var i={};try{i.id=o.getElementsByTagName("guid",e.children,!1)[0].children[0].data}catch(s){}try{i.title=o.getElementsByTagName("title",e.children,!1)[0].children[0].data}catch(s){}try{i.link=o.getElementsByTagName("link",e.children,!1)[0].children[0].data}catch(s){}try{i.description=o.getElementsByTagName("description",e.children,!1)[0].children[0].data}catch(s){}try{i.pubDate=new Date(o.getElementsByTagName("pubDate",e.children,!1)[0].children[0].data)}catch(s){}t.items.push(i)})}else{t.type="atom";try{t.id=o.getElementsByTagName("id",n.children,!1)[0].children[0].data}catch(s){}try{t.title=o.getElementsByTagName("title",n.children,!1)[0].children[0].data}catch(s){}try{t.link=o.getElementsByTagName("link",n.children,!1)[0].attribs.href}catch(s){}try{t.description=o.getElementsByTagName("subtitle",n.children,!1)[0].children[0].data}catch(s){}try{t.updated=new Date(o.getElementsByTagName("updated",n.children,!1)[0].children[0].data)}catch(s){}try{t.author=o.getElementsByTagName("email",n.children,!0)[0].children[0].data}catch(s){}t.items=[],o.getElementsByTagName("entry",n.children).forEach(function(e,n,r){var i={};try{i.id=o.getElementsByTagName("id",e.children,!1)[0].children[0].data}catch(s){}try{i.title=o.getElementsByTagName("title",e.children,!1)[0].children[0].data}catch(s){}try{i.link=o.getElementsByTagName("link",e.children,!1)[0].attribs.href}catch(s){}try{i.description=o.getElementsByTagName("summary",e.children,!1)[0].children[0].data}catch(s){}try{i.pubDate=new Date(o.getElementsByTagName("updated",e.children,!1)[0].children[0].data)}catch(s){}t.items.push(i)})}this.dom=t}i.super_.prototype.done.call(this)},s._emptyTags={area:1,base:1,basefont:1,br:1,col:1,frame:1,hr:1,img:1,input:1,isindex:1,link:1,meta:1,param:1,embed:1},s.reWhitespace=/^\s*$/,s.prototype.dom=null,s.prototype.reset=function(){this.dom=[],this._done=!1,this._tagStack=[],this._tagStack.last=function(){return this.length?this[this.length-1]:null}},s.prototype.done=function(){this._done=!0,this.handleCallback(null)},s.prototype.writeTag=function(t){this.handleElement(t)},s.prototype.writeText=function(t){if(this._options.ignoreWhitespace&&s.reWhitespace.test(t.data))return;this.handleElement(t)},s.prototype.writeComment=function(t){this.handleElement(t)},s.prototype.writeDirective=function(t){this.handleElement(t)},s.prototype.error=function(t){this.handleCallback(t)},s.prototype._options=null,s.prototype._callback=null,s.prototype._done=!1,s.prototype._tagStack=null,s.prototype.handleCallback=function(t){if(typeof this._callback!="function"){if(t)throw t;return}this._callback(t,this.dom)},s.prototype.isEmptyTag=function(e){var t=e.name.toLowerCase();return t.charAt(0)=="/"&&(t=t.substring(1)),this._options.enforceEmptyTags&&!!s._emptyTags[t]},s.prototype.handleElement=function(t){this._done&&this.handleCallback(new Error("Writing to the handler after done() called is not allowed without a reset()")),this._options.verbose||(delete t.raw,(t.type=="tag"||t.type=="script"||t.type=="style")&&delete t.data);if(!this._tagStack.last())t.type!=n.Text&&t.type!=n.Comment&&t.type!=n.Directive?t.name.charAt(0)!="/"&&(this.dom.push(t),this.isEmptyTag(t)||this._tagStack.push(t)):this.dom.push(t);else if(t.type!=n.Text&&t.type!=n.Comment&&t.type!=n.Directive)if(t.name.charAt(0)=="/"){var r=t.name.substring(1);if(!this.isEmptyTag(t)){var i=this._tagStack.length-1;while(i>-1&&this._tagStack[i--].name!=r);if(i>-1||this._tagStack[0].name==r)while(i<this._tagStack.length-1)this._tagStack.pop()}}else this._tagStack.last().children||(this._tagStack.last().children=[]),this._tagStack.last().children.push(t),this.isEmptyTag(t)||this._tagStack.push(t);else this._tagStack.last().children||(this._tagStack.last().children=[]),this._tagStack.last().children.push(t)};var o={testElement:function(t,n){if(!n)return!1;for(var r in t)if(r=="tag_name"){if(n.type!="tag"&&n.type!="script"&&n.type!="style")return!1;if(!t.tag_name(n.name))return!1}else if(r=="tag_type"){if(!t.tag_type(n.type))return!1}else if(r=="tag_contains"){if(n.type!="text"&&n.type!="comment"&&n.type!="directive")return!1;if(!t.tag_contains(n.data))return!1}else if(!n.attribs||!t[r](n.attribs[r]))return!1;return!0},getElements:function(t,n,r,i){function a(e){return function(t){return t==e}}r=r===undefined||r===null||!!r,i=isNaN(parseInt(i))?-1:parseInt(i);if(!n)return[];var s=[],u;for(var f in t)typeof t[f]!="function"&&(t[f]=a(t[f]));o.testElement(t,n)&&s.push(n);if(i>=0&&s.length>=i)return s;if(r&&n.children)u=n.children;else{if(!(n instanceof Array))return s;u=n}for(var l=0;l<u.length;l++){s=s.concat(o.getElements(t,u[l],r,i));if(i>=0&&s.length>=i)break}return s},getElementById:function(t,n,r){var i=o.getElements({id:t},n,r,1);return i.length?i[0]:null},getElementsByTagName:function(t,n,r,i){return o.getElements({tag_name:t},n,r,i)},getElementsByTagType:function(t,n,r,i){return o.getElements({tag_type:t},n,r,i)}};return t.Parser=r,t.DefaultHandler=s,t.RssHandler=i,t.ElementType=n,t.DomUtils=o,t}()}),define("runtime-browser/domutils",["require","datajslib!htmlparser","datajslib!underscore"],function(e,t,n){var r=t.DomUtils,i=function(e){return e?e.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;"):""},s=function(e,t,r){var i="";return e?(e.children?n.each(e.children,function(e){i+=s(e,!0)}):e.type==="text"?i+=e.data:!r&&e.name==="img"&&e.attribs?i+=e.attribs.alt||e.attribs.title||"":e.name==="br"&&(i+="\n"),t?i:i.trim()):i},o=function(e,t){if(t&&t.children){var n=r.getElementsByTagName(e,t.children,!1,1);if(n&&n.length>0)return n[0]}return null},u=function(e,t){var n=o(e,t);return n?s(n):""},a=function(e,t){var r=[];if(!e)return[];switch(e.type){case"text":r.push(e.data);break;case"directive":r.push("<!"+e.data+">");break;case"comment":r.push("<!--"+i(e.data)+"-->");break;case"tag":r.push("<"+e.name),e.attribs&&n.each(e.attribs,function(e,t){r.push(" "+t+'="'+i(e)+'"')}),e.children&&(r.push(">"),n.each(e.children,function(e){r=r.concat(a(e,!0))}),r.push("</"+e.name+">"))}return t?r:r.join("")},f=function(e,n){n=n||function(){};var r=null,i=null;if(e["joshfire:dom"])return n(null,e["joshfire:dom"]);r=new t.DefaultHandler(function(t,r){return e["joshfire:dom"]=r||[],t?n():n(null,e["joshfire:dom"])},{verbose:!1,ignoreWhitespace:!1}),i=new t.Parser(r),i.parseComplete(e.articleBody||e.content)},l=n.extend({},r,{getTextContent:s,getElementChild:o,getElementChildText:u,toHTML:a,parseHTML:f});return l}),define("runtime-browser/iso8601",[],function(){var e=/^([0-9]{4})(\-([0-9]{2})(\-([0-9]{2})(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?(Z|(([\-+])([0-9]{2}):([0-9]{2})))?)?)?)?$/,t=/^PT([0-9]+)H([0-9]+)M([0-9]+)S$/;return{toDate:function(t){var n=null,r=0,i=null,s=null;return t?(n=t.trim().match(e),n?(i=new Date(2010,0,1),n[13]?(i.setUTCFullYear(n[1],0,1),i.setUTCHours(0,0,0,0),n[3]&&i.setUTCMonth(n[3]-1),n[5]&&i.setUTCDate(n[5]),n[7]&&i.setUTCHours(n[7]),n[8]&&i.setUTCMinutes(n[8]),n[10]&&i.setUTCSeconds(n[10]),n[12]&&i.setUTCMilliseconds(Number("0."+n[12])*1e3),n[14]&&(r=Number(n[16])*60+Number(n[17]),r*=n[15]=="-"?1:-1,s=i.getTime()+r*60*1e3,i.setTime(s))):(i.setFullYear(n[1],0,1,0,0,0,0),n[3]&&i.setMonth(n[3]-1),n[5]&&i.setDate(n[5]),n[7]&&i.setHours(n[7]),n[8]&&i.setMinutes(n[8]),n[10]&&i.setSeconds(n[10]),n[12]&&i.setMilliseconds(Number("0."+n[12])*1e3)),i):null):null},fromString:function(e){var t=null;return e?(t=new Date(e),isNaN(t.getTime())&&(t=this.toDate(e)),this.fromDate(t)):""},fromDate:function(e){var t=function(e){return e<10?"0"+e:e},n=function(e){return e<10?"00"+e:e<100?"0"+e:e};return!!e&&e instanceof Date&&!isNaN(e.getTime())?e.getUTCFullYear()+"-"+t(e.getUTCMonth()+1)+"-"+t(e.getUTCDate())+"T"+t(e.getUTCHours())+":"+t(e.getUTCMinutes())+":"+t(e.getUTCSeconds())+(e.getUTCMilliseconds()!==0?"."+n(e.getUTCMilliseconds()):"")+"Z":""},fromDateNoTime:function(e){var t=function(e){return e<10?"0"+e:e};return!!e&&e instanceof Date&&!isNaN(e.getTime())?!e||isNaN(e.getTime())?"":e.getHours()===0&&e.getMinutes()===0&&e.getSeconds()===0&&e.getMilliseconds()===0?e.getFullYear()+"-"+t(e.getMonth()+1)+"-"+t(e.getDate()):e.getUTCFullYear()+"-"+t(e.getUTCMonth()+1)+"-"+t(e.getUTCDate()):""},toDuration:function(e){if(!e)return 0;var n=e.match(t);if(!n)return 0;var r=0;return n[2]&&(r+=Number(n[2])*3600*1e3),n[3]&&(r+=Number(n[3])*60*1e3),n[4]&&(r+=Number(n[4])*1e3),r},fromDuration:function(e){var t=Math.round(e/1e3),n=Math.floor(t/3600),r=0;return t-=n*3600,r=Math.floor(t/60),t-=r*60,"PT"+n+"H"+r+"M"+t+"S"}}}),define("runtime-browser/uri",[],function(){var e=function(e){this.value=e||"",this.length=this.value.length};return e.prototype={toString:function(){return this.value.toString()},valueOf:function(){return this.value},defrag:function(){var t=this.value.indexOf("#");return t<0?this:new e(this.value.slice(0,t))},scheme:function(){var e=this.value.match(/^[a-z0-9\-\+\.]+:/i);return e?e.shift():null},heirpart:function(){var t=this.value,n=t.indexOf("?");return n>=0?t=t.substring(0,n):(n=t.indexOf("#"),n>=0&&(t=t.substring(0,n))),n=this.scheme(),n&&(t=t.slice(n.length)),new e.HeirPart(t)},querystring:function(){var e=this.value.indexOf("?");if(e<0)return null;var t=this.value.indexOf("#");return t<0?this.value.slice(e):this.value.substring(e,t)},fragment:function(){var e=this.value.indexOf("#");return e<0?null:this.value.slice(e)},isAbsolute:function(){return this.scheme()&&this.heirpart().toString()&&!this.fragment()},toAbsolute:function(){if(!this.scheme()||!this.heirpart().toString())throw"URI must have a scheme and a hierpart.";var e=this.resolveReference(this);return e.defrag()},removeDotSegments:function(e){var t="",n=null;while(e.length>0)e.substr(0,3)==="../"||e.substr(0,2)==="./"?e=e.slice(e.indexOf("/")):e==="/."?e="/":e.substr(0,3)==="/./"?e=e.slice(2):e.substr(0,4)==="/../"||e==="/.."?(e==="/.."?e="/":e=e.slice(3),n=t.lastIndexOf("/"),n>=0?t=t.substring(0,n):t=""):e.substr(0,2)===".."||e.substr(0,1)==="."?(e=e.slice(e.indexOf(".")),n=e.indexOf("."),n>=0&&(e=e.slice(n))):(e.substr(0,1)==="/"&&(t+="/",e=e.slice(1)),n=e.indexOf("/"),n<0?(t+=e,e=""):(t+=e.substring(0,n),e=e.slice(n)));return t},resolveReference:function(t){typeof t=="string"&&(t=new e(t));if(t instanceof e){var n={scheme:"",authority:"",path:"",query:"",fragment:""},r=null;return t.scheme()?(n.scheme=t.scheme(),r=t.heirpart().authority(),n.authority+=r?"//"+r:"",n.path=this.removeDotSegments(t.heirpart().path()),r=t.querystring(),n.query+=r?r:""):(r=t.heirpart().authority(),r?(n.authority=r?"//"+r:"",n.path=this.removeDotSegments(t.heirpart().path()),r=t.querystring(),n.query+=r?r:""):(r=t.heirpart().path(),r===""?(n.path=this.heirpart().path(),r=t.querystring(),r?n.query+=r?r:"":(r=this.querystring(),n.query+=r?r:"")):(r.substring(0,1)==="/"?n.path=this.removeDotSegments(r):(this.heirpart().path()?(r=this.heirpart().path().lastIndexOf("/"),r>=0&&(n.path=this.heirpart().path().substring(0,++r)),n.path+=t.heirpart().path()):n.path="/"+r,n.path=this.removeDotSegments(n.path)),r=t.querystring(),n.query+=r?r:""),r=this.heirpart().authority(),n.authority=r?"//"+r:""),n.scheme=this.scheme()),r=t.fragment(),n.fragment=r?r:"",new e(n.scheme+n.authority+n.path+n.query+n.fragment)}throw"Expected an URI or a String"}},e.HeirPart=function(e){this.value=e||"",this.length=this.value.length},e.HeirPart.prototype={toString:function(){return this.value.toString()},valueOf:function(){return this.value},authority:function(){if("//"!==this.value.substring(0,2))return null;var t=this.value.slice(2),n=t.indexOf("/");return n>=0&&(t=t.substr(0,n)),new e.Authority(t)},path:function(){var e=this.authority();return e?this.value.slice(e.length+2):this.toString()}},e.Authority=function(e){this.value=e||"",this.length=this.value.length},e.Authority.prototype={toString:function(){return this.value.toString()},valueOf:function(){return this.value},userinfo:function(){var e=this.value.indexOf("@");return e<0?null:this.value.substr(0,e)},host:function(){var e=this.value,t=e.indexOf("@");t>=0&&(e=e.slice(++t));if(e.indexOf("[")===0){t=e.indexOf("]");if(t>0)return e.substring(0,++t)}return t=e.lastIndexOf(":"),t>=0?e.substring(0,t):e},port:function(){var e=this.value,t=e.indexOf("@");return t>=0&&(e=e.slice(t)),e.indexOf("[")===0&&(t=e.indexOf("]"),t>0&&(e=e.slice(t))),t=e.lastIndexOf(":"),t<0?null:(e=e.slice(++t),e.length===0?null:e)}},e}),define("runtime-browser/errors/paramerror",["./error"],function(e){var t=function(t,n){e.call(this,t,n),this.name="invalid parameter"};return t.prototype=new e,t}),define("runtime-browser/rssconverter",["require","datajslib!http","datajslib!underscore","datajslib!htmlparser","datajslib!domutils","datajslib!iso8601","datajslib!uri","datajslib!errors/paramerror","datajslib!errors/proxyerror"],function(e,t,n,r,i,s,o,u,a){return{fetch:function(e,s){var a=e&&e.filter?e.filter:{},f=null,l=!1;if(!a.url)return s(new u("No RSS feed URL received"));f=a.url,f.search(/^http(s?)\:\/\//i)===-1&&(f="http://"+f);var c=new r.DefaultHandler(function(e,t){var r=null,s=i.getElementsByTagName("base",t,!0,1);s&&s.length&&(s=s[0]),s&&s.attribs&&s.attribs.href?r=(new o(s.attribs.href)).defrag():r=(new o(f)).defrag();var u=i.getElementsByTagName("link",t,!0),a=n.find(u,function(e){return e.attribs&&e.attribs.href&&e.attribs.rel&&e.attribs.type&&e.attribs.rel==="alternate"&&e.attribs.type==="application/rss+xml"?!0:!1});a&&(l=r.resolveReference(a.attribs.href).toString())},{ignoreWhitespace:!0,verbose:!1,enforceEmptyTags:!0});t.request({url:f,dataType:"text"},function(e,n){if(e)return s(e);if(!n)return s();var i=new r.Parser(c);i.parseComplete(n);if(!l)return s(null,n);t.request({url:l,dataType:"text"},function(e,t){return e?s(e):t?s(null,t):s()})})},process:function(e,t,u){var f=t&&t.filter?t.filter:{};if(!e)return u(null,{entries:[]});var l=function(e){try{return e?parseInt(e,10):null}catch(t){return null}},c=new o("http://example.org"),h=null,p=null,d=new r.DefaultHandler(function(e,t){if(e||!t||!t[0]){h=null,p=null;return}h="",n.each(t,function(e){h&&(h+=" "),h+=i.getTextContent(e)});var r=i.getElementsByTagName("img",t,!0),s=null;r&&(s=n.find(r,function(e){return e.attribs&&e.attribs.src&&e.attribs.src.search(/\.doubleclick\.net/)===-1&&e.attribs.src.search(/\.pheedo\.com/)===-1&&e.attribs.src.search(/\.feedburner\.com/)===-1&&e.attribs.src.search(/\.fsdn\.com/)===-1}),s&&(p={"@type":"ImageObject",itemType:"ImageObject",contentURL:c.resolveReference(s.attribs.src).toString()},s.attribs.alt&&(p.name=s.attribs.alt),l(s.attribs.width)&&(p.width=l(s.attribs.width)),l(s.attribs.height)&&(p.height=l(s.attribs.height))));return},{ignoreWhitespace:!0,verbose:!1,enforceEmptyTags:!0}),v=new r.Parser(d),m=new r.DefaultHandler(function(e,t){if(e)return u(new a("RSS feed could not be parsed as XML"));var r=i.getElementsByTagName("rdf:RDF",t,!1,1),o="rdf";if(!r||r.length===0)r=i.getElementsByTagName("rss",t,!1,1),o="rss";if(!r||r.length===0)r=i.getElementsByTagName("feed",t,!1,1),o="atom";if(!r||r.length===0)return u(new a("Not a valid RSS feed: root element not found"));r=r[0];var c=[],d=r;if(o==="rss"){d=i.getElementChild("channel",r);if(!d)return u(new a('Not a valid RSS feed: "channel" tag not found'))}c=i.getElementsByTagName(o==="atom"?"entry":"item",d.children,!1);var m=n.map(c,function(e){var t={"@type":"BlogPosting",itemType:"BlogPosting",name:i.getElementChildText("title",e)},r=null,u=null,a=0;u=i.getElementChildText("link",e),!u&&e.attribs&&e.attribs["rdf:about"]&&(u=e.attribs["rdf:about"]);if(!u&&o==="atom"){var d=n.find(n.toArray(i.getElementsByTagName("link",e.children,!1)),function(e){return e.attribs.rel==="alternate"});d&&d.attribs&&(u=d.attribs.href)}if(u){if(u.indexOf("?")>=0){var m=u.substring(u.indexOf("?")+1);for(a=0;a<2;a++)m=m.replace(/&utm_([a-z]+)(\=[^&]*)?(?=&|$)|^utm_([a-z]+)(\=[^&]*)?(&|$)/g,"");var g=u.indexOf("?")+(m&&m.substring(0,1)!=="#"?1:0);u=u.substring(0,g)+m}t.url=u}u=i.getElementChildText(o==="atom"?"summary":"description",e),u&&(v.parseComplete(u),h&&(u=h)),u&&(t.description=u),u=i.getElementChildText(o==="atom"?"content":"content:encoded",e),u||(u=i.getElementChildText(o==="atom"?"summary":"description",e)),u&&(t.articleBody=u),u=i.getElementChildText("dc:date",e),u||(u=i.getElementChildText("pubDate",e)),u?t.datePublished=s.fromString(u):o==="atom"&&(u=i.getElementChildText("published",e),u&&(t.datePublished=u.replace(" ","T"))),o==="atom"&&(u=i.getElementChildText("updated",e),u&&(t.dateModified=u.replace(" ","T"))),u=i.getElementChildText("dc:creator",e),u||(u=i.getElementChildText("author",e)),u&&(t.author={"@type":"Person",itemType:"Person",name:u}),u=i.getElementChildText("dc:subject",e),u&&(t.about={"@type":"Thing",itemType:"Thing",name:u}),f.disableThumbnailExtraction||(p=null,v.parseComplete(t.articleBody),p&&(t.image=p)),t.image||(u=i.getElementChild("media:thumbnail",e),u&&u.attribs&&u.attribs.url&&(t.image={"@type":"ImageObject",itemType:"ImageObject",contentURL:u.attribs.url},l(u.attribs.width)&&(t.image.width=l(u.attribs.width)),l(u.attribs.height)&&(t.image.height=l(u.attribs.height)))),t.image||(r=i.getElementsByTagName("media:content",e.children,!1),u=n.find(r,function(e){return e.attribs&&e.attribs.url&&(e.attribs.medium&&e.attribs.medium==="image"||e.attribs.type&&e.attribs.type.indexOf("image/")===0)}),u&&(t.image={"@type":"ImageObject",itemType:"ImageObject",contentURL:u.attribs.url},l(u.attribs.width)&&(t.image.width=l(u.attribs.width)),l(u.attribs.height)&&(t.image.height=l(u.attribs.height))));for(a=0;a<e.children.length;a++)u=e.children[a],u.type==="tag"&&(u.name.indexOf("jf:")===0||u.name.indexOf("joshfire:")===0)&&(t[u.name]=i.getTextContent(u));c=i.getElementsByTagName("category",e);if(c){var y=[];for(a in c){if(!c.hasOwnProperty(a))continue;if(c[a]&&c[a].children&&c[a].children.length)for(var b in c[a].children){if(!c[a].children.hasOwnProperty(b))continue;y.push(c[a].children[b].data)}}t.keywords=y}return t});return u(null,{entries:m})},{ignoreWhitespace:!0,verbose:!1,enforceEmptyTags:!1}),g=new r.Parser(m);g.parseComplete(e)},find:function(e,t){var n=this;n.fetch(e,function(r,i){if(r)return t(r,null);n.process(i,e,function(e,n){return t(e,n)})})}}}),define("databases/feed/rss",["datajslib!rssconverter"],function(e){return{desc:{options:{schema:{url:{title:"URL",description:"RSS feed or HTML page that links to the RSS feed",required:!0,type:"string"},disableThumbnailExtraction:{title:"Thumbnail extraction",description:"The datasource uses the first image that appears in the article as thumbnail by default, falling back to RSS properties. Check this box to use only RSS properties (media:thumbnail and media:content).",type:"boolean","default":!1}},form:["url",{type:"advancedfieldset",items:[{key:"disableThumbnailExtraction",inlinetitle:"Use only RSS properties to extract thumbnails"}]}]},runtimes:["hosted","browser","nodejs","win8"],corsRequest:!0,maxAge:120,outputType:"BlogPosting"},fetch:e.fetch,process:function(t,n,r){e.process(t,n,function(e,t){return t&&t.entries&&(n.limit||(n.limit=20),n.skip>0?n.limit>0?t.entries=t.entries.slice(n.skip,n.skip+n.limit):t.entries=t.entries.slice(n.skip):n.limit>0&&(t.entries=t.entries.slice(0,n.limit))),r(e,t)})},find:e.find}}),function(){var e=this,t=function(e,t,n){if(!e||!t)return e;var r={},o=function(e,t){var i=null,u=null;if(!e||e!==Object(e))return null;if(e.required){if(e["default"]){r[t]=e["default"];return}e.type==="object"||e.properties?r[t]={}:e.type==="array"||e.items?r[t]=[]:e.type==="string"?r[t]="":e.type==="number"||e.type==="integer"?r[t]=0:e.type==="boolean"?r[t]=!1:r[t]={}}else if(e["default"]&&n){r[t]=e["default"];return}i=e.properties;if(i)for(var a in i)i.hasOwnProperty(a)&&o(i[a],t+"."+a);e.items&&(u=e.items,s(u)&&(u=u[0]),o(u,t+"[]"))};o(t,"");for(var u in r)r.hasOwnProperty(u)&&i(e,u,r[u])},n=function(e,t){var n=t.replace(/\./g,".properties.").replace(/\[.*\](\.|$)/g,".items$1"),i=r(e,n);return i?i["default"]:null},r=function(e,t){var n=e,r=t.split("."),i=null,s=null,o=/\[([0-9]*)\](?:\.|$)/;for(var u=0;u<r.length;u++){if(typeof n!="object")return null;i=r[u],s=i.match(o);if(s){i=i.replace(o,"");if(!_.isArray(n[i]))return null;n=n[i][parseInt(s[1],10)]}else n=n[i]}return n},i=function(e,t,n){var r=t.split("."),i=function(e,t,n){var r=null,o=/\[([0-9]*)\]$/,u=t.shift(),a=0;if(t.length>0){r=u.match(o);if(r){u=u.replace(o,""),s(e[u])||(e[u]=[]),e=e[u];if(r[1]!=="")a=parseInt(r[1],10),e[a]||(e[a]={}),i(e[a],t,n);else for(var f=0;f<e.length;f++)i(e[f],t,n);return}typeof e[u]!="object"&&(e[u]={}),e=e[u],i(e,t,n)}else r=u.match(o),r?(u=u.replace(o,""),s(e[u])||(e[u]=[]),a=parseInt(r[1],10),e[u][a]||(e[u][a]=n)):e[u]||(e[u]=n)};r[0]||r.shift(),i(e,r,n)},s=Array.isArray||function(e){return Object.prototype.toString.call(e)=="[object Array]"};typeof define!="undefined"?define("runtime-browser/jsonform-defaults",[],function(){return{setDefaultValues:t}}):typeof module!="undefined"&&module.exports?module.exports={setDefaultValues:t}:(e.JSONForm=e.JSONForm||{},e.JSONForm.setDefaultValues=t,e.JSONForm.setObjKey=i,e.JSONForm.getSchemaKeyDefaultValue=n)}();;
;

define("addons/splashscreen/loaded",[],function(){return function(a){return{generate:function(a,b){try{var c=document.getElementsByClassName("_joshfire_factory_splashscreen");c&&c.length&&(c[0].style.opacity="0",setTimeout(function(){c[0].style.display="none"},700))}catch(d){}b()}}}})

;(function(a,b){var c=function(c,e){var f=null,g=null,h=0;return!c||!a||!a.config||!a.config.datasources||!a.config.datasources[c]?null:(e=e||{},f=a.config.datasources[c],e.params&&(f=d(f,e.params)),b(["datajslib!collection"],function(a){if(Object.prototype.toString.call(f)==="[object Array]")if(typeof e.child!="undefined")g=a.getCollection(f[e.child]),g.name=f[e.child].name,g.config=f[e.child];else{g={children:[],find:function(a,b){var c=f.length,d=!1,e=[],h=0,i=function(a,f){c-=1;if(d)return;a&&(d=!0),f&&e.push(f);if(a||c===0)return b(a,{entries:e})};for(h=0;h<g.children.length;h++)g.children[h].find(a,i)}};for(h=0;h<f.length;h++)g.children[h]=a.getCollection(f[h]),g.children[h].name=f[h].name,g.children[h].config=f[h]}else g=a.getCollection(f),g.name=f.name,g.config=f},null,!0),g)},d=function(a,b){if(!a)return null;var c=function(a){return a.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&")},d=0,e=JSON.stringify(a,null,2),f=null;for(d in b)b.hasOwnProperty(d)&&(f=new RegExp("{{"+c(d)+"}}","g"),e=e.replace(f,b[d]));return JSON.parse(e)};a.getDataSource=c})(window.Joshfire.factory,require);
(function(a,b){a(["datajslib!utils/getthumbnail","datajslib!utils/getthumbnailurl"],function(a,c){b.schemaio=b.schemaio||{},b.schemaio.utils=b.schemaio.utils||{},b.schemaio.utils.getThumbnail=a,b.schemaio.utils.getThumbnailUrl=c},null,!0)})(require,Joshfire);
(function(a,b){var c=function(a){return a=a||document.getElementById("body")||document.documentElement,Object.prototype.toString.call(a)==="[object String]"?document.querySelector?document.querySelector(a):document.getElementById(a):a},d=function(b){var d=[],f=null,g,h,i=[];if(!b)return[];d=a.config.addons||[];for(g=0;g<d.length;g++){f=d[g];for(h=0;h<f.hooks.length;h++)if(b===f.hooks[h]){i.push(new e(f,b));break}}return i.run=i.render=function(a,b,d){var e=0,f=c(a),g=null;d=d||function(){return};var h=0,j=function(a){a&&(g=a),h+=1;if(h===i.length)return d(g)};b=b||{},b.replaceContent&&delete b.replaceContent;for(e=0;e<i.length;e++)i[e].render(f,b,j)},i.startActivity=function(a,b,c){a=a||{},b=b||function(){return},c=c||function(){return};if(i.length>0)i[0].startActivity(a,b,c);else if(c)return c("No installed add-on for the given intent")},i},e=function(a,d){var e=this,f=!1,g=[],h=function(a){f?a():g.push(a)};this.config=a,this.intent=a.intent,this.run=this.render=function(a,b,d){b=b||{},d=d||function(){return},h(function(){if(!e.addon)return d("Add-on not available");if(!e.addon.generate)return d("Invalid add-on, no generate function");var f=c(a),g=f;e.addon.generate(b,function(a,c){return a?d(a):(c?(b.replaceContent?e.setContent(g,c):(g=document.createElement("div"),e.setContent(g,c),f.appendChild(g)),e.addon.enhance&&e.addon.enhance(g,b)):b.replaceContent&&e.setContent(g,""),d())})})},this.generate=function(a,b){a=a||{},b=b||function(){return},h(function(){if(!e.addon)return b("Add-on not available");if(!e.addon.generate)return b("Invalid add-on, no generate function");e.addon.generate(a,b)})},this.setContent=function(a,b){function e(a,b){b(a);for(var c in a.childNodes)e(a.childNodes[c],b)}function f(a,b){return a.nodeName&&a.nodeName.toUpperCase()===b.toUpperCase()}function g(a){var b=a.text||a.textContent||a.innerHTML||"",c=a.getAttribute("src"),d=document.getElementsByTagName("head")[0]||document.documentElement,e=document.createElement("script");e.type="text/javascript",c&&e.setAttribute("src",c),b&&e.appendChild(document.createTextNode(b)),d.insertBefore(e,d.firstChild),a.parentNode&&a.parentNode.removeChild(a)}var d=c(a);d.innerHTML=(""+b).trim();var h=[],i=null;e(d,function(a){a.nodeName&&a.nodeName.toUpperCase()==="SCRIPT"&&(!a.type||a.type==="text/javascript")&&h.push(a)});for(i in h)g(h[i])},this.enhance=function(a,b){b=b||{};if(!this.addon)return;if(!this.addon.enhance)return;var d=c(a);this.addon.enhance(d,b)},this.startActivity=function(a,b,c){a=a||{},b=b||function(){return},c=c||function(){return},h(function(){if(!e.addon)return c("Add-on not available");if(!e.addon.startActivity)return c("Add-on does not implement a startActivity function");e.addon.startActivity(a,b,c)})},b(["addons/"+this.config.name+"/"+d],function(a){e.addon=a({options:e.config.options||{},deploy:Joshfire.factory.config.deploy}),f=!0;for(var b=0,c=g.length;b<c;b++)g.pop()()})};a.getAddOns=d})(window.Joshfire.factory,require);
})();
